# vi: ts=8 sw=8 sts=8

package Tongues::Deutsch;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Western European CP1252 / ISO 8859-1 / Latin-1

# Extra characters (ISO & Windows)
#   
#     
# Extra characters (Mac)
#   
#     

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::WestEuroWin;

$charset = new Charsets::WestEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun
 # Noun gender
 [ 'in',	'',	'',		'',	'n' ],	# regular enough?
 [ 'in',	'e',	'',		'',	'n' ],	# regular enough?
 #  Noun plural - Five plural groups
 #   1. Unchanged or with an umlaut
 #    masc. in -el -en -er
 #    neut. in -chen -el -en -er -sel (unchanged)
 #    fem.  only Mutter, Tochter
 #[ '',		'',	'',		's',	'n' ],
 [ '',		'',	'',		's',	'n' ],

 #   2. -s
 #    most Germanized English words
 #    all nouns ending in a vowel except e
 [ 's',		'',	'',		's',	'n' ],	# also genitive

 #   3. -(e)n
 #    fem. and masc. nouns in -e add -n
 #    nearly all fem. nouns no matter the ending
 [ 'en',	'',	'',		's',	'n' ],	# weak nouns, all cases
 [ 'n',		'',	'',		's',	'n' ],

 #   4. Umlaut and -er
 #    mainly monosyllabic neut. nouns
 #    singular o u a become   
 #    a few masc. words
 [ 'er',	'',	'',		's',	'n' ],
 [ 'er',	'',	'',		's',	'n' ],

 #   5. Umlaut and -e
 #    mainly monosyllabic masc. nouns
 #    singular o u a *sometimes* become   
 #    only a few monosyllabic fem. nouns
 #    no neuter nouns take umlaut+e, but masc. and monosyllabic neut. take -e
 [ 'e',		'',	'',		's',	'n' ],
 [ 'e',	'',	'',		's',	'n' ],

 [ 'nisse',	'nis',	'',		's',	'n' ],
 [ 'nnen',	'n',	'',		's',	'n' ],

 [ 'ens',	'',	'',		'',	'n' ],	# weak genitive singular
 [ 'ns',	'',	'',		'',	'n' ],

 [ 'nisse',	'nis',	'',		's',	'n' ],
 [ 'nissen',	'nis',	'',		's',	'n' ],
 [ 'nisses',	'nis',	'',		'',	'n' ],	# genitive singular
 # Adjectives
 # Adjective cases
 [ 'er',	'',	'',		'',	'a' ],	# nm, df, gf, gp
 [ 'e',		'',	'',		'',	'a' ],	# nf, np, af, ap
 [ 'es',	'',	'',		'',	'a' ],	# nn, an
 [ 'en',	'',	'',		'',	'a' ],	# am, dp, gm, gn
 [ 'em',	'',	'',		'',	'a' ],	# dm, dn

 # Adjective comparitive
 #[ 'er',	'',	'more ',	'',	'a' ],

 # Adjective superlative
 [ 'st',	'',	'most ',	'',	'a' ],
 [ 'est',	'',	'most ',	'',	'a' ],
 # Verb past participle
 [ 'ge-t',	'en',	'',		'ed',	'v' ],	# weak
 [ 'ge-en',	'en',	'',		'ed',	'v' ],	# strong
 # Verb gerund
 # Verb present participle
 [ 'end',	'en',	'',		'ing',	'v' ],	# doing
 # Verb present
 [ 'e',		'en',	'',		'',	'v' ],	# I do, you (imperitive) do
 [ 'st',	'en',	'',		'',	'v' ],	# you (informal) do
 [ 't',		'en',	'',		's',	'v' ],	# he, you (plural) do
 #[ 'en',	'en',	'',		'',	'v' ],	# we, you (formal), they do
 # Verb past
 [ 'te',	'en',	'',		'ed',	'v' ],	# I, he did
 [ 'test',	'en',	'',		'ed',	'v' ],	# you (informal) did
 [ 'ten',	'en',	'',		'ed',	'v' ],	# we, you (formal), they did
 [ 'tet',	'en',	'',		'ed',	'v' ],	# you (plural) did
 # Verb present subjunctive
 [ 'est',	'en',	'(may) ',	'',	'v' ],	# you (informal) (may) do
 [ 'et',	'en',	'(may) ',	'',	'v' ],	# you (plural) (may) do

];

#Fields:
# x -> translatioin			all words
# r -> root				irregular declensions
# t -> type:				all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjugation
#	adv -> adverb
# g -> gender:				nouns, pronouns, adjectives
#	m -> masculine			default
#	f -> feminine
#	n -> neuter
# n -> number:				articles, pronouns, verbs, adjectives, nouns
#	s -> singular			default
#	p -> plural
# p -> person:				pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person			default
#   -> plural:				nouns
#	n -> add +n suffix
#	en -> add +en suffix
#	er -> add +er suffix
#	 -> umlaut
#	er -> umlaut + add +er suffix
#	s -> add +s suffix
#	se -> add +se suffix (if ending is is)
#	- -> no change
# c -> conjugation:			verbs
#	p -> present
#	s -> past
#	inf -> infinitive		default
#	pp -> past participle
#	pr -> present participle
#	g -> gerund
#	imp -> imperative
#   -> case:				nouns
#	n -> nominative
#	a -> accusative
#	d -> dative
#	g -> genitive

# German to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 'der'		=> { 'x' => 'the',
		     '#' => 'nm, df, gf, gp',
 		     't' => 'art' },
 'die'		=> { 'x' => 'the',
		     '#' => 'nf, np, af, ap',
 		     't' => 'art' },
 'das'		=> { 'x' => 'the',
		     '#' => 'nn, an',
 		     't' => 'art',
		     'g' => 'n' },
 'den'		=> { 'x' => 'the',
		     '#' => 'am, dp',
 		     't' => 'art' },
 'dem'		=> { 'x' => 'the',
		     '#' => 'dm, dn',
 		     't' => 'art' },
 'des'		=> { 'x' => 'the',
		     '#' => 'gm, gn',
 		     't' => 'art' },

 #   Indefinite articles
 'ein'		=> { 'x' => 'a',
		     '#' => 'an',
 		     't' => 'art' },
 'eine'		=> { 'x' => 'a',
		     '#' => 'an',
 		     't' => 'art' },
 'einen'	=> { 'x' => 'a',
		     '#' => 'an',
 		     't' => 'art' },
 'einem'	=> { 'x' => 'a',
		     '#' => 'an',
 		     't' => 'art' },
 'einer'	=> { 'x' => 'a',
		     '#' => 'an',
 		     't' => 'art' },
 'eines'	=> { 'x' => 'a',
		     '#' => 'an',
 		     't' => 'art' },

 #  Pronouns & possessive adjectives
 #   1st person
 'ich'		=> { 'x' => 'i',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'mir'		=> { 'x' => 'me' },
 'mich'		=> { 'x' => 'me' },
 'mein'		=> { 'x' => 'mine',
		     '#' => 'poss adj.',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'wir'		=> { 'x' => 'we',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 'p' },
 #   2nd person
 'du'		=> { 'x' => 'you',
		     '#' => 'informal',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 's' },
 'dir'		=> { 'x' => 'you' },
 'dich'		=> { 'x' => 'you' },
 'dein'		=> { 'x' => 'your',
		     '#' => 'poss adj.',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 's' },
 #   3rd person
 'er'		=> { 'x' => 'he',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'm' },
 'sie'		=> { 'x' => 'you',
		     '#' => 'she, they' },
 'es'		=> { 'x' => 'it',
 		     't' => 'pro',
		     'p' => '3',
		     'n' => 's',
		     'g' => 'n' },
 'man'		=> { 'x' => 'one',
 		     't' => 'pro',
		     'p' => '3' },
 'ihn'		=> { 'x' => 'him',
		     '#' => 'it' },
 'seiner'	=> { 'x' => 'he/it',
		     '#' => 'gen.' },
 'seine'	=> { 'x' => 'her/their',
		     '#' => 'nom/acc fem/plural' },
 'selbar'	=> { 'x' => 'self',
		     '#' => 'pronoun' },

 #   Indefinite pronouns
 'etwas'	=> { 'x' => 'something' },
 'jemand'	=> { 'x' => 'somebody',
		     '#' => 'someone',
		     't' => 'pro' },
 'niemand'	=> { 'x' => 'nobody',
		     '#' => 'noone',
 		     't' => 'pro' },

 #  Interrogatives
 'wann'		=> { 'x' => 'when' },
 'was'		=> { 'x' => 'what',
		     '#' => 'interrogative/relative pronoun' },
 'welche'	=> { 'x' => 'which',
		     '#' => 'interrogative/relative pronoun' },
 'wie'		=> { 'x' => 'how',
		     '#' => 'as, like' },

 #  Other functional words
 'ab'		=> { 'x' => 'off',
		     '#' => 'away, down, from',
 		     't' => 'p' },
 'aber'		=> { 'x' => 'but',
		     '#' => 'however, again : conjunction, adverb' },
 'all'		=> { 'x' => 'all',
		     '#' => 'every, everything',
 		     't' => 'a' },
 'allein'	=> { 'x' => 'alone',
		     '#' => 'single; but; adj. and adv.' },
 'als'		=> { 'x' => 'as',
 		     't' => 'p' },
 'also'		=> { 'x' => 'therefore',
		     '#' => 'so' },
 'an'		=> { 'x' => 'at',
		     '#' => 'on, to, against, in, by',
 		     't' => 'p' },
 'anstatt'	=> { 'x' => 'instead of',
 		     't' => 'p' },
 'auch'		=> { 'x' => 'also',
		     '#' => 'as well, too' },
 'auf'		=> { 'x' => 'on',
 		     't' => 'p' },
 'aus'		=> { 'x' => 'out',
		     '#' => 'out of',
 		     't' => 'p' },
 'ausser'	=> { 'x' => 'except',
 		     't' => 'p' },
 'ausserhalb'	=> { 'x' => 'outside',
		     '#' => 'beyond',
 		     't' => 'p' },
 'bei'		=> { 'x' => 'by',
		     '#' => 'at, near, while, during',
 		     't' => 'p' },
 'bis'		=> { 'x' => 'until',
 		     't' => 'p' },
 'bitte'	=> { 'x' => 'please',
 		     't' => 'interj' },
 'da'		=> { 'x' => 'there',
		     '#' => 'here then as when since' },
 'damit'	=> { 'x' => 'thereby',
		     '#' => 'with or by it',
		     't' => 'adv',
		     '#' => 'conj. in order to, so (that)' },
 'dann'		=> { 'x' => 'then' },
 'da'		=> { 'x' => 'that' },
 'denen'	=> { 'x' => 'to these',
		     '#' => 'to those' },
 'diese'	=> { 'x' => 'this' },
 'dieser'	=> { 'x' => 'this' },
 'doch'		=> { 'x' => 'but' },
 'durch'	=> { 'x' => 'through',
 		     't' => 'p' },
 'entlang'	=> { 'x' => 'along',
 		     't' => 'p' },
 'etliche'	=> { 'x' => 'some',
		     '#' => 'a few; pron. & adj. pl.' },
 'etwa'		=> { 'x' => 'about',
		     '#' => 'nearly, perhaps' },
 'fr'		=> { 'x' => 'for',
 		     't' => 'p' },
 'gegen'	=> { 'x' => 'towards',
		     '#' => 'against',
 		     't' => 'p' },
 'gegenber'	=> { 'x' => 'opposite',
 		     't' => 'p' },
 'gegessen'	=> { 'x' => 'eaten',
		     'r' => 'essen',
		     't' => 'v',
		     'c' => 'pp' },
 'heute'	=> { 'x' => 'today',
		     '#' => 'temporal adverb',
 		     't' => 'adv' },
 'hier'		=> { 'x' => 'here' },
 'immer'	=> { 'x' => 'always',
		     '#' => 'ever' },
 'in'		=> { 'x' => 'in',
 		     't' => 'p' },
 'ja'		=> { 'x' => 'yes',
		     '#' => 'interjection' },
 'jede'		=> { 'x' => 'every',
		     '#' => 'f. pron, adj.' },
 'jeden'	=> { 'x' => 'every' },
 'jeder'	=> { 'x' => 'every',
		     '#' => 'm. pron, adj.' },
 'jedes'	=> { 'x' => 'every',
		     '#' => 'n. pron, adj.' },
 'jedoch'	=> { 'x' => 'however',
		     '#' => 'yet',
 		     't' => 'adv' },
 'jetzt'	=> { 'x' => 'now',
 		     't' => 'adv' },
 'kein'		=> { 'x' => 'no',
		     't' => 'a',
		     '#' => 'KEY WORD - not any',
		     '#' => 'inflects like adj., but isn\'t it an adv.?' },
 'manchmal'	=> { 'x' => 'sometimes' },
 'mann'		=> { 'x' => 'man',
		     '#' => '? noun or pronoun?' },
 'mehr'		=> { 'x' => 'more' },
 'mit'		=> { 'x' => 'with',
		     '#' => 'at, by',
 		     't' => 'p' },
 'nach'		=> { 'x' => 'from',
		     '#' => 'after, according to',
 		     't' => 'p' },
 'nahe'		=> { 'x' => 'near',
		     '#' => 'close' },
 'nein'		=> { 'x' => 'no',
		     '#' => 'interjection' },
 'nie'		=> { 'x' => 'never' },
 'nicht'	=> { 'x' => 'not',
		     '#' => 'KEY WORD - not' },
 'noch'		=> { 'x' => 'still',
		     '#' => 'yet, besides' },
 'nur'		=> { 'x' => 'only',
		     '#' => 'solely, merely, but' },
 'oberhalb'	=> { 'x' => 'above',
 		     't' => 'p' },
 'oder'		=> { 'x' => 'or' },
 'oft'		=> { 'x' => 'often',
 		     't' => 'adv' },
 'ohne'		=> { 'x' => 'without',
 		     't' => 'p' },
 'per'		=> { 'x' => 'per',
		     '#' => 'by',
 		     't' => 'p' },
 'seit'		=> { 'x' => 'since',
 		     't' => 'p' },
 'selten'	=> { 'x' => 'seldom',
		     '#' => 'rare, rarely; adj + adv' },
 'sich'		=> { 'x' => 'self',
		     '#' => 'KEY WORD - reflexive particle' },
 'so'		=> { 'x' => 'so',
		     '#' => 'adv. thus; conj. so that' },
 'sowie'	=> { 'x' => 'as soon as',
		     '#' => 'just as',
 		     't' => 'conj' },
 'trotz'	=> { 'x' => 'in spite of',
 		     't' => 'p' },
 'ber'		=> { 'x' => 'over' },
 'unterhalb'	=> { 'x' => 'under',
		     '#' => 'beneath',
 		     't' => 'p' },
 'um'		=> { 'x' => 'around',
 		     't' => 'about' },
 'und'		=> { 'x' => 'and' },
 'unter'	=> { 'x' => 'under',
 		     't' => 'p' },
 'viel'		=> { 'x' => 'much',
		     '#' => 'many' },
 'von'		=> { 'x' => 'from',
		     '#' => 'of, by',
 		     't' => 'p' },
 'vor'		=> { 'x' => 'before',
		     '#' => 'in front of',
 		     't' => 'p' },
 'vorher'	=> { 'x' => 'previously',
		     '#' => 'before',
 		     't' => 'adv' },
 'whrend'	=> { 'x' => 'during',
		     '#' => 'while',
 		     't' => 'p' },
 'wegen'	=> { 'x' => 'because of',
		     '#' => 'owing to',
 		     't' => 'p' },
 'wem'		=> { 'x' => 'whom' },
 'wen'		=> { 'x' => 'whom' },
 'wenn'		=> { 'x' => 'if',
		     '#' => 'KEY WORD' },
 'wer'		=> { 'x' => 'who' },
 'wessen'	=> { 'x' => 'whose',
		     '#' => 'pronoun/possessive adjective' },
 'wo'		=> { 'x' => 'where' },
 'zu'		=> { 'x' => 'to',
 		     't' => 'p',
		     '#' => 'also a particle like "to" in English' },
 'zum'		=> { 'x' => 'to the',
		     '#' => 'contraction = zu + dem' },
 'zur'		=> { 'x' => 'to the',
		     '#' => 'contraction = zu + der' },
 'zurck'	=> { 'x' => 'back',
		     '#' => 'backwards, behind' },
 'zwischen'	=> { 'x' => 'between',
 		     't' => 'p' },

 #   Partitive / Contractions
 'beim'		=> { 'x' => 'by the',
		     '#' => 'bei+dem' },
 'im'		=> { 'x' => 'in the',
		     '#' => 'in+dem' },

 # Numbers: cardinal and ordinal
 'erste'	=> { 'x' => 'first' },
 'einmal'	=> { 'x' => 'once',
		     '#' => 'adverb?' },
 'zwei'		=> { 'x' => 'two' },
 'zweite'	=> { 'x' => 'second',
 		     't' => 'a' },
 'zwo'		=> { 'x' => 'two',
		     '#' => 'alt.' },
 'drei'		=> { 'x' => 'three' },
 'dritt'	=> { 'x' => 'third',
 		     't' => 'a' },
 'vier'		=> { 'x' => 'four' },
 'fnf'		=> { 'x' => 'five' },
 'sechs'	=> { 'x' => 'six' },
 'sieben'	=> { 'x' => 'seven' },
 'acht'		=> { 'x' => 'eight' },
 'neun'		=> { 'x' => 'nine' },
 'zehn'		=> { 'x' => 'ten' },
 'zwanzig'	=> { 'x' => 'twenty' },
 'hunnert'	=> { 'x' => 'hundred' },

 # Days and months
 'sonntag'	=> { 'x' => 'sunday',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'sonntags'	=> { 'x' => 'on sundays',
 		     't' => 'adv' },
 'montag'	=> { 'x' => 'monday',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'sonntags'	=> { 'x' => 'on sundays',
 		     't' => 'adv' },
 'dienstag'	=> { 'x' => 'tuesday',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'dienstags'	=> { 'x' => 'on tuesdays',
 		     't' => 'adv' },
 'mittwoch'	=> { 'x' => 'wednesday' },
 'donnerstag'	=> { 'x' => 'thursday',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'donnerstags'	=> { 'x' => 'on thursdays',
 		     't' => 'adv' },
 'freitag'	=> { 'x' => 'friday',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'freitags'	=> { 'x' => 'on fridays',
 		     't' => 'adv' },
 'samstag'	=> { 'x' => 'saturday',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'samstags'	=> { 'x' => 'on saturdays',
 		     't' => 'adv' },
 'januar'	=> { 'x' => 'january' },
 'februar'	=> { 'x' => 'february' },
 'mrz'		=> { 'x' => 'march' },
 'april'	=> { 'x' => 'april' },
 'mai'		=> { 'x' => 'may' },
 'juni'		=> { 'x' => 'june' },
 'juli'		=> { 'x' => 'july' },
 'august'	=> { 'x' => 'august' },
 'september'	=> { 'x' => 'september' },
 'oktober'	=> { 'x' => 'october' },
 'november'	=> { 'x' => 'november' },
 'dezember'	=> { 'x' => 'december' },
 
 # Key verbs
 #  haben - to have
 'haben'	=> { 'x' => 'have',
		     '#' => 'KEY aux',
 		     't' => 'v' },
  'hast'	=> { 'x' => 'have',
		     '#' => 'KEY aux',
		     'r' => 'haben',
 		     't' => 'v',
		     'c' => 'p',
		     'p' => '2',
		     'n' => 's' },
  'hat'		=> { 'x' => 'have',
		     '#' => 'KEY aux',
		     'r' => 'haben',
 		     't' => 'v',
		     'c' => 'p',
		     'p' => '3',
		     'n' => 's' },
  'hatte'	=> { 'x' => 'had',
		     '#' => 'KEY aux',
		     'r' => 'haben',
 		     't' => 'v',
		     'c' => 's',
		     'p' => '3',
		     'n' => 's' },

 #  geben - to give
 'geben'	=> { 'x' => 'give',
 		     't' => 'v' },
  'gibt'	=> { 'x' => 'gave',
 		     'r' => 'geben',
		     't' => 'v',
		     'c' => 'p',
		     'p' => '3',
		     'n' => 's' },
 
 #  knnen - can / to be able to
 'knnen'	=> { 'x' => 'can',
		     '#' => 'be able to',
 		     't' => 'v' },
  'kann'	=> { 'x' => 'can',
		     '#' => '3s pres',
 		     'r' => 'knnen',
		     't' => 'v' },
  'konnte'	=> { 'x' => 'could',
		     '#' => '3s past',
 		     'r' => 'knnen',
		     't' => 'v' },
  'gekonnt'	=> { 'x' => 'been able to',
		     '#' => 'pp',
 		     'r' => 'knnen',
		     't' => 'v' },
 #  liegen - to lie, to be (situated)
 'liegen'	=> { 'x' => 'lie',
		     '#' => 'be (situated); irreg.',
 		     't' => 'v' },
  'gelegen'	=> { 'x' => 'lied',
		     '#' => 'been',
  		     'r' => 'liegen',
		     't' => 'v',
		     'c' => 'pp' },
  'lag'		=> { 'x' => 'lied',
		     '#' => 'was',
  		     'r' => 'liegen',
		     't' => 'v',
		     'c' => 't' },

 #  mssen - must / to have to
 'mssen'	=> { 'x' => 'must',
		     '#' => 'have to',
 		     't' => 'v' },

 #  sein - to be
 'sein'		=> { 'x' => 'be',
		     '#' => 'also pron. his/its masc/neut',
 		     't' => 'v' },
  'bin'		=> { 'x' => 'am',
 		     'r' => 'sein',
		     't' => 'v',
		     'c' => 'p',
		     'p' => '1',
		     'n' => 's' },
  'bist'	=> { 'x' => 'are',
 		     'r' => 'sein',
		     't' => 'v',
		     'c' => 'p',
		     'p' => '2',
		     'n' => 's' },
  'ist'		=> { 'x' => 'is',
		     'r' => 'sein',
 		     't' => 'v',
		     'c' => 'p',
		     'p' => '3',
		     'n' => 's' },
  'sind'	=> { 'x' => 'are',
		     'r' => 'sein',
 		     't' => 'v',
		     'c' => 'p',
		     'p' => '3',
		     '#' => '1 they/we',
		     'n' => 'p' },
  'seid'	=> { 'x' => 'are',
		     'r' => 'sein',
 		     't' => 'v',
		     'c' => 'p',
		     'p' => '2',
		     'n' => 'p' },
  'war'		=> { 'x' => 'was',
 		     'r' => 'sein',
 		     't' => 'v',
		     'c' => 's',
		     'p' => '3',
		     'n' => 's' },
  'gewesen'	=> { 'x' => 'been',
 		     'r' => 'sein',
 		     't' => 'v',
		     'c' => 'pp' },

 #  tun - to do
 'tun'		=> { 'x' => 'do',
		     '#' => 'KEY WORD, also we/they do',
 		     't' => 'v' },
  'tue'		=> { 'x' => 'do',
 		     'r' => 'tun',
		     't' => 'v' },
  'tust'	=> { 'x' => 'do',
 		     'r' => 'tun',
		     't' => 'v' },
  'tut'		=> { 'x' => 'does',
 		     'r' => 'tun',
		     't' => 'v' },
  'tat'		=> { 'x' => 'did',
		     '#' => 'past',
 		     'r' => 'tun',
		     't' => 'v',
		     'c' => 't' },
  'getan'	=> { 'x' => 'done',
		     '#' => 'past participle',
 		     'r' => 'tun',
		     't' => 'v',
		     'c' => 'pp' },

 #  werden - to become
 'werden'	=> { 'x' => 'become',
		     '#' => 'will, KEY WORD, aux.',
 		     't' => 'v' },
  'wirst'	=> { 'x' => 'become',
		     '#' => 'du',
 		     'r' => 'werden',
 		     't' => 'v',
		     'c' => 'p',
		     'p' => '2',
		     'n' => 's' },
  'wird'	=> { 'x' => 'become',
		     '#' => 'er',
 		     'r' => 'werden',
 		     't' => 'v',
		     'c' => 'p',
		     'p' => '3',
		     'n' => 's' },
  'wurde'	=> { 'x' => 'became',
 		     'r' => 'werden',
		     't' => 'v',
		     'c' => 'pp' },

 #  drfen - may / to be allowed
 'drfen'	=> { 'x' => 'may',
		     '#' => 'be allowed',
 		     't' => 'v' },
  'darf'	=> { 'x' => 'may',
  		     'r' => 'drfen',
		     't' => 'v' },

 #  wollen - to want
 'wollen'	=> { 'x' => 'want',
 		     't' => 'v' },
  'will'	=> { 'x' => 'want',
  		     'r' => 'wollen',
		     't' => 'v',
		     #'p' => '1 & 3',
		     'n' => 's',
		     'c' => 'p' },
  'willst'	=> { 'x' => 'want',
  		     'r' => 'wollen',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'p' },

 # Vocabulary
 'aal'		=> { 'x' => 'eel',
		     '#' => 'animal',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'abend'	=> { 'x' => 'evening',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'ader'		=> { 'x' => 'vein',
		     '#' => 'artery',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'adjektiv'	=> { 'x' => 'adjective',
 		     't' => 'n' },
 'hnlich'	=> { 'x' => 'similar',
 		     't' => 'a' },
 'anbieten'	=> { 'x' => 'offer',
 		     't' => 'v' },
 'ander'	=> { 'x' => 'other',
		     '#' => 'different',
 		     't' => 'a' },
 'anders'	=> { 'x' => 'otherwise',
		     '#' => 'differently, else' },
 'anerkennen'	=> { 'x' => 'acknowledge',
		     '#' => 'recognize',
 		     't' => 'v',
		     '#' => 'trans.' },
 'anerkennung'	=> { 'x' => 'acknowledgment',
		     '#' => 'recognition',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'anfhrungszeichen'	=> { 'x' => 'quote marks',
		     '#' => 'inverted commas',
 			     't' => 'n',
			     'n' => 'p' },
 'angeboren'	=> { 'x' => 'inborn',
 		     't' => 'a' },
 'angebot'	=> { 'x' => 'bid',
		     '#' => 'offer, supply',
 		     't' => 'n',
		     'g' => 'n' },
 'angenehm'	=> { 'x' => 'pleasant',
 		     't' => 'a' },
 'anspruch'	=> { 'x' => 'claim',
		     '#' => 'demand; right, entitlement',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'antwort'	=> { 'x' => 'answer',
 		     't' => 'n' },
 'antworten'	=> { 'x' => 'answer',
 		     't' => 'v' },
 'apfel'	=> { 'x' => 'apple',
		     't' => 'n' },
 'arbeit'	=> { 'x' => 'work',
 		     't' => 'n',
		     'g' => 'f' },
 'arbeiten'	=> { 'x' => 'work',
 		     't' => 'v' },
 'argentinien'	=> { 'x' => 'argentina',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'artikel'	=> { 'x' => 'article',
 		     't' => 'n',
		     'g' => 'm' },
 'as'		=> { 'x' => 'ace',
		     '#' => 'playing cards',
		     't' => 'n',
		     'g' => 'n' },
 'a'		=> { 'x' => 'ate',
		     'r' => 'essen',
		     't' => 'v',
		     'c' => 'p',
		     '#' => 'all persons singular (2ns sing.?)',
		     'n' => 's' },
 'aen'		=> { 'x' => 'ate',
		     'r' => 'essen',
		     't' => 'v',
		     'c' => 'p',
		     '#' => 'all persons plural',
		     'n' => 'p' },
 'aufgabe'	=> { 'x' => 'task',
		     '#' => 'duty; problem; delivery',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'aufgeben'	=> { 'x' => 'deliver',
		     '#' => 'give up, surrender; post; irreg.',
 		     't' => 'v' },
 'auge'		=> { 'x' => 'eye',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'ausgang'	=> { 'x' => 'exit',
		     '#' => 'door; result, end; going out',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'auslnder'	=> { 'x' => 'foreigner',
 		     't' => 'n' },
 'automatisch'	=> { 'x' => 'automatic',
 		     't' => 'a' },
 'bald'		=> { 'x' => 'soon' },
 'bank'		=> { 'x' => 'bench',
 		     't' => 'n',
		     'g' => 'f' },
 'bargeld'	=> { 'x' => 'cash',
 		     't' => 'n' },
 'bauch'	=> { 'x' => 'belly',
		     '#' => 'stomach',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'beenden'	=> { 'x' => 'end',
		     '#' => 'conclude, finish',
 		     't' => 'v' },
 'befehlen'	=> { 'x' => 'command',
 		     't' => 'v' },
 'begegnen'	=> { 'x' => 'meet',
 		     't' => 'v' },
 'beginnen'	=> { 'x' => 'begin',
 		     't' => 'v' },
 'beglcken'	=> { 'x' => 'make happy',
		     '#' => 'multiword',
 		     't' => 'v' },
 'begriff'	=> { 'x' => 'idea',
		     '#' => 'concept, understanding',
 		     't' => 'n' },
 'beispiel'	=> { 'x' => 'example',
 		     't' => 'n' },
 'beien'	=> { 'x' => 'bite',
 		     't' => 'v' },
 'bekannt'	=> { 'x' => 'familiar',
 		     't' => 'a' },
 'belgien'	=> { 'x' => 'belgium',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'benutzen'	=> { 'x' => 'use',
		     '#' => 'make use of',
 		     't' => 'v' },
 'benutzenung'	=> { 'x' => 'utilization',
 		     't' => 'n',
		     'g' => 'f' },
 'bequem'	=> { 'x' => 'comfortable',
 		     't' => 'a' },
 'besetzen'	=> { 'x' => 'occupy',
 		     't' => 'v' },
 'besonders'	=> { 'x' => 'separately',
		     '#' => 'especially',
 		     't' => 'adv' },
 'bett'		=> { 'x' => 'bed',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'en' },
 'bewusst'	=> { 'x' => 'aware',
		     '#' => 'conscious',
 		     't' => 'a' },
 'bier'		=> { 'x' => 'beer',
 		     't' => 'n',
		     'g' => 'n' },
 'bieten'	=> { 'x' => 'offer',
		     '#' => 'irreg.',
 		     't' => 'v' },
 'bilden'	=> { 'x' => 'form',
		     '#' => 'educate',
 		     't' => 'v' },
 'bildung'	=> { 'x' => 'education',
		     '#' => 'formation',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'billig'	=> { 'x' => 'cheap',
		     '#' => 'reasonable, just',
 		     't' => 'a' },
 'billigen'	=> { 'x' => 'sanction',
		     '#' => 'approve of',
 		     't' => 'v' },
 'binden'	=> { 'x' => 'tie',
 		     't' => 'v' },
 'bitten'	=> { 'x' => 'ask',
 		     't' => 'v' },
 'bleiben'	=> { 'x' => 'stay',
		     '#' => 'remain; irreg.',
 		     't' => 'v' },
 'blenden'	=> { 'x' => 'blind',
		     '#' => 'dazzle',
 		     't' => 'v' },
 'blick'	=> { 'x' => 'look',
		     '#' => 'glance',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'blicken'	=> { 'x' => 'look',
		     '#' => 'glance',
 		     't' => 'v' },
 'blut'		=> { 'x' => 'blood',
 		     't' => 'n',
		     'g' => 'n' },
 'boden'	=> { 'x' => 'ground',
		     '#' => 'soil; bottom, bed',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => '' },
 'bohne'	=> { 'x' => 'bean',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'bse'		=> { 'x' => 'mean',
		     '#' => 'angry',
 		     't' => 'a' },
 'brasilien'	=> { 'x' => 'brazil',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'brechen'	=> { 'x' => 'break',
 		     't' => 'v' },
 'brennen'	=> { 'x' => 'burn',
		     '#' => 'sting; irreg.',
 		     't' => 'v' },
 'brezel'	=> { 'x' => 'pretzel',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'brief'	=> { 'x' => 'letter',
		     '#' => 'as in mail',
 		     't' => 'n' },
 'bruder'	=> { 'x' => 'brother',
 		     't' => 'n' },
 'brust'	=> { 'x' => 'breast',
		     '#' => 'chest',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'brssel'	=> { 'x' => 'brussels',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'bube'		=> { 'x' => 'jack',
		     '#' => 'playing cards',
		     't' => 'n',
		     'g' => 'm' },
 'buch'		=> { 'x' => 'book',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => '-er' },
 'buchstabe'	=> { 'x' => 'character',
		     '#' => 'letter',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'n' },
 'buchstabieren'	=> { 'x' => 'spell',
 			     't' => 'v' },
 'bulle'	=> { 'x' => 'bull',
 		     't' => 'n',
		     'g' => 'm' },
 'chef'		=> { 'x' => 'boss',
		     '#' => 'head',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 's' },
 'chile'	=> { 'x' => 'chile',
		     '#' => 'proper, place',
 		     't' => 'n',
		     'g' => 'n' },
 'chilene'	=> { 'x' => 'chilean',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'n' },
 'chilenisch'	=> { 'x' => 'chilean',
 		     't' => 'a' },
 'china'	=> { 'x' => 'china',
		     '#' => 'proper, place',
 		     't' => 'n',
		     'g' => 'n' },
 'chinese'	=> { 'x' => 'chinese',
		     '#' => 'chinaman',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'n' },
 'chinesisch'	=> { 'x' => 'chinese',
		     '#' => 'adj. and noun?' },
 'dame'		=> { 'x' => 'queen',
		     '#' => 'playing cards',
		     't' => 'n',
		     'g' => 'f' },
 'dnemark'	=> { 'x' => 'denmark',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'dnisch'	=> { 'x' => 'danish',
		     '#' => 'adj. and neut. noun' },
 'danken'	=> { 'x' => 'thank',
 		     't' => 'v' },
 'dankbar'	=> { 'x' => 'grateful',
 		     't' => 'a' },
 'degen'	=> { 'x' => 'sword',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => '-' },
 'denken'	=> { 'x' => 'think',
 		     't' => 'v' },
 'deutsch'	=> { 'x' => 'german',
		     '#' => 'noun or adjective?' },
 'deutschland'	=> { 'x' => 'germany',
 		     't' => 'n' },
 'donau'	=> { 'x' => 'danube',
 		     't' => 'n',
		     'g' => 'f' },
 'dialekt'	=> { 'x' => 'dialect',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'dit'		=> { 'x' => 'diet',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => '-' },
 'diamant'	=> { 'x' => 'diamond',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'en' },
 'dienen'	=> { 'x' => 'serve',
 		     't' => 'v' },
 'ding'		=> { 'x' => 'thing',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'doppel'	=> { 'x' => 'double',
		     '#' => 'duplicate',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => '-' },
 'doppeln'	=> { 'x' => 'double',
 		     't' => 'v' },
 'doppelpunkt'	=> { 'x' => 'colon',
		     '#' => '":"',
 		     't' => 'n',
		     'g' => 'm' },
 'doppelt'	=> { 'x' => 'double',
 		     't' => 'a' },
 'dorf'		=> { 'x' => 'village',
		     '#' => 'hamlet',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'dorn'		=> { 'x' => 'thorn',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'en' },
 'dornig'	=> { 'x' => 'thorny',
 		     't' => 'a' },
 'einfach'	=> { 'x' => 'simple',
		     '#' => 'plain, single',
 		     't' => 'a' },
 'eingang'	=> { 'x' => 'entrance',
		     '#' => 'arrival, recepit (of letter etc); introduction, beginning',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'eingeben'	=> { 'x' => 'enter',
 		     't' => 'v' },
 'einstrzen'	=> { 'x' => 'demolish',
		     '#' => 'fall in',
 		     't' => 'v' },
 'eis'		=> { 'x' => 'ice',
		     '#' => 'ice-cream',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'elektrizitt'	=> { 'x' => 'electricity',
 		     't' => 'n' },
 'elefant'	=> { 'x' => 'elephant',
 		     't' => 'n',
		     'g' => 'm' },
 'engel'	=> { 'x' => 'angel',
		     '#' => 'animate',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => '-' },
 'england'	=> { 'x' => 'england',
 		     't' => 'n' },
 'englisch'	=> { 'x' => 'english',
		     '#' => 'noun or adjective?' },
 'enthalten'	=> { 'x' => 'contain',
		     '#' => 'include',
 		     't' => 'v' },
 'ergebnis'	=> { 'x' => 'result',
 		     't' => 'n' },
 'erhalten'	=> { 'x' => 'receive',
		     '#' => 'maintain, support',
 		     't' => 'v' },
 'erkenntnis'	=> { 'x' => 'recognition',
		     '#' => 'realization',
		     't' => 'n',
		     'g' => 'f',
		     'p' => 'se' },
 'erklren'	=> { 'x' => 'explain',
		     '#' => 'declare',
 		     't' => 'v' },
 'erklrung'	=> { 'x' => 'declaration',
		     '#' => 'explanation',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'erlauben'	=> { 'x' => 'permit',
		     '#' => 'allow',
 		     't' => 'v' },
 'errichten'	=> { 'x' => 'erect',
		     '#' => '?',
 		     't' => 'v' },
 'erwarten'	=> { 'x' => 'expect',
 		     't' => 'v' },
 'essen'	=> { 'x' => 'eat',
		     '#' => 'irreg.',
 		     't' => 'v' },
 'fahrrad'	=> { 'x' => 'bicycle',
		     '#' => 'bike; cf swiss velo',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'fahrzeug'	=> { 'x' => 'vehicle',
		     '#' => 'cf vehikel',
 		     't' => 'n',
		     'g' => 'n' },
 'fallen'	=> { 'x' => 'fall',
 		     't' => 'v' },
 'familie'	=> { 'x' => 'family',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'fehlen'	=> { 'x' => 'miss',
		     '#' => 'lack, faltar',
 		     't' => 'v' },
 'fein'		=> { 'x' => 'fine',
		     '#' => 'delicate, elegant, distinguished',
 		     't' => 'a',
		     '#' => 'acute, sensitive' },
 'fenster'	=> { 'x' => 'window',
 		     't' => 'n' },
 'feuer'	=> { 'x' => 'fire',
 		     't' => 'n',
		     'g' => 'n' },
 'fick'		=> { 'x' => 'fuck',
 		     't' => 'n' },
 'ficken'	=> { 'x' => 'fuck',
 		     't' => 'v' },
 'finden'	=> { 'x' => 'find',
 		     't' => 'v' },
  'fand'	=> { 'x' => 'found',
  		     'r' => 'finden',
		     't' => 'v',
		     'p' => '3',
		     's' => 's',
		     'c' => 's' },
  'gefunden'	=> { 'x' => 'found',
  		     'r' => 'finden',
		     't' => 'v',
		     'c' => 'pp' },
 'finnisch'	=> { 'x' => 'finnish',
		     '#' => 'adj. and neut. noun' },
 'flamme'	=> { 'x' => 'flame',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'fleck'	=> { 'x' => 'spot',
		     '#' => 'stain',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'fleisch'	=> { 'x' => 'meat',
		     '#' => 'flesh',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'fliegen'	=> { 'x' => 'fly',
 		     't' => 'v' },
 'folgen'	=> { 'x' => 'follow',
 		     't' => 'v' },
 'foto'		=> { 'x' => 'photo',
 		     't' => 'n',
		     'g' => 'n',
		     '#' => 'swiss: f',
		     'p' => 's' },
 'frage'	=> { 'x' => 'question',
 		     't' => 'n' },
 'fragen'	=> { 'x' => 'ask',
		     '#' => 'question, enquire',
 		     't' => 'v' },
 'frankreich'	=> { 'x' => 'france',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'franzsisch'	=> { 'x' => 'french',
 		     't' => 'a' },
 'frau'		=> { 'x' => 'woman',
 		     't' => 'n',
		     'g' => 'f' },
 'frulein'	=> { 'x' => 'young woman',
 		     't' => 'n',
		     'g' => 'n' },
 'frei'		=> { 'x' => 'free',
		     '#' => 'libre',
 		     't' => 'a' },
 'freiheit'	=> { 'x' => 'freedom',
 		     't' => 'n',
		     'g' => 'f' },
 'fremd'	=> { 'x' => 'foreign',
		     '#' => 'strange',
 		     't' => 'a' },
 'frequenz'	=> { 'x' => 'frequency',
 		     't' => 'n',
		     'g' => 'f' },
 'frhlich'	=> { 'x' => 'happy',
		     '#' => 'merry',
 		     't' => 'a' },
 'frh'		=> { 'x' => 'early',
		     '#' => 'adj. and adv.' },
 'frhe'	=> { 'x' => 'morning',
		     '#' => 'early hour',
 		     't' => 'n',
		     'g' => 'f' },
 'frhjahr'	=> { 'x' => 'srping',
		     '#' => 'the season',
 		     't' => 'n',
		     'g' => 'n' },
 'frhling'	=> { 'x' => 'spring',
		     '#' => 'the season',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'frhstck'	=> { 'x' => 'breakfast',
 		     't' => 'n',
		     'g' => 'n' },
 'frhstcken'	=> { 'x' => 'breakfast',
		     '#' => 'have breakfast',
 		     't' => 'v' },
 'funktionieren'	=> { 'x' => 'function',
 			     't' => 'v' },
 'frchten'	=> { 'x' => 'fear',
 		     't' => 'v' },
 'fu'		=> { 'x' => 'foot',
 		     't' => 'n',
		     'g' => 'm' },
 'gast'		=> { 'x' => 'guest',
		     '#' => 'person',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'ganz'		=> { 'x' => 'whole',
 		     't' => 'a' },
 'gefallen'	=> { 'x' => 'please',
 		     't' => 'v' },
 'gegenwrtig'	=> { 'x' => 'currently',
 		     't' => 'adv' },
 'gehen'	=> { 'x' => 'go',
 		     't' => 'v' },
 'gehorchen'	=> { 'x' => 'obey',
 		     't' => 'v' },
 'geist'	=> { 'x' => 'spirit',
		     '#' => 'mind, intellect, ghost',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'er' },
 'gelb'		=> { 'x' => 'yellow',
 		     't' => 'a' },
 'geld'		=> { 'x' => 'money',
 		     't' => 'n' },
 'gelingen'	=> { 'x' => 'succeed',
 		     't' => 'v' },
 'genau'	=> { 'x' => 'exact',
		     '#' => 'precise, right',
 		     't' => 'a' },
 'general'	=> { 'x' => 'general',
 		     't' => 'n' },
 'geniebar'	=> { 'x' => 'enjoyable',
		     '#' => 'edible, drinkable',
 		     't' => 'a' },
 'genieen'	=> { 'x' => 'enjoy',
		     '#' => 'eat, drink; irreg.',
 		     't' => 'v' },
 'gengen'	=> { 'x' => 'be enough',
		     '#' => 'be sufficient',
 		     't' => 'v' },
 'gern'		=> { 'x' => 'gladly',
		     '#' => 'adverb -> translates "like, like to" in English' },
 'geschehen'	=> { 'x' => 'happen',
 		     't' => 'v' },
 'geschichte'	=> { 'x' => 'story',
		     '#' => 'history',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'geschlecht'	=> { 'x' => 'sex',
		     '#' => '; race; species, kind; family, blood;',
 		     't' => 'n',
		     '#' => 'gender',
		     'g' => 'n',
		     'p' => 'er' },
 'geschlechtlicht'	=> { 'x' => 'sexual',
		     '#' => 'venereal',
 			     't' => 'a' },
 'geschmack'	=> { 'x' => 'taste',
		     '#' => 'flavour; liking',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'geschmackvoll'	=> { 'x' => 'tasteful',
 			     't' => 'a' },
 'geschwister'	=> { 'x' => 'siblings',
		     '#' => 'brothers and sisters',
 		     't' => 'n',
		     'n' => 'p' },
 'gesellschaft'	=> { 'x' => 'society',
		     '#' => 'company, firm, party',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'gesicht'	=> { 'x' => 'face',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'gesichtskreis'	=> { 'x' => 'horizon',
 			     't' => 'n',
			     'g' => 'm' },
 'gestern'	=> { 'x' => 'yesterday',
		     '#' => 'temporal adverb',
 		     't' => 'adv' },
 'gestikulieren'	=> { 'x' => 'gesticulate',
 			     't' => 'v' },
 'gesundheit'	=> { 'x' => 'health',
 		     't' => 'n',
		     'g' => 'f' },
 'glas'		=> { 'x' => 'glass',
		     '#' => 'both senses as english',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'glauben'	=> { 'x' => 'believe',
 		     't' => 'v' },
 'gleich'	=> { 'x' => 'soon',
		     '#' => 'equal' },
 'gleichgewicht'	=> { 'x' => 'balance',
		     '#' => 'equilibrium',
 			     't' => 'n' },
 'glck'	=> { 'x' => 'luck',
		     '#' => 'fortune; happiness',
 		     't' => 'n',
		     'g' => 'n' },
 'glcklich'	=> { 'x' => 'happy',
		     '#' => 'lucky',
 		     't' => 'a' },
 'glckspilz'	=> { 'x' => 'lucky one',
		     '#' => 'lucky person; multiword',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'gott'		=> { 'x' => 'god',
 		     't' => 'n',
		     'g' => 'm' },
 'gratulieren'	=> { 'x' => 'congratulate',
 		     't' => 'v' },
 'grau'		=> { 'x' => 'grey',
		     '#' => 'am. gray',
 		     't' => 'a' },
 'graupeln'	=> { 'x' => 'sleet',
 		     't' => 'v' },
 'graupelregen'	=> { 'x' => 'sleet',
		     '#' => 'uncountable?',
 		     't' => 'n',
		     'g' => 'm' },
 'grenze'	=> { 'x' => 'border',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'griechisch'	=> { 'x' => 'greek',
		     '#' => 'adj. and neut. noun' },
 'gro'		=> { 'x' => 'big',
 		     't' => 'a' },
 'gru'		=> { 'x' => 'greeting',
 		     't' => 'n' },
 'gut'		=> { 'x' => 'good',
 		     't' => 'a' },
 'hagel'	=> { 'x' => 'hail',
		     '#' => 'as rain, snow; uncountable?',
 		     't' => 'n',
		     'g' => 'm' },
 'hagelkorn'	=> { 'x' => 'hailstone',
 		     't' => 'n',
		     'g' => 'n' },
 'hageln'	=> { 'x' => 'hail',
 		     't' => 'v' },
 'hagelschauer'	=> { 'x' => 'hailstorm',
 		     't' => 'n',
		     'g' => 'm' },
 'hallo'	=> { 'x' => 'hello',
 		     't' => 'interj' },
 'hand'		=> { 'x' => 'hand',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'hufig'	=> { 'x' => 'frequent',
 		     't' => 'a' },
 'haus'		=> { 'x' => 'house',
 		     't' => 'n' },
 'haut'		=> { 'x' => 'skin',
		     '#' => 'hide, membrane, film',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'heilig'	=> { 'x' => 'holy',
 		     't' => 'a' },
 'heien'	=> { 'x' => 'be called',
		     '#' => 'multiword;',
 		     't' => 'v',
		     '#' => 'mean, name, command' },
 'helfen'	=> { 'x' => 'help',
 		     't' => 'v' },
 'hell'		=> { 'x' => 'bright',
		     '#' => 'light, clear',
 		     't' => 'a' },
 'helle'	=> { 'x' => 'brightness',
		     '#' => 'daylight, clearness',
 		     't' => 'n',
		     'g' => 'f' },
 'herbst'	=> { 'x' => 'autumn',
		     '#' => 'am. fall',
 		     't' => 'n',
		     'g' => 'm' },
 'herr'		=> { 'x' => 'lord',
		     '#' => 'mister, sir?',
 		     't' => 'n',
		     'g' => 'm' },
 'herz'		=> { 'x' => 'heart',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'en' },
 'hilfe'	=> { 'x' => 'help',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'himmel'	=> { 'x' => 'sky',
		     '#' => 'heaven',
 		     't' => 'n',
		     'g' => 'm' },
 'hoffen'	=> { 'x' => 'hope',
		     '#' => 'hope for',
 		     't' => 'v' },
 'hoffnung'	=> { 'x' => 'hope',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'hollndisch'	=> { 'x' => 'dutch',
		     '#' => 'adj. and neut. noun' },
 'hren'	=> { 'x' => 'hear',
		     '#' => 'listen, listen to',
 		     't' => 'v' },
 'hotel'	=> { 'x' => 'hotel',
 		     't' => 'n' },
 'indien'	=> { 'x' => 'india',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'information'	=> { 'x' => 'information',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'interessant'	=> { 'x' => 'interesting',
 		     't' => 'a' },
 'interesse'	=> { 'x' => 'interest',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'n' },
 'interessieren'	=> { 'x' => 'interest',
 			     't' => 'v' },
 'irland'	=> { 'x' => 'ireland',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'islndisch'	=> { 'x' => 'icelandic',
		     '#' => 'adj. and neut. noun' },
 'italien'	=> { 'x' => 'italy',
 		     't' => 'n' },
 'italienisch'	=> { 'x' => 'italian',
		     '#' => 'adj. and neut. noun' },
 'jahr'		=> { 'x' => 'year',
 		     't' => 'n',
		     'g' => 'g' },
 'jahrhundert'	=> { 'x' => 'century',
 		     't' => 'n' },
 'japan'	=> { 'x' => 'japan',
		     '#' => 'proper, place',
 		     't' => 'n',
		     'g' => 'n' },
 'japaner'	=> { 'x' => 'japanese',
		     '#' => 'person',
 		     't' => 'n',
		     'g' => 'm' },
 'japanisch'	=> { 'x' => 'japanese',
		     '#' => 'n neut. language; adj.' },
 'kalt'		=> { 'x' => 'cold',
 		     't' => 'a' },
 'klte'	=> { 'x' => 'cold',
 		     't' => 'n',
		     'g' => 'f' },
 'kamm'		=> { 'x' => 'comb',
		     '#' => 'crest; ridge',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'kampf'	=> { 'x' => 'battle',
 		     't' => 'n' },
 'kanada'	=> { 'x' => 'canada',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'karo'		=> { 'x' => 'diamond',
		     '#' => 'playing cards',
		     't' => 'n' },
 'kartographie'	=> { 'x' => 'cartography',
 		     't' => 'n' },
 'kaschmir'	=> { 'x' => 'kashmir',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'kaufen'	=> { 'x' => 'buy',
 		     't' => 'v' },
 'kehle'	=> { 'x' => 'throat',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'kennen'	=> { 'x' => 'know',
		     '#' => 'as in conocer',
 		     't' => 'v' },
 'kind'		=> { 'x' => 'child',
		     '#' => 'person',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'kino'		=> { 'x' => 'cinema',
 		     't' => 'n',
		     'g' => 'n' },
 'kissen'	=> { 'x' => 'cushion',
		     '#' => 'pillow',
 		     't' => 'n',
		     'g' => 'n' },
 'klar'		=> { 'x' => 'clear',
		     '#' => 'certainly',
 		     't' => 'a' },
 'klicken'	=> { 'x' => 'click',
 		     't' => 'v' },
 'klon'		=> { 'x' => 'clone',
 		     't' => 'n' },
 'kommen'	=> { 'x' => 'come',
 		     't' => 'v' },
 'konferenz'	=> { 'x' => 'conference',
		     '#' => 'meeting',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'knig'	=> { 'x' => 'king',
 		     't' => 'n',
		     'g' => 'm' },
 'kopf'		=> { 'x' => 'head',
 		     't' => 'n' },
 'koreanisch'	=> { 'x' => 'korean',
		     '#' => 'adj. and neut. noun' },
 'kosmonaut'	=> { 'x' => 'cosmonaut',
 		     't' => 'n' },
 'kraft'	=> { 'x' => 'power',
		     '#' => 'strength, force',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'krank'	=> { 'x' => 'sick',
		     '#' => 'ill',
 		     't' => 'a' },
 'krnken'	=> { 'x' => 'hurt',
		     '#' => 'offend',
 		     't' => 'v' },
 'krankenhaus'	=> { 'x' => 'hospital',
 		     't' => 'n',
		     'g' => 'n' },
 'krankheit'	=> { 'x' => 'disease',
		     '#' => 'illness',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'kreuz'	=> { 'x' => 'cross',
		     '#' => 'clubs (playing cards)',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'krieg'	=> { 'x' => 'war',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'kuh'		=> { 'x' => 'cow',
 		     't' => 'n',
		     'g' => 'f' },
 'kunst'	=> { 'x' => 'art',
 		     't' => 'n' },
 'ku'		=> { 'x' => 'kiss',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'ksse' },
 'krbis'	=> { 'x' => 'pumpkin',
 		     't' => 'n',
		     'g' => 'm' },
 'kssen'	=> { 'x' => 'kiss',
 		     't' => 'v' },
 'laden'	=> { 'x' => 'load',
		     '#' => 'irreg.',
 		     't' => 'v' },
 'land'		=> { 'x' => 'land',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'lauf'		=> { 'x' => 'run',
		     '#' => 'race',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'laufen'	=> { 'x' => 'walk',
		     '#' => 'run; skate',
 		     't' => 'v' },
 'laut'		=> { 'x' => 'loud',
 		     't' => 'a' },
 'leben'	=> { 'x' => 'live',
 		     't' => 'v',
		     'g' => 'n' },
 'lecken'	=> { 'x' => 'lick',
 		     't' => 'v' },
 'lehren'	=> { 'x' => 'teach',
 		     't' => 'n' },
 'leisten'	=> { 'x' => 'achieve',
		     '#' => 'do; perform; give; offer',
 		     't' => 'v' },
 'leistenung'	=> { 'x' => 'achievement',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'leiten'	=> { 'x' => 'lead',
		     '#' => 'guide, manage, direct, conduct',
 		     't' => 'v' },
 'leitung'	=> { 'x' => 'direction',
		     '#' => 'management, lead, circuit',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'lenken'	=> { 'x' => 'steer',
		     '#' => 'direct, lead',
 		     't' => 'v' },
 'lenkung'	=> { 'x' => 'steering',
		     '#' => 'direction',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'lernen'	=> { 'x' => 'learn',
 		     't' => 'v' },
 'lesen'	=> { 'x' => 'read',
 		     't' => 'v' },
 'leute'	=> { 'x' => 'people',
 		     't' => 'n' },
 'leutnant'	=> { 'x' => 'lieutenant',
 		     't' => 'n' },
 'licht'	=> { 'x' => 'light',
		     '#' => 'candle, lamp',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'lieb'		=> { 'x' => 'dear',
 		     't' => 'a' },
 'liebe'	=> { 'x' => 'love',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'lieben'	=> { 'x' => 'love',
		     '#' => 'like',
 		     't' => 'v' },
 'liechtenstein'	=> { 'x' => 'liechtenstein',
		     '#' => 'proper, place',
			     't' => 'n' },
 'link'		=> { 'x' => 'left',
 		     't' => 'a' },
 'linkisch'	=> { 'x' => 'awkward',
		     '#' => 'clumsy',
 		     't' => 'a' },
 'linke'	=> { 'x' => 'left',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'links'	=> { 'x' => 'left',
		     '#' => 'on the left',
 		     't' => 'adv' },
 'luchs'	=> { 'x' => 'lynx',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'luft'		=> { 'x' => 'air',
		     '#' => 'draft, breeze',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'lunge'	=> { 'x' => 'lung',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'lust'		=> { 'x' => 'pleasure',
		     '#' => 'joy, desire',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'luxemburg'	=> { 'x' => 'luxembourg',
		     '#' => 'proper, place',
		     't' => 'n' },
 'machen'	=> { 'x' => 'make',
 		     't' => 'v' },
 'mdchen'	=> { 'x' => 'girl',
 		     't' => 'n',
		     'g' => 'n' },
 'mag'		=> { 'x' => '(I/he) likes',
 		     'r' => 'mgen',
		     't' => 'v' },
 'masse'	=> { 'x' => 'mass',
		     '#' => 'bulk, substance',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'migen'	=> { 'x' => 'moderate',
 		     't' => 'v' },
 'maus'		=> { 'x' => 'mouse',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'mensch'	=> { 'x' => 'person',
		     '#' => 'human being',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'n' },
 'messer'	=> { 'x' => 'knife',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => '-' },
 'messing'	=> { 'x' => 'brass',
 		     't' => 'n',
		     'g' => 'n' },
 'mexiko'	=> { 'x' => 'mexico',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'milch'	=> { 'x' => 'milk',
 		     't' => 'n',
		     'g' => 'f' },
 'milchstrae'	=> { 'x' => 'galaxy',
 		     't' => 'n' },
 'mittag'	=> { 'x' => 'mid-day',
		     '#' => 'or two words?',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'minute'	=> { 'x' => 'minute',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'mitte'	=> { 'x' => 'middle',
		     '#' => 'center, centre; mean',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'mochte'	=> { 'x' => 'liked',
 		     'r' => 'mgen',
		     'c' => 'p' },
 'modifizieren'	=> { 'x' => 'modify',
 		     't' => 'v' },
 'mgen'	=> { 'x' => 'like',
		     't' => 'v' },
  'gemocht'	=> { 'x' => 'liked',
 		     'r' => 'mgen',
		     't' => 'v',
		     'c' => 'pp' },
 'mglich'	=> { 'x' => 'possible',
 		     't' => 'a' },
 'morgen'	=> { 'x' => 'morning',
		     '#' => 'also adv. tomorrow',
 		     't' => 'n',
		     'g' => 'm' },
 'mnchen'	=> { 'x' => 'munich',
 		     't' => 'n' },
 'mund'		=> { 'x' => 'mouth',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'er' },
 'musik'	=> { 'x' => 'music',
 		     't' => 'n',
		     'g' => 'f' },
 'mutter'	=> { 'x' => 'mother',
 		     't' => 'n',
		     'g' => 'f' },
 'nchst'	=> { 'x' => 'next',
 		     't' => 'a' },
 'nabel'	=> { 'x' => 'navel',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => '-' },
 'nachmittag'	=> { 'x' => 'afternoon',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'nachricht'	=> { 'x' => 'report',
		     '#' => 'information',
 		     't' => 'n',
		     'g' => 'f' },
 'nachrichten'	=> { 'x' => 'news',
 		     't' => 'n',
		     'g' => 'f',
		     'n' => 'p' },
 'nacht'	=> { 'x' => 'night',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'nackt'	=> { 'x' => 'naked',
		     '#' => 'nude',
 		     't' => 'a' },
 'nadel'	=> { 'x' => 'needle',
		     '#' => 'pin',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'name'		=> { 'x' => 'name',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'n' },
 'nase'		=> { 'x' => 'nose',
 		     't' => 'n' },
 'nation'	=> { 'x' => 'nation',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'natur'	=> { 'x' => 'nature',
 		     't' => 'n',
		     'g' => 'f' },
 'nehmen'	=> { 'x' => 'take',
		     '#' => 'receive; irreg.',
 		     't' => 'v' },
  'nimmt'	=> { 'x' => 'received',
  		     'r' => 'nehmen',
		     't' => 'v' },
 'neider'	=> { 'x' => 'down',
		     '#' => 'adv.; low, inferior adj.' },
 'nektar'	=> { 'x' => 'nectar',
 		     't' => 'n',
		     'g' => 'm' },
 'neu'		=> { 'x' => 'new',
		     '#' => 'modern, recent, fresh',
 		     't' => 'a' },
 'niederlande'	=> { 'x' => 'netherlands',
		     '#' => 'proper, place, plural',
 		     't' => 'n' },
 'norwegen'	=> { 'x' => 'norway',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'norwegisch'	=> { 'x' => 'norwegian',
		     '#' => 'adj. and neut. noun' },
 'nummer'	=> { 'x' => 'number',
 		     't' => 'n' },
 'ntzlich'	=> { 'x' => 'advantageous',
		     '#' => 'profitable',
 		     't' => 'a' },
 'offizier'	=> { 'x' => 'officer',
 		     't' => 'n' },
 'ffnen'	=> { 'x' => 'open',
 		     't' => 'v' },
 'onkel'	=> { 'x' => 'uncle',
 		     't' => 'n' },
 'sterreich'	=> { 'x' => 'austria',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'paar'		=> { 'x' => 'pair',
		     '#' => 'but adj. = couple (of)',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'passen'	=> { 'x' => 'fit',
 		     't' => 'v' },
 'passieren'	=> { 'x' => 'happen',
 		     't' => 'v' },
 'person'	=> { 'x' => 'person',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'pfennig'	=> { 'x' => 'pfennig',
		     '#' => 'penny',
 		     't' => 'n',
		     'g' => 'm' },
 'philosophie'	=> { 'x' => 'philosophy',
 		     't' => 'n' },
 'pik'		=> { 'x' => 'spade',
		     '#' => 'playing cards',
		     't' => 'n' },
 'pilz'		=> { 'x' => 'mushroom',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'platz'	=> { 'x' => 'place',
		     '#' => 'square',
 		     't' => 'n',
		     'g' => 'm' },
 'polizei'	=> { 'x' => 'police',
 		     't' => 'n' },
 'polnisch'	=> { 'x' => 'polish',
		     '#' => 'adj. and neut. noun' },
 'portugiesisch'	=> { 'x' => 'portuguese',
		     '#' => 'adj. and neut. noun' },
 'programm'	=> { 'x' => 'program',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'programmieren'	=> { 'x' => 'program',
 			     't' => 'v' },
 'projekt'	=> { 'x' => 'project',
 		     't' => 'n' },
 'prozent'	=> { 'x' => 'percent' },
 'punkt'	=> { 'x' => 'point',
		     '#' => 'dot, spot; full stop; item',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'qulen'	=> { 'x' => 'torment',
 		     't' => 'v' },
 'raten'	=> { 'x' => 'advise',
		     '#' => 'guess',
 		     't' => 'v' },
 'reaktion'	=> { 'x' => 'reaction',
 		     't' => 'n',
		     'g' => 'f' },
 'recht'	=> { 'x' => 'right',
		     '#' => 'just',
 		     't' => 'n' },
 'redakteur'	=> { 'x' => 'editor',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'regen'	=> { 'x' => 'rain',
		     '#' => 'uncountable?',
 		     't' => 'n',
		     'g' => 'm' },
 'regenbogen'	=> { 'x' => 'rainbow',
 		     't' => 'n',
		     'g' => 'm' },
 'regnen'	=> { 'x' => 'snow',
 		     't' => 'v' },
 'reihe'	=> { 'x' => 'file',
		     '#' => 'row, rank, series, queue',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'reise'	=> { 'x' => 'journey',
		     '#' => 'trip, tour',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'reisebro'	=> { 'x' => 'travel agent',
		     '#' => 'tourist office',
 		     't' => 'n',
		     'g' => 'n' },
 'reisen'	=> { 'x' => 'travel',
 		     't' => 'v' },
 'replubik'	=> { 'x' => 'republic',
		     't' => 'n' },
 'rezession'	=> { 'x' => 'recession',
 		     't' => 'n' },
 'riechen'	=> { 'x' => 'smell',
		     '#' => 'irreg.',
 		     't' => 'v' },
 'rose'		=> { 'x' => 'rose',
 		     't' => 'n',
		     'g' => 'f' },
 'ruf'		=> { 'x' => 'cry',
		     '#' => 'call, reputation, summons',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'rune'		=> { 'x' => 'rune',
 		     't' => 'n',
		     'g' => 'f' },
 'russisch'	=> { 'x' => 'russian',
		     '#' => 'adj. and neut noun' },
 'sagen'	=> { 'x' => 'say',
 		     't' => 'v' },
 'satt'		=> { 'x' => 'full',
		     '#' => 'satisfied; sick of, tired of; satiated; rich, deep',
 		     't' => 'a' },
 'schaden'	=> { 'x' => 'hurt',
		     '#' => 'harm',
 		     't' => 'v' },
 'schein'	=> { 'x' => 'light',
		     '#' => 'appearance, semblance',
 		     't' => 'n',
		     '#' => 'shine, banknote, certificate',
		     'g' => 'm',
		     '#' => 'licence, receipt',
		     'p' => 'e' },
 'scheinbar'	=> { 'x' => 'apparent',
 		     't' => 'a' },
 'scheinen'	=> { 'x' => 'seem',
		     '#' => 'shine; irreg.',
 		     't' => 'v' },
 'scheie'	=> { 'x' => 'shit',
 		     't' => 'n',
		     'g' => 'f' },
 'schenken'	=> { 'x' => 'give',
		     '#' => 'present, pardon',
 		     't' => 'v' },
 'schicken'	=> { 'x' => 'send',
 		     't' => 'v' },
 'schlafwagen'	=> { 'x' => 'sleeping-car',
		     '#' => '(two words?)',
 		     't' => 'n' },
 'schlauch'	=> { 'x' => 'tube',
		     '#' => 'hose',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'schlecht'	=> { 'x' => 'bad',
		     '#' => 'wicked, inferior',
 		     't' => 'a' },
 'schlechthin'	=> { 'x' => 'simply',
		     '#' => 'perfectly',
 		     't' => 'adv' },
 'schmecken'	=> { 'x' => 'taste',
 		     't' => 'v' },
 'schmerzen'	=> { 'x' => 'ache',
		     '#' => 'pain, grieve',
 		     't' => 'v' },
 'schnee'	=> { 'x' => 'snow',
		     '#' => 'uncountable?',
 		     't' => 'n',
		     'g' => 'm' },
 'schneeball'	=> { 'x' => 'snowball',
 		     't' => 'n',
		     'g' => 'm' },
 'schneeflocken'	=> { 'x' => 'snowflake',
			     't' => 'n',
			     'g' => 'g' },
 'schneemann'	=> { 'x' => 'snowman',
 		     't' => 'n',
		     'g' => 'm' },
 'schneepflug'	=> { 'x' => 'snowplough',
		     '#' => 'snowplow?',
 		     't' => 'n',
		     'g' => 'm' },
 'schneien'	=> { 'x' => 'snow',
 		     't' => 'v' },
 'schokolade'	=> { 'x' => 'chocolate',
		     't' => 'n' },
 'schn'	=> { 'x' => 'beautiful',
 		     't' => 'a' },
 'schreiben'	=> { 'x' => 'write',
 		     't' => 'v' },
 'schrieb'	=> { 'x' => 'wrote',
 		     'r' => 'schreiben',
		     't' => 'v',
		     'c' => 'p' },
 'schuh'	=> { 'x' => 'shoe',
 		     't' => 'n' },
 'schuldig'	=> { 'x' => 'guilty',
		     '#' => 'indebted',
 		     't' => 'a' },
 'schlacht'	=> { 'x' => 'battle',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'schlachten'	=> { 'x' => 'slaughter',
 		     't' => 'v' },
 'schlafen'	=> { 'x' => 'sleep',
 		     't' => 'v' },
 'schlag'	=> { 'x' => 'blow',
		     '#' => 'stroke; punch, slap; ...',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'schlagen'	=> { 'x' => 'strike',
		     '#' => 'beat; defeat; ...',
 		     't' => 'v' },
 'schutz'	=> { 'x' => 'protection',
		     '#' => 'defence, shelter',
 		     't' => 'n',
		     'g' => 'm' },
 'schwarz'	=> { 'x' => 'black',
		     '#' => 'adj. and neut. noun',
 		     't' => 'a' },
 'schweden'	=> { 'x' => 'sweden',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'schwedisch'	=> { 'x' => 'swedish',
		     '#' => 'adj. and neut. noun' },
 'schweiz'	=> { 'x' => 'switzerland',
		     '#' => 'proper, place',
 		     't' => 'n',
		     'g' => 'f' },
 'schwester'	=> { 'x' => 'sister',
 		     't' => 'n' },
 'schwimmen'	=> { 'x' => 'swim',
 		     't' => 'v' },
 'seele'	=> { 'x' => 'soul',
		     '#' => 'mind',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'sehen'	=> { 'x' => 'see',
 		     't' => 'v' },
 'seite'	=> { 'x' => 'page',
 		     't' => 'n' },
 'sekunde'	=> { 'x' => 'second',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'setzen'	=> { 'x' => 'put',
 		     't' => 'v' },
# 'setzen sich'	=> { 'x' => 'sit',
# 		     't' => 'v' },
 'sex'		=> { 'x' => 'sex',
 		     't' => 'n',
		     'g' => 'm' },
 'sexuell'	=> { 'x' => 'sexual',
 		     't' => 'a' },
 'singapur'	=> { 'x' => 'singapore',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'singen'	=> { 'x' => 'sing',
		     '#' => 'irreg.',
 		     't' => 'v' },
  'sang'	=> { 'x' => 'sang',
  		     'r' => 'singen',
		     't' => 'v',
		     'c' => 't' },
 'sohn'		=> { 'x' => 'son',
 		     't' => 'n',
		     'g' => 'm' },
 'solidaritt'	=> { 'x' => 'solidarity',
 		     't' => 'n' },
 'sommer'	=> { 'x' => 'summer',
 		     't' => 'n',
		     'g' => 'm' },
 'sonne'	=> { 'x' => 'sun',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'sorte'	=> { 'x' => 'sort',
		     '#' => 'type',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'spanien'	=> { 'x' => 'spain',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'spanisch'	=> { 'x' => 'spanish',
		     '#' => 'adj. and neut. noun' },
 'spt'		=> { 'x' => 'late' },
 'spter'	=> { 'x' => 'later',
		     '#' => '?' },
 'spiegel'	=> { 'x' => 'mirror',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => '-' },
 'spiegeln'	=> { 'x' => 'reflect',
		     '#' => 'shine',
 		     't' => 'v' },
 'spiegelung'	=> { 'x' => 'reflection',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'sprache'	=> { 'x' => 'language',
		     '#' => 'speech',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'sprechen'	=> { 'x' => 'speak',
		     '#' => 'talk',
 		     't' => 'v' },
 'staat'	=> { 'x' => 'state',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'en' },
 'starten'	=> { 'x' => 'start',
		     '#' => 'take off',
 		     't' => 'v' },
 'stecken'	=> { 'x' => 'stick',
		     '#' => 'put',
 		     't' => 'v' },
 'stehen'	=> { 'x' => 'stand',
 		     't' => 'v' },
 'stehlen'	=> { 'x' => 'steal',
		     '#' => 'irreg.; pilfer',
 		     't' => 'v' },
  'gestohlen'	=> { 'x' => 'stolen',
  		     'r' => 'stehlen',
		     't' => 'v',
		     'c' => 'pp' },
  'stiehlt'	=> { 'x' => 'steals',
  		     'r' => 'stehlen',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'p' },
  'stahl'	=> { 'x' => 'stole',
  		     'r' => 'stehlen',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
 'steif'	=> { 'x' => 'stiff',
		     '#' => 'rigid',
 		     't' => 'a' },
 'steigen'	=> { 'x' => 'climb',
		     '#' => 'rise, mount; irreg.',
 		     't' => 'v' },
 'steil'	=> { 'x' => 'steep',
 		     't' => 'a' },
 'stein'	=> { 'x' => 'stone',
		     '#' => 'rock',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'steinbock'	=> { 'x' => 'ibex',
 		     't' => 'n',
		     'g' => 'm' },
 'stellen'	=> { 'x' => 'place',
		     '#' => 'put, arrange, adjust, set, provide, supply',
 		     't' => 'v' },
 'sterben'	=> { 'x' => 'die',
		     '#' => 'irreg.',
 		     't' => 'v' },
 'stern'	=> { 'x' => 'star',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'stimme'	=> { 'x' => 'voice',
		     '#' => 'vote',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'stimmen'	=> { 'x' => 'agree',
		     '#' => 'vote, be correct',
 		     't' => 'v' },
 'stimmung'	=> { 'x' => 'mood',
		     '#' => 'feeling',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'stirn'	=> { 'x' => 'brow',
		     '#' => 'forehead',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'studieren'	=> { 'x' => 'study',
 		     't' => 'v' },
 'stunde'	=> { 'x' => 'hour',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'suchen'	=> { 'x' => 'search',
		     '#' => 'search for, seek',
 		     't' => 'v' },
 'sdafrika'	=> { 'x' => 'south africa',
		     '#' => 'proper, place; multiword',
 		     't' => 'n' },
 'tag'		=> { 'x' => 'day',
 		     't' => 'n' },
 'taiwan'	=> { 'x' => 'taiwan',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'tasse'	=> { 'x' => 'cup',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'tastatur'	=> { 'x' => 'keyboard',
 		     't' => 'n',
		     'g' => 'f' },
 'teil'		=> { 'x' => 'part',
 		     't' => 'n' },
 'terrorismus'	=> { 'x' => 'terrorism',
 		     't' => 'n' },
 'teuer'	=> { 'x' => 'expensive',
		     '#' => 'irreg.',
 		     't' => 'a' },
 'teufel'	=> { 'x' => 'devil',
 		     't' => 'n',
		     'g' => 'm' },
 'thailndisch'	=> { 'x' => 'thai',
		     '#' => 'adj. and neut. noun' },
 'thema'	=> { 'x' => 'theme',
		     '#' => 'subject',
 		     't' => 'n',
		     'g' => 'n' },
 'theorie'	=> { 'x' => 'theory',
 		     't' => 'n' },
 'titel'	=> { 'x' => 'title',
 		     't' => 'n' },
 'tochter'	=> { 'x' => 'daughter',
 		     't' => 'n',
		     'g' => 'f' },
 'tote'		=> { 'x' => 'corpse',
		     '#' => 'dead man',
 		     't' => 'n',
		     'g' => 'm' },
 'tten'	=> { 'x' => 'kill',
 		     't' => 'v' },
 'tragen'	=> { 'x' => 'carry',
		     '#' => 'wear',
 		     't' => 'v' },
 'trauen'	=> { 'x' => 'trust',
 		     't' => 'v' },
 'treffen'	=> { 'x' => 'meeting',
 		     't' => 'n',
		     'g' => 'n' },
 'tschechisch'	=> { 'x' => 'czech',
		     '#' => 'adj. and neut. noun' },
 'tunika'	=> { 'x' => 'tunic',
		     '#' => 'plural: tunikken',
 		     't' => 'n',
		     'g' => 'f' },
 'trke'	=> { 'x' => 'turk',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'n',
		     '#' => '?' },
 'trkei'	=> { 'x' => 'turkey',
		     '#' => 'the country',
 		     't' => 'n',
		     'g' => 'f' },
 'trkisch'	=> { 'x' => 'turkish',
		     '#' => 'adj. and neut. noun' },
 'bersetzen'	=> { 'x' => 'translate',
 		     't' => 'v' },
 'bersetzer'	=> { 'x' => 'translator',
 		     't' => 'n',
		     'g' => 'm' },
 'bersetzung'	=> { 'x' => 'translation',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'en' },
 'bertragen'	=> { 'x' => 'transport',
		     '#' => 'transmit, broadcast; ...',
 		     't' => 'v',
		     '#' => 'irreg.' },
 'uhr'		=> { 'x' => 'hour',
 		     't' => 'n' },
 'umzug'	=> { 'x' => 'removal',
		     '#' => 'procession',
 		     't' => 'n',
		     'g' => 'm' },
 'ungarisch'	=> { 'x' => 'hungarian',
		     '#' => 'adj. and neut. noun' },
 'uniform'	=> { 'x' => 'uniform',
 		     't' => 'n' },
 'universitt'	=> { 'x' => 'university',
 		     't' => 'n' },
 'untergehen'	=> { 'x' => 'perish',
		     '#' => 'die, sink; irreg.',
 		     't' => 'v' },
 'untersttzen'	=> { 'x' => 'support',
		     '#' => 'back',
 		     't' => 'v' },
 'untersttzung'	=> { 'x' => 'support',
 			     't' => 'n',
			     'g' => 'f',
			     'p' => 'en' },
 'uruguay'	=> { 'x' => 'uruguay',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'vehikel'	=> { 'x' => 'vehicle',
		     '#' => 'cf fahrzeug',
 		     't' => 'n',
		     'g' => 'n' },
 'velo'		=> { 'x' => 'bicycle',
		     '#' => 'bike; swiss german; cf fahrrad',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 's' },
 'vene'		=> { 'x' => 'vein',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'verbrennen'	=> { 'x' => 'burn',
 		     't' => 'v' },
 'vereinten'	=> { 'x' => 'united',
		     '#' => 'in Die vereinten Nationen...',
 		     't' => 'a' },
 'verkaufen'	=> { 'x' => 'sell',
 		     't' => 'v' },
 'verletzen'	=> { 'x' => 'wound',
		     '#' => 'violate',
 		     't' => 'v' },
 'verlieben'	=> { 'x' => 'fall in love',
		     '#' => 'multiword; refl. only',
 		     't' => 'v' },
 'verlust'	=> { 'x' => 'loss',
 		     't' => 'n',
		     'g' => 'm' },
 'versuchen'	=> { 'x' => 'try',
		     '#' => 'test, tempt',
 		     't' => 'v' },
 'versuchung'	=> { 'x' => 'temptation',
 		     't' => 'n' },
 'verzeihen'	=> { 'x' => 'forgive',
		     '#' => 'pardon',
 		     't' => 'v' },
 'vietnam'	=> { 'x' => 'vietnam',
		     '#' => 'proper, place',
		     't' => 'n' },
 'vorherig'	=> { 'x' => 'previous',
 		     't' => 'a' },
 'vorschlag'	=> { 'x' => 'suggestion',
		     '#' => 'proposal',
 		     't' => 'n',
		     'g' => 'm' },
 'vorschlagen'	=> { 'x' => 'propose',
		     '#' => 'suggest; irreg.',
 		     't' => 'v' },
 'vorstellen'	=> { 'x' => 'introduce',
		     '#' => 'sich=imagine',
 		     't' => 'v' },
 'warten'	=> { 'x' => 'wait',
 		     't' => 'v' },
 'wecken'	=> { 'x' => 'wake',
		     '#' => 'rouse',
 		     't' => 'v' },
 'wecker'	=> { 'x' => 'alarm clock',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => '-' },
 'weg'		=> { 'x' => 'way',
 		     't' => 'n' },
 'weihnacht'	=> { 'x' => 'christmas',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'e' },
 'wein'		=> { 'x' => 'wine',
 		     't' => 'n',
		     'g' => 'm' },
 'wei'		=> { 'x' => 'white',
		     '#' => 'also form of v. wissen',
 		     't' => 'a' },
 'weit'		=> { 'x' => 'far',
		     '#' => 'wide, broad',
 		     't' => 'a' },
 'welt'		=> { 'x' => 'world',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'werfen'	=> { 'x' => 'throw',
		     '#' => 'irreg.',
 		     't' => 'v' },
 'wert'		=> { 'x' => 'worth',
		     '#' => 'valued' },
 'wieder'	=> { 'x' => 'again',
		     '#' => 'back' },
 'wiederholen'	=> { 'x' => 'repeat',
 		     't' => 'v' },
 'wiederholung'	=> { 'x' => 'repetition',
 		     't' => 'n',
		     'g' => 'f' },
 'wilkommen'	=> { 'x' => 'welcome',
 		     't' => 'a' },
 'winter'	=> { 'x' => 'winter',
 		     't' => 'n',
		     'g' => 'm' },
 'wissen'	=> { 'x' => 'know',
		     '#' => 'as in saber',
 		     't' => 'v' },
 'woche'	=> { 'x' => 'week',
 		     't' => 'n' },
 'wochenende'	=> { 'x' => 'weekend',
 		     't' => 'n',
		     'g' => 'n' },
 'wort'		=> { 'x' => 'word',
		     '#' => 'written; spoken is -e plural',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'er' },
 'wrterbuch'	=> { 'x' => 'dictionary',
 		     't' => 'n' },
 'wunderbar'	=> { 'x' => 'wonderful',
 		     't' => 'a' },
 'zahlen'	=> { 'x' => 'pay',
 		     't' => 'v' },
 'zhlen'	=> { 'x' => 'count',
 		     't' => 'v' },
 'zeichen'	=> { 'x' => 'sign',
		     '#' => 'signal',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => '-' },
 'zeigen'	=> { 'x' => 'show',
 		     't' => 'v' },
 'zeit'		=> { 'x' => 'time',
 		     't' => 'n' },
 'zeitung'	=> { 'x' => 'newspaper',
 		     't' => 'n' },
 'zentrum'	=> { 'x' => 'centre',
 		     't' => 'n',
		     'g' => 'n' },
 'zeile'	=> { 'x' => 'line',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'zersprengen'	=> { 'x' => 'explode',
		     '#' => 'blow up, burst',
 		     't' => 'v' },
 'ziege'	=> { 'x' => 'goat',
 		     't' => 'n',
		     'g' => 'f',
		     'p' => 'n' },
 'ziel'		=> { 'x' => 'aim',
		     '#' => 'end, goal, target',
 		     't' => 'n',
		     'g' => 'n',
		     'p' => 'e' },
 'zins'		=> { 'x' => 'interest',
		     '#' => 'financial only',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'zopf'		=> { 'x' => 'plait',
		     '#' => 'am. braid',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
 'zusammen'	=> { 'x' => 'together',
		     '#' => 'adv' },
 'zusammentreffen'	=> { 'x' => 'meet',
		     '#' => 'coincide; irreg.',
 			     't' => 'v' },
 'zwerg'	=> { 'x' => 'dwarf',
 		     't' => 'n',
		     'g' => 'm',
		     'p' => 'e' },
);
}

1;

