# vi: ts=8 sw=8 sts=8

package Tongues::Maori;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Western European CP1252 / ISO 8859-1 / Latin-1

# Alphabetical order
# A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
# a b c d e f g h i j k l m n o p q r s t u v w x y z

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::WestEuroWin;

$charset = new Charsets::WestEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun case
 # Noun gender
 # Noun plural
 # Adjectives
 # Verb past participle
 # Verb gerund
 # Verb present participle
 # Verb present
 # Verb present subjunctive
 # Verb imperitive
 # Verb preterite
 # Verb imperfect
 # Verb future
 # Verb conditional
 # Adverb
 # Higher level transformations
];

#Fields:
# x -> translatioin		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjugation
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default
#	f -> feminine
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	d -> dual
#	t -> trial
#	p -> plural
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
# c -> conjugation:
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative
# c -> case:
#	n -> nominative 		default
#	a -> accusative 

# Maori to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 #    Masculine
 #     Singular
 #     Plural
 #    Feminine
 #     Singular
 #     Plural
 #   Indefinite articles
 #  Pronouns & possessive adjectives
 #   1st person inclusive
 'taaua'	=> { 'x' => 'we' },
 'taatou'	=> { 'x' => 'we' },
 #   1st person exclusive
 'au'		=> { 'x' => 'i' },
 'ahau'		=> { 'x' => 'i' },
 'maaua'	=> { 'x' => 'we' },
 'maatou'	=> { 'x' => 'we' },
 #   2nd person
 'koe'		=> { 'x' => 'you' },
 'koorua'	=> { 'x' => 'you' },
 'koutou'	=> { 'x' => 'you' },
 #   3rd person
 'ia'		=> { 'x' => 'he' },
 'raaua'	=> { 'x' => 'they' },
 'raatou'	=> { 'x' => 'they' },
 #   Other pronouns
 #  Interrogatives
 #  Other functional words
 'de'	=> { 'x' => 'of',
	     't' => 'p' },
 #   Partitive / Contractions
 # Numbers: cardinal and ordinal
 'tahi'		=> { 'x' => 'one' },
 'rua'		=> { 'x' => 'two' },
 'toru'		=> { 'x' => 'three' },
 'whaa'		=> { 'x' => 'four' },
 'rima'		=> { 'x' => 'five' },
 'ono'		=> { 'x' => 'six' },
 'whitu'	=> { 'x' => 'seven' },
 'waru'		=> { 'x' => 'eight' },
 'iwa'		=> { 'x' => 'nine' },
 'tekau'	=> { 'x' => 'ten' },
 'ngahuru'	=> { 'x' => 'ten' },
 'ran'		=> { 'x' => 'hundred' },
 'mano'		=> { 'x' => 'thousand' },
 # Days and months
 # Key verbs
 # Vocabulary
 'rarangi'  => { 'x' => 'article',
	         't' => 'n' },
);
}

1;

