# vi: ts=8 sw=8

package Tongues::Portugues;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Western European CP1252 / ISO 8859-1 / Latin-1

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::WestEuroWin;

$charset = new Charsets::WestEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun gender
 # Noun plural
 [ 'as',	'a',	'',		's',	'n' ],
 [ 'es',	'e',	'',		's',	'n' ],
 [ 'os',	'o',	'',		's',	'n' ],
 [ 'ns',	'm',	'',		's',	'n' ],
 [ 'es',	'o',	'',		's',	'n' ],
 # Adjectives
 [ 'a',		'o',	'',		'',	'a' ],	# fem.
 [ 'os',	'o',	'',		'',	'a' ],	# pl.
 [ 'as',	'o',	'',		'',	'a' ],	# fem. pl.
 [ 's',		'',	'',		'',	'a' ],	# pl.
 # Verb past participle
 [ 'ado',	'ar',	'',		'ed',	'v' ],
 [ 'ada',	'ar',	'',		'ed',	'v' ],
 [ 'ados',	'ar',	'',		'ed',	'v' ],
 [ 'adas',	'ar',	'',		'ed',	'v' ],
 [ 'ido',	'er',	'',		'ed',	'v' ],
 [ 'ida',	'er',	'',		'ed',	'v' ],
 [ 'idos',	'er',	'',		'ed',	'v' ],
 [ 'idas',	'er',	'',		'ed',	'v' ],
 [ 'ido',	'ir',	'',		'ed',	'v' ],
 [ 'ida',	'ir',	'',		'ed',	'v' ],
 [ 'idos',	'ir',	'',		'ed',	'v' ],
 [ 'idas',	'ir',	'',		'ed',	'v' ],
 # Verb gerund
 [ 'ando',	'ar',	'',		'ing',	'v' ],
 [ 'endo',	'er',	'',		'ing',	'v' ],
 [ 'indo',	'ir',	'',		'ing',	'v' ],
 # Verb present participle
 # Verb present
 [ 'o',		'ar',	'(i) ',		'',	'v' ],
 [ 'o',		'er',	'(i) ',		'',	'v' ],
 [ 'o',		'ir',	'(i) ',		'',	'v' ],
 [ 'as',	'ar',	'(you) ',	'',	'v' ],
 [ 'es',	'er',	'(you) ',	'',	'v' ],
 [ 'is',	'ir',	'(you) ',	'',	'v' ],
 [ 'a',		'ar',	'(he) ',	's',	'v' ],
 [ 'e',		'er',	'(he) ',	's',	'v' ],
 [ 'e',		'ir',	'(he) ',	's',	'v' ],
 [ 'amos',	'ar',	'(we) ',	'',	'v' ],
 [ 'emos',	'er',	'(we) ',	'',	'v' ],
 [ 'imos',	'ir',	'(we) ',	'',	'v' ],
 [ 'ais',       'ar',   '(you) ',       '',     'v' ],
 [ 'eis',       'er',   '(you) ',       '',     'v' ],
 [ 'is',        'ir',   '(you) ',       '',     'v' ],
 [ 'am',	'ar',	'(they) ',	'',	'v' ],
 [ 'em',	'er',	'(they) ',	'',	'v' ],
 [ 'em',	'ir',	'(they) ',	'',	'v' ],
 # Verb present subjunctive, imperitive
 [ 'e',		'ar',	'',		'',	'v' ],
 [ 'a', 	'ar',	'',     	'',	'v' ],
 [ 'e',		'ar',	'',     	'',	'v' ],
 [ 'emos',	'ar',	'let\'s ',     	'',	'v' ],
 [ 'ai',        'ar',   '',             '',     'v' ],
 [ 'em',	'ar',	'',     	'',	'v' ],
 [ 'a',		'er',	'',		'',	'v' ],
 [ 'e', 	'er',	'',     	'',	'v' ],
 [ 'a',		'er',	'',     	'',	'v' ],
 [ 'amos',	'er',	'let\'s ',     	'',	'v' ],
 [ 'ei',        'er',   '',             '',     'v' ],
 [ 'am',	'er',	'',     	'',	'v' ],
 [ 'a',		'ir',	'',		'',	'v' ],
 [ 'e', 	'ir',	'',     	'',	'v' ],
 [ 'a',		'ir',	'',     	'',	'v' ],
 [ 'amos',	'ir',	'let\'s ',     	'',	'v' ],
 [ 'i',         'ir',   '',             '',     'v' ],
 [ 'am',	'ir',	'',     	'',	'v' ],
 # Verb preterite
 [ 'ei',	'ar',	'(i) ',		'ed',	'v' ],
 [ 'i',		'er',	'(i) ',		'ed',	'v' ],
 [ 'i',		'ir',	'(i) ',		'ed',	'v' ],
 [ 'aste',	'ar',	'(you) ',	'ed',	'v' ],
 [ 'este',	'er',	'(you) ',	'ed',	'v' ],
 [ 'iste',	'ir',	'(you) ',	'ed',	'v' ],
 [ 'ou',	'ar',	'(he) ',	'ed',	'v' ],
 [ 'eu',	'er',	'(he) ',	'ed',	'v' ],
 [ 'io',	'ir',	'(he) ',	'ed',	'v' ],
 [ 'amos',	'ar',	'(we) ',	'ed',	'v' ],
 [ 'emos',	'er',	'(we) ',	'ed',	'v' ],
 [ 'imos',	'ir',	'(we) ',	'ed',	'v' ],
 [ 'astes',	'ar',	'(you) ',	'ed',	'v' ],
 [ 'estes',	'er',	'(you) ',	'ed',	'v' ],
 [ 'istes',	'ir',	'(you) ',	'ed',	'v' ],
 [ 'aram',	'ar',	'(they) ',	'ed',	'v' ],
 [ 'eram',	'er',	'(they) ',	'ed',	'v' ],
 [ 'iram',	'ir',	'(they) ',	'ed',	'v' ],
 # Verb imperfect
 [ 'ava',	'ar',	'(he) ',	'ed',	'v' ],	# I
 [ 'ia',	'er',	'(he) ',	'ed',	'v' ],	# I
 [ 'ia',	'ir',	'(he) ',	'ed',	'v' ],	# I
 [ 'avas',	'ar',	'(you) ',	'ed',	'v' ],
 [ 'ias',	'er',	'(you) ',	'ed',	'v' ],
 [ 'ias',	'ir',	'(you) ',	'ed',	'v' ],
 [ 'vamos',	'ar',	'(we) ',	'ed',	'v' ],
 [ 'amos',	'er',	'(we) ',	'ed',	'v' ],
 [ 'amos',	'ir',	'(we) ',	'ed',	'v' ],
 [ 'veis',	'ar',	'(you) ',	'ed',	'v' ],
 [ 'veis',	'er',	'(you) ',	'ed',	'v' ],
 [ 'eis',	'ir',	'(you) ',	'ed',	'v' ],
 [ 'avan',	'ar',	'(they) ',	'ed',	'v' ],
 [ 'an',	'er',	'(they) ',	'ed',	'v' ],
 [ 'an',	'ir',	'(they) ',	'ed',	'v' ],
 # Verb future
 [ 'arei',	'ar',	'(i) will ',	'',	'v' ],
 [ 'ars',	'ar',	'(you) will ',	'',	'v' ],
 [ 'ar',	'ar',	'(he) will ',	'',	'v' ],
 [ 'aremos',	'ar',	'(we) will ',	'',	'v' ],
 [ 'areis',	'ar',	'(you) will ',	'',	'v' ],
 [ 'aro',	'ar',	'(they) will ',	'',	'v' ],
 [ 'erei',	'er',	'(i) will ',	'',	'v' ],
 [ 'ers',	'er',	'(you) will ',	'',	'v' ],
 [ 'er',	'er',	'(he) will ',	'',	'v' ],
 [ 'eremos',	'er',	'(we) will ',	'',	'v' ],
 [ 'ereis',	'er',	'(you) will ',	'',	'v' ],
 [ 'ero',	'er',	'(they) will ',	'',	'v' ],
 [ 'irei',	'ir',	'(i) will ',	'',	'v' ],
 [ 'irs',	'ir',	'(you) will ',	'',	'v' ],
 [ 'ir',	'ir',	'(he) will ',	'',	'v' ],
 [ 'iremos',	'ir',	'(we) will ',	'',	'v' ],
 [ 'ireis',	'ir',	'(you) will ',	'',	'v' ],
 [ 'iro',	'ir',	'(they) will ',	'',	'v' ],
 # Verb conditional
 [ 'aria',	'ar',	'(i) would ',	'',	'v' ],
 [ 'arias',	'ar',	'(you) would ',	'',	'v' ],
 [ 'aria',	'ar',	'(he) would ',	'',	'v' ],
 [ 'aramos',	'ar',	'(we) would ',	'',	'v' ],
 [ 'areis',	'ar',	'(you) would ',	'',	'v' ],
 [ 'ariam',	'ar',	'(they) would ',	'',	'v' ],
 [ 'eria',	'er',	'(i) would ',	'',	'v' ],
 [ 'erias',	'er',	'(you) would ',	'',	'v' ],
 [ 'eria',	'er',	'(he) would ',	'',	'v' ],
 [ 'eramos',	'er',	'(we) would ',	'',	'v' ],
 [ 'ereis',	'er',	'(you) would ',	'',	'v' ],
 [ 'eriam',	'er',	'(they) would ',	'',	'v' ],
 [ 'iriam',	'ir',	'(they) would ',	'',	'v' ],
 # Some other quasi-regular conjugations
 [ 'quei',	'car',	'(i) ', 	'ed ',	'v' ],  # 1p s. preterite
 [ 'que',	'car',	'',     	'',	'v' ],  # 1p s. pres. subj., imp.
 [ 'que',	'car',	'',     	'',	'v' ],  # 2p s. pres. subj., imp.
 [ 'que',	'car',	'',     	'',	'v' ],  # 3p s. pres. subj., imp.
 [ 'quemos',	'car',	'let\'s',     	'',	'v' ],  # 1p pl. pres. subj., imp.
 [ 'quei',	'car',	'',     	'',	'v' ],  # 2p pl. pres. subj., imp.
 [ 'que',	'car',	'',     	'',	'v' ],  # 3p pl. pres. subj., imp.

 # Adverb
 # Higher level transformations
];

#Fields:
# x -> translatioin		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjugation
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default
#	f -> feminine		default for words ending in a
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	p -> plural		default for words ending in s
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
# c -> conjugation:
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative

# Portuguese to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 'o'		=> { 'x' => 'the',
		     't' => 'art',
		     'n' => 's',
		     'g' => 'm' },
 'os'		=> { 'x' => 'the',
		     't' => 'art',
		     'n' => 'p',
		     'g' => 'm' },
 'a'		=> { 'x' => 'the',
		     't' => 'art',
		     'n' => 's',
		     'g' => 'f' },
 'as'		=> { 'x' => 'the',
		     't' => 'art',
		     'n' => 's',
		     'g' => 'f' },
 #   Indefinite articles
 'uma'		=> { 'x' => 'a',
		     '#' => 'an',
 		     't' => 'art',
		     'n' => 's',
		     'g' => 'f' },
 #  Pronouns & possessive adjectives
 #   1st person singular
 'eu'		=> { 'x' => 'i',
		     't' => 'pro' },
 'meu'		=> { 'x' => 'my',
		     't' => 'a' },
 'minha'	=> { 'x' => 'my',
		     't' => 'a' },
 #   2nd person singular
 'tu'		=> { 'x' => 'you',
		     't' => 'pro', },
 'teu'		=> { 'x' => 'your',
		     't' => 'a' },
 'tua'		=> { 'x' => 'you',
		     't' => 'a' },
 #   3rd person singular
 'ele'	        => { 'x' => 'he',
		     't' => 'pro', },
 'ela'	        => { 'x' => 'she',
		     't' => 'pro', },
 'voc' 	=> { 'x' => 'you',
		     '#' => 'it only exists in Brazilian Portuguese, and is 3rd person',
		     't' => 'pro' },
 'seu'  	=> { 'x' => 'your',
		     '#' => 'actually 3rd person, but *can* mean "your" in Brazilian Portuguese',
		     't' => 'a' },
 'sua'  	=> { 'x' => 'your',
		     't' => 'a' },
 #   1st person plural
 'ns'		=> { 'x' => 'we',
		     't' => 'pro' },
 'nosso'	=> { 'x' => 'my',
		     't' => 'a' },
 #   2nd person plural
 'vs'		=> { 'x' => 'you',
		     't' => 'pro', },
 'vosso'	=> { 'x' => 'your',
		     't' => 'a' },
 #   3rd person plural
 'eles'	        => { 'x' => 'they',
		     't' => 'pro', },
 'elas'	        => { 'x' => 'they',
		     't' => 'pro', },
 'vocs' 	=> { 'x' => 'you',
		     '#' => 'it only exists in Brazilian Portuguese, and is 3rd person',
		     't' => 'pro' },
 #  Other functional words
 'aqui' 	=> { 'x' => 'here',
		     't' => 'adv' },
 'atravs'	=> { 'x' => 'through',
		     '#' => 'also prep?',
		     't' => 'adv' },
 'com'		=> { 'x' => 'with',
 		     't' => 'p' },
 'de'		=> { 'x' => 'of',
 		     't' => 'p' },
 'e'            => { 'x' => 'and' },
 'esta'         => { 'x' => 'this' },
 'mais'		=> { 'x' => 'more' },
 'outro'	=> { 'x' => 'another',
		     '#' => 'other?',
		     't' => 'a' },
 'para'		=> { 'x' => 'to',
		     '#' => 'for',
 		     't' => 'p' },
 'por'		=> { 'x' => 'for',
		     '#' => 'by, per',
		     't' => 'p' },
 'quando'	=> { 'x' => 'when' },
 'que'		=> { 'x' => 'that',
		     '#' => 'than' },
 'se'		=> { 'x' => 'if' },
 'tambm'	=> { 'x' => 'also',
		     '#' => 'too' },
 'to'		=> { 'x' => 'so' },
 'tudo'		=> { 'x' => 'everything' },
 'vrios'	=> { 'x' => 'various',
 		     't' => 'a',
		     'n' => 'p' },
 #   Partitive / Contractions
 # Numbers: cardinal and ordinal
 'zero'		=> { 'x' => 'zero' },
 'dois'		=> { 'x' => 'two' },
 'trs'		=> { 'x' => 'three' },
 'quatro'	=> { 'x' => 'four' },
 'cinco'	=> { 'x' => 'five' },
 'seis'		=> { 'x' => 'six' },
 'sete' 	=> { 'x' => 'seven' },
 'oito'		=> { 'x' => 'eight' },
 'nove' 	=> { 'x' => 'nine' },
 'dez'		=> { 'x' => 'ten' },
 'cem'		=> { 'x' => 'hundred' },
 'mil'		=> { 'x' => 'thousand' },

 'primeiro'	=> { 'x' => 'first',
 		     't' => 'a' },
 'segundo'	=> { 'x' => 'second',
 		     't' => 'a' },
 'terceiro'	=> { 'x' => 'third',
 		     't' => 'a' },

 # Days and months
 'domingo'	=> { 'x' => 'sunday' },
 'segunda-feira'	=> { 'x' => 'monday' },
 'tera-feira'	=> { 'x' => 'tuesday' },
 'quarta-feira'	=> { 'x' => 'wednesday' },
 'quinta-feira'	=> { 'x' => 'thursday' },
 'sexta-feira'	=> { 'x' => 'friday' },
 'sbado'	=> { 'x' => 'saturday' },
 'janeiro'	=> { 'x' => 'january' },
 'fevereiro'	=> { 'x' => 'february' },
 'maro'	=> { 'x' => 'march' },
 'abril'	=> { 'x' => 'april' },
 'maio'		=> { 'x' => 'may' },
 'junho'	=> { 'x' => 'june' },
 'julho'	=> { 'x' => 'july' },
 'agosto'	=> { 'x' => 'august' },
 'setembro'	=> { 'x' => 'september' },
 'outubro'	=> { 'x' => 'october' },
 'novembro'	=> { 'x' => 'november' },
 'dezembro'	=> { 'x' => 'december' },
 # Key verbs
 'ser'  	=> { 'x' => 'be',
		     't' => 'v' },
 'sou'  	=> { 'x' => '(i) am',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 's'   	=> { 'x' => '(you) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 ''    	=> { 'x' => '(it) is',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'somos'  	=> { 'x' => '(we) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'sois'  	=> { 'x' => '(you) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'so'  	=> { 'x' => '(they) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'fui'  	=> { 'x' => '(i) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'foste'  	=> { 'x' => '(you) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'foi'  	=> { 'x' => '(he) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'fomos'  	=> { 'x' => '(we) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'fostes'  	=> { 'x' => '(i) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'foram'  	=> { 'x' => '(i) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'era'  	=> { 'x' => '(i) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'eras'  	=> { 'x' => '(you) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'era'  	=> { 'x' => '(he) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'ramos'  	=> { 'x' => '(we) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'reis'  	=> { 'x' => '(i) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'eram'  	=> { 'x' => '(i) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },

 'estar'	=> { 'x' => 'be',
		     't' => 'v' },
 'estou'  	=> { 'x' => '(i) am',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'ests'   	=> { 'x' => '(you) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'est'    	=> { 'x' => '(it) is',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'estamos'  	=> { 'x' => '(we) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'estais'  	=> { 'x' => '(you) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'esto'  	=> { 'x' => '(they) are',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'p' },
 'estive'  	=> { 'x' => '(i) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'estiveste'  	=> { 'x' => '(you) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'esteve'  	=> { 'x' => '(he) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'estivemos'  	=> { 'x' => '(we) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'estiveste'  	=> { 'x' => '(you) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'estiveram'  	=> { 'x' => '(they) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 't' },
 'estava'  	=> { 'x' => '(i) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'estavas'  	=> { 'x' => '(you) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'estava'  	=> { 'x' => '(he) was',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'estvamos'  	=> { 'x' => '(we) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'estveis'  	=> { 'x' => '(you) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },
 'estavam'  	=> { 'x' => '(they) were',
		     'r' => 'ser',
		     't' => 'v',
		     'c' => 'i' },

 'ter'  	=> { 'x' => 'have',
		     't' => 'v' },
 'tenho'  	=> { 'x' => '(i) have',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'p' },
 'tens'  	=> { 'x' => '(you) have',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'p' },
 'temos'  	=> { 'x' => '(we) have',
		     '#' => 'this is not really necessary, because this is a regular form',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'p' },
 'tem'  	=> { 'x' => '(he) has',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'p' },
 'tendes'  	=> { 'x' => '(you) have',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'p' },
 'tm'  	=> { 'x' => '(they) have',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'p' },
 'tive'  	=> { 'x' => '(i) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 't' },
 'tiveste'  	=> { 'x' => '(you) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 't' },
 'teve'  	=> { 'x' => '(we) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 't' },
 'tivemos'  	=> { 'x' => '(he) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 't' },
 'tivestes'  	=> { 'x' => '(you) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 't' },
 'tiveram'  	=> { 'x' => '(they) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 't' },
 'tinha'  	=> { 'x' => '(i) had',
		     'r' => 'have',
		     't' => 'i',
		     'c' => 't' },
 'tinhas'  	=> { 'x' => '(you) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'i' },
 'tinha'  	=> { 'x' => '(we) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'i' },
 'tnhamos'  	=> { 'x' => '(he) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'i' },
 'tnheis'  	=> { 'x' => '(you) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'i' },
 'tinham'  	=> { 'x' => '(they) had',
		     'r' => 'have',
		     't' => 'v',
		     'c' => 'i' },

 'vou'  	=> { 'x' => 'will',
		     '#' => 'means also "(i) go"',
		     'r' => 'ir',
		     't' => 'v',
		     'c' => 'p' },
 'vais'  	=> { 'x' => 'will',
		     '#' => '(you) go',
		     'r' => 'ir',
		     't' => 'v',
		     'c' => 'p' },
 'vai'  	=> { 'x' => 'will',
		     '#' => '(he or you) goes',
		     'r' => 'ir',
		     't' => 'v',
		     'c' => 'p' },
 'vamos'  	=> { 'x' => 'will',
		     '#' => '(we) go',
		     'r' => 'ir',
		     't' => 'v',
		     'c' => 'p' },
 'ides'  	=> { 'x' => 'will',
		     '#' => '(you) go',
		     'r' => 'ir',
		     't' => 'v',
		     'c' => 'p' },
 'vo'  	=> { 'x' => 'will',
		     '#' => '(them or you) go',
		     'r' => 'ir',
		     't' => 'v',
		     'c' => 'p' },

 # Vocabulary
 'aberto'	=> { 'x' => 'open',
 		     't' => 'a' },
 'abrir'	=> { 'x' => 'open',
		     't' => 'v' },
 'acessar'	=> { 'x' => 'access',
		     't' => 'v' },
 'afiliao'	=> { 'x' => 'affiliation',
 		     't' => 'n' },
 'alemanha'	=> { 'x' => 'germany',
 		     't' => 'n' },
 'alemo'	=> { 'x' => 'german',
		     '#' => 'adj. and noun?' },
 'amor'		=> { 'x' => 'love',
 		     't' => 'n' },
 'arquivo'	=> { 'x' => 'file',
		     't' => 'n' },
 'artigo'	=> { 'x' => 'article',
 		     't' => 'n' },
 'assemblia'	=> { 'x' => 'assembly',
 		     't' => 'n' },
 'assistncia'	=> { 'x' => 'assistance',
 		     't' => 'n' },
 'associao'	=> { 'x' => 'association',
 		     't' => 'n' },
 'assunto'	=> { 'x' => 'subject',
		     't' => 'n' },
 'astronauta'	=> { 'x' => 'astronaut',
 		     't' => 'n' },
 'atualmente'	=> { 'x' => 'currently',
		     '#' => 'presently, now',
 		     't' => 'adv' },
 'austrlia'	=> { 'x' => 'australia',
 		     't' => 'n' },
 'ustria'	=> { 'x' => 'austria',
		     '#' => 'proper, place',
		     't' => 'n' },
 'auxiliar'	=> { 'x' => 'help',
		     't' => 'v' },
 'beber'	=> { 'x' => 'drink',
 		     't' => 'v' },
 'bebida'	=> { 'x' => 'drink',
 		     't' => 'n' },
 'beleza'	=> { 'x' => 'beauty',
 		     't' => 'n' },
 'blgica'	=> { 'x' => 'belgium',
		     '#' => 'proper, place',
		     't' => 'n' },
 'bonito'	=> { 'x' => 'pretty',
 		     't' => 'a' },
 'brasileiro'	=> { 'x' => 'brazilian',
		     't' => 'a' },
 'cair' 	=> { 'x' => 'fall',
		     't' => 'v' },
 'cabea'	=> { 'x' => 'head',
		     't' => 'n' },
 'cadastrar' 	=> { 'x' => 'subscribe',
		     't' => 'v' },
 'cantar'	=> { 'x' => 'sing',
 		     't' => 'v' },
 'caracter'	=> { 'x' => 'character',
		     '#' => 'letter',
 		     't' => 'n' },
 'cidade' 	=> { 'x' => 'city',
		     't' => 'n' },
 'clicar'	=> { 'x' => 'click',
 		     't' => 'v' },
 'codificao'	=> { 'x' => 'encoding',
		     '#' => 'codification',
 		     't' => 'n' },
 'coisa'	=> { 'x' => 'thing',
 		     't' => 'n' },
 'comer'	=> { 'x' => 'eat',
		     't' => 'v' },
 'completar'	=> { 'x' => 'complete',
 		     't' => 'v' },
 'compreender'	=> { 'x' => 'understand',
		     't' => 'v' },
 'comunidade'	=> { 'x' => 'community',
 		     't' => 'n' },
 'configurar' 	=> { 'x' => 'configure',
		     't' => 'v' },
 'conectar' 	=> { 'x' => 'connect',
		     't' => 'v' },
 'conexo' 	=> { 'x' => 'connection',
		     't' => 'n' },
 'considerao'	=> { 'x' => 'consideration',
 		     't' => 'n' },
 'consrcio'	=> { 'x' => 'consortium',
 		     't' => 'n' },
 'constituio'	=> { 'x' => 'constitution',
 		     't' => 'n' },
 'contedo' 	=> { 'x' => 'contents',
		     't' => 'n' },
 'cooperao'	=> { 'x' => 'cooperation',
 		     't' => 'n' },
 'coreano'	=> { 'x' => 'korean',
		     '#' => 'noun and adj.?' },
 'corpo'	=> { 'x' => 'body',
 		     't' => 'n' },
 'criar' 	=> { 'x' => 'create',
		     't' => 'n' },
 'data' 	=> { 'x' => 'date',
		     't' => 'n' },
 'declarao'	=> { 'x' => 'declaration',
 		     't' => 'n' },
 'deixar' 	=> { 'x' => 'leave',
		     't' => 'v' },
 'departamento'	=> { 'x' => 'department',
 		     't' => 'n' },
 'destino'	=> { 'x' => 'fate',
		     't' => 'n' },
 'dica' 	=> { 'x' => 'tip',
		     't' => 'n' },
 'dicionrio'	=> { 'x' => 'dictionary',
 		     't' => 'n' },
 'dignidade'	=> { 'x' => 'dignity',
 		     't' => 'n' },
 'dinamarqus'	=> { 'x' => 'danish',
		     '#' => 'noun and adj.?' },
 'direito'	=> { 'x' => 'right',
		     '#' => 'as in "human rights"',
 		     't' => 'n' },
 'diretrio'	=> { 'x' => 'directory',
 		     't' => 'n' },
 'discar'	=> { 'x' => 'dial',
		     't' => 'v' },
 'dissoluo'	=> { 'x' => 'dissolution',
 		     't' => 'n' },
 'documento'	=> { 'x' => 'document',
 		     't' => 'n' },
 'domnio'	=> { 'x' => 'domain',
		     't' => 'v' },
 'educao'	=> { 'x' => 'education',
 		     't' => 'n' },
 'electricidade'	=> { 'x' => 'electricity',
 			     't' => 'n' },
 'eletrnico'	=> { 'x' => 'electronic',
		     't' => 'a' },
 'encontrar'	=> { 'x' => 'find',
		     't' => 'v' },
 'endereo' 	=> { 'x' => 'address',
		     't' => 'n' },
 'entrevista'	=> { 'x' => 'interview',
 		     't' => 'n' },
 'envelope'	=> { 'x' => 'envelope',
 		     't' => 'n' },
 'enviar' 	=> { 'x' => 'send',
		     't' => 'v' },
 'episdio'	=> { 'x' => 'episode',
 		     't' => 'n' },
 'espanhol'	=> { 'x' => 'spanish',
		     '#' => 'noun and adj.?' },
 'esperanto'	=> { 'x' => 'esperanto',
 		     't' => 'n' },
 'esprito'	=> { 'x' => 'spirit',
 		     't' => 'n' },
 'estudar'	=> { 'x' => 'study',
		     't' => 'v' },
 'excluir'	=> { 'x' => 'exclude',
 		     't' => 'v' },
 'exemplo'	=> { 'x' => 'example',
 		     't' => 'n' },
 'expresso'	=> { 'x' => 'expression',
 		     't' => 'n' },
 'fcil'	=> { 'x' => 'easy',
 		     't' => 'a' },
 'facilitar'	=> { 'x' => 'facilitate',
		     't' => 'v' },
 'falar'	=> { 'x' => 'say',
		     't' => 'v' },
 'famlia'	=> { 'x' => 'family',
 		     't' => 'n' },
 'fechar'	=> { 'x' => 'close',
		     't' => 'v' },
 'ferramenta'	=> { 'x' => 'ferramentas',
		     't' => 'n' },
 'fim'  	=> { 'x' => 'end',
 		     't' => 'n' },
 'finlands'	=> { 'x' => 'finnish',
		     '#' => 'noun and adj.?' },
 'fonte'  	=> { 'x' => 'source',
 		     't' => 'n' },
 'francs'	=> { 'x' => 'french',
		     '#' => 'noun and adj.?' },
 'frase'	=> { 'x' => 'phrase',
 		     't' => 'n' },
 'fuga'		=> { 'x' => 'trail',
		     't' => 'n' },
 'garota'	=> { 'x' => 'girl',
 		     't' => 'n' },
 'graa'	=> { 'x' => 'grace',
		     '#' => '"de graa" means "for free"',
 		     't' => 'n' },
 'gratuito'	=> { 'x' => 'free',
 		     't' => 'a' },
 'guardar'	=> { 'x' => 'store',
		     't' => 'n' },
 'grego'	=> { 'x' => 'greek',
		     '#' => 'noun and adj.?' },
 'holands'	=> { 'x' => 'dutch',
		     '#' => 'noun and adj.?' },
 'hngaro'	=> { 'x' => 'hungarian',
		     '#' => 'noun and adj.?' },
 'cone'	=> { 'x' => 'icon',
		     't' => 'n' },
 'idioma'	=> { 'x' => 'language',
 		     't' => 'n' },
 'ignorar'	=> { 'x' => 'ignore',
 		     't' => 'v' },
 'implementao'	=> { 'x' => 'implementation',
 			     't' => 'n' },
 'imprimir'	=> { 'x' => 'print',
		     't' => 'v' },
 'incluir'	=> { 'x' => 'include',
		     't' => 'v' },
 'informao'	=> { 'x' => 'information',
 		     't' => 'n' },
 'inglaterra'	=> { 'x' => 'england',
		     '#' => 'proper, place',
		     't' => 'n' },
 'ingls'	=> { 'x' => 'english',
		     '#' => 'noun and adj.?' },
 'inicial'	=> { 'x' => 'initial',
 		     't' => 'a' },
 'inserir'	=> { 'x' => 'enter',
		     '#' => 'insert?',
 		     't' => 'v' },
 'instalao'	=> { 'x' => 'installation',
 		     't' => 'n' },
 'instalar'	=> { 'x' => 'install',
		     't' => 'v' },
 'internacional'	=> { 'x' => 'international',
 			     't' => 'a' },
 'interpretar'	=> { 'x' => 'interpret',
 		     't' => 'v' },
 'islands'	=> { 'x' => 'icelandic',
		     '#' => 'noun and adj.?' },
 'italiano'	=> { 'x' => 'italian',
		     '#' => 'noun and adj.?' },
 'janela' 	=> { 'x' => 'window',
		     't' => 'n' },
 'japo'	=> { 'x' => 'japan',
 		     't' => 'n' },
 'jornal'	=> { 'x' => 'newspaper',
 		     't' => 'n' },
 'ler'  	=> { 'x' => 'read',
		     't' => 'v' },
 'letra'	=> { 'x' => 'letter',
 		     't' => 'n' },
 'limitao'	=> { 'x' => 'limitation',
 		     't' => 'n' },
 'linda'	=> { 'x' => 'beautiful',
		     '#' => 'pretty?',
 		     't' => 'a' },
 'lngua'	=> { 'x' => 'language',
		     '#' => 'tongue?',
 		     't' => 'n' },
 'lista' 	=> { 'x' => 'list',
		     't' => 'n' },
 'listar' 	=> { 'x' => 'list',
		     't' => 'v' },
 'livre' 	=> { 'x' => 'free',
		     't' => 'a' },
 'livro'	=> { 'x' => 'book',
 		     't' => 'n' },
 'lugar'	=> { 'x' => 'place',
		     't' => 'n' },
 'maisculo'	=> { 'x' => 'capital',
		     '#' => 'as in "captial letter"',
 		     't' => 'a' },
 'mar'		=> { 'x' => 'sea',
 		     't' => 'n' },
 'membro'	=> { 'x' => 'member',
 		     't' => 'n' },
 'menina'	=> { 'x' => 'girl',
 		     't' => 'n' },
 'mensagem'	=> { 'x' => 'message',
		     't' => 'n' },
 'minuto'	=> { 'x' => 'minute',
 		     't' => 'n' },
 'misria'	=> { 'x' => 'misery',
 		     't' => 'n' },
 'moa'		=> { 'x' => 'girl',
 		     't' => 'n' },
 'mundo'	=> { 'x' => 'world',
 		     't' => 'n' },
 'nadar'	=> { 'x' => 'swim',
 		     't' => 'v' },
 'navegador'	=> { 'x' => 'browser',
 		     't' => 'n' },
 'necessrio'	=> { 'x' => 'necessary',
 		     't' => 'a' },
 'nome' 	=> { 'x' => 'name',
		     't' => 'n' },
 'noruega'	=> { 'x' => 'norway',
		     '#' => 'proper, place',
		     't' => 'n' },
 'noruegus'	=> { 'x' => 'norwegian',
		     '#' => 'noun and adj.?' },
 'novo'		=> { 'x' => 'new',
 		     't' => 'a' },
 'nmero'	=> { 'x' => 'number',
 		     't' => 'n' },
 'onde' 	=> { 'x' => 'where',
 		     't' => 'n' },
 'operacional' 	=> { 'x' => 'operational',
 		     't' => 'a' },
 'opinio'	=> { 'x' => 'opinion',
 		     't' => 'n' },
 'opresso'	=> { 'x' => 'oppression',
 		     't' => 'n' },
 'organizao'	=> { 'x' => 'organization',
 		     't' => 'n' },
 'pacfica'	=> { 'x' => 'peaceful',
 		     't' => 'a' },
 'pagar'	=> { 'x' => 'pay',
		     't' => 'v' },
 'pgina'	=> { 'x' => 'page',
 		     't' => 'n' },
 'palavra'	=> { 'x' => 'word',
 		     't' => 'n' },
 'pas'		=> { 'x' => 'country',
 		     't' => 'n' },
 'paraguai'	=> { 'x' => 'paraguay',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'partir'	=> { 'x' => 'split',
		     '#' => 'divide, share, distribute, break open',
		     't' => 'v',
		     '#' => 'split open, cut, set off, st' },
 'passar'	=> { 'x' => 'pass',
 		     't' => 'v' },
 'passaporte'	=> { 'x' => 'passport',
 		     't' => 'n' },
 'paz'		=> { 'x' => 'peace',
 		     't' => 'n' },
 'pergunta'	=> { 'x' => 'question',
 		     't' => 'n' },
 'perseguio'	=> { 'x' => 'persecution',
 		     't' => 'n' },
 'pesquisar'	=> { 'x' => 'research',
 		     't' => 'v' },
 'plataforma'	=> { 'x' => 'platform',
 		     't' => 'n' },
 'poema'	=> { 'x' => 'poem',
 		     't' => 'n' },
 'polons'	=> { 'x' => 'polish',
		     '#' => 'noun and adj.?' },
 'portugal'	=> { 'x' => 'portugal',
		     '#' => 'proper, place',
		     't' => 'n' },
 'portugus'	=> { 'x' => 'portuguese',
		     '#' => 'noun and adj.?' },
 'produto'	=> { 'x' => 'product',
 		     't' => 'n' },
 'programa'	=> { 'x' => 'program',
 		     't' => 'n' },
 'projeto'	=> { 'x' => 'project',
 		     't' => 'n' },
 'proteco'	=> { 'x' => 'protection',
		     '#' => 'spelling?? (proteo)',
 		     't' => 'n' },
 'querer'	=> { 'x' => 'want',
		     't' => 'v' },
 'raa'		=> { 'x' => 'race',
		     '#' => 'as in nationality',
 		     't' => 'n' },
 'receber'	=> { 'x' => 'receive',
		     't' => 'v' },
 'reduzir'	=> { 'x' => 'reduce',
 		     't' => 'v' },
 'religio'	=> { 'x' => 'religion',
 		     't' => 'n' },
 'representao'	=> { 'x' => 'representation',
 			     't' => 'n' },
 'reputao'	=> { 'x' => 'reputation',
 		     't' => 'n' },
 'restringir'	=> { 'x' => 'restrict',
 		     't' => 'v' },
 'reunio'	=> { 'x' => 'meeting',
 		     't' => 'n' },
 'russo'	=> { 'x' => 'russian',
		     '#' => 'noun and adj.?' },
 'salvar'	=> { 'x' => 'save',
		     't' => 'v' },
 'seguro'	=> { 'x' => 'safe',
		     't' => 'a' },
 'senha'	=> { 'x' => 'password',
		     't' => 'n' },
 'servidor'	=> { 'x' => 'server',
		     't' => 'n' },
 'sexo'		=> { 'x' => 'sex',
 		     't' => 'n' },
 'smbolo'	=> { 'x' => 'symbol',
 		     't' => 'n' },
 'simplificar'	=> { 'x' => 'simplify',
 		     't' => 'v' },
 'sistema'	=> { 'x' => 'system',
		     't' => 'n' },
 'sol'		=> { 'x' => 'sun',
 		     't' => 'n' },
 'sozinho'	=> { 'x' => 'alone',
 		     't' => 'a' },
 'sueco'	=> { 'x' => 'swedish',
		     '#' => 'noun and adj.?' },
 'suear'	=> { 'x' => 'dream',
 		     't' => 'v' },
 'sufrgio'	=> { 'x' => 'suffrage',
 		     't' => 'n' },
 'sugerir'	=> { 'x' => 'suggest',
		     't' => 'v' },
 'tailndia'	=> { 'x' => 'thailand',
		     '#' => 'proper, place',
		     't' => 'n' },
 'tailands'	=> { 'x' => 'thai',
		     '#' => 'noun and adj.?' },
 'tcheco'	=> { 'x' => 'czech',
		     '#' => 'noun and adj.?' },
 'tela' 	=> { 'x' => 'screen',
		     't' => 'v' },
 'telefone'	=> { 'x' => 'telephone',
 		     't' => 'n' },
 'tentao'	=> { 'x' => 'temptation',
 		     't' => 'n' },
 'territrio'	=> { 'x' => 'territory',
 		     't' => 'n' },
 'tipo'		=> { 'x' => 'type',
 		     't' => 'n' },
 'tirania'	=> { 'x' => 'tyranny',
 		     't' => 'n' },
 'ttulo'	=> { 'x' => 'title',
 		     't' => 'n' },
 'trabalhar'	=> { 'x' => 'work',
		     't' => 'v' },
 'tradutor'	=> { 'x' => 'translator',
 		     't' => 'n' },
 'triste'	=> { 'x' => 'sad',
 		     't' => 'a' },
 'turco'	=> { 'x' => 'turkish',
		     '#' => 'noun and adj.?' },
 'ltimo'	=> { 'x' => 'last',
 		     't' => 'a' },
 'nico'	=> { 'x' => 'unique',
 		     't' => 'a' },
 'universal'	=> { 'x' => 'universal',
 		     't' => 'a' },
 'universidade'	=> { 'x' => 'university',
 		     't' => 'n' },
 'urso'		=> { 'x' => 'bear',
 		     't' => 'n' },
 'uruguai'	=> { 'x' => 'uruguay',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'usar' 	=> { 'x' => 'use',
 		     't' => 'v' },
 'usurio'	=> { 'x' => 'user',
 		     't' => 'n' },
 'utilizar' 	=> { 'x' => 'utilize',
 		     't' => 'v' },
 'vender'	=> { 'x' => 'sell',
 		     't' => 'v' },
 'verso'	=> { 'x' => 'version',
 		     't' => 'n' },
 'vida' 	=> { 'x' => 'life',
		     't' => 'v' },
 'vietnam'	=> { 'x' => 'vietnam',
		     '#' => 'proper, place',
		     't' => 'n' },
 'viver'	=> { 'x' => 'live',
		     't' => 'v' },
);
}

1;

