/******************************************************************************
 * Locke's   __                      __         NIM Server Software           *
 * ___ ___  (__)__    __ __   __ ___|  | v5     Version 5 (ALPHA)             *
 * |  /   \  __|  \__/  |  | |  |      |        unreleased+revamped 2004      *
 * |       ||  |        |  \_|  | ()   |                                      *
 * |    |  ||  |  |__|  |       |      |                                      *
 * |____|__|___|__|  |__|\___/__|______|        http://www.nimud.org/nimud    *
 *   n a m e l e s s i n c a r n a t e          dedicated to chris cool       *
 ******************************************************************************/

#if defined(BSD)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "nimud.h"
#include "net.h"
#include "skills.h"


char *  const   where_name  [] =
{
    "<as earring>",
    "<as earring>",
    "<placed on head>",
    "<fastened to helm>",
    "<tied about forehead>",
    "<over face>",
    "<piercing nose>",
    "<around neck>",
    "<around neck>",
    "<as shirt>",
    "<on body>",
    "<strapped to back>",
    "<on legs>",
    "<as pants>",
    "<as stockings>",
    "<on feet>",
    "<around ankle>",
    "<around ankle>",
    "<on arms>",
    "<around wrist>",
    "<around wrist>",
    "<on hands>",
    "<on finger>",
    "<on finger>",
    "<as shield>",
    "<about body>",
    "<floating nearby>",
    "<worn about waist>",
    "<over loins>",
    "<pinned on garment>",
    "<thrust through belt>",
    "<thrust through belt>",
    "<thrust through belt>",
    "<thrust through belt>",
    "<thrust through belt>",
    "<held in hand>",
    "<held in hand>",
    "<slung over shoulder>",
    "<slung over shoulder>",
    "<wield primary>",
    "<wield secondary>",
    "<fastened to waist>",
    "<fastened to waist>",
    "<fastened to waist>",
    "<on shoulders>",
};




#define MAX_ITERATION       (IS_IMMORTAL(ch) ? 16 : 4)      /* for old ver */



void show_scene_to_char( PLAYER_DATA *ch, SCENE_INDEX_DATA *pScene, 
                        int dist, int dir )
{
    PLAYER_DATA *rch;
    char buf[MAX_STRING_LENGTH];
    char *p;
    int count;

    /*
     * Count the number of people.
     */
    count = 0;
    for ( rch = pScene->people; rch != NULL;  rch = rch->next_in_scene )
    {
       if ( rch != ch && can_see( ch, rch ) )
       {
           count++;
           if ( IS_NPC(ch) && IS_SET(ch->act, ACT_NOSCAN) )  count--;
       }
    }
    
    if ( scene_is_dark( pScene )
      || !check_blind( ch )
      || count <= 0 )
    return;
    
    switch ( dist )
    {
       case 1: sprintf( buf, "Nearby to the %s you can see", dir_name[dir] ); break;
       case 2: sprintf( buf, "%s of here, you see", capitalize(dir_name[dir]) ); break;
       case 3: sprintf( buf, "In the distance to the %s is", dir_name[dir] ); break;
       case 4: sprintf( buf, "Far away %s from you is",      dir_name[dir] ); break;
      default: break;
    }       
    
    for( rch = pScene->people;  rch != NULL;  rch = rch->next_in_scene )
    {        
        if ( !can_see( ch, rch )
          || (IS_NPC(ch) && IS_SET(ch->act, ACT_NOSCAN))
          || ch == rch )
        continue;
        
        if ( --count < 0 ) break;

        strcat( buf, " " );
        strcat( buf, NAME(rch) );

        if ( count > 1 )
        strcat( buf, "," );
        
        if ( count == 1 )
        strcat( buf, " and" );        
        
        if ( count == 0 )
        strcat( buf, ".\n\r" );
    }
        
    p = format_string( str_dup(buf) );    
    page_to_actor( buf, ch );
    free_string( p );
    return;
}        



/*
 * Brand new version.
 */
void scan_direction( PLAYER_DATA *ch, int dir )
{ 
    SCENE_INDEX_DATA *pScene;
    EXIT_DATA *pExit;
    char buf[MAX_STRING_LENGTH];
    int dist;
    
    if ( (pExit = ch->in_scene->exit[dir]) == NULL
      || IS_SET(pExit->exit_info, EX_CLOSED) )
    return;    

    for( dist = 1; dist < 5; dist++ )
    {
        if ( (pScene = pExit->to_scene) == NULL )
        break;
        
        show_scene_to_char( ch, pScene, dist, dir );

        if ( (pExit = pScene->exit[dir]) == NULL )
        break;

        if ( IS_SET(pExit->exit_info, EX_CLOSED)
          || IS_SET(pExit->exit_info, EX_CONCEALED) )
        {
            if ( !IS_SET(pExit->exit_info, EX_CONCEALED) )
            {
                sprintf( buf, "A closed %s%s blocks your view.\n\r",
                         pExit->keyword,
                         dist == 1 ? " nearby"          :
                         dist == 2 ? " close by"        :
                         dist == 3 ? " in the distance" : "" );
                page_to_actor( buf, ch );
            }

            break;
        }
        else
        if ( IS_SET(pExit->exit_info, EX_ISDOOR)
          && !IS_SET(pExit->exit_info, EX_CONCEALED) )
        {
            sprintf( buf, "You can see an opened %s%s.\n\r",
                     pExit->keyword,
                     dist == 1 ? " nearby"          :
                     dist == 2 ? " close by"        :
                     dist == 3 ? " in the distance" : "" );
            page_to_actor( buf, ch );
        }
        else
        if ( !IS_SET(pExit->exit_info, EX_ISDOOR)
          && !IS_SET(pExit->exit_info, EX_CONCEALED)
          && !MTD(pExit->keyword) )
        {
            sprintf( buf, "You are able to peer through a%s %s.\n\r",
                     IS_VOWEL(pExit->keyword[0]) ? "n" : "",
                     pExit->keyword );
            page_to_actor( buf, ch );
        }
    }
    
    return;
}




char *format_prop_to_char( PROP_DATA *prop, PLAYER_DATA *ch, bool fShort )
{
    static char buf[MAX_STRING_LENGTH];
    char pre[MAX_STRING_LENGTH];

    buf[0] = '\0';

    if ( fShort )
    {
        pre[0] = '\0';
        if ( IS_PROP_STAT(prop, ITEM_INVIS)     ) strcat( pre, "invisible "  );
        if ( IS_AFFECTED(ch, AFF_DETECT_MAGIC)
          && IS_PROP_STAT(prop, ITEM_MAGIC)     ) strcat( pre, "shimmering " );
        if ( IS_PROP_STAT(prop, ITEM_GLOW)      ) strcat( pre, "glowing "    );
        if ( IS_PROP_STAT(prop, ITEM_BURNT)   ) strcat( pre, "charred "    );
	if ( IS_PROP_STAT(prop, ITEM_BURNING) ) strcat( pre, "burning " );
        if ( IS_PROP_STAT(prop, ITEM_HUM)       ) strcat( pre, "humming "    );

        if ( TOO_BIG(ch,prop)   && (prop->carried_by == ch
          || IS_SET(prop->extra_flags, ITEM_INVENTORY)) ) strcat(pre, "loose " );
        else
        if ( TOO_SMALL(ch,prop) && (prop->carried_by == ch
          || IS_SET(prop->extra_flags, ITEM_INVENTORY)) ) strcat(pre, "tight " );

        switch ( prop->item_type )
        {
            default: break;
    case ITEM_LIGHT:
         if( prop->value[1] )
         {
            int percent = PERCENTAGE( prop->value[0], prop->value[1] );
            if ( prop->value[1] < prop->value[0] ) percent = 100;

            if ( !prop->value[0] && VAL_SET(prop,3,LIGHT_FILLABLE) )
            strcat( pre, "empty " );
            else
            strcat( pre,
             !IS_LIT(prop)  ? "unlit "    :
             percent <  10 ? "very dim " :
             percent <  20 ? "dim "      :
                             "lit " );
         }
        break;
    case ITEM_DRINK_CON: if ( !prop->value[0] ) strcat( pre, "empty " ); break;
    case ITEM_ARMOR:
         if( prop->value[0] )
         {
            int percent;

            percent = PERCENTAGE( prop->value[0], prop->pIndexData->value[0] );
            strcat( pre,
             percent <  10 ? "destroyed "     :
             percent <  20 ? "poor "          :
             percent <  30 ? "badly damaged " :
             percent <  40 ? "damaged "       :
             percent <  50 ? "worn "          :
             percent <  80 ? "somewhat used " :
                             "new " );
         }
        break;
    case ITEM_GEM:
         if( prop->value[1] )
         {
            int percent;
            percent = PERCENTAGE( prop->value[1], prop->value[2] );

            strcat( pre,
             percent <  10 ? "darkened " :
             percent <  20 ? "pulsing "  :
                             "" );
         }
        break;
        }

        if ( pre[0] != '\0' )
        {
            sprintf( buf, "%s %s%s", IS_VOWEL(pre[0]) ? "an" : "a", pre,
                     smash_article( STR(prop, short_descr) ) );
        }
        else
        strcat( buf, STR(prop,short_descr) );

        if ( prop->item_type == ITEM_DRINK_CON
          && IS_SET(prop->extra_flags, ITEM_INVENTORY)
          && prop->value[0] != 0 )
        {
            if ( prop->value[2] < 0 || prop->value[2] >= LIQ_MAX )
            {
                prop->value[2] = 0;
                bug( "Invalid liquid on prop %d.", prop->pIndexData->vnum );
            }

            strcat( buf, " of " );
            strcat( buf, liq_table[prop->value[2]].liq_name );
        }
    }
    else
    {
        PLAYER_DATA *puller;

        if ( IS_PROP_STAT(prop, ITEM_INVIS)     )   strcat( buf, "(Invisible) ");
        if ( IS_AFFECTED(ch, AFF_DETECT_EVIL)
          && IS_PROP_STAT(prop, ITEM_EVIL)   )   strcat( buf, "(Red Aura) "    );
        if ( IS_AFFECTED(ch, AFF_DETECT_MAGIC)
          && IS_PROP_STAT(prop, ITEM_MAGIC)  )   strcat( buf, "(Yellow Glow) " );
        if ( IS_PROP_STAT(prop, ITEM_GLOW)      )   strcat( buf, "(Glowing) "  );
        if ( IS_PROP_STAT(prop, ITEM_HUM)       )   strcat( buf, "(Humming) "  );

        if ( (puller = hitched( prop )) == NULL )
        {
            if ( ch->furniture == prop )
            {
                strcat( buf, capitalize(STR(prop,short_descr)) );
                strcat( buf, " upon which you are " );
                strcat( buf, position_name( ch->position ) );
                strcat( buf, " is here.\n\r" );
            }
            else
            strcat( buf, STR(prop, description) );
        }
        else
        {
            strcat( buf, "   " );
            strcat( buf, capitalize(STR(prop, short_descr)) );
            strcat( buf, " is hitched to " );
            strcat( buf, PERS(puller, ch) );
            strcat( buf, ".\n\r" );
        }       
        
        if ( prop->item_type == ITEM_FURNITURE 
          && prop->contains != NULL )
        {
            char xbuf[MAX_STRING_LENGTH];

            if ( prop->contains->next_content != NULL
              && can_see_prop(ch, prop->contains)
              && can_see_prop(ch, prop->contains->next_content) )
             sprintf( xbuf, "      A few items are on %s.\n\r",
                     STR(prop,short_descr) );
            else
            if ( can_see_prop(ch, prop->contains) )
             sprintf( xbuf, "      %s is on %s.\n\r",
                     capitalize(STR(prop->contains,short_descr)),
                     STR(prop,short_descr) );

            strcat( buf, xbuf );
        }
    }

    return buf;
}




/*
 * Show a list to a character.
 * Can coalesce duplicated items.
 */
void show_list_to_char( PROP_DATA *list, PLAYER_DATA *ch, bool fShort, bool fShowNothing )
{
    char buf[MAX_STRING_LENGTH];
    char **prgpstrShow;
    int *prgnShow;
    char *pstrShow;
    PROP_DATA **lastprop;
    PROP_DATA *prop;
    int nShow;
    int iShow;
    int count;
    bool fCombine;

    if ( ch->desc == NULL )
        return;
    
    /*
     * Alloc space for output lines.
     */
    count = 0;
    for ( prop = list; prop != NULL; prop = prop->next_content ) count++;
    prgpstrShow	= alloc_mem( count * sizeof(char *) );
    prgnShow    = alloc_mem( count * sizeof(int)    );
    lastprop     = alloc_mem( count * sizeof(PROP_DATA *) );
    nShow       = 0;

    /*
     * Format the list of props.
     */
    for ( prop = list; prop != NULL; prop = prop->next_content )
    {
        if ( prop->wear_loc == WEAR_NONE
          && can_see_prop( ch, prop )
          && hitched(prop) == NULL )
        {
	    if ( prop->item_type == ITEM_FURNITURE
	    &&   prop->value[0]/100 > 0
            &&  (OCCUPADO(prop) || ch->furniture == prop) )
		continue;
            pstrShow = str_dup( format_prop_to_char( prop, ch, fShort ) );
            fCombine = FALSE;

            if ( IS_NPC(ch) || IS_SET(ch->act2, PLR_COMBINE) )
            {
                /*
                 * Look for duplicates, case sensitive.
                 * Matches tend to be near end so run loop backwards.
                 */
                for ( iShow = nShow - 1; iShow >= 0 && iShow < count; iShow-- )
                {
                    if ( !strcmp( prgpstrShow[iShow], pstrShow )
                      && prop->item_type != ITEM_MONEY )
                    {
                    ++prgnShow[iShow];
                    lastprop[iShow] = prop;
                    fCombine = TRUE;
                    break;
                    }
                }
            }

            /*
             * Couldn't combine, or didn't want to.
             */
            if ( !fCombine )
            {
                 prgpstrShow [nShow] = str_dup( pstrShow );
                 prgnShow    [nShow] = 1;
                 lastprop     [nShow] = prop;
                 nShow++;
            }

            free_string( pstrShow );
        }
    }

    /*
     * Output the formatted list.
     */
    for ( iShow = 0; iShow < nShow; iShow++ )
    {
        if ( fShort )
        send_to_actor( " ", ch );

        if ( IS_NPC(ch) || IS_SET(ch->act2, PLR_COMBINE) )
        {
            if ( prgnShow[iShow] > 1 )
            {
                if (fShort) sprintf( buf, "%s ", numberize(prgnShow[iShow]) );
                else
                {
                    char *pt;

                    pt = STR(lastprop[iShow], description_plural);
                    if ( MTD(pt) )
                    {
                         sprintf( buf, "%s %s are here.\n\r",
                                       numberize(prgnShow[iShow]),
                                       pluralize(STR(lastprop[iShow],short_descr))  );
                         buf[0] = UPPER(buf[0]);
                    }
                    else buf[0] = '\0';
                }
                send_to_actor( buf, ch );
            }
        }

        if (fShort)
        {
            send_to_actor( prgnShow[iShow] > 1 ? pluralize( STR(lastprop[iShow],short_descr) ) :
                                                prgpstrShow[iShow],
                          ch );
            send_to_actor( "\n\r", ch );
        }
        else
        {
            if ( buf[0] == '\0' && prgnShow[iShow] > 1 )
            {
                sprintf( buf, STR(lastprop[iShow], description_plural),
                              numberize(prgnShow[iShow]) );
                send_to_actor( capitalize(buf), ch );
            }
            else
            if ( prgnShow[iShow] <= 1 )
            send_to_actor( capitalize(prgpstrShow[iShow]), ch );
        }

        free_string( prgpstrShow[iShow] );
        lastprop[iShow] = NULL;
    }

    if ( fShowNothing && nShow == 0 )
    {
        if ( IS_NPC(ch) || IS_SET(ch->act2, PLR_COMBINE) )
            send_to_actor( "     ", ch );
        send_to_actor( "Nothing.\n\r", ch );
    }

    /*
     * Clean up.
     */
    free_mem( prgpstrShow, count * sizeof(char *)      );
    free_mem( prgnShow,    count * sizeof(int)         );
    free_mem( lastprop,     count * sizeof(PROP_DATA *)  );

    return;
}



/*
 * Show a list to a character.
 * Can coalesce duplicated items.
 */
char *show_list_to_char2( PROP_DATA *list, PLAYER_DATA *ch, char *prefix )
{
    static char final[MAX_STRING_LENGTH*4];
    char **prgpstrShow;
    int *prgnShow;
    char *pstrShow;
    PROP_DATA **lastprop;
    PROP_DATA *prop;
    int nShow;
    int iShow;
    int count;
    bool fCombine;

    if ( ch->desc == NULL )
        return "";
    
    /*
     * Alloc space for output lines.
     */
    count = 0;
    for ( prop = list; prop != NULL; prop = prop->next_content ) count++;
    prgpstrShow = alloc_mem( count * sizeof(char *) );
    prgnShow    = alloc_mem( count * sizeof(int)    );
    lastprop     = alloc_mem( count * sizeof(PROP_DATA *) );
    nShow       = 0;

    /*
     * Format the list of props.
     */
    for ( prop = list; prop != NULL; prop = prop->next_content )
    {
        if ( prop->wear_loc == WEAR_NONE
          && can_see_prop( ch, prop )
          && hitched(prop) == NULL )
        {
	    if ( prop->item_type == ITEM_FURNITURE
	    &&   prop->value[0]/100 > 0
            &&  (OCCUPADO(prop) || ch->furniture == prop) )
		continue;
            pstrShow = str_dup( format_prop_to_char( prop, ch, TRUE ) );
            fCombine = FALSE;

            if ( IS_NPC(ch) || IS_SET(ch->act2, PLR_COMBINE) )
            {
                /*
                 * Look for duplicates, case sensitive.
                 * Matches tend to be near end so run loop backwards.
                 */
                for ( iShow = nShow - 1; iShow >= 0 && iShow < count; iShow-- )
                {
                    if ( !strcmp( prgpstrShow[iShow], pstrShow )
                      && prop->item_type != ITEM_MONEY )
                    {
                    ++prgnShow[iShow];
                    lastprop[iShow] = prop;
                    fCombine = TRUE;
                    break;
                    }
                }
            }

            /*
             * Couldn't combine, or didn't want to.
             */
            if ( !fCombine )
            {
                 prgpstrShow [nShow] = str_dup( pstrShow );
                 prgnShow    [nShow] = 1;
                 lastprop     [nShow] = prop;
                 nShow++;
            }

            free_string( pstrShow );
        }
    }

    /*
     * Output the formatted list.
     */

    sprintf( final, nShow == 1 ? prefix
                              : replace_string( prefix, " is ", " are " ) );
    
    for ( iShow = 0; iShow < nShow; iShow++ )
    {
        if ( iShow == nShow-1 && nShow != 1 )
        strcat( final, "and " );

        if ( prgnShow[iShow] > 1 )
        {
            strcat( final, numberize(prgnShow[iShow]) );
            strcat( final, " " );
            strcat( final, pluralize( STR(lastprop[iShow],short_descr) ) );
        }
        else
        strcat( final, prgpstrShow[iShow] );

        if ( iShow != nShow-1 )
        strcat( final, ", " );


        free_string( prgpstrShow[iShow] );
        lastprop[iShow] = NULL;
    }

    if ( nShow == 0 ) strcat( final, "nothing" );
    
    strcat( final, ". " );

    {
        char *p;
        p = format_string( str_dup( final ) );
        sprintf( final, "%s", p );
        free_string( p );
    }

    /*
     * Clean up.
     */
    free_mem( prgpstrShow, count * sizeof(char *)      );
    free_mem( prgnShow,    count * sizeof(int)         );
    free_mem( lastprop,     count * sizeof(PROP_DATA *)  );
    
    return final;
}



void show_peek_to_char( PROP_DATA *list, PLAYER_DATA *ch, bool fShort,
                        bool fShowNothing, int percent )
{
    char buf[MAX_STRING_LENGTH];
    char **prgpstrShow;
    int *prgnShow;
    char *pstrShow;
    PROP_DATA **lastprop;
    PROP_DATA *prop;
    int nShow;
    int iShow;
    int count;
    bool fCombine;

    if ( ch->desc == NULL )
	return;
    
    /*
     * Alloc space for output lines.
     */
    count = 0;
    for ( prop = list; prop != NULL; prop = prop->next_content )
	count++;
    prgpstrShow	= alloc_mem( count * sizeof(char *) );
    prgnShow    = alloc_mem( count * sizeof(int)    );
    lastprop     = alloc_mem( count * sizeof(PROP_DATA *) );
    nShow	= 0;

    /*
     * Format the list of props.
     */
    for ( prop = list; prop != NULL; prop = prop->next_content )
    {
        bool fHitched = (hitched(prop) != NULL ? TRUE : FALSE);

    if ( prop->wear_loc == WEAR_NONE
      && can_see_prop( ch, prop )
      && percent > number_percent( )
      && occupant(prop)
      && !fHitched )
	{
        pstrShow = percent > number_percent( ) ?
                   format_prop_to_char( prop, ch, fShort ) : "something";
	    fCombine = FALSE;

        if ( IS_NPC(ch) || IS_SET(ch->act2, PLR_COMBINE) )
        {
		/*
		 * Look for duplicates, case sensitive.
         * Matches tend to be near end so run loop backwards.
		 */
		for ( iShow = nShow - 1; iShow >= 0; iShow-- )
		{
            if ( ( !strcmp( prgpstrShow[iShow], pstrShow )
                 || lastprop[iShow] == prop )
               && prop->item_type != ITEM_MONEY )
		    {
            ++prgnShow[iShow];
            lastprop[iShow] = prop;
			fCombine = TRUE;
			break;
		    }
		}
	    }

	    /*
	     * Couldn't combine, or didn't want to.
	     */
	    if ( !fCombine )
	    {
		prgpstrShow [nShow] = str_dup( pstrShow );
		prgnShow    [nShow] = 1;
        lastprop     [nShow] = prop;
		nShow++;
	    }
	}
    }

    /*
     * Output the formatted list.
     */
    for ( iShow = 0; iShow < nShow; iShow++ )
    {
    if ( IS_NPC(ch) || IS_SET(ch->act2, PLR_COMBINE) )
	{
        if ( prgnShow[iShow] > 1 )
	    {
        if (fShort) sprintf( buf, "%s ", numberize( prgnShow[iShow] ) );
        else
        {
            char *pt;
            pt = STR(lastprop[iShow], description_plural);

            if ( pt == NULL || pt[0] == '\0' )
            {
                sprintf( buf, "%s %s are here.\n\r", 
                numberize( prgnShow[iShow] ), pluralize( STR(lastprop[iShow],short_descr) ) );
                buf[0] = UPPER(buf[0]);
            }
            else
            {
                buf[0] = '\0';
            }
        }
        send_to_actor( buf, ch );
	    }
	}

    if (fShort)
    {
      send_to_actor( prgnShow[iShow] > 1 ? pluralize( STR(lastprop[iShow],short_descr) ) :
                    prgpstrShow[iShow], ch );
      send_to_actor( "\n\r", ch );
    }
    else
    {
        if ( buf[0] == '\0' && prgnShow[iShow] > 1 )
        {
            sprintf( buf, STR(lastprop[iShow], description_plural), numberize( prgnShow[iShow] ) );
            send_to_actor( capitalize(buf), ch );
        }
        else
        if ( prgnShow[iShow] <= 1 )
        send_to_actor( capitalize(prgpstrShow[iShow]), ch );
    }
        free_string( prgpstrShow[iShow] );
        lastprop[iShow] = NULL;
    }

    if ( fShowNothing && nShow == 0 )
    {
    if ( IS_NPC(ch) || IS_SET(ch->act2, PLR_COMBINE) )
        send_to_actor( "     ", ch );
    send_to_actor( "Nothing.\n\r", ch );
    }

    /*
     * Clean up.
     */
    free_mem( prgpstrShow, count * sizeof(char *)      );
    free_mem( prgnShow,    count * sizeof(int)         );
    free_mem( lastprop,     count * sizeof(PROP_DATA *)  );

    return;
}



/*
 *  Shows a actor/player to character (look in scene)
 */
void show_actor_to_actor_0( PLAYER_DATA *victim, PLAYER_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    char *p;

    buf[0] = '\0';
    if ( victim == NULL || ch == NULL ) return;

    if ( victim->rider != NULL )
        return;

    if ( victim->riding == ch )
        return;



    /*
     * FOR HOODS by Morgenes, Ported by Daurven, Mods by Locke
     */
    if ( !is_hooded( victim ) ) {
        if ( victim->position == POS_STANDING
        && victim->riding   == NULL
        && victim->furniture == NULL
        && STR(victim, long_descr)[0] != '\0' ) {
            strcat( buf, "   " );
            strcat( buf, STR(victim, long_descr) );

            p = format_string( str_dup( capitalize(buf) ) );
            STC( p, ch );
            free_string( p );
         return;
  }
    } else {
        if ( victim->position == POS_STANDING
        && victim->riding   == NULL
        && victim->furniture == NULL ) {
            strcat( buf, "   " );
            strcat( buf, "There is a hooded figure nearby." );

            p = format_string( str_dup( capitalize(buf) ) );
            STC( p, ch );
            free_string( p );
         return;
  }
 }

/*    send_to_actor( "ACTOR/PLR ", ch );       */

    if ( IS_AFFECTED(victim, AFF_INVISIBLE)   ) strcat( buf, "(Invisible) "   );
    if ( IS_AFFECTED(victim, AFF_HIDE)        ) strcat( buf, "(Hiding) "      );
    if ( IS_AFFECTED(victim, AFF_CHARM)       ) strcat( buf, "(Charmed) "     );
    if ( IS_AFFECTED(victim, AFF_PASS_DOOR)   ) strcat( buf, "(Translucent) " );
    if ( IS_AFFECTED(victim, AFF_FAERIE_FIRE) ) strcat( buf, "(Pink Aura) "   );
    if ( IS_AFFECTED(victim, AFF_SANCTUARY)   ) strcat( buf, "(White Aura) "  );
    if ( !IS_NPC(victim)
      && IS_AFFECTED(victim, AFF_FLYING)      ) strcat( buf, "(Floating) "    );
    if ( !IS_NPC(victim)
      && victim->desc == NULL                 ) strcat( buf, "(Linkless) "    );

    if ( victim->position == POS_STANDING
      && victim->riding   == NULL
      && victim->furniture == NULL
      && STR(victim, long_descr)[0] != '\0' )
    {
    strcat( buf, STR(victim, long_descr) );

    p = format_string( str_dup( capitalize(buf) ) );
    send_to_actor( p, ch );
    free_string( p );

	return;
    }

    strcat( buf, capitalize( PERS( victim, ch ) ) );

    switch ( victim->position )
    {
    case POS_DEAD:     strcat( buf, " is dead" );               break;
    case POS_MORTAL:   strcat( buf, " is mortally wounded" );   break;
    case POS_INCAP:    strcat( buf, " is incapacitated" );      break;
    case POS_STUNNED:  strcat( buf, " is lying here stunned" ); break;
    case POS_SLEEPING: strcat( buf, " is sleeping" );           break;
    case POS_SITTING:  strcat( buf, " is sitting" );            break;
    case POS_RESTING:  strcat( buf, " is resting" );            break;
    case POS_STANDING:
     if ( victim->rider == ch ) strcat( buf, " that you are riding" );
     strcat( buf, " is here" );
     break;
    case POS_FIGHTING:
      strcat( buf, " is here, fighting " );
      if ( victim->fighting == NULL )
          strcat( buf, "thin air" );
      else if ( victim->fighting == ch )
          strcat( buf, "YOU" );
      else if ( victim->in_scene == victim->fighting->in_scene )
      {
          strcat( buf, PERS( victim->fighting, ch ) );
      }
      else
          strcat( buf, "someone who left" );
     break;
    }

    if ( victim->riding != NULL && victim->riding->in_scene == victim->in_scene )
    {
        sprintf( buf2, ", mounted on %s.\n\r",
                          PERS(victim->riding, ch) );
        strcat( buf, buf2 );
    }
    else
    {
    if ( victim->furniture != NULL )
    {
        sprintf( buf2, " on %s.\n\r", can_see_prop( ch, victim->furniture ) ?
                                      STR(victim->furniture, short_descr)  :
                                      "something" );
        strcat( buf, buf2 );
    }
    else
    strcat( buf, ".\n\r" );
    }

    p = format_string( str_dup(capitalize(buf)) );
    send_to_actor( p, ch );
    free_string( p );

    if ( victim->hitched_to != NULL && can_see_prop( ch, victim->hitched_to ) )
    {
        sprintf( buf, "  %s has been hitched to %s.\n\r", NAME(victim),
                 STR(victim->hitched_to,short_descr) );
        send_to_actor( buf, ch );
    }
    return;
}



/*
 * Shows the generated prop description to a character.
 * (Once again I must say, Thanks to Kalgen for this AWESOME idea)
 * Thanks to Morgenes for the bug fixes/improvements.
 *                                    - Locke
 * Locations not displayed:
 * on belt (sheathed)
 * stockings (if wearing pants/leggings)
 * nose (if wearing something on face)
 * ankles (if wearing stockings)
 * shirt (if body wear)
 * loins (if wearing pants/leggings)
 */
void show_equipment( PLAYER_DATA *ch, PLAYER_DATA *tch )
{
   PROP_DATA *prop;
   PROP_DATA *prop2;
   char buf[MAX_STRING_LENGTH];
   char descr[MAX_STRING_LENGTH];
   char *finalstr;
   bool fBool;
   int oType;
   char *p1;
   char *p2;

   buf[0] = '\0';
   descr[0] = '\0';

   fBool = FALSE;

   p1 = NULL;
   p2 = NULL;

   if( ( ( prop = get_eq_char( ch, WEAR_FINGER_L ) ) != NULL
    && can_see_prop( tch, prop ) )
    && ( ( prop2 = get_eq_char( ch, WEAR_FINGER_R ) ) != NULL
    && can_see_prop( tch, prop2 ) )
    && !strcmp( (p1 = str_dup(format_prop_to_char(prop,  ch, TRUE))),
                (p2 = str_dup(format_prop_to_char(prop2, ch, TRUE))) ) )
   {
      sprintf( buf, "%s is wearing two %s on %s fingers",
                    HE_SHE( ch ),  pluralize( STR(prop,short_descr) ),   HIS_HER( ch ) );
      strcat( descr, buf );
   }
   else
   {
      if( ( prop = get_eq_char( ch, WEAR_FINGER_L ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "On %s left hand, %s has placed %s",
          HIS_HER( ch ), HE_SHE( ch ), PERSO(prop, ch) );
         strcat( descr, buf );
         fBool = TRUE;
      }
     
      if( ( prop = get_eq_char( ch, WEAR_FINGER_R ) ) != NULL
       && can_see_prop( tch, prop ) )
      { 
         if( fBool )
            sprintf( buf, ", while %s encircles a finger on %s right",
             PERSO(prop, ch), HIS_HER( ch ) );
         else
            sprintf( buf, "On %s right hand %s %s",
             HIS_HER( ch ), is_are( PERSO(prop, ch) ), PERSO(prop, ch) );
         strcat( descr, buf );
      }
   }

   free_string( p1 );
   free_string( p2 );
   p1 = NULL;
   p2 = NULL;

   if( ( prop = get_eq_char( ch, WEAR_HANDS ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      if( descr[0] != '\0' )
         sprintf( buf, ", with %s covering %s hands.",
          format_prop_to_char(prop, ch, TRUE), HIS_HER(ch) );
      else
         sprintf( buf, "%s %s worn on %s hands.",
          format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
      strcat( descr, buf );
   }
   else if( descr[0] != '\0' )
      strcat( descr, ".  " );

   fBool = FALSE;

   if( ( prop = get_eq_char( ch, WEAR_HEAD ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s is wearing %s on %s head.  ",
       HE_SHE( ch ), format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_FOREHEAD ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "Around %s forehead, %s has wrapped %s.",
       HIS_HER(ch), HE_SHE(ch), PERSO(prop, ch) );
      strcat( descr, buf );
   }

   if( ( ( prop = get_eq_char( ch, WEAR_NECK_1 ) ) != NULL
    && can_see_prop( tch, prop ) )
    && ( ( prop2 = get_eq_char( ch, WEAR_NECK_2 ) ) != NULL
    && can_see_prop( tch, prop2 ) )
    && !str_cmp( (p1 = str_dup(format_prop_to_char(prop,  ch, TRUE))),
                 (p2 = str_dup(format_prop_to_char(prop2, ch, TRUE))) ) )
   {
      sprintf( buf, "Two %s are clasped around %s neck.  ",
       pluralize( STR(prop, short_descr) ), HIS_HER( ch ) );
      strcat( descr, buf );
   }
   else
   {
      if( ( prop = get_eq_char( ch, WEAR_NECK_1 ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         if( get_eq_char( ch, WEAR_NECK_2 ) == NULL )
            sprintf( buf, "%s %s worn about %s neck.  ",
             PERSO(prop,ch), is_are( PERSO(prop,ch) ),
             HIS_HER( ch ) );
         else
            sprintf( buf, "%s and ", PERSO(prop,ch) );
         strcat( descr, buf );
         fBool = TRUE;
      }
   
      if( ( prop = get_eq_char( ch, WEAR_NECK_2 ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         if( fBool )
            sprintf( buf, "%s are worn about %s neck.  ",
             PERSO(prop,ch), HIS_HER( ch ) );
         else
            sprintf( buf, "%s %s worn about %s neck.  ",
             PERSO(prop,ch), is_are( PERSO(prop,ch) ), HIS_HER( ch ) );
         strcat( descr, buf );
      }
   }

   free_string( p1 );
   free_string( p2 );
   p1 = NULL;
   p2 = NULL;

   fBool = FALSE;

   if( ( ( prop = get_eq_char( ch, WEAR_EAR_L ) ) != NULL
    && can_see_prop( tch, prop ) )
    && ( ( prop2 = get_eq_char( ch, WEAR_EAR_R ) ) != NULL
    && can_see_prop( tch, prop2 ) )
    && !str_cmp( (p1 = str_dup(format_prop_to_char(prop,  ch, TRUE))),
                 (p2 = str_dup(format_prop_to_char(prop2, ch, TRUE))) ) )
   {
      sprintf( buf, "%s has a pair of %s on %s ears.",
       HE_SHE( ch ), pluralize( STR(prop, short_descr) ), HIS_HER( ch ) );
      strcat( descr, buf );
   } 
   else
   {
      if( ( prop = get_eq_char( ch, WEAR_EAR_L ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "%s pierces %s left ear",
          PERSO(prop,ch), HIS_HER( ch ) );
         strcat( descr, buf );
         fBool = TRUE;
      }
      if( ( prop = get_eq_char( ch, WEAR_EAR_R ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         if( fBool )
            sprintf( buf, " and %s %s worn on %s right",
             PERSO(prop,ch), is_are( PERSO(prop,ch) ), HIS_HER( ch ) );
         else
            sprintf( buf, "%s right ear is pierced with %s",
             HIS_HER( ch ), PERSO(prop,ch) );
         strcat( descr, buf );
      }
      if( fBool || get_eq_char( ch, WEAR_EAR_R ) )
         strcat( descr, ".  " );
   }

   free_string( p1 );
   free_string( p2 );
   p1 = NULL;
   p2 = NULL;

   if( ( prop = get_eq_char( ch, WEAR_NOSE ) ) != NULL
    && can_see_prop( tch, prop )
    && get_eq_char( ch, WEAR_FACE ) == NULL )
   {
      sprintf( buf, "%s %s piercing %s nose.",
       format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_FACE ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s covers %s face.", format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   fBool = FALSE;
   
   if( ( prop = get_eq_char( ch, WEAR_ABOUT ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s %s draped over %s ",
       format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
      strcat( descr, buf );
      if( ( prop = get_eq_char( ch, WEAR_BODY ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "%s.  ",
          smash_article( format_prop_to_char(prop, ch, TRUE) ) );
      }
      else
      if( ( prop = get_eq_char( ch, WEAR_SHIRT ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "%s.  ",
          smash_article( format_prop_to_char(prop, ch, TRUE) ) );
      }
      else
         sprintf( buf, "naked chest.  " );
      strcat( descr, buf );
   }
   else if( ( prop = get_eq_char( ch, WEAR_BODY ) ) != NULL 
    && (get_eq_char( tch, WEAR_SHIRT ) == NULL 
     || get_eq_char( tch, WEAR_PANTS ) == NULL)
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s %s being worn on %s body.",
       format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
      strcat( descr, buf );
   }
   
   if( ( prop = get_eq_char( ch, WEAR_SHIRT ) ) != NULL
    && !get_eq_char( ch, WEAR_ABOUT ) 
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s fits over %s torso.",
       format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }
  
   if( ( prop = get_eq_char( ch, WEAR_ARMS ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s %s protecting %s arms.  ",
       format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_PANTS ) ) != NULL
    && can_see_prop( tch, prop )
    && get_eq_char( ch, WEAR_LEGS ) == NULL )
   {
      sprintf( buf, "%s is wearing %s on %s legs.  ",
       HE_SHE( ch ), format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop2 = get_eq_char( ch, WEAR_LEGS ) ) != NULL
    && can_see_prop( tch, prop2 ) )
   {
       prop = get_eq_char( ch, WEAR_PANTS );
      if ( prop == NULL )
      sprintf( buf, "%s is wearing %s on %s legs.  ",
       HE_SHE( ch ), format_prop_to_char(prop2, ch, TRUE), HIS_HER( ch ) );
      else
      sprintf( buf, "%s %s worn over %s on %s legs.  ",
       format_prop_to_char(prop2, ch, TRUE), is_are( format_prop_to_char(prop2, ch, TRUE) ),
       format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );

      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_STOCKING ) ) != NULL
    && can_see_prop( tch, prop )
    && get_eq_char( ch, WEAR_PANTS ) == NULL )
   {
      sprintf( buf, "%s has pulled %s onto %s legs.  ",
       HE_SHE( ch ), format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_FEET ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s wears %s as footwear.  ",
       HE_SHE( ch ), format_prop_to_char(prop, ch, TRUE) );
      strcat( descr, buf );
   }

   if( descr[0] != '\0' )
   {
      send_to_actor( "\n\r", tch );
      send_to_actor( "   ", tch );

      finalstr = format_string( str_dup( descr ) );
      send_to_actor( finalstr, tch );
      free_string( finalstr );

      descr[0] = '\0';
   }

   if( ( ( prop = get_eq_char( ch, WEAR_WRIST_L ) ) != NULL
    && can_see_prop( tch, prop ) )
    && ( ( prop2 = get_eq_char( ch, WEAR_WRIST_R ) ) != NULL
    && can_see_prop( tch, prop2 ) )
    && !str_cmp( (p1 = str_dup(format_prop_to_char(prop,  ch, TRUE))),
                 (p2 = str_dup(format_prop_to_char(prop2, ch, TRUE))) ) )
   {
      sprintf( buf, "Two %s are on each wrist.  ", pluralize( STR(prop, short_descr) ) );
      strcat( descr, buf );
   }
   else
   {
      if( ( prop = get_eq_char( ch, WEAR_WRIST_L ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "Encircling %s left wrist, %s %s",
          HIS_HER( ch ), is_are( format_prop_to_char(prop, ch, TRUE) ), format_prop_to_char(prop, ch, TRUE) );
         strcat( descr, buf );
         fBool = TRUE;
      }
    
      if( ( prop = get_eq_char( ch, WEAR_WRIST_R ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         if( fBool )
            sprintf( buf, " and %s on %s right.  ",
             format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
         else
            sprintf( buf,
             "Encircling %s right wrist %s %s.  ",
             HIS_HER( ch ), is_are( format_prop_to_char(prop, ch, TRUE) ), format_prop_to_char(prop, ch, TRUE) );
         strcat( descr, buf );
      }
      else if( fBool )
         strcat( descr, ".  " );
   }

   free_string( p1 );
   free_string( p2 );
   p1 = NULL;
   p2 = NULL;

   if ( get_eq_char( ch, WEAR_STOCKING ) == NULL )
   {

   fBool = FALSE;

   if( ( ( prop = get_eq_char( ch, WEAR_ANKLE_L ) ) != NULL
    && can_see_prop( tch, prop ) )
    && ( ( prop2 = get_eq_char( ch, WEAR_ANKLE_R ) ) != NULL
    && can_see_prop( tch, prop2 ) )
    && !str_cmp( (p1 = str_dup(format_prop_to_char(prop,  ch, TRUE))),
                 (p2 = str_dup(format_prop_to_char(prop2, ch, TRUE))) ) )
   {
      sprintf( buf, "%s has two %s fastened around %s ankles.",
       HE_SHE( ch ), pluralize( STR(prop, short_descr) ), HIS_HER( ch ) );
      strcat( descr, buf );
   } 
   else
   {
      if( ( prop = get_eq_char( ch, WEAR_ANKLE_L ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "%s %s fastened around %s left ankle",
          format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
         strcat( descr, buf );
         fBool = TRUE;
      }
      if( ( prop = get_eq_char( ch, WEAR_ANKLE_R ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         if( fBool )
            sprintf( buf, " and %s %s around %s right",
             format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
         else
            sprintf( buf, "Around %s right ankle %s %s",
            HIS_HER( ch ), is_are( format_prop_to_char(prop, ch, TRUE) ), format_prop_to_char(prop, ch, TRUE) );
         strcat( descr, buf );
      }
      if( fBool || get_eq_char( ch, WEAR_ANKLE_R ) )
         strcat( descr, ".  " );
   }

   }

   fBool = FALSE;

   if( ( ( prop = get_eq_char( ch, WEAR_SHOULDER_L ) ) != NULL
    && can_see_prop( tch, prop ) )
    && ( ( prop2 = get_eq_char( ch, WEAR_SHOULDER_R ) ) != NULL
    && can_see_prop( tch, prop2 ) )
    && !str_cmp( (p1 = str_dup(format_prop_to_char(prop,  ch, TRUE))),
                 (p2 = str_dup(format_prop_to_char(prop2, ch, TRUE))) ) )
   {
      sprintf( buf, "%s has two %s slung over %s shoulders.",
       HE_SHE( ch ), pluralize( STR(prop,short_descr) ), HIS_HER( ch ) );
      strcat( descr, buf );
   } 
   else
   {
      if( ( prop = get_eq_char( ch, WEAR_SHOULDER_L ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "%s %s slung over %s left shoulder",
          format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
         strcat( descr, buf );
         fBool = TRUE;
      }
      if( ( prop = get_eq_char( ch, WEAR_SHOULDER_R ) ) != NULL
       && can_see_prop( tch, prop ) )
      {
         if( fBool )
            sprintf( buf, " and %s %s slung over %s right",
             format_prop_to_char(prop, ch, TRUE), is_are( format_prop_to_char(prop, ch, TRUE) ), HIS_HER( ch ) );
         else
            sprintf( buf, "Over %s right shoulder %s %s",
            HIS_HER( ch ), is_are( format_prop_to_char(prop, ch, TRUE) ), format_prop_to_char(prop, ch, TRUE) );
         strcat( descr, buf );
      }
      if( fBool || get_eq_char( ch, WEAR_SHOULDER_R ) )
         strcat( descr, ".  " );
   }

   free_string( p1 );
   free_string( p2 );
   p1 = NULL;
   p2 = NULL;

   fBool = FALSE;

   prop  = get_eq_char( ch, WEAR_HOLD_1 );
   if ( !prop )  prop  = get_eq_char( ch, WEAR_WIELD_1 );

   prop2 = get_eq_char( ch, WEAR_HOLD_2 );
   if ( !prop2 ) prop2 = get_eq_char( ch, WEAR_WIELD_2 );

   if( prop != NULL     && can_see_prop( tch, prop )
    && prop2 != NULL    && can_see_prop( tch, prop2 )
    && (prop->wear_loc == prop2->wear_loc-1)
    && !str_cmp( (p1 = str_dup(format_prop_to_char(prop,  ch, TRUE))),
                 (p2 = str_dup(format_prop_to_char(prop2, ch, TRUE))) ) )
   {
      sprintf( buf, "Two %s are %s in %s hands.",
                       pluralize( STR(prop, short_descr) ),
                       prop->wear_loc == WEAR_WIELD_1 ? "wielded" : "held",
                       HIS_HER( ch ) );
      strcat( descr, buf );
   }
   else
   if ( prop != NULL 
        && can_see_prop( tch, prop ) 
        && prop->wear_loc == WEAR_WIELD_1
        && IS_SET(prop->wear_flags, ITEM_TWO_HANDED) )
   {
        sprintf( buf, "%s %s gripped in both %s hands.\n\r",
                      format_prop_to_char( prop, ch, TRUE ),
                      is_are( format_prop_to_char( prop, ch, TRUE ) ),
                      HIS_HER( ch ) );
        strcat( descr, buf );
   }
   else  
   {
      oType = ITEM_TRASH;
      if( prop != NULL && can_see_prop( tch, prop ) )
      {
         sprintf( buf, "%s %s %s in one hand",
             format_prop_to_char(prop, ch, TRUE),
             is_are( format_prop_to_char(prop, ch, TRUE) ),
             prop->wear_loc == WEAR_WIELD_1 ? "being wielded" : "held" );
         strcat( descr, buf );
         fBool = TRUE;
      }

      if( prop2 != NULL && can_see_prop( tch, prop2 ) )
      {
         if( fBool )
         {
            if ( prop->wear_loc == (prop2->wear_loc-1) )
            sprintf( buf, ", with %s in the other.",
                     format_prop_to_char(prop2, ch, TRUE) );
            else
            sprintf( buf, ", and %s %s %s in the other.  ",
                format_prop_to_char(prop2, ch, TRUE),
                is_are( format_prop_to_char(prop2, ch, TRUE) ),
                prop2->wear_loc == WEAR_WIELD_2 ? "wielded" : "held" );
         }
         else
         {
            sprintf( buf, "%s %s being %s.  ",
                format_prop_to_char(prop2, ch, TRUE),
                is_are( format_prop_to_char(prop2, ch, TRUE) ),
                prop2->wear_loc == WEAR_WIELD_2 ? "wielded" : "held" );
         } 
         strcat( descr, buf );
      }
      else
      if( fBool ) strcat( descr, ".  " );
   }

   free_string( p1 );
   free_string( p2 );
   p1 = NULL;
   p2 = NULL;

   if( ( prop = get_eq_char( ch, WEAR_BACK ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s is strapped to %s back.  ",
        format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }
   
   fBool = FALSE;
   
   if( ( prop = get_eq_char( ch, WEAR_FLOATING ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s is floating near %s head.  ",
       format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_WAIST ) ) != NULL
    && can_see_prop( tch, prop )
    && (get_eq_char( ch, WEAR_LOIN ) == NULL 
     && get_eq_char( ch, WEAR_PANTS ) == NULL) )
   {
      sprintf( buf, "%s fits snugly around %s waist.  ",
       format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_SHOULDERS ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s is wearing %s.  ",
       format_prop_to_char(prop, ch, TRUE), HIS_HER( ch ) );
      strcat( descr, buf );
   }

   if( ( prop2 = get_eq_char( ch, WEAR_LOIN ) ) != NULL
    && can_see_prop( tch, prop2 )
    && get_eq_char( ch, WEAR_PANTS ) == NULL )
   {
      sprintf( buf, "%s %s wrapped loosely below %s %s.  ",
       format_prop_to_char(prop2, ch, TRUE), is_are( format_prop_to_char(prop2, ch, TRUE) ),
       HIS_HER( ch ), prop ? smash_article(format_prop_to_char(prop, ch, TRUE)) : "waist" );
      strcat( descr, buf );
   }

   if( ( prop = get_eq_char( ch, WEAR_SHIELD ) ) != NULL
    && can_see_prop( tch, prop ) )
   {
      sprintf( buf, "%s serves to protect %s from attacks.  ",
        format_prop_to_char(prop, ch, TRUE), HIM_HER( ch ) );
      strcat( descr, buf );
   }
   
   if( descr[0] != '\0' )
   {
      send_to_actor( "\n\r", tch );
      send_to_actor( "   ", tch );

      finalstr = format_string( str_dup( descr ) );
      send_to_actor( finalstr, tch );
      free_string( finalstr );
   }

   return;
}



void show_equipment_table( PLAYER_DATA *ch, PLAYER_DATA *victim )
{
    PROP_DATA *prop;
    char buf[MAX_STRING_LENGTH];
    int iWear;
    bool found = FALSE;

    if ( ch != victim )
       act( "$N is using:", ch, NULL, victim, TO_CHAR );
  else send_to_actor( "You are using:\n\r", ch );

    for ( iWear = 0; iWear < MAX_WEAR; iWear++ )
    {

    if ( ( prop = get_eq_char( victim, iWear ) ) == NULL )
        continue;

    sprintf( buf, "%-24s %s\n\r", iWear == WEAR_WIELD_1
                               && IS_SET(prop->wear_flags, ITEM_TWO_HANDED)
                                 ? "<wielded both hands>" : where_name[iWear],
                                  can_see_prop( ch, prop ) ?
                                      format_prop_to_char(prop, ch, TRUE)
                                      : "something" );
    send_to_actor( buf, ch );
	found = TRUE;
    }

    if ( !found )  send_to_actor( "     Nothing.\n\r", ch );
    return;
}



/*
 * Shows the actually character (look at someone)
 */
void show_actor_to_actor_1( PLAYER_DATA *victim, PLAYER_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    char final[MAX_STRING_LENGTH];

    if ( can_see( victim, ch ) )
    {
         act( "$n looks at you.", ch, NULL, victim, TO_VICT    );
         if ( ch != victim )
         act( "$n looks at $N.",  ch, NULL, victim, TO_NOTVICT );
         else act( "$n looks $mself over.", ch, NULL, NULL, TO_NOTVICT );
    }

    if ( !MTD(STR(victim,description)) )
    send_to_actor( STR(victim, description), ch );

    show_equipment( victim, ch );

    send_to_actor( "\n\r",   ch );

    final[0] = '\0';

    sprintf( buf, "     %s looks %s.\n\r",
                        capitalize(PERS( victim, ch )),
                                  STRING_HITS(victim) );
    strcat( final, buf );

    switch ( victim->position )
    {
        case POS_DEAD:
          sprintf( buf, "%s is dead. ", capitalize(PERS(victim,ch)) );
          strcat( final, buf );
         break;
        case POS_MORTAL:
          sprintf( buf, "%s has wounds that may bring death. ",
                   capitalize(PERS(victim,ch)) );
          strcat( final, buf );
         break;
        case POS_INCAP:
          sprintf( buf, "%s has been rendered incapacitated. ",
                   capitalize(PERS(victim,ch)) );
          strcat( final, buf );
         break;
        case POS_STUNNED:
          sprintf( buf, "%s is stunned. ",
                   capitalize(PERS(victim,ch)) );
          strcat( final, buf );
         break;
        case POS_SLEEPING:
          if ( victim->furniture )
          sprintf( buf, "%s is fast asleep upon %s. ",
                        capitalize(HE_SHE(victim)), PERSO(victim->furniture,ch) );
          else
          sprintf( buf, "%s is fast asleep. ",
                        capitalize(HE_SHE(victim)) );
          strcat( final, buf );
         break;
        case POS_SITTING:
          if ( victim->furniture )
          sprintf( buf, "%s is sitting on %s. ",
                        capitalize(HE_SHE(victim)), PERSO(victim->furniture,ch) );
          else
          sprintf( buf, "%s is sitting. ",
                        capitalize(HE_SHE(victim)) );
          strcat( final, buf );
         break;
        case POS_RESTING:
          if ( victim->furniture )
          sprintf( buf, "%s rests on %s. ",
                        capitalize(HE_SHE(victim)), PERSO(victim->furniture,ch) );
          else
          sprintf( buf, "%s is resting. ",
                        capitalize(HE_SHE(victim)) );
          strcat( final, buf );
         break;
        case POS_FIGHTING:
          if ( victim->fighting != NULL )
          {
          sprintf( buf, "     %s is fighting with %s.\n\r",
                        PERS( victim, ch ),  PERS( victim->fighting, ch ) );
          strcat( final, buf );
          }
         break;
          default:  break;
    }

if ( victim->size )
{
    sprintf( buf, "%s stands the height of %s handspans",
             capitalize(HE_SHE(victim)),
             numberize( victim->size/2 ) );

    strcat( final, buf );

    if ( ch->size != victim->size )
    {
        sprintf( buf, ", which is roughly %s halfspans %s than you.\n\r",
             numberize( abs(ch->size - victim->size) ),
             ch->size - victim->size > 0 ? "shorter" : "taller" );
        strcat( final, buf );
    }
    else
    strcat( final, ".\n\r" );
}
    if ( !IS_NPC(victim) )
    {
        char *race_name;
        int age;
        int race;

        race = race_lookup( victim->race );

        if ( race == 0 )
        {
            age = 0;
            race_name = "human";
        }
        else
        {
            age = PERCENTAGE(GET_AGE(victim),RACE(race,base_age) );
            race_name = RACE(race,race_name);
        }

        sprintf( buf, "%s is a%s %s.\n\r",
                 capitalize(NAME(victim)),
                 age < 30 ? " young" :
                 age < 40 ? " matured" :
                 age < 60 ? " middle-aged" :
                 age < 80 ? "n aging" :
                 age < 90 ? "n elderly" : "n ancient",
                 race_name );
        strcat( final, buf );
    }
    
    if ( victim->riding != NULL )
    {
        sprintf( buf, "     %s rides atop %s.\n\r", HE_SHE(victim), NAME(victim->riding) );
	strcat( final, buf );
    }
    else
    if ( victim->rider != NULL )
    {
        sprintf( buf, "     %s is being ridden by %s.\n\r", HE_SHE(victim), capitalize(NAME(victim->rider)) );
        strcat( final, buf );
    }

    if ( GET_PC(victim,condition[COND_DRUNK],0) > 50 )
    {
        sprintf( buf, " %s looks a bit tipsy and smells of drink.\n\r",
                 HE_SHE(victim) );
        strcat( final, buf );
    }

	{
		char *p;
                p = format_string( str_dup(final) );
		send_to_actor( "   ", ch );
		send_to_actor( p, ch );
		free_string( p );
	}

    send_to_actor( "\n\r", ch );

    if ( (IS_IMMORTAL(ch) && IS_SET(ch->act2, WIZ_EQUIPMENT)) )
    {
       show_equipment_table( ch, victim );
       show_inventory( victim, ch, FALSE );
    }

    return;
}


 /*
  *  Show whose in the scene (all actors/players)
  */
void show_actor_to_char( PLAYER_DATA *list, PLAYER_DATA *ch )
{
    PLAYER_DATA *rch;
    bool fDark = 0;

    for ( rch = list; rch != NULL; rch = rch->next_in_scene )
    {
        if ( rch == ch )
            continue;

        if ( !IS_NPC(rch)
        &&   GET_PC(rch,wizinvis,0) >= GET_PC(ch,level,0)
        &&   get_trust( ch ) < get_trust( rch ) )
             continue;

        if ( can_see( ch, rch ) )
        {
             show_actor_to_actor_0( rch, ch );
        }
        else
        if ( scene_is_dark( ch->in_scene ) && IS_AFFECTED(rch, AFF_INFRARED) )
        fDark++;
    }

    if ( fDark > 0 )
    {
        char buf[MAX_STRING_LENGTH];

        sprintf( buf, "%s red eyes stare at you from the darkness.\n\r",
                      numberize( fDark * 2 ) );
        send_to_actor( buf, ch );
    }
    return;
}

 
bool check_blind( PLAYER_DATA *ch )
{
    if ( !IS_NPC(ch) && IS_SET(ch->act2, WIZ_HOLYLIGHT) )
    return TRUE;

    if ( IS_AFFECTED(ch, AFF_BLIND) )
    {
    send_to_actor( "It is impossible to see a thing!\n\r", ch );
    return FALSE;
    }

    return TRUE;
}

bool actor_look_list( PLAYER_DATA *ch, PROP_DATA *list, char *arg )
{
    PROP_DATA *prop;
    char *pdesc;
    char buf[MAX_STRING_LENGTH];
    char argnew[MAX_STRING_LENGTH];
    int count;
    int number;
    
    number = number_argument( arg, argnew );
    count  = 0;

    for ( prop = list; prop != NULL; prop = prop->next_content )
    {
        if ( can_see_prop( ch, prop ) )
        {
            pdesc = get_extra_descr( arg, prop->extra_descr );
            if ( pdesc == NULL )
            pdesc = get_extra_descr( arg, prop->pIndexData->extra_descr );

            if ( pdesc != NULL )
            {
                    page_to_actor( pdesc, ch );
                    return TRUE;
            }
        }

        if ( is_name( argnew, STR(prop, name) ) && ++count == number )
        {
            char fub[MAX_STRING_LENGTH];
            
            page_to_actor( STR(prop, real_description), ch );
            switch ( prop->item_type )
            {
              default:
                break;

              case ITEM_LIGHT:
                 if( prop->value[1] )
                 {
                     int percent = PERCENTAGE( prop->value[0], prop->value[1] );

                     sprintf( buf, "\n\r%s %s %s\n\r",
                                  capitalize( STR(prop, short_descr) ),
                                  is_are( STR(prop, short_descr) ),
                       !IS_LIT(prop)  ? "extinguished."              :
                       percent <  10 ? "flickering and sputtering." :
                       percent <  20 ? "flickering."                :
                       percent <  30 ? "flickering slightly."       :
                       percent <  40 ? "providing ample light."     :
                                       "glowing brightly." );
                     send_to_actor( buf, ch );
                 }
              break;

              case ITEM_BOARD:
                  cmd_note( ch, "list" );
                break;

              case ITEM_DRINK_CON:
                 if ( prop->value[1] <= 0 )
                     {
                         send_to_actor( "It is empty.\n\r", ch );
                         break;
                     }
              break;

              case ITEM_RANGED_WEAPON:
               if ( prop->value[3] > 0 )
                  {
                      sprintf( buf, "It has %d round%s ready to fire.\n\r",
                               prop->value[3],
                               prop->value[3] > 1 ? "s" : "" );
                      send_to_actor( buf, ch );
                  }
                  else send_to_actor( "It is not readied.\n\r", ch );
              break;

              case ITEM_ARMOR:
                    if( prop->value[1] )
                    {
                       int percent;

                       percent = PERCENTAGE( prop->value[1], prop->value[2] );
                       sprintf( buf, "\n\r%s %s\n\r",
                                     capitalize( STR(prop, short_descr) ),
                        percent <  10 ? "is in pieces." :
                        percent <  20 ? "is nearly destroyed." :
                        percent <  30 ? "is heavily damaged." :
                        percent <  40 ? "is in poor shape." :
                        percent <  50 ? "looks worn." :
                        percent <  60 ? "is starting to look rather worn." :
                        percent <  70 ? "looks well used." :
                        percent <  80 ? "is slightly damaged." :
                        percent <  90 ? "is in fair shape." :
                        percent < 100 ? "is in good shape." :
                                        "looks excellent." );
                       send_to_actor( buf, ch );
                    }
              case ITEM_CLOTHING:
              		if ( prop->carried_by != ch
              		  || prop->wear_loc <= WEAR_BELT_1 )
              		{
                    if ( TOO_BIG(ch, prop) )
                    send_to_actor( "It looks too big for you to wear.\n\r", ch );
               else if ( TOO_SMALL(ch, prop) )
                    send_to_actor( "It looks too small for you to wear.\n\r", ch );
               else send_to_actor( "You could probably wear it.\n\r", ch );
                    }
              break;

            case ITEM_GEM:
             if( prop->value[1] )
             {
                int percent = PERCENTAGE( prop->value[1], prop->value[2] );

                sprintf( buf, "\n\r%s %s\n\r",
                         capitalize( STR(prop, short_descr) ),
                         percent <  10 ? "is dimly flickering."   :
                         percent <  20 ? "is slowly pulsating."   :
                         percent <  30 ? "is quickly pulstating." :
                         percent <  40 ? "is glowing."            :
                         percent <  50 ? "glows brightly."
                                       : "radiates with energy."    );
                send_to_actor( buf, ch );
             }
            break;
            }

            if ( prop->item_type == ITEM_FURNITURE && prop->contains != NULL )
            {
                sprintf( fub, "On %s is ", format_prop_to_char( prop, ch, TRUE ) );
                page_to_actor( show_list_to_char2( prop->contains, ch, fub ), ch );
            }

            if ( has_occupant( prop ) )
            show_occupants_to_char( prop, ch );
            return TRUE;
        }
    }

    return FALSE;
}



/*
 * The scan command.
 */
void cmd_scan( PLAYER_DATA *ch, char *argument )
{
       SCENE_INDEX_DATA *pScene;
       bool fMatch = FALSE; 

       ansi_color( B_BLUE, ch );
       ansi_color( BOLD, ch );

       pScene = ch->in_scene;
       if ( ch->in_scene == NULL ) return;

       if ( pScene->exit[DIR_NORTH] != NULL
        && pScene->exit[DIR_NORTH]->to_scene != NULL
        && pScene->exit[DIR_NORTH]->to_scene->people != NULL  ) {
       send_to_actor( "You glance north:\n\r", ch );
       cmd_look( ch, "north" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_SOUTH] != NULL
        && pScene->exit[DIR_SOUTH]->to_scene != NULL
        && pScene->exit[DIR_SOUTH]->to_scene->people != NULL  ) {
       send_to_actor( "You glance south:\n\r", ch );
       cmd_look( ch, "south" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_EAST] != NULL
        && pScene->exit[DIR_EAST]->to_scene != NULL
        && pScene->exit[DIR_EAST]->to_scene->people != NULL  ) {
       send_to_actor( "You glance east:\n\r", ch );
       cmd_look( ch, "east" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_WEST] != NULL
        && pScene->exit[DIR_WEST]->to_scene != NULL
        && pScene->exit[DIR_WEST]->to_scene->people != NULL  ) {
       send_to_actor( "You glance west:\n\r", ch );
       cmd_look( ch, "west" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_UP] != NULL
        && pScene->exit[DIR_UP]->to_scene != NULL
        && pScene->exit[DIR_UP]->to_scene->people != NULL  ) {
       send_to_actor( "You glance up:\n\r", ch );
       cmd_look( ch, "up" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_DOWN] != NULL
        && pScene->exit[DIR_DOWN]->to_scene != NULL
        && pScene->exit[DIR_DOWN]->to_scene->people != NULL  ) {
       send_to_actor( "You glance down:\n\r", ch );
       cmd_look( ch, "down" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_NE] != NULL
        && pScene->exit[DIR_NE]->to_scene != NULL
        && pScene->exit[DIR_NE]->to_scene->people != NULL  ) {
       send_to_actor( "You glance northeast:\n\r", ch );
       cmd_look( ch, "northeast" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_NW] != NULL
        && pScene->exit[DIR_NW]->to_scene != NULL
        && pScene->exit[DIR_NW]->to_scene->people != NULL  ) {
       send_to_actor( "You glance northwest:\n\r", ch );
       cmd_look( ch, "northwest" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_SE] != NULL
        && pScene->exit[DIR_SE]->to_scene != NULL
        && pScene->exit[DIR_SE]->to_scene->people != NULL  ) {
       send_to_actor( "You glance southeast:\n\r", ch );
       cmd_look( ch, "southeast" );
       fMatch = TRUE;
       }

       if ( pScene->exit[DIR_SW] != NULL
        && pScene->exit[DIR_SW]->to_scene != NULL
        && pScene->exit[DIR_SW]->to_scene->people != NULL  ) {
       send_to_actor( "You glance southwest:\n\r", ch );
       cmd_look( ch, "southwest" );
       fMatch = TRUE;
       }

       if ( !fMatch ) send_to_actor( "There is no one nearby.\n\r", ch );
       ansi_color( NTEXT, ch );
};

/*
 * Syntax:  look
 *          look auto
 *          look [direction]
 *          look [prop]
 *          look [person]
 *          look [description]
 */
void cmd_look( PLAYER_DATA *ch, char *argument )
{
    char buf  [MAX_STRING_LENGTH];
    char arg1 [MAX_INPUT_LENGTH];
    char arg2 [MAX_INPUT_LENGTH];
    EXIT_DATA *pexit;
    SCENE_INDEX_DATA *in_scene;
    PLAYER_DATA *victim;
    PROP_DATA *prop;
    int door;

    in_scene = ch->in_scene;

    if ( is_number( argument ) ) {cmd_note( ch, argument ); return;}

/*    if ( ch->desc == NULL )
    return;   */

    if ( ch->position < POS_SLEEPING )
    {
	send_to_actor( "It is impossible to see anything but stars!\n\r", ch );
	return;
    }

    if ( ch->position == POS_SLEEPING )
    {
	send_to_actor( "It is impossible to see anything, you're sleeping!\n\r", ch );
	return;
    }

    if ( !check_blind( ch ) )
	return;

    if ( !IS_NPC(ch)
      && !IS_SET(ch->act, WIZ_HOLYLIGHT)
      && scene_is_dark( in_scene ) )
    {
        bool fLight = FALSE;
        int direction = 0;
        int count = 0;
        EXIT_DATA *pExit;
        char buf[MAX_INPUT_LENGTH];

        buf[0] = '\0';
        for( direction = 0; direction < MAX_DIR; direction++ )
        {
             pExit = in_scene->exit[direction];
             if ( pExit != NULL
               && !IS_SET(pExit->exit_info, EX_CLOSED)
               && pExit->to_scene != NULL
               && !scene_is_dark( pExit->to_scene ) ) 
             count++;
        }

        for( direction = 0;  direction < MAX_DIR; direction++ )
        {
             pExit = in_scene->exit[direction];
             if ( pExit != NULL 
               && !IS_SET(pExit->exit_info, EX_CLOSED)
               && pExit->to_scene != NULL
               && !scene_is_dark( pExit->to_scene ) )
             {
                  strcat( buf, dir_name[direction] );
                  fLight = TRUE;
                  switch( --count ) 
                  {
                        case 1: strcat( buf, " and " ); break;
                        case 0: break;
                       default: strcat( buf, ", " ); break;
                  }
             }
        }

        if (!fLight)
        {
             send_to_actor( "   You are surrounded by darkness.\n\r", ch );
        }
        else
        {
             act( "You can see light to the $t.", ch, buf, NULL, TO_CHAR);
        }

        show_actor_to_char( in_scene->people, ch );
        return;
    }

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0' || !str_cmp( arg1, "auto" ) )
    {
	/* 'look' or 'look auto' */
    if ( !IS_NPC(ch) &&  HAS_ANSI(ch) )
    {
    ansi_color( NTEXT, ch );
    ansi_color( color_table[PC(ch,colors[COLOR_SCENE_TITLE])].code, ch );
    }

    if ( ch->desc && ch->desc->client ) {
      char temp[MAX_STRING_LENGTH];

      sprintf( temp, "TITLE %s", in_scene->name );
      sendcli( ch->desc, temp );

      cmd_exits( ch, "clientauto" );
    }
    else {
    send_to_actor( in_scene->name, ch );

    if ( !IS_NPC(ch) && IS_SET(ch->act2, PLR_AUTOEXIT) )
        cmd_exits( ch, "auto" );
    }

    if ( !IS_NPC(ch) && HAS_ANSI(ch) )
    {
    ansi_color( NTEXT, ch );
    }

    if ( IS_SET(in_scene->scene_flags, SCENE_SAVE) )
    send_to_actor( "*", ch );
    
    send_to_actor( "\n\r", ch );

    if ( arg1[0] == '\0'
    || ( !IS_NPC(ch) && !IS_SET(ch->act2, PLR_BRIEF) ) 
    || MTP((in_scene->description)) )
    {
        SCENE_INDEX_DATA *template;
        
        template = get_scene_index( in_scene->template );
        if ( template != NULL && !MTP( template->description ) )
        {
            send_to_actor( "   ", ch );  /*indent*/
            display_interp( ch, template->description );
        }
    }

    /*
     * Worldgen
     */
    if ( in_scene->terrain != 0 )
            show_terrain( ch, in_scene );

        if ( !MTP((in_scene->description)) && !IS_SET(ch->act2,PLR_BRIEF) )
        {
        send_to_actor( "   ", ch ); /* indent */
        display_interp( ch, in_scene->description );
        }


    ansi_color( NTEXT, ch );
    send_to_actor( "\n\r", ch );
    if ( !IS_NPC(ch) ) {
    ansi_color( color_table[PC(ch,colors[COLOR_PROP_LIST])].code, ch );
	show_list_to_char( in_scene->contents, ch, FALSE, FALSE );
	show_actor_to_char( in_scene->people,   ch );

    ansi_color( NTEXT, ch );

    if ( ch->desc) sendcli( ch->desc, in_scene->client );
    }
	return;
    }

    if ( !str_cmp( arg1, "i" ) || !str_cmp( arg1, "in" ) )
    {
        /* 'look in' */
        if ( arg2[0] == '\0' )
        {
            send_to_actor( "Look in what?\n\r", ch );
            return;
        }

        if ( ( prop = get_prop_here( ch, arg2 ) ) == NULL )
        {
            send_to_actor( "You do not see that here.\n\r", ch );
            return;
        }

        switch ( prop->item_type )
        {
        default: send_to_actor( "That is not a container.\n\r", ch );       
            break;

        case ITEM_LIGHT:
             if ( prop->value[1] != 0 )
             {
             int percent = PERCENTAGE(prop->value[0], prop->value[1]);

             if ( prop->value[0] <= 0 )
             {
                 send_to_actor( "It is empty.\n\r", ch );
                 break;
             }
             else
             {
             sprintf( buf, "It %s fuel.\n\r",
             percent < 10 ? "contains a low amount of"      :
             percent < 40 ? "contains less than half its capacity of" :
             percent < 70 ? "contains an ample amount of"   :
             percent < 90 ? "is mostly filled with"         : "is full of" );
             send_to_actor( buf, ch );
             }
             }
             else send_to_actor( "It contains nothing.\n\r", ch );
        break;

        case ITEM_DRINK_CON:
             if ( prop->value[1] <= 0 )
                 {
                     send_to_actor( "It is empty.\n\r", ch );
                     break;
                 }
             else
             {
             int percent = PERCENTAGE(prop->value[0], prop->value[1]);

             sprintf( buf, "It %s of a%s %s liquid.\n\r",
                      percent < 10 ? "has but a drop"         :
                      percent < 40 ? "is less than half full" :
                      percent < 60 ? "is half full"           :
                      percent < 70 ? "is near full"           : "full",
                      IS_VOWEL(liq_table[prop->value[2]].liq_color[0]) ? "n" : "",
                      liq_table[prop->value[2]].liq_color );
             send_to_actor( buf, ch );
             }
        break;

        case ITEM_CONTAINER:
        case ITEM_CORPSE_NPC:
        case ITEM_CORPSE_PC:
            if ( IS_SET(prop->value[1], CONT_CLOSED) )
            {
                send_to_actor( "It is closed.\n\r", ch );
                break;
            }

            if ( prop->item_type == ITEM_CONTAINER )
            {
                 sprintf( buf, "%s contains ",
                     format_prop_to_char( prop, ch, TRUE ) );
                 buf[0] = UPPER(buf[0]);
            }
            else
            {
            	act( "$n searches the remains of $p.", ch, prop, NULL, TO_SCENE );
            	sprintf( buf, "Searching the remains of %s reveals ",
                     format_prop_to_char( prop, ch, TRUE ) );
            }

            page_to_actor( show_list_to_char2( prop->contains, ch, buf ), ch );
        break;
	}
        return;
    }

    if ( !str_cmp( arg1, "o" ) || !str_cmp( arg1, "on" ) )
    {
        /* 'look on' */
        if ( arg2[0] == '\0' )
        {
            send_to_actor( "Look on what?\n\r", ch );
            return;
        }

        if ( (prop = get_prop_here( ch, arg2 )) == NULL )
        {
            send_to_actor( "You do not see that here.\n\r", ch );
            return;
        }
        
        if ( prop->item_type != ITEM_FURNITURE || prop->contains  == NULL )
        act( "There is nothing on $p.", ch, prop, NULL, TO_CHAR );
        else
        {
        if ( prop->contains->next_content != NULL )
        sprintf( buf, "On %s are ", format_prop_to_char( prop, ch, TRUE ) );
        else
        sprintf( buf, "On %s is ", format_prop_to_char( prop, ch, TRUE ) );

        send_to_actor( show_list_to_char2( prop->contains, ch, buf ), ch );
        }
        
        if ( has_occupant( prop ) )
        show_occupants_to_char( prop, ch );
        return;
    }

    if ( !str_cmp( arg1, "a" ) || !str_cmp( arg1, "at" ) )
    {
	strcpy( arg1, arg2 );
    arg2[0] = '\0';
    }

    if ( (victim = get_actor_scene( ch, arg1 )) != NULL )
    {
	show_actor_to_actor_1( victim, ch );
	return;
    }

    if ( actor_look_list( ch, ch->carrying, arg1 ) )
    return;

    if ( actor_look_list( ch, in_scene->contents, arg1 ) )
    return;

    {
        char *pdesc;
        
        pdesc = get_extra_descr( arg1, in_scene->extra_descr );        
        if ( pdesc != NULL )
        {
            page_to_actor( pdesc, ch );
            return;
        }
    }

    if ( ( door = get_dir( arg1 ) ) == MAX_DIR )
    {
	send_to_actor( "You do not see that here.\n\r", ch );
	return;
    }

    /* 'look direction' */
    if ( ( pexit = in_scene->exit[door] ) == NULL )
    {
        sprintf( buf, "There is nothing of note %sward from here.\n\r",
                      dir_name[door] );
        send_to_actor( buf, ch );
        return;
    }

    if ( pexit->to_scene != NULL && scene_is_dark( pexit->to_scene ) )
    {
        sprintf( buf, "To the %s is darkness.\n\r", dir_name[door] );
        send_to_actor( buf, ch );
        return;
    }

    if ( !MTD(pexit->description) )  send_to_actor( pexit->description, ch );
    else
    if ( MTD(pexit->keyword) || !IS_SET( pexit->exit_info, EX_ISDOOR ) )
    {
        sprintf( buf, "There is nothing of note %sward from here.\n\r",
                      dir_name[door] );
        send_to_actor( buf, ch );
    }

    if ( !MTD(pexit->keyword)
      && !IS_SET(pexit->exit_info, EX_SECRET)
      && !IS_SET(pexit->exit_info, EX_CONCEALED) )
    {
        if ( IS_SET(pexit->exit_info, EX_CLOSED) )
        act( "The $T is closed.", ch, NULL, pexit->keyword, TO_CHAR );
        else if ( IS_SET(pexit->exit_info, EX_ISDOOR) )
        act( "The $T is open.",   ch, NULL, pexit->keyword, TO_CHAR );
    }

    if ( (( IS_SET(pexit->exit_info, EX_WINDOW)
      && !IS_SET(pexit->exit_info, EX_CLOSED) )
      || IS_SET(pexit->exit_info, EX_TRANSPARENT) )
      && pexit->to_scene != NULL )
    {
        act( "Through the $t you see $T:", ch, pexit->keyword, pexit->to_scene->name, TO_CHAR );

        if ( !MTD(pexit->to_scene->description) )
        {
        send_to_actor( "   ", ch ); /* indent */
        send_to_actor( pexit->to_scene->description, ch );
        }

        show_list_to_char( pexit->to_scene->contents, ch, FALSE, FALSE );
/*        show_actor_to_char( pexit->to_scene->people,   ch ); */
    }

    if ( !IS_SET(pexit->exit_info, EX_CLOSED)
      && !IS_SET(pexit->exit_info, EX_CONCEALED) )
    {
         if ( !IS_SET(pexit->exit_info, EX_WINDOW) )
         act( "$n glances $t.", ch, dir_name[door], NULL, TO_SCENE );
         else
         act( "$n peers through the $t.", ch, pexit->keyword, NULL, TO_SCENE ); 
    }
   
    scan_direction( ch, door );

    return;
}




/*
 * Thanks to Zrin for auto-exit part.
 * Syntax:  exits
 *          exits auto
 */
void cmd_exits( PLAYER_DATA *ch, char *argument )
{
    extern char * const dir_name[];
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    EXIT_DATA *pexit;
    PROP_DATA *prop;
    bool found;
    bool fAuto;
    int door;

    buf[0] = '\0';
    fAuto  = !str_cmp( argument, "auto" );

    if ( !str_cmp( argument, "clientauto" ) ) {

          send_to_actor( "SHOWEXITS", ch );
          if ( ch->desc ) {

          for( door = 0;  door < MAX_DIR; door++ )
          {
	      	if ( ( pexit = ch->in_scene->exit[door] ) != NULL
		&&   pexit->to_scene != NULL
	        &&   !IS_SET(pexit->exit_info, EX_CONCEALED) )
                {
                        send_to_actor( " ", ch );
                        send_to_actor( dir_letter[door], ch );
                }
          }
          }

          return;
    }

    if ( scene_is_dark( ch->in_scene ) && !IS_SET(ch->act2, WIZ_HOLYLIGHT) )
      {
          if ( !fAuto ) 
          send_to_actor( "It's too dark to see anything!\n\r", ch );
          return;
	}


    if ( !check_blind( ch ) )
	return;

    strcpy( buf, fAuto ? " [" : "" );

    found = FALSE;
    for ( door = 0; door < MAX_DIR; door++ )
    {
	if ( ( pexit = ch->in_scene->exit[door] ) != NULL
	&&   pexit->to_scene != NULL
        &&   !IS_SET(pexit->exit_info, EX_CONCEALED) )
	{
            if ( fAuto )
            {
                if ( (IS_SET(pexit->exit_info, EX_CLOSED)
                   && IS_SET(pexit->exit_info, EX_SECRET))
                  || IS_SET(pexit->exit_info, EX_CONCEALED) )
                continue;
                
                strcat( buf, dir_letter[door] );
                found = TRUE;
            }
	    else
	    {
            if ( !IS_SET(pexit->exit_info, EX_CLOSED)
              && !MTD(pexit->keyword) )
            {
                sprintf( buf + strlen(buf), "A %s leads %s to %s.\n\r",
                         pexit->keyword, dir_name[door],
                         scene_is_dark( pexit->to_scene )  ?  "darkness"
                                            : pexit->to_scene->name );
                found = TRUE;
            }
            else
            if ( !IS_SET(pexit->exit_info, EX_SECRET) )
            {
                sprintf( buf + strlen(buf), "%s%s%s is %s%s.\n\r",
                  door != DIR_UP && door != DIR_DOWN ? "To the " : "",
                  door != DIR_UP && door != DIR_DOWN ? dir_name[door]
                                               : capitalize(dir_name[door]),
                  door == DIR_UP || door == DIR_DOWN ? "ward from here" : "",
                  IS_SET(pexit->exit_info, EX_CLOSED) ? "a closed " : "",
                  IS_SET(pexit->exit_info, EX_CLOSED) ? pexit->keyword :
                  scene_is_dark( pexit->to_scene )  ?  "darkness"
                                            : pexit->to_scene->name );
                found = TRUE;
            }
	    }
	}
    }
    
    for ( prop = ch->in_scene->contents;  prop != NULL;  prop =prop->next_content )
    {
    	if ( prop->item_type == ITEM_FURNITURE
    	  && IS_SET(prop->value[1], FURN_EXIT)
         && !IS_SET(prop->value[1], FURN_NOSHOW)
          && can_see_prop( ch, prop ) )
    	{
		if ( !fAuto )
		sprintf( buf2, "%s", STR(prop, action_descr) );
		else
		sprintf( buf2, " (%s)", STR(prop, short_descr) );
 
    		send_to_actor( buf2, ch );
    		found = TRUE;
    	}
    }

    if ( !found )
        strcat( buf, fAuto ? "" : "There are no obvious ways out.\n\r" );

    if ( fAuto )
        strcat( buf, "]" );

    if ( buf[1] != ']' ) send_to_actor( buf, ch );
    return;
}


#define STAT(func, str1, str2, str3, str4, str5 ) \
                if (func(ch) <  8) sprintf(buf, str1 );       \
           else if (func(ch) < 12) sprintf(buf, str2 );       \
           else if (func(ch) < 16) sprintf(buf, str3 );       \
           else if (func(ch) < 22) sprintf(buf, str4 );       \
           else                    sprintf(buf, str5 );


/*
 * Syntax:  score
 */
void cmd_score( PLAYER_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    int i, race;

    if ( IS_NPC(ch) )
    {
        cmd_stat( ch, "self" );
        return;
    }

    ansi_color( NTEXT, ch );
    
    buf2[0] = '\0';

    race = race_lookup( ch->race );
    if ( race != 0 )
    {
         i = PERCENTAGE(GET_AGE(ch),RACE(race,base_age));
         sprintf( buf, "You are a%s %s named %s%s%s,\n\r",
                   i < 30 ? " young" :
                   i < 60 ? " middle-aged" :
                   i < 80 ? "n aging" :
                   i < 90 ? "n elderly" : "n ancient",
                   RACE(race,race_name), ch->name, 
                   PC(ch,title) != NULL && PC(ch,title) != &str_empty[0] ? " " : "",
                   PC(ch,title) != NULL && PC(ch,title) != &str_empty[0] ? PC(ch,title) : ""   );
    }
    else
    {
         i = PERCENTAGE(GET_AGE(ch),100);
         sprintf( buf, "You are a%s %s named %s%s%s,\n\r",
                   i < 30 ? " young" :
                   i < 60 ? " middle-aged" :
                   i < 80 ? "n aging" :
                   i < 90 ? "n elderly" : "n ancient", 
         ch->sex == SEX_MALE ? "man" : "woman", ch->name, 
                   PC(ch,title) != NULL && PC(ch,title) != &str_empty[0] ? " " : "",
                   PC(ch,title) != NULL && PC(ch,title) != &str_empty[0] ? PC(ch,title) : ""   );
    }

    strcat( buf2, buf );

    {
        int p = PC(ch,played);
        int ct = (int) (current_time - PC(ch,logon));

        i = (p + ct)/7200;
    }

    if ( i > 0 )
    {
    if ( i > 24 )
    {
        int days;
        int hours;
        char smuf[MAX_STRING_LENGTH];

        days  = i / 24;
        hours = i % 24;

        sprintf( smuf, "%s", numberize( days ) );
        sprintf( buf, " with a total of %s day%s and %s hour%s of play,\n\r",
                      smuf, days == 1 ? "" : "s",
                      numberize( hours ), hours == 1 ? "" : "s" );
    }
    else
    {
        sprintf( buf, " with a total of %s hour%s of play,\n\r",
                      numberize( i ), i == 1 ? "" : "s" );
    }
    strcat( buf2, buf );
    }

    sprintf( buf, " %s halfspans tall,\n\r", numberize( ch->size ) );
    strcat( buf2, buf );
    
    sprintf( buf, " you are %s years old,\n\r ", numberize( GET_AGE(ch) ) );
    strcat( buf2, buf );

    if ( PC(ch,birth_month) == weather_info.month )
    {

        int daydist = PC(ch,birth_day) - weather_info.day;

        if ( daydist == -1 )
        strcat( buf2, " yesterday was your birthday, " );
        else
        if ( daydist == 0 )
        strcat( buf2, " today is your birthday, " );
        else
        if ( daydist == 1 )
        strcat( buf2, " tomorrow is your birthday, " );
        else
        if ( daydist > 1 && daydist < 30 )
        {
            sprintf( buf, " your birthday is in %s days,", numberize(daydist) );
            strcat( buf2, buf );
        }
    }
    strcat( buf2, "\n\r" );
    
    sprintf( buf, " and you are currently %s and %s.\n\r\n\r",
                  STRING_HITS(ch), STRING_MOVES(ch) );
    strcat( buf2, buf );

    {
	char *p;

    p = format_indent( str_dup(buf2), " ", 76, TRUE );
    ansi_color( BLUE, ch );
    ansi_color( BOLD, ch );
    send_to_actor( "\n\r", ch );
    send_to_actor( p, ch );
    ansi_color( NTEXT, ch );
    free_string( p );
    }

    ansi_color( B_CYAN, ch );
    ansi_color( BOLD, ch );
    sprintf( buf, "\n\rYou are level %d with %d experience points.\n\r", ch->exp_level, ch->exp );
    send_to_actor( buf, ch );
    ansi_color( NTEXT, ch ); 

  
    if ( ch->carry_number != 0 )
    {
        sprintf( buf2, "%s", numberize( ch->carry_number ) );
        sprintf( buf, "\n\rYou are carrying %s item%s weighing roughly %s stone%s.\n\r",
                      buf2,
                      ch->carry_number == 1 ? "" : "s",
                      numberize( ch->carry_weight/2+1 ),
                      ch->carry_weight == 1 ? "" : "s" );
        send_to_actor( buf, ch );
    }

    
    {
    char *p;
    char fbuf[MAX_STRING_LENGTH];
    
    fbuf[0] = '\0';
    
    sprintf( fbuf, "You possess a " );

    STAT(get_curr_int, "moronic",
                       "simpleton's",
                       ch->sex == SEX_MALE   ? "man's" :
                       ch->sex == SEX_FEMALE ? "woman's" : "eunich's",
                       "scholarly",
                       "genius" );
    strcat( fbuf, buf );

    STAT(get_curr_str, " frail ",
                       " ",
                       " hardened ",
                       " muscular ",
                       " mighty " );
    strcat( fbuf, buf );

    STAT(get_curr_wis, " physique, with a head like a hole,  ",
                       " body, ",
                       " build, with good common sense,  ",
                       " frame, with a wise air about you,  ",
                       " physique, with a perceptive mind,  " );
    strcat( fbuf, buf );

    STAT(get_curr_dex, " and you are lithargic and" ,
                       " and you are ",
                       " and you are nimble and",
                       " and you are quick and",
                       " and you are like lightning and" );
    strcat( fbuf, buf );
     
    STAT(get_curr_con, " quite faint-hearted.  ",
                       " of average health.  ",
                       " healthy.  ",
                       " brazen.  ",
                       " very healthy.  " );
    strcat( fbuf, buf );
    
    p = format_indent( str_dup(fbuf), " ", 76, TRUE );
    ansi_color( RED, ch );
    send_to_actor( "\n\r", ch );
    cmd_fight( ch, "" );
    ansi_color( BOLD, ch );
    send_to_actor( p, ch );
    free_string( p );
    }

    if ( ch->userdata )
    {
    ansi_color( YELLOW, ch );
    sprintf( buf, "You are %s, %s, %s",
         (PC(ch,condition[COND_DRUNK] ) > 50) ? "drunk" : 
         (PC(ch,condition[COND_DRUNK] ) > 20) ? "tipsy" : 
                                                "sober",
         
         (PC(ch,condition[COND_THIRST]) >   50) ? "quenched"    :
         (PC(ch,condition[COND_THIRST]) >   10) ? "not thirsty" :
         (PC(ch,condition[COND_THIRST]) >= -10) ? "thirsty"     : 
                                                  "dehydrated",
         
         (PC(ch,condition[COND_FULL]  ) >  50) ? "full"            :
         (PC(ch,condition[COND_FULL]  ) >  20) ? "satisfied"       :
         (PC(ch,condition[COND_FULL]  ) >   0) ? "slightly hungry" :
         (PC(ch,condition[COND_FULL]  ) > -30) ? "hungry"          :
         (PC(ch,condition[COND_FULL]  ) > -50) ? "famished"        :
                                                 "starving" );
    send_to_actor( buf,  ch );
    }

    switch ( ch->position )
    {
    case POS_DEAD:     send_to_actor( ", and have died.\n\r",              ch ); break;
    case POS_MORTAL:   send_to_actor( ", and have been mortally wounded.\n\r",  ch ); break;
    case POS_INCAP:    send_to_actor( ", and have been incapacitated.\n\r",     ch ); break;
    case POS_STUNNED:  send_to_actor( ", and, unfortunately, stunned.\n\r",     ch ); break;
    case POS_SLEEPING:
    case POS_SITTING:
    case POS_RESTING:
    if ( ch->furniture )
    {
        sprintf( buf, ", and %s on %s.\n\r", position_name(ch->position),
                 STR(ch->furniture,short_descr) );
    } 
    else
    sprintf( buf, ", and %s.\n\r", position_name(ch->position) );
    
    send_to_actor( buf, ch );
    break;
    case POS_STANDING:
     if ( ch->riding != NULL )
     {
        sprintf( buf, ", and you are riding %s.\n\r", NAME(ch->riding) );
        send_to_actor( buf, ch );
     }
     else
     send_to_actor( ".\n\r", ch );
    break;
    case POS_FIGHTING:
     if ( ch->riding != NULL )
     {
        sprintf( buf, ", and engaged in combat with %s.\n\r", NAME(ch->fighting) );
        send_to_actor( buf, ch );
     }
     else
     send_to_actor( ", and fighting.\n\r",          ch );
    break;
    }

    ansi_color( NTEXT, ch );
    ansi_color( BOLD, ch );
     if ( ch->bounty <=   0  ) send_to_actor( "You are a protected citizen", ch );
else if ( ch->bounty <   500  ) send_to_actor( "You are noted as a troublemaker", ch );
else if ( ch->bounty <  10000 ) send_to_actor( "You are considered a criminal", ch );
else if ( ch->bounty < 25000 ) send_to_actor( "You are a fugitive from justice", ch );
                           else send_to_actor( "You are a feared outlaw", ch );

    if ( ch->owed > 0 ) {
      if ( ch->owed > 500 )
    send_to_actor( ", and owed a sizeable reward.\n\r", ch ); 
     else
    send_to_actor( ", and owed a reward.\n\r", ch );
    }
    else
    send_to_actor( ".\n\r", ch );

    ansi_color( NTEXT, ch );  

    if ( ch->karma > 0 ) { ansi_color( GREEN, ch ); ansi_color( BOLD, ch );  }
    if ( ch->karma > 100 ) { ansi_color( B_BLUE, ch ); ansi_color( CYAN, ch ); ansi_color( BOLD, ch ); } 
    if ( ch->karma < -100 ) { ansi_color( B_RED, ch ); }
    else ansi_color( YELLOW, ch );

     if ( ch->karma  == 0 ) send_to_actor( "Your karma is neutral.\n\r", ch );
else if ( ch->karma < -50 ) send_to_actor( "You've got bad karma written all over you.\n\r", ch);
else if ( ch->karma >  50 ) send_to_actor( "Your karma is ultrapositive.\n\r", ch );
else if ( ch->karma  <  0 ) send_to_actor( "Your karma is negative.\n\r", ch );
                       else send_to_actor( "Your karma is positive.\n\r", ch );
    ansi_color( BLUE, ch );
    ansi_color( NTEXT, ch );
    ansi_color( BLUE, ch );

     if ( ch->mana > 100 ) send_to_actor( "You feel energized.\n\r", ch );
else if ( ch->mana > 75  ) send_to_actor( "Your mind is clear.\n\r", ch );
else if ( ch->mana > 50  ) send_to_actor( "You feel focused.\n\r", ch );
else if ( ch->mana > 25  ) send_to_actor( "Your thoughts are blurred.\n\r", ch );
else if ( ch->mana >  0  ) send_to_actor( "Your mind is weak.\n\r", ch );
else send_to_actor( "Your mind is unfocused; your thoughts are a cacophony of voices.\n\r", ch );

    ansi_color( NTEXT, ch );

    sprintf( buf, "You are currently speaking %s.\n\r", lang_table[ch->speaking].name );
    send_to_actor( buf, ch );

    if ( IS_AFFECTED(ch,AFF_HIDE) && IS_AFFECTED(ch,AFF_SNEAK) )
    send_to_actor( "You are attempting camoflage and stealth.\n\r", ch );
    else
    if ( IS_AFFECTED(ch,AFF_HIDE) )
    send_to_actor( "You are trying to camoflage yourself.\n\r", ch );
    else
    if ( IS_AFFECTED(ch,AFF_SNEAK) )
    send_to_actor( "You are trying to move with stealth.\n\r", ch );

    if ( IS_IMMORTAL(ch) )
    {
    ansi_color( BOLD, ch );
    send_to_actor( "\n\rYou are", ch );

    switch ( get_trust( ch ) )
    {
           default: if ( IS_IMMORTAL(ch) )
                           sprintf( buf, " an immortal,"   );
                      else buf[0] = '\0';
                    break;
   case LEVEL_HERO: sprintf( buf, " an avatar." ); break;
case LEVEL_BUILDER: sprintf( buf, " a builder," ); break;
  case LEVEL_ANGEL: sprintf( buf, " an angel."  ); break;
case LEVEL_SUPREME: sprintf( buf, " supreme."   ); break;
case LEVEL_DEMIGOD: sprintf( buf, " a demigod." ); break;
    case MAX_LEVEL: sprintf( buf, ch->sex == SEX_MALE ? " God." : " Goddess." ); break;
    }
    
    send_to_actor( buf, ch );

    if ( get_trust( ch ) == LEVEL_BUILDER ) {
    sprintf( buf, " with a building security of %d.\n\r", PC(ch,security) );
    }
    else sprintf( buf, " [%d]\n\r", PC(ch,security) );
    send_to_actor( buf, ch );
    }

    ansi_color( NTEXT, ch );
    return;
}



char *	const	day_name	[] =
{
    "Lexo, the Great Hawk",
    "Franto, the Falcon",
    "Halag, the Owl",
    "Ruz, the Black Crow",
    "Kalamak, the Phoenix",
    "Aedo, the Golden Eagle",
    "Franwag, the Grey Gull"                         /*  7 */
};

char *  const   month_name  [] =
{
    "Iak, Season of the Ice Cougar, in winter",
    "Vod, Season of the White Fox, in winter",
    "Lin, Season of the Robin, in spring",
    "Nemi, Season of the Crystalfish, in spring",
    "Grez, the Crop Wind, planting season",
    "Po, Season of the Squirrel, in summer",
    "Rani, the Dry Winds, in summer",
    "Falak, the Dragon's Breath, late summer",
    "Kae, Harvest Moon",
    "Ret, Season of the Changing Oak, in autumn",
    "Jal, the New Frost, in autumn",
    "Yun, the Ice Weaver, in winter"
};

static char * const sky_look[8] =
{
    "cloudless",
    "cloudy",
    "obscured by storm clouds",
    "lit by flashes of lightning",
    "slightly overcast",
    "scattered with a few flakes",
    "filled with flakes",
    "a blizzard of white"
};


/*
 * Syntax:  time
 *          time full
 */
void cmd_time( PLAYER_DATA *ch, char *argument )
{
    extern char str_boot_time[];
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    char descr[MAX_STRING_LENGTH];
    sh_int wind;
    int day;

    if ( !check_blind( ch ) )
    {
        send_to_actor( "You have no clue what is going on, you are blind!\n\r",
                      ch );
        return;
    }

    one_argument( argument, arg );

    day     = weather_info.day + 1;

    switch ( weather_info.hour )
    {
        case  1:
        case  2: sprintf( descr, "late night"       ); break;
        case  3:
        case  4: sprintf( descr, "early morning"    ); break;
        case  5: sprintf( descr, "just before dawn" ); break;
        case  6: sprintf( descr, "dawn"             ); break;
        case  7: 
        case  8: sprintf( descr, "early morning"    ); break;
        case  9:
        case 10:
        case 11: sprintf( descr, "morning"          ); break;
        case 12: sprintf( descr, "midday"           ); break;
        case 13:
        case 15: sprintf( descr, "afternoon"        ); break;
        case 14: sprintf( descr, "mid-afternoon"    ); break;
        case 16:
        case 17: sprintf( descr, "evening"          ); break;
        case 18: sprintf( descr, "twilight"         ); break;
        case 19: sprintf( descr, "dusk"             ); break;
        case 20:
        case 21:
        case 22: sprintf( descr, "night"            ); break;
        case 23: sprintf( descr, "late at night"    ); break;
        case  0: sprintf( descr, "midnight"         ); break;
    }

    sprintf( buf, "It is %s on the day of %s.  The %s of the month of %s, in the year %s. ",
             descr,
             day_name[day % 7],
             day ==  1 ? "first"          :
             day ==  2 ? "second"         :
             day ==  3 ? "third"          :
             day ==  4 ? "fourth"         :
             day ==  5 ? "fifth"          :
             day ==  6 ? "sixth"          :
             day ==  7 ? "seventh"        :
             day ==  8 ? "eighth"         :
             day ==  9 ? "ninth"          :
             day == 10 ? "tenth"          :
             day == 11 ? "eleventh"       :
             day == 12 ? "twelfth"        :
             day == 13 ? "thirteenth"     :
             day == 14 ? "fourteenth"     :
             day == 15 ? "fifteenth"      :
             day == 16 ? "sixteenth"      :
             day == 17 ? "seventeenth"    :
             day == 18 ? "eighteenth"     :
             day == 19 ? "nineteenth"     :
             day == 20 ? "twentieth"      :
             day == 21 ? "twenty-first"   :
             day == 22 ? "twenty-second"  :
             day == 23 ? "twenty-third"   :
             day == 24 ? "twenty-fourth"  :
             day == 25 ? "twenty-fifth"   :
             day == 26 ? "twenty-sixth"   :
             day == 27 ? "twenty-seventh" :
             day == 28 ? "twenty-eighth"  :
             day == 29 ? "twenty-ninth"   :
                         "thirtieth",
             month_name[weather_info.month],
             numberize( weather_info.year ) );

    if ( !IS_OUTSIDE(ch) )
    {
        strcat( buf, "You cannot tell what the weather is like from here." );
    }
    else
    {
    wind = weather_info.windspeed;
    descr[0] = '\0';
    sprintf( descr, "The sky is %s ",
                  (weather_info.month < 2 || weather_info.month == 11) ?
                  sky_look[weather_info.sky+4] : sky_look[weather_info.sky] );
    strcat( buf, descr );
    sprintf( descr, "and a %s %sward %s blows.\n\r",
             (weather_info.temperature < 35) ? "cold" :
             (weather_info.temperature < 60) ? "cool" :
             (weather_info.temperature < 90) ? "warm" : "hot",
             dir_name[abs(weather_info.winddir) % 4],
             wind <= 20 ? "breeze"   :
             wind <= 50 ? "wind"     :
             wind <= 80 ? "gust"     :
                          "torrent"    );
    strcat( buf, descr );


    }

    if ( IS_OUTSIDE(ch) && weather_info.hour < 4 )
    {
        if ( weather_info.sky != SKY_CLOUDLESS
          && weather_info.moon_phase != MOON_NEW )
        strcat( buf, "The moon is behind a cloud." );
        else
        switch ( weather_info.moon_phase )
        {
        case MOON_NEW:
            strcat( buf, "No moon graces the heavens tonight." ); break;
        case MOON_WAXING_CRESCENT:
            strcat( buf, "The waxing moon is a mere crescent in the night sky." ); break;
        case MOON_WAXING_HALF:
            strcat( buf, "The waxing half-moon can be seen on this night." ); break;
        case MOON_WAXING_THREE_QUARTERS:
            strcat( buf, "Tonight, the moon is near full, a waxing gibbous of celestial harmony." ); break;
        case MOON_FULL:
            strcat( buf, "The moon is full." ); break;
        case MOON_WANING_THREE_QUARTERS:
            strcat( buf, "The moon is in the phase of a waning gibbous." ); break;
        case MOON_WANING_HALF:
            strcat( buf, "A waning half-moon treks across the night sky." ); break;
        case MOON_WANING_CRESCENT:
            strcat( buf, "Only a sliver of moon remains in the heavens." ); break;
          default:
            if ( IS_IMMORTAL(ch) )
            strcat( buf, "Something is drastically wrong with the moon!  E-gads!" );
           break;
        }
    }

    {
        char *p;
        p = format_string( str_dup( buf ) );
        send_to_actor( p, ch );
        free_string( p );
    }

    if ( IS_IMMORTAL( ch ) && !str_cmp( arg, "full" ) )
    {
        extern int num_hour;
        extern int pulse_zone;
        extern int pulse_actor;
        extern int pulse_violence;
        extern int autosave_counter;

        sprintf( buf, "%s\n\rBooted at:   %s\rSystem time: %s\r",
         VERSION_STR,  str_boot_time, (char *) ctime( &current_time )  );
        send_to_actor( buf, ch );

        sprintf( buf,
 "Temp: %4d  Windspd: %4d  Dir:    %4d  Month: %4d  Hour: %4d  Phase: %4d\n\r"
 "Sky:  %4d  MMHG:    %4d  Change: %4d  Day:   %4d  Year: %4d  Next:  %4d\n\r"
 "Sun:  %4d  NumHour: %4d  zone:   %4d  Mob:   %4d  Fgt:  %4d  Auto:  %4d\n\r",
               weather_info.temperature,   weather_info.windspeed,
               weather_info.winddir %4,    weather_info.month,
               weather_info.hour,             weather_info.moon_phase,
               weather_info.sky,           weather_info.mmhg,
               weather_info.change,        weather_info.day,
               weather_info.year,             weather_info.next_phase,
               weather_info.sunlight,      num_hour/60,
               pulse_zone,                 pulse_actor,
               pulse_violence,             autosave_counter );
        send_to_actor( buf, ch );
    }

    return;
}



void help_to_char( char *text, PLAYER_DATA *ch, bool fPage )
{
   char buf[MAX_STRING_LENGTH*2];
   const char *str;
   const char *i;
   char *point;

   if( text == NULL || *text == '\0' )
   {
      send_to_actor( "\n\r\n\r", ch );
      return;
   }

   point = buf;
   str = text;
   while( *str != '\0' )
   {
      if( *str != '$' )
      {
         *point++ = *str++;
         continue;
      }

      ++str;
      switch( *str )
      {
         default: i = " "; break;
        case '0': i = !HAS_ANSI(ch) ? "" :  BLACK;   break;
        case '1': i = !HAS_ANSI(ch) ? "" :  BLUE;    break;
        case '2': i = !HAS_ANSI(ch) ? "" :  GREEN;   break;
        case '3': i = !HAS_ANSI(ch) ? "" :  CYAN;    break;
        case '4': i = !HAS_ANSI(ch) ? "" :  RED;     break;
        case '5': i = !HAS_ANSI(ch) ? "" :  PURPLE;  break;
        case '6': i = !HAS_ANSI(ch) ? "" :  YELLOW;  break;
        case '7': i = !HAS_ANSI(ch) ? "" :  GREY;    break;
        case 'B': i = !HAS_ANSI(ch) ? "" :  BOLD;    break;
        case 'I': i = !HAS_ANSI(ch) ? "" :  INVERSE; break;
        case 'F': i = !HAS_ANSI(ch) ? "" :  FLASH;   break;
        case 'N': i = !HAS_ANSI(ch) ? "" :  NTEXT;   break;
        case 'n': i = "\n\r"; break;
        case '$': i = "$";    break;
      } 
      ++str;
      while( (*point = *i) != '\0' )
         ++point, ++i;      
   }
   *point = '\0';

   if ( fPage ) page_to_actor( buf, ch );
   else         send_to_actor( buf, ch );
   return;
}





/*
 * Syntax:  help [keyword]
 */
void cmd_help( PLAYER_DATA *ch, char *argument )
{
   char argall[MAX_INPUT_LENGTH];
   char arg1[MAX_INPUT_LENGTH];
   char buf[MAX_STRING_LENGTH];
   char buf2[MAX_STRING_LENGTH];
   HELP_DATA *pHelp;
    
   if ( ch->desc == NULL ) return;

   strcpy( buf, "" );
   strcpy( buf2, "" );

   argument = one_argument( argument, arg1 );

   if ( arg1[0] == '\0' )
      strcpy( arg1, "summary");

   if ( !str_cmp( arg1, "show" ) || !str_cmp( arg1, "index" ) ) {
      int col = 0;
      int i;
      int hclass[MAX_HELP_CLASS];


      argument = one_argument( argument, arg1 );

      if ( arg1[0] == '\0' ) {

         for ( i=0; i < MAX_HELP_CLASS; i++ )
            hclass[i] = 0;

         sprintf( buf, "\nThe following classes of help files are available:\n\n#     Class           Description\n\r" );
         send_to_actor( buf, ch );
         sprintf( buf, "-     -----           -----------\n\r" );
         send_to_actor( buf, ch );

         for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next ) {
            if ( pHelp->level > get_trust( ch ) )
               continue;
            hclass[pHelp->class]++;
         }

         for ( i=0; i < MAX_HELP_CLASS; i++ ) {
            sprintf(buf, "%-5d %-15s %s\n", hclass[i], help_class_table[i].name, help_class_table[i].desc);
            send_to_actor( buf, ch );
         }
         sprintf( buf, "\nType 'help show [class]' to see available helps for a specific class.\n\r" );
         send_to_actor( buf, ch );

         return;

      } else {
         for ( i=0; i < MAX_HELP_CLASS; i++ ) {
            if ( !str_prefix( arg1, help_class_table[i].name )) {
               for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next ) {

                  if ( pHelp->level > get_trust( ch ) )
                  continue;

                  if ( pHelp->class == i) {
                     col++;
                     sprintf( buf2, "%-26s", trunc_fit( string_proper ( pHelp->name ), 25 ) );
                     strcat( buf, buf2 );
                     if ( col % 3 == 0 ) {
                       strcat( buf, "\n\r" );
                       send_to_actor( buf, ch );
                       strcpy( buf, "" );
                       strcpy( buf2, "" );
                     }
                  }
               }
               send_to_actor( buf, ch );
               return;
            }
         }
      }
   }

   if ( arg1[0] == '#' ) {
      char buf[MAX_STRING_LENGTH];
      bool fFound = FALSE;        

      strcpy (argument, arg1);

      argument++;
      if ( *argument == '\0' ) return;

      sprintf( buf, "Related Topics:\n\r" );
      for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next ) {

         if ( pHelp->level > get_trust( ch ) )
            continue;

         if ( strstr( pHelp->text, argument ) ) {
            if ( !ch->desc || CONNECTED(ch->desc)
               || ch->desc->connected == CON_DOC_MENU ) {
                  fFound = TRUE;
                  strcat( buf, pHelp->keyword );
                  strcat( buf, "\n\r" );
               }
         }
      }
      page_to_actor( buf, ch );
      if ( !fFound ) page_to_actor( "None.", ch );
      return;
   }

   /*
    * Tricky argument handling so 'help a b' doesn't match a.
    */
   strcpy ( argall, arg1 );
   while ( argument[0] != '\0' ) {
      argument = one_argument( argument, arg1 );
      if ( arg1[0] != '\0' )
         strcat( argall, " " );
      strcat( argall, arg1 );
   }

   for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next ) {
	if ( pHelp->level > get_trust( ch ) )
	    continue;

        if ( is_prename( argall, pHelp->keyword ) )
        {
            if ( !ch->desc || CONNECTED(ch->desc)
              || ch->desc->connected == CON_DOC_MENU )
            {
                char buf[MAX_STRING_LENGTH];

                buf[0] = '\0';
                
/*
                first_arg( pHelp->keyword, buf, FALSE ); 
*/
		if ( pHelp->class != HELP_CLASS_TIPS
                  && pHelp->class != HELP_CLASS_NANNY ) {
                sprintf(buf, "\n%s (%s)\n",
                    pHelp->name, help_class_table[pHelp->class].name );

                if ( HAS_ANSI(ch) ) send_to_actor( BOLD, ch );
                send_to_actor( buf, ch );
                if ( HAS_ANSI(ch) ) send_to_actor( NTEXT, ch );
		}
                send_to_actor( "\n\r", ch );

                if ( str_cmp( pHelp->syntax, "" ) ) {
                   if ( HAS_ANSI(ch) ) send_to_actor( BOLD, ch );
                   send_to_actor( "Syntax:  ", ch );
                   if ( HAS_ANSI(ch) ) send_to_actor( NTEXT, ch );
                   sprintf(buf, "%s\n", pHelp->syntax);
                   send_to_actor( buf, ch );
                }

/*
                if ( HAS_ANSI(ch) ) send_to_actor( BOLD, ch );
                send_to_actor( "Description:\n", ch );
                if ( HAS_ANSI(ch) ) send_to_actor( NTEXT, ch );
 */

                if ( pHelp->text[0] == '.' )
                help_to_char( pHelp->text+1, ch, TRUE );
                else
                help_to_char( pHelp->text  , ch, TRUE );

                if ( IS_IMMORTAL(ch) && str_cmp( pHelp->immtext, "") )  {
                   if ( HAS_ANSI(ch) ) page_to_actor( BOLD, ch );
                   page_to_actor( "\nImmtext:\n", ch );
                   if ( HAS_ANSI(ch) ) page_to_actor( NTEXT, ch );
                   if ( pHelp->immtext[0] == '.' )
                      help_to_char( pHelp->immtext+1, ch, TRUE );
                   else
                      help_to_char( pHelp->immtext  , ch, TRUE );
                }

                if ( str_cmp( pHelp->example, "") )  {
                   if ( HAS_ANSI(ch) ) page_to_actor( BOLD, ch );
                   page_to_actor( "\nExample:\n", ch );
                   if ( HAS_ANSI(ch) ) page_to_actor( NTEXT, ch );
                   if ( pHelp->example[0] == '.' )
                      help_to_char( pHelp->example+1, ch, TRUE );
                   else
                      help_to_char( pHelp->example  , ch, TRUE );
                }

                if ( str_cmp( pHelp->seealso, "") )  {
                   if ( HAS_ANSI(ch) ) page_to_actor( BOLD, ch );
                   page_to_actor( "\nSee also: ", ch );
                   if ( HAS_ANSI(ch) ) page_to_actor( NTEXT, ch );
                   sprintf(buf, "%s\n", pHelp->seealso);
                   page_to_actor( buf, ch );
                } else {
                   page_to_actor( "\n", ch );
                }
                return;
            }

            /*
             * Strip leading '.' to allow initial blanks.
             */
            if ( pHelp->text[0] == '.' )
            help_to_char( pHelp->text+1, ch, FALSE );
            else
            help_to_char( pHelp->text  , ch, FALSE );

            if ( str_cmp( pHelp->immtext, "") )  {
               if ( pHelp->immtext[0] == '.' )
                  help_to_char( pHelp->immtext+1, ch, TRUE );
               else
                  help_to_char( pHelp->immtext  , ch, TRUE );
            }
            return;
        }
    }

    send_to_actor( "No help on that word.\n\r", ch );
    return;
}


void old_cmd_help( PLAYER_DATA *ch, char *argument )
{
    char argall[MAX_INPUT_LENGTH];
    char argone[MAX_INPUT_LENGTH];
    HELP_DATA *pHelp;
    int trust;
    
    if ( ch->desc == NULL ) return;

    trust = get_trust( ch );

    if ( argument[0] == '\0' )
	argument = "summary";

    /*
     * Help Index --- return the whole list of keywords
     * Improvements by Anthony Goins
     */
    if ( !str_cmp( argument, "index" ) ) {
        int col = 0;
        bool fFound = FALSE;
        char buf[MAX_STRING_LENGTH];
        char buf2[MAX_STRING_LENGTH];

        sprintf( buf, "Help Index:\n\r" );
        for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next ) {
         if ( pHelp->level <  LEVEL_HERO
         &&   pHelp->level <= get_trust( ch )
         &&   pHelp->level >=  0 ) {
             col++;
             fFound = TRUE;
    one_argument( pHelp->keyword, buf2 );
       sprintf( buf2, "%-26s", trunc_fit( buf2, 26 ) );
             strcat( buf, buf2 );
       if ( ++col % 3 == 0 )
     strcat( buf, "\n\r" );
      }
  }
        if ( !fFound ) send_to_actor( "None.", ch );
        else
         page_to_actor( buf, ch );
     return;
 }

    if ( !str_cmp( argument, "wizindex" ) ) {
        int col = 0;
        bool fFound = FALSE;
        char buf[MAX_STRING_LENGTH];
        char buf2[MAX_STRING_LENGTH];

        sprintf( buf, "Immortal Help Index:\n\r" );
        for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next ) {
         if ( pHelp->level >  LEVEL_HERO
         &&   pHelp->level <= get_trust( ch )
         &&   pHelp->level >= 0 ) {
             col++;
             fFound = TRUE;
    one_argument( pHelp->keyword, buf2 );
       sprintf( buf2, "%-26s", trunc_fit( buf2, 26 ) );
             strcat( buf, buf2 );
       if ( ++col % 3 == 0 )
     strcat( buf, "\n\r" );
      }
  }
        if ( !fFound ) send_to_actor( "None.", ch );
        else
         page_to_actor( buf, ch );
     return;
 }

    if ( argument[0] == '#' )
    {
        char buf[MAX_STRING_LENGTH*2];
        bool fFound = FALSE;        
        
        
        argument++;
        if ( *argument == '\0' ) return;

        sprintf( buf, "Related Topics:\n\r" );
        for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
        {

            if ( pHelp->level > trust )
            continue;

            if ( strstr( pHelp->text, argument ) )
            {
                if ( !ch->desc || CONNECTED(ch->desc)
                  || ch->desc->connected == CON_DOC_MENU )
                {
                    fFound = TRUE;
                    strcat( buf, pHelp->keyword );
                    strcat( buf, "\n\r" );
                }
            }
        }

        page_to_actor( buf, ch );
        if ( !fFound ) page_to_actor( "None.", ch );
        return;
    }


    /*
     * Tricky argument handling so 'help a b' doesn't match a.
     */
    argall[0] = '\0';
    while ( argument[0] != '\0' )
    {
	argument = one_argument( argument, argone );
	if ( argall[0] != '\0' )
	    strcat( argall, " " );
	strcat( argall, argone );
    }

    for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
    {
    if ( pHelp->level > trust )
	    continue;

        if ( is_prename( argall, pHelp->keyword ) )
        {
            if ( !ch->desc || CONNECTED(ch->desc)
              || ch->desc->connected == CON_DOC_MENU )
            {
                char buf[MAX_STRING_LENGTH];
                
                first_arg( pHelp->keyword, buf, FALSE );
                if ( HAS_ANSI(ch) ) send_to_actor( BOLD, ch );
                send_to_actor( buf, ch );
                if ( HAS_ANSI(ch) ) send_to_actor( NTEXT, ch );
                send_to_actor( "\n\r", ch );

                if ( pHelp->text[0] == '.' )
                help_to_char( pHelp->text+1, ch, TRUE );
                else
                help_to_char( pHelp->text  , ch, TRUE );
                return;
            }

            /*
             * Strip leading '.' to allow initial blanks.
             */
            if ( pHelp->text[0] == '.' )
            help_to_char( pHelp->text+1, ch, FALSE );
            else
            help_to_char( pHelp->text  , ch, FALSE );
            return;
        }
    }

    send_to_actor( "No help on that word.\n\r", ch );
    return;
}



char *get_name_level( int level ) 
{
     switch( level ) {
          case 0: return "Newbie"; break;
          case 1:
          case 2:
          case 3:
          case 4: 
          case 5: return "Freshling"; break;
          case 6:
          case 7:
          case 8:
          case 9: 
          case 10: return "Adventurer"; break;
          case 11:
          case 12:
          case 13:
          case 14:
          case 15:
          case 16:
          case 17:
          case 18:
          case 19: return "Veteran";  break;    
          case 20:     
          case 21: 
          case 22:
          case 23:
          case 24:
          case 25:
          case 26: 
          case 27:
          case 28:
          case 29: return "Dungeoneer"; break;
          case 30:
          case 31:
          case 32:
          case 33:
          case 34:
          case 35:
          case 36:
          case 37:
          case 38:   
          case 39: return "Heroic"; break;
          case 40:
          case 41:
          case 42:
          case 43:
          case 44:
          case 45:
          case 46:
          case 47:
          case 48:
          case 49:
          case 50: return "Avatar"; break;  
          case 99: return "Transcendant"; break;
         case 100:
         case 101: return "Godlike"; break;
          default: return "Legendary"; break; 
        
       }
       return "Mortal"; 
} 
  


/*
 * New 'who' command originally by Alander of Rivers of Mud.
 * NEW 'who' command by Locke of The Isles.
 * Syntax:  who
 *          who [number] [number]
 *          who immortal
 */
void cmd_who( PLAYER_DATA *ch, char *argument )
{
    static int max = 0;
    char buf[MAX_STRING_LENGTH];
    CONNECTION_DATA *d;
    extern int num_hour;
    extern int imc_active;  /* from imc */
    int iLevelLower;
    int iLevelUpper;
    int nNumber;
    int nMatch;
    int gMatch;
    bool fImmortalOnly;

    /*
     * Set default arguments.
     */
    iLevelLower    = 0;
    iLevelUpper    = MAX_LEVEL;
    fImmortalOnly  = FALSE;

    /*
     * Parse arguments.
     */
    nNumber = 0;
    for ( ;; )
    {
	char arg[MAX_STRING_LENGTH];

	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;

	if ( is_number( arg ) )
	{
	    switch ( ++nNumber )
	    {
	    case 1: iLevelLower = atoi( arg ); break;
	    case 2: iLevelUpper = atoi( arg ); break;
	    default:
		send_to_actor( "Only two level numbers allowed.\n\r", ch );
		return;
	    }
	}
	else
	{
	    arg[3]    = '\0';
        if ( !str_prefix( arg, "immortal" ) )
	    {
		fImmortalOnly = TRUE;
	    }
        }
    }

    /*
     * Now show matching chars.
     */
    nMatch = 0;
    gMatch = 0;
    buf[0] = '\0';
    for ( d = connection_list; d != NULL; d = d->next )
    {
	PLAYER_DATA *wch;

	/*
	 * Check for match against restrictions.
	 * Don't use trust as that exposes trusted mortals.
	 */
    if ( !CONNECTED(d) || !can_see( ch, d->character ) )
        continue;

#ifdef NOWHO_BUILDERS
    if ( d->connected != CON_PLAYING && !IS_IMMORTAL(ch) ) continue;
#endif

	wch   = ( d->original != NULL ) ? d->original : d->character;

    if ( IS_NPC(wch) ) continue;

    if ( PC(wch,level) < iLevelLower
        ||   PC(wch,level) > iLevelUpper
        || ( fImmortalOnly  && PC(wch,level) < LEVEL_HERO ) )
	    continue;

        /*
         * Format it up.
         */
         if (IS_HERO(wch) && PC(wch,level) == get_trust( wch ) )
         {
         char doe[MAX_STRING_LENGTH];
         gMatch++;

         if ( d->connected != CON_PLAYING )
         sprintf( doe, " -Creating- " );
         else
         {
         switch ( PC(wch,level) )
         {
                         default: sprintf( doe, " Immortal    " ); break;
                 case LEVEL_HERO: sprintf( doe, " Hero        " ); break;
              case LEVEL_BUILDER: sprintf( doe, " Builder     " ); break;
              case LEVEL_SUPREME: sprintf( doe, " Writer      " ); break;
              case LEVEL_DEMIGOD: sprintf( doe, " Director    " ); break;
                  case MAX_LEVEL: sprintf( doe, " Producer    " ); break;
         }
         }
               if ( HAS_ANSI(ch) ) {
    strcat( buf, NTEXT ); strcat( buf, BOLD ); }
               sprintf( buf + strlen(buf), "[%-13s] ", doe );
               sprintf( doe, PC(wch,constellation), wch->name );
               strcat( buf, doe );
               strcat( buf, "\n\r" );
        }
        else
        {
        nMatch++;
        if ( IS_IMMORTAL(ch) )
        sprintf( buf + strlen(buf), "[Mortal %2d] %s %s\n\r", 
wch->exp_level, NAME(wch), PC(wch,title) );
        else
        {
/* use for no visible morts in game: */
/*        if ( !CONNECTED(ch->desc) )  */ {
           char buf2[MAX_STRING_LENGTH];

          sprintf( buf2, "%s%s%s", wch->name, 
PC(wch,title) != NULL && PC(wch,title) != &str_empty[0] ? " " : "",
PC(wch,title) != NULL && PC(wch,title) != &str_empty[0] ? PC(wch,title) : "" );
         if ( HAS_ANSI(ch) ) {
    if ( wch->karma > 0 ) { strcat( buf, GREEN ); strcat( buf, BOLD );  }
         else
    if ( wch->karma > 100 ) { strcat( buf, B_BLUE ); strcat( buf, CYAN ); strcat( buf, BOLD ); } 
         else
    if ( wch->karma < -100 ) { strcat( buf, RED ); }
    else strcat( buf, YELLOW );
        } 
 
        sprintf( buf + strlen(buf), "[ %-11s ] %s%-28s Level %s\n\r", 
get_name_level(wch->exp_level),  
HAS_ANSI(ch) ? NTEXT : "", trunc_fit( buf2, 28 ),
capitalize(numberize(wch->exp_level)) 
); 
        } 

        }
        }
    }

    ansi_color( BOLD, ch );
    send_to_actor( buf, ch );
    ansi_color( NTEXT, ch );

    if (nMatch > 0)
    {
        sprintf( buf, "There %s %d visible player%s",
             nMatch > 1 ? "are" : "is",  nMatch,
             nMatch > 1 ? "s" : "" );
        send_to_actor( buf, ch );
    }

    if (gMatch > 0 || fImmortalOnly)
    {
        if (nMatch > 0)
        sprintf( buf, " and %d visible god%s", gMatch,
                      gMatch == 1 ? "" : "s" );
        else
        sprintf( buf, "There %s %d visible god%s",
             gMatch > 1 ? "are" : "is",  gMatch,
             gMatch > 1 ? "s" : "" );

        send_to_actor( buf, ch );
    }

    max = nMatch + gMatch > max ? nMatch + gMatch : max;
    sprintf( buf, ", with a high of %d.\n\r", max );
    if ( max > 0 ) send_to_actor( buf, ch );

    if ( num_hour / (60 * PULSE_PER_SECOND) < 60 )
    {
    sprintf( buf, "Next reboot will occur in %d minute%s.\n\r",
         num_hour / (60 * PULSE_PER_SECOND),
         num_hour / (60 * PULSE_PER_SECOND) != 1 ? "s" : "" );
    }
    else
    {
    sprintf( buf, "Next reboot will occur in %d hour%s.\n\r",
        num_hour / (60 * PULSE_PER_SECOND * 60)+1,
        num_hour / (60 * PULSE_PER_SECOND * 60)+1 != 1 ? "s" : "" );
    }
    send_to_actor( buf, ch );
    if ( imc_active )
    send_to_actor( "Feeling lonely? Try typing 'imc' for relayed chat.\n\r", ch );
    return;
}



void show_inventory( PLAYER_DATA *ch, PLAYER_DATA *tch, bool fPeek )
{
    PROP_DATA *prop;
    bool ShowPockets = FALSE;
    bool ShowNothing = TRUE;
    
    for ( prop = ch->carrying; prop != NULL; prop = prop->next_content )
    {
        if ( fPeek
          && ch != tch
          && !IS_IMMORTAL(tch) 
          && !skill_check( tch, gsn_peek, 50 ) )
        continue;

        if ( !can_see_prop( tch, prop ) )
        continue;
        
        if ( prop->wear_loc == WEAR_NONE )
        ShowPockets = TRUE;
            
        if ( prop->item_type != ITEM_CONTAINER )
        continue;
        
        if ( IS_SET(prop->value[1], CONT_CLOSED) )
        {
            send_to_actor( capitalize(STR(prop,short_descr)), ch );
            send_to_actor( " is closed.\n\r", ch );
            ShowNothing = FALSE;
            continue;
        }
        
        if ( prop->contains == NULL )
        continue;
        
        ShowNothing = FALSE;
        act( "$p contains:", tch, prop, NULL, TO_CHAR );
        if ( fPeek )
        show_peek_to_char( prop->contains, tch, TRUE, TRUE, LEARNED(tch,gsn_peek) );
        else
        show_list_to_char( prop->contains, tch, TRUE, TRUE );
    }
    
    if ( !ShowPockets )
    {
    if ( ShowNothing )
    act( "$E $t carrying nothing of note.", 
         tch, tch == ch ? "are" : "is", ch, TO_CHAR );
    return;
    }

    act( "Tucked away in the folds of $S clothing is:", 
         tch, NULL, ch, TO_CHAR );

    if ( fPeek )
    show_peek_to_char( ch->carrying, tch, TRUE, TRUE, LEARNED(tch,gsn_peek) );
    else
    show_list_to_char( ch->carrying, tch, TRUE, TRUE );
    return;
}



void cmd_title( PLAYER_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    if ( IS_NPC( ch ) ) return;

    free_string( PC(ch,title ));
    PC(ch,title) = str_dup( "" );
    if ( *argument == '\0' ) send_to_actor( "Title cleared.\n\r", ch );
    else { PC(ch,title) = str_dup( argument );  
           send_to_actor( "You are now known as ", ch );
           send_to_actor( NAME(ch), ch );
           send_to_actor( " ", ch );
           send_to_actor( PC(ch,title), ch );
           send_to_actor( ".\n\r", ch );
           sprintf( buf, "%s changes %s title to %s %s.\n\r", 
                    NAME(ch), HIS_HER(ch),
                    NAME(ch), PC(ch,title) );
           add_history( ch, buf );
    }  
    return;
}   

/*
 * Syntax:  inventory
 */
void cmd_inventory( PLAYER_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    PROP_DATA *h;

    ansi_color( color_table[PC(ch,colors[COLOR_PROP_LIST])].code, ch );

    show_inventory( ch, ch, FALSE );
    if ( ch->riding )
    {
        send_to_actor( "In the packs of your mount:\n\r", ch );
        show_list_to_char( ch->riding->carrying, ch, TRUE, TRUE );
    }

    h = get_eq_char( ch, WEAR_HOLD_1 );
    if ( h == NULL ) h = get_eq_char( ch, WEAR_WIELD_1 );
    if ( h != NULL )
    {
        sprintf( buf, (get_eq_char( ch, WEAR_WIELD_2 ) == NULL) 
                   && (get_eq_char( ch, WEAR_HOLD_2 )  == NULL) ?
                     "You %s %s in your hands.\n\r"             :
                     "You %s %s in your primary hand.\n\r",
                 h->wear_loc ==  WEAR_HOLD_1 ? "hold" : "wield",
                 PERSO(h, ch) );
        send_to_actor( buf, ch );
    }

    h = get_eq_char( ch, WEAR_HOLD_2 );
    if ( h == NULL ) h = get_eq_char( ch, WEAR_WIELD_2 );
    if ( h != NULL )
    {
        sprintf( buf, "In your secondary hand, you %s %s.\n\r",
                 h->wear_loc ==  WEAR_HOLD_2 ? "hold" : "wield",
                 PERSO(h, ch) );
        send_to_actor( buf, ch );
    }

    show_belt( ch, ch ); 

    ansi_color( NTEXT, ch );
    return;
}


void show_belt( PLAYER_DATA *victim, PLAYER_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    PROP_DATA *prop;
    int belt;

    buf[0] = '\0';

    for ( belt = WEAR_BELT_1;  belt < WEAR_BELT_5; belt++ )
    {
        if ( (prop = get_eq_char( victim, belt )) == NULL )
        continue;

        strcat( buf, "   " );
        strcat( buf, PERSO(prop, ch) );
        strcat( buf, "\n\r" );
    }

    if ( buf[0] != '\0' )
    {
        act( "From $N$y belt hangs:", ch, NULL, victim, TO_CHAR );
        send_to_actor( buf, ch );
    }
    return;
}



/*
 * Syntax:  peek [person]
 */
void cmd_peek( PLAYER_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    PLAYER_DATA *victim;
    
    one_argument( argument, arg );
    
    if ( arg[0] == '\0' || (victim = get_actor_scene( ch, arg )) == NULL )
    {
        send_to_actor( "Peek into whose inventory?\n\r", ch );
        return;
    }

    if ( !skill_check( ch, gsn_peek, 0 ) )
    {
        act( "$n is caught peeking at $N's inventory!", ch, NULL, victim, TO_NOTVICT );
        act( "You catch $n peeking at your inventory!", ch, NULL, victim, TO_VICT );
        send_to_actor( "You were caught!\n\r", ch );
    }

    show_inventory( victim, ch, TRUE );
    show_belt( victim, ch );
    return;
}



/*
 * Syntax:  equipment
 */
void cmd_equipment( PLAYER_DATA *ch, char *argument )
{
      if ( !IS_NPC(ch) ) 
    ansi_color( color_table[PC(ch,colors[COLOR_PROP_LIST])].code, ch );
  show_equipment_table( ch, ch );
      if ( !IS_NPC(ch) ) 
    ansi_color( NTEXT, ch );
    return;
}




/*
 * Syntax:  compare [prop] [prop]
 */
void cmd_compare( PLAYER_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    PROP_DATA *prop1;
    PROP_DATA *prop2;
    int value1;
    int value2;
    char *msg;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    if ( arg1[0] == '\0' )
    {
	send_to_actor( "Compare what to what?\n\r", ch );
	return;
    }

    if ( ( prop1 = get_prop_inv( ch, arg1 ) ) == NULL )
    {
	send_to_actor( "You do not have that item.\n\r", ch );
	return;
    }

    if ( arg2[0] == '\0' )
    {
	for ( prop2 = ch->carrying; prop2 != NULL; prop2 = prop2->next_content )
	{
	    if ( prop2->wear_loc != WEAR_NONE
	    &&   can_see_prop( ch, prop2 )
	    &&   prop1->item_type == prop2->item_type
	    && ( prop1->wear_flags & prop2->wear_flags & ~ITEM_TAKE) != 0 )
		break;
	}

	if ( prop2 == NULL )
	{
	    send_to_actor( "You aren't wearing anything comparable.\n\r", ch );
	    return;
	}
    }
    else
    {
    if ( ( prop2 = get_prop_inv( ch, arg2 ) ) == NULL )
	{
	    send_to_actor( "You do not have that item.\n\r", ch );
	    return;
	}
    }
	    
    msg		= NULL;
    value1	= 0;
    value2	= 0;

    if ( prop1 == prop2 )
    msg = "You compare $p to itself.  It looks about the same.";
    else
    if ( prop1->item_type == prop2->item_type )
    {
       switch ( prop1->item_type )
       {
       default:
           msg = "It is impossible to to compare $p and $P.";
           break;

       case ITEM_GEM:
           value1 = prop1->value[1];
           value2 = prop2->value[1];
           if ( prop1->value[0] != prop2->value[0] )
           msg = "You shouldn't compare those gems; they are of a different element.";
           break;

       case ITEM_ARMOR:
           value1 = prop1->value[0] / 2;
           value2 = prop2->value[0] / 2;
           break;

       case ITEM_WEAPON:
       case ITEM_RANGED_WEAPON:
           value1 = prop1->value[1] + prop1->value[2] / 2;
           value2 = prop2->value[1] + prop2->value[2] / 2;
           break;

       case ITEM_AMMO:
           value1 = prop1->value[1];
           value2 = prop2->value[1];
           break;
       }
    }

    if ( msg == NULL )
    {
	     if ( value1 == value2 ) msg = "$p and $P look about the same.";
	else if ( value1  > value2 ) msg = "$p looks better than $P.";
	else                         msg = "$p looks worse than $P.";
    }

    act( msg, ch, prop1, prop2, TO_CHAR );

    value1 = prop1->weight - prop2->weight;

         if ( value1 == 0 ) msg = NULL;
    else if ( value1 >  3 ) msg = "$p is heavier than $P.";
    else if ( value1 >  0 ) msg = "$p is slightly heavier than $P.";
    else if ( value1 > -3 ) msg = "$P is slightly heavier than $p.";
    else                    msg = "$P is heavier than $p.";

    if ( msg != NULL )     act( msg, ch, prop1, prop2, TO_CHAR );
    return;
}




/*
 * Syntax:  credits
 */
void cmd_credits( PLAYER_DATA *ch, char *argument )
{
    cmd_help( ch, "nimud" );
    return;
}




/*
 * Syntax:  consider [person]
 */
void cmd_consider( PLAYER_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    PLAYER_DATA *victim;
    char *buf = '\0';
    int hp;
    int curr_wis, curr_con, curr_str, curr_dex, curr_int;
    int plr_wis, plr_con, plr_str, plr_dex, plr_int;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_actor( "Consider killing whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_actor_scene( ch, arg ) ) == NULL )
    {
	send_to_actor( "They're not here.\n\r", ch );
	return;
    }

    cmd_look( ch, arg );
    if ( ch == victim ) { cmd_score( ch, "" ); return; }

    curr_wis = get_curr_wis(victim);
    curr_dex = get_curr_dex(victim);
    curr_con = get_curr_con(victim);
    curr_int = get_curr_int(victim);
    curr_str = get_curr_str(victim);

    plr_wis  = get_curr_wis( ch );
    plr_con  = get_curr_con( ch );
    plr_str  = get_curr_str( ch );
    plr_dex  = get_curr_dex( ch );
    plr_int  = get_curr_int( ch );

    {
      char buf2[MAX_STRING_LENGTH];
      int count = 0;
      buf2[0] = '\0';

    if ( curr_dex > plr_dex ) { strcat( buf2, "$E is more agile than you.\n\r" );    }
    if ( curr_str > plr_str ) { strcat( buf2, "$E is stronger than you.\n\r"  );  }
    if ( curr_int < 5 ) strcat( buf2, "$N has no mind to speak of.\n\r" );  
    if ( curr_wis < plr_wis ) strcat( buf2, "$N is unwise.\n\r" );
    act( buf2, ch, NULL, victim, TO_CHAR );

    if ( curr_con > plr_con ) { act( "You are vital and hardened compared to $N." , ch, NULL, victim, TO_CHAR ); }
                         else { act( "$N is more vital and hardened than you.", ch, NULL, victim, TO_CHAR ); } 

     }

    sprintf( arg, "$E is " );
    if ( ( IS_NPC(ch) && (IS_SET(victim->act, ACT_GOOD) || 
IS_SET(victim->act, ACT_BOUNTY)) ) || !IS_NPC(ch) || victim->bounty > 0 ) {
     if ( victim->bounty <    70 ) strcat( arg, "a noted troublemaker.\n\r" );
else if ( victim->bounty <   500 ) strcat( arg, "an untrustworthy thief.\n\r" );
else if ( victim->bounty <  1000 ) strcat( arg, "considered a criminal.\n\r" );
else if ( victim->bounty < 10000 ) strcat( arg, "a fugitive from justice.\n\r" );
else                               strcat( arg, "a feared outlaw.\n\r" );
    act( arg, ch, NULL, victim, TO_CHAR );
    }

    hp     = ch->hit - victim->hit;

   if ( GET_AC(ch) > GET_AC(victim) ) act( "You are well armored compared to $N.", ch, NULL, victim, TO_CHAR ); 

 if ( hp >=  80 ) buf = "You are much healthier than $M.";
 if ( hp <=  60 ) buf = "You are healthier than $M.";
 if ( hp <=  30 ) buf = "You are a bit healthier than $M.";
 if ( hp <=   5 ) buf = "You are a slight bit healthier than $M.";
 if ( hp <=   0 ) buf = "$E is a slight bit healthier than you.";
 if ( hp <= -10 ) buf = "$E is a bit healthier than you.";
 if ( hp <= -30 ) buf = "$E is healthier than you.";
 if ( hp <= -80 ) buf = "$E is much healthier than you.";

    act( buf, ch, NULL, victim, TO_CHAR );
    return;
}




/*
 * Syntax:  report
 */
void cmd_report( PLAYER_DATA *ch, char *argument )
{
    char buf[MAX_INPUT_LENGTH];

    sprintf( buf, "You report that you are %s and %s.\n\r",
                  STRING_HITS(ch), STRING_MOVES(ch)  );
    send_to_actor( buf, ch );

    sprintf( buf, "$n reports that $e is %s and %s.",
                  STRING_HITS(ch), STRING_MOVES(ch) );
    act( buf, ch, NULL, NULL, TO_SCENE );

    return;
}




/*
 * Syntax:  socials
 */
void cmd_socials( PLAYER_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int iSocial;
    int col;
 
    col = 0;
    for ( iSocial = 0; social_table[iSocial].name[0] != '\0'; iSocial++ )
    {
	sprintf( buf, "%-12s", social_table[iSocial].name );
	send_to_actor( buf, ch );
	if ( ++col % 6 == 0 )
	    send_to_actor( "\n\r", ch );
    }
 
    if ( col % 6 != 0 )
	send_to_actor( "\n\r", ch );
    return;
}



/*
 * Contributed by Alander.  Updates by Krayy.
 * Syntax:  commands
 */
void cmd_commands( PLAYER_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char buf1[MAX_STRING_LENGTH];
    int cmd;
    int col;
 
    col = 0;
    sprintf( buf, "\r\n" );

    for ( cmd = 0; command_table[cmd].name[0] != '\0'; cmd++ )
    {
            if ( command_table[cmd].level == -1 ) continue;

        if ( command_table[cmd].level <  LEVEL_HERO
        &&   command_table[cmd].level <= get_trust( ch )
        &&   command_table[cmd].position >= POS_DEAD )
	{
            if ( command_table[cmd].level == -2 )
            {
                strcat( buf, "\n\r" );
                strcat( buf, "\n\r" );

                if ( HAS_ANSI(ch) )
                strcat( buf, BOLD );

                strcat( buf, command_table[cmd].name );

                if ( HAS_ANSI(ch) )
                strcat( buf, NTEXT );

                strcat( buf, "\n\r" );
                col = 0;

            } else {

         sprintf( buf1, "%-12s", command_table[cmd].name );
         strcat( buf, buf1 );

         if ( ++col % 6 == 0 )
         strcat( buf, "\n" );
        }

	}
    }
 
    if ( col % 6 != 0 )
    strcat( buf, "\n\r" );

    page_to_actor( buf, ch );
    return;
}




void cmd_wizlist ( PLAYER_DATA *ch, char *argument )
{

    cmd_help ( ch, "wizlist" );
    return;

}



void cmd_history( PLAYER_DATA *ch, char *argument ) {

/*
 * History
 */
    char arg[MAX_STRING_LENGTH];
    PLAYER_DATA *pActor;

    if ( IS_NPC(ch) ) return;

    one_argument( argument, arg );

    if ( arg[0] == '\0' ) pActor = ch;
    else {

    pActor = get_actor_world( ch, arg );  

    if ( !pActor || IS_NPC(pActor) ) {
              sprintf( arg, "%s will most likely die in obscurity.\n\r", 
                       NAME(pActor) );
              send_to_actor( arg, ch );
              return;
        }
    }

    send_to_actor( "\n\r", ch ); 
    sprintf( arg, "The History of %s\n\r\n\rAs recorded by Edigzorr the Great\n\r\n\r%s\n\r", 
             NAME(pActor), PC(pActor,history) );
    
    page_to_actor( arg, ch );
    return;
}          


void add_history( PLAYER_DATA *ch, char *_out )
    {
    char buf[MAX_STRING_LENGTH];

    if ( !IS_NPC(ch) ) {
    sprintf( buf, "%s\n\r%s", PC(ch,history),  _out );
    free_string( PC(ch,history) );
    PC(ch,history) = str_dup( buf );
    
    }
    return;
}

