/******************************************************************************
 * Locke's   __                      __         NIM Server Software           *
 * ___ ___  (__)__    __ __   __ ___|  | v5     Version 5 (ALPHA)             *
 * |  /   \  __|  \__/  |  | |  |      |        unreleased+revamped 2004      *
 * |       ||  |        |  \_|  | ()   |                                      *
 * |    |  ||  |  |__|  |       |      |                                      *
 * |____|__|___|__|  |__|\___/__|______|        http://www.nimud.org/nimud    *
 *   n a m e l e s s i n c a r n a t e          dedicated to chris cool       *
 ******************************************************************************/

#if defined(BSD)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "nimud.h"
#include "defaults.h"



void update_money( PROP_DATA *prop )
{
    char buf[MAX_STRING_LENGTH];
    char buf1[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    char buf3[MAX_STRING_LENGTH];

    if ( prop->item_type != ITEM_MONEY )
    {
        bug( "update_money: item is not of type money.", 0 );
        return;
    }

    if ( prop->value[1] >= MAX_COIN ) prop->value[1] = MAX_COIN-1;
    if ( prop->value[1] < 0 ) prop->value[1] = 0;

    if ( prop->value[0] > 1  )
    {
    prop->pIndexData = get_prop_index( PROP_VNUM_MONEY_SOME );
    if ( prop->value[0] <= 10 )       strcpy( buf, "few"               ); else
    if ( prop->value[0] <= 100 )      strcpy( buf, "small pile of"     ); else
    if ( prop->value[0] <= 1000 )     strcpy( buf, "pile of"           ); else
    if ( prop->value[0] <= 10000 )    strcpy( buf, "large pile of"     ); else
    if ( prop->value[0] <= 100000 )   strcpy( buf, "heaping mound of"  ); else
    if ( prop->value[0] <= 1000000 )  strcpy( buf, "small hill of"     ); else
    if ( prop->value[0] <= 10000000 ) strcpy( buf, "mountain of"       );
                                else strcpy( buf, "whole shitload of" );

    sprintf( buf1, prop->pIndexData->short_descr, numberize( prop->value[0] ),
                   coin_table[prop->value[1]].long_name );
    sprintf( buf2, prop->pIndexData->description, buf,
                   coin_table[prop->value[1]].long_name );
    sprintf( buf3, prop->pIndexData->name, coin_table[prop->value[1]].long_name );
    free_string( prop->name );        prop->name        = str_dup( buf3 );
    free_string( prop->description ); prop->description = str_dup( buf2 );
    free_string( prop->short_descr ); prop->short_descr = str_dup( buf1 );
    prop->weight         = (prop->value[0] * coin_table[prop->value[1]].weight) / 20;
    prop->cost           = prop->value[0] * coin_table[prop->value[1]].convert;
    }
    else
    if ( prop->value[0] == 1 )
    {
    prop->pIndexData = get_prop_index( PROP_VNUM_MONEY_ONE );
    sprintf( buf1, prop->pIndexData->short_descr,
                   coin_table[prop->value[1]].long_name );
    sprintf( buf2, prop->pIndexData->description,
                   coin_table[prop->value[1]].long_name );
    sprintf( buf3, prop->pIndexData->name, coin_table[prop->value[1]].long_name);
    free_string( prop->name );           prop->name        = str_dup( buf3 );
    free_string( prop->description );    prop->description = str_dup( buf2 );
    free_string( prop->short_descr );    prop->short_descr = str_dup( buf1 );
    prop->weight         = coin_table[prop->value[1]].weight;
    prop->cost           = coin_table[prop->value[1]].convert;
    }

    return;
}

PROP_DATA *create_money( int amount, int type )
{
    PROP_DATA *prop;

    if ( amount <= 0 )
    {
    bug( "Create_money: zero or negative money %d.", amount );
    amount = 1;
    }

    if ( amount == 1 )
    {
    prop = create_prop( get_prop_index( PROP_VNUM_MONEY_ONE ), 0 );
    prop->item_type      = ITEM_MONEY;
    prop->value[0]       = 1;
    prop->value[1]       = type;
    prop->wear_loc       = -1;
    prop->weight         = coin_table[type].weight;
    prop->cost           = coin_table[type].convert;
    update_money( prop );
    }
    else
    {
	prop = create_prop( get_prop_index( PROP_VNUM_MONEY_SOME ), 0 );
    prop->value[0]       = amount;
    prop->value[1]       = type;
    prop->item_type      = ITEM_MONEY;
    prop->wear_loc       = -1;
    prop->weight         = amount * coin_table[type].weight / 16;
    prop->cost           = amount * coin_table[type].convert;
    update_money( prop );
    }

    return prop;
}

/*
 * Conglomerate several money items.
 */
void merge_money( PLAYER_DATA *ch )
{
    PROP_DATA *prop;
    PROP_DATA *prop_next;
    int sub_totals[MAX_COIN];
    int type;

    for ( type = 0; type < MAX_COIN; type++ )
    {
        sub_totals[type] = tally_one_coin( ch->carrying, type, FALSE );
    }

    for ( prop = ch->carrying;  prop != NULL;  prop = prop_next )
    {
        prop_next = prop->next_content;
        if ( prop->item_type == ITEM_MONEY )
        {
            prop_from_char( prop );
            extract_prop( prop );
        }
    }

    for ( type = 0; type < MAX_COIN; type++ )
    {
        if ( sub_totals[type] > 0 )
        {
            prop_to_char( (prop = create_money( sub_totals[type], type )), ch );
            prop->wear_loc = hand_empty( ch );
        }
    }
    return;
}

void merge_money_prop( PROP_DATA *prop )
{
    PROP_DATA *iprop;
    PROP_DATA *prop_next;
    int sub_totals[MAX_COIN];
    int type;

    for ( type = 0; type < MAX_COIN; type++ )
    {
        sub_totals[type] = tally_one_coin( prop->contains, type, FALSE );
    }

    for ( iprop = prop->contains;  iprop != NULL;  iprop = prop_next )
    {
        prop_next = iprop->next_content;
        if ( iprop->item_type == ITEM_MONEY )
        {
            prop_from_prop( iprop );
            extract_prop( iprop );
        }
    }

    for ( type = 0; type < MAX_COIN; type++ )
    {
        if ( sub_totals[type] > 0 )
          prop_to_prop( create_money( sub_totals[type], type ), prop );
    }
    return;
}


void merge_money_scene( SCENE_INDEX_DATA *scene )
{
    PROP_DATA  *iprop;
    PROP_DATA *prop_next;
    int sub_totals[MAX_COIN];
    int type;

    for ( type = 0; type < MAX_COIN; type++ )
    {
        sub_totals[type] = tally_one_coin( scene->contents, type, FALSE );
    }

    for ( iprop = scene->contents;  iprop != NULL;  iprop = prop_next )
    {
        prop_next = iprop->next_content;
        if ( iprop->item_type == ITEM_MONEY )
        {
            prop_from_scene( iprop );
            extract_prop( iprop );
        }
    }

    for ( type = 0; type < MAX_COIN; type++ )
    {
        if ( sub_totals[type] > 0 )
          prop_to_scene( create_money( sub_totals[type], type ), scene );
    }
    return;
}

/*
 * Create a set of 'money' props.
 */
void create_amount( int amount, PLAYER_DATA *pActor, SCENE_INDEX_DATA *pScene,
                    PROP_DATA *pProp )
{
    PROP_DATA *prop;
    int type;

    if ( amount <= 0 )
    return;

    if ( amount == 1 )
    {
    prop = create_money( 1, MAX_COIN-1 );
    if ( pActor ) prop_to_char_money( prop, pActor );
    if ( pScene ) prop_to_scene( prop, pScene );
    if ( pProp ) prop_to_prop( prop, pProp );
    }
    else
    if ( amount < 30 && (amount % 10 != 0) )
    {
    prop = create_money( amount, MAX_COIN-1 );
    if ( pActor ) prop_to_char_money( prop, pActor );
    if ( pScene ) prop_to_scene( prop, pScene );
    if ( pProp ) prop_to_prop( prop, pProp );
    }
    else
    {
        for ( type = 0; type < MAX_COIN; type++ )
        {

        if ( amount/coin_table[type].convert <= 0 )
            continue;

        prop = create_money( amount/coin_table[type].convert, type );
        amount %= coin_table[type].convert;

        if ( pActor ) prop_to_char_money( prop, pActor );
        if ( pScene ) prop_to_scene( prop, pScene );
        if ( pProp ) prop_to_prop( prop, pProp );
        }
    }

    if ( pActor ) merge_money( pActor );
    if ( pProp ) merge_money_prop( pProp );
    return;
}


/*
 * Tally the number of coins a character has.
 */
int tally_coins( PLAYER_DATA *actor )
{
    PROP_DATA *prop;
    int total = 0;

    for ( prop = actor->carrying;  prop != NULL; prop = prop->next_content )
    {
        if ( prop->item_type == ITEM_MONEY
          && prop->value[1] >= 0
          && prop->value[1] < MAX_COIN )
        total += prop->value[0] * coin_table[prop->value[1]].convert;

        if ( ( prop->value[1] < 0 || prop->value[1] >= MAX_COIN ) 
          && prop->item_type == ITEM_MONEY )
        {
        bug( "Tally_coins: prop (vnum %d) bad coinage",
             prop->pIndexData->vnum );
        prop->value[1] = URANGE( 0, prop->value[1], MAX_COIN-1 );
        }

        if ( prop->item_type == ITEM_CONTAINER )
        {
            PROP_DATA *iprop;

            for ( iprop = prop->contains; iprop != NULL; iprop = iprop->next_content )
            {
                if ( iprop->item_type == ITEM_MONEY
                  && iprop->value[1] >= 0
                  && iprop->value[1] < MAX_COIN )
                total += iprop->value[0] * coin_table[iprop->value[1]].convert;

                if ( ( iprop->value[1] < 0 || iprop->value[1] >= MAX_COIN )
                  && iprop->item_type == ITEM_MONEY )
                {
                bug( "Tally_coins: iprop (vnum %d) bad coinage",
                     iprop->pIndexData->vnum );
                iprop->value[1] = URANGE( 0, iprop->value[1], MAX_COIN-1 );
                }
            }
        }
    }

    return total;
}

/*
 * Tally the number of A SINGLE TYPE OF coin a character has.
 */
int tally_one_coin( PROP_DATA *prop_list, int type, bool fContents )
{
    PROP_DATA *prop;
    int total = 0;

    for ( prop = prop_list;  prop != NULL; prop = prop->next_content )
    {
        if ( prop->item_type == ITEM_MONEY
          && prop->value[1] >= 0
          && prop->value[1] < MAX_COIN
          && prop->value[1] == type )
        total += prop->value[0];

        if ( ( prop->value[1] < 0 || prop->value[1] >= MAX_COIN ) 
           && prop->item_type == ITEM_MONEY )
        {
        bug( "Tally_one_coin: prop (vnum %d) bad coinage",
             prop->pIndexData->vnum );
        prop->value[1] = URANGE( 0, prop->value[1], MAX_COIN-1 );
        }

        if ( prop->item_type == ITEM_CONTAINER
           && fContents )
        {
            PROP_DATA *iprop;

            for ( iprop = prop->contains; iprop != NULL;
                  iprop = iprop->next_content )
            {
                if ( iprop->item_type == ITEM_MONEY
                  && iprop->value[1] >= 0
                  && iprop->value[1] < MAX_COIN
                  && iprop->value[1] == type )
                total += iprop->value[0];

                if ( ( iprop->value[1] < 0 || iprop->value[1] >= MAX_COIN )
                  && iprop->item_type == ITEM_MONEY )
                {
                bug( "Tally_one_coin: iprop (vnum %d) bad coinage",
                     iprop->pIndexData->vnum );
                iprop->value[1] = URANGE( 0, iprop->value[1], MAX_COIN-1 );
                }
            }
        }
    }

    return total;
}


/*
 * Create a cost out of the blue.
 */
char *name_amount( int amount )
{
    static char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    int type;

    if ( amount < 20 && amount != 0 )
    {
    sprintf( buf, "%d copper", amount );
    return buf;
    }

    buf[0] = '\0';

    for ( type = 0; ; type++ )
    {
    if (type >= MAX_COIN || amount == 0 )
        break;

    if ( amount/coin_table[type].convert <= 0 )
        continue;

    sprintf( buf2, "%s%s %s", buf[0] != '\0' ? ", " : "",
                              numberize( (amount/coin_table[type].convert) ),
                              coin_table[type].long_name );
    strcat( buf, buf2 );
    amount %= coin_table[type].convert;
    }
    if ( buf[0] == '\0' ) sprintf( buf, "nothing" );

    return buf;
}

/*
 * Nail unused money items.
 */
void strip_empty_money( PLAYER_DATA *ch )
{
    PROP_DATA *prop;
    PROP_DATA *prop_next;
    PROP_DATA *iprop;
    PROP_DATA *iprop_next;

    for ( prop = ch->carrying;  prop != NULL;  prop = prop_next )
    {
        prop_next = prop->next_content;
        if ( prop->item_type == ITEM_MONEY  && prop->value[0] <= 0 )
        {
            prop_from_char( prop );
            extract_prop( prop );
        }

        for ( iprop = prop->contains;  iprop != NULL; iprop = iprop_next )
        {
        iprop_next = iprop->next_content;
        if ( iprop->item_type == ITEM_MONEY  && iprop->value[0] <= 0 )
        {
            prop_from_prop( iprop );
            extract_prop( iprop );
        }
        }
    }
    return;
}
     

/*
 * Ok, here's the tricky part... managing two money totals at once,
 * this is the routine that makes the change if you give the dude
 * too much.  This is for shops mainly, but can be used elsewhere.
 */
char *sub_coins( int amount, PLAYER_DATA *ch )
{
    PROP_DATA *prop;
    PROP_DATA *iprop;
    int money = 0;

    if ( tally_coins( ch ) < amount )
    {
        bug( "Sub_coins: ch doesn't have enough money (%d)", amount );
        return "nothing";
    }

    for ( prop = ch->carrying;  prop != NULL;  prop = prop->next_content )
    {
        if ( money >= amount ) break;
        if ( prop->item_type == ITEM_MONEY )
        {
            while ( prop->value[0] > 0 && money < amount )
            {
                 money += coin_table[prop->value[1]].convert;
                 prop->value[0]--;
            }
            if ( prop->value[0] > 0 ) update_money( prop );
        }

        for ( iprop = prop->contains;  iprop != NULL;  iprop = iprop->next_content )
        {
        if ( money >= amount ) break;
        if ( iprop->item_type == ITEM_MONEY )
        {
            while ( iprop->value[0] > 0 && money < amount )
            {
                 money += coin_table[iprop->value[1]].convert;
                 iprop->value[0]--;
            }
            if ( iprop->value[0] > 0 ) update_money( iprop );
        }
        }
    }

    strip_empty_money( ch );
    if ( money > amount ) create_amount( money - amount, ch, NULL, NULL );
    return name_amount( money - amount );
}



void cmd_money( PLAYER_DATA *ch, char *argument )
{
     send_to_char( "You have ", ch );
     send_to_char( name_amount( tally_coins(ch) ), ch );
     send_to_char( ".\n\r", ch );
     return;
}
