# vi: ts=8 sw=8 sts=8

package Tongues::Espanol;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $charset $grammar %dictionary);

use Exporter   ();
# set the version for version checking
@ISA         = qw(Exporter);
@EXPORT      = qw(charset grammar dictionary);
@EXPORT_OK   = qw();
$VERSION     = 0.00;

# Character set Windows Western European CP1252 / ISO 8859-1 / Latin-1

# Old alphabetical order
# A B C CH D E F G H I J K L LL M N  O P Q R S T U V W X Y Z
# a b c ch d e f g h i j k l ll m n  o p q r s t u v w x y z
# New alphabetical order
# A B C D E F G H I J K L M N  O P Q R S T U V W X Y Z
# a b c d e f g h i j k l m n  o p q r s t u v w x y z

sub new {
	my $class = shift;
	my $self = bless {}, $class;
	return $self;
}

sub charset {
	return $charset;
}

sub grammar {
	return $grammar;
}

sub dictionary {
	return \%dictionary;
}

sub BEGIN {

require Charsets::WestEuroWin;

$charset = new Charsets::WestEuroWin;

# Table for converting words to root form
$grammar = [
 # Noun gender
 [ 'a',		'o',	'',		'',	'n' ],
 [ 'a',		'',	'',		'',	'n' ],
 # Noun plural
 [ 's',		'',	'',		's',	'n' ],	# 1
 [ 'es',	'',	'',		's',	'n' ],	# 2
 [ 'es',	'',	'',		's',	'n' ],	# 2b
 [ 'es',	'',	'',		's',	'n' ],
 [ 'es',	'',	'',		's',	'n' ],
 [ 'es',	'',	'',		's',	'n' ],
 [ 'es',	'',	'',		's',	'n' ],
 [ 'anes',	'n',	'',		's',	'n' ],	# 3
 [ 'enes',	'n',	'',		's',	'n' ],
 [ 'ines',	'n',	'',		's',	'n' ],
 [ 'ones',	'n',	'',		's',	'n' ],
 [ 'unes',	'n',	'',		's',	'n' ],
 [ 'ases',	's',	'',		's',	'n' ],
 [ 'eses',	's',	'',		's',	'n' ],
 [ 'ises',	's',	'',		's',	'n' ],
 [ 'oses',	's',	'',		's',	'n' ],
 [ 'uses',	's',	'',		's',	'n' ],
 [ 'ces',	'z',	'',		's',	'n' ],	# 5
 # Adjectives
 [ 'a',		'o',	'',		'',	'a' ],
 [ 'os',	'o',	'',		'',	'a' ],
 [ 'as',	'o',	'',		'',	'a' ],
 [ 's',		'',	'',		'',	'a' ],
 [ 'es',	'',	'',		'',	'a' ],
 [ 'simo',	'o',	'very ',	'',	'a' ],
 [ 'sima',	'o',	'very ',	'',	'a' ],
 [ 'simos',	'o',	'very ',	'',	'a' ],
 [ 'simas',	'o',	'very ',	'',	'a' ],
 # Verb past participle
 [ 'ado',	'ar',	'',		'ed',	'v' ],
 [ 'ada',	'ar',	'',		'ed',	'v' ],
 [ 'ados',	'ar',	'',		'ed',	'v' ],
 [ 'adas',	'ar',	'',		'ed',	'v' ],
 [ 'ido',	'er',	'',		'ed',	'v' ],
 [ 'do',	'er',	'',		'ed',	'v' ],
 [ 'ida',	'er',	'',		'ed',	'v' ],
 [ 'idos',	'er',	'',		'ed',	'v' ],
 [ 'idas',	'er',	'',		'ed',	'v' ],
 [ 'ido',	'ir',	'',		'ed',	'v' ],
 [ 'ida',	'ir',	'',		'ed',	'v' ],
 [ 'idos',	'ir',	'',		'ed',	'v' ],
 [ 'idas',	'ir',	'',		'ed',	'v' ],
 # Verb gerund
 [ 'ando',	'ar',	'',		'ing',	'v' ],
 [ 'iendo',	'er',	'',		'ing',	'v' ],
 [ 'iendo',	'ir',	'',		'ing',	'v' ],
 # Verb present participle
 [ 'ante',	'ar',	'',		'ing',	'v' ],
 [ 'iente',	'er',	'',		'ing',	'v' ],
 [ 'iente',	'ir',	'',		'ing',	'v' ],
 # Verb imperitive
 [ 'ad',	'ar',	'',		'',	'v' ],	# 2pp
 [ 'ed',	'er',	'',		'',	'v' ],	# 2pp
 [ 'id',	'ir',	'',		'',	'v' ],	# 2pp
 # Verb present
 [ 'o',		'ar',	'(i) ',		'',	'v' ],
 [ 'o',		'er',	'(i) ',		'',	'v' ],
 [ 'o',		'ir',	'(i) ',		'',	'v' ],
 [ 'as',	'ar',	'(you) ',	'',	'v' ],
 [ 'es',	'er',	'(you) ',	'',	'v' ],
 [ 'es',	'ir',	'(you) ',	'',	'v' ],
 [ 'a',		'ar',	'(it) ',	's',	'v' ],
 [ 'e',		'er',	'(it) ',	's',	'v' ],
 [ 'e',		'ir',	'(it) ',	's',	'v' ],
 [ 'amos',	'ar',	'(we) ',	'',	'v' ],	# also pret.
 [ 'emos',	'er',	'(we) ',	'',	'v' ],
 [ 'imos',	'ir',	'(we) ',	'',	'v' ],	# also pret.
 [ 'is',	'ar',	'(you) ',	'',	'v' ],	# castillian
 [ 'is',	'er',	'(you) ',	'',	'v' ],	# castillian
 [ 's',	'ir',	'(you) ',	'',	'v' ],	# castillian
 [ 'an',	'ar',	'(they) ',	'',	'v' ],
 [ 'en',	'er',	'(they) ',	'',	'v' ],
 [ 'en',	'ir',	'(they) ',	'',	'v' ],
 # Verb present subjunctive, imperitive
 [ 'es',	'ar',	'(you) ',	'',	'v' ],
 [ 'as',	'er',	'(you) ',	'',	'v' ],
 [ 'as',	'ir',	'(you) ',	'',	'v' ],
 [ 'e',		'ar',	'(it) ',	'(s)',	'v' ],	# i
 [ 'a',		'er',	'(it) ',	'(s)',	'v' ],	# i
 [ 'a',		'ir',	'(it) ',	'(s)',	'v' ],	# i
 [ 'emos',	'ar',	'(we) ',	'',	'v' ],
 [ 'amos',	'er',	'(we) ',	'',	'v' ],
 [ 'amos',	'ir',	'(we) ',	'',	'v' ],
 [ 'is',	'ar',	'(you) ',	'',	'v' ],	# castillian
 [ 'is',	'er',	'(you) ',	'',	'v' ],	# castillian
 [ 'is',	'ir',	'(you) ',	'',	'v' ],	# castillian
 [ 'en',	'ar',	'(they) ',	'',	'v' ],
 [ 'an',	'er',	'(they) ',	'',	'v' ],
 [ 'an',	'ir',	'(they) ',	'',	'v' ],
 # Verb preterite
 [ '',		'ar',	'(i) ',		'ed',	'v' ],
 [ '',		'er',	'(i) ',		'ed',	'v' ],
 [ '',		'ir',	'(i) ',		'ed',	'v' ],
 [ 'aste',	'ar',	'(you) ',	'ed',	'v' ],
 [ 'iste',	'er',	'(you) ',	'ed',	'v' ],
 [ 'iste',	'ir',	'(you) ',	'ed',	'v' ],
 [ '',		'ar',	'(it) ',	'ed',	'v' ],
 [ 'i',	'er',	'(it) ',	'ed',	'v' ],
 [ 'i',	'ir',	'(it) ',	'ed',	'v' ],
 #[ 'amos',	'ar',	'(we) ',	'ed',	'v' ],	# also pres.
 [ 'imos',	'er',	'(we) ',	'ed',	'v' ],
 #[ 'imos',	'ir',	'(we) ',	'ed',	'v' ],	# also pres.
 [ 'asteis',	'ar',	'(you) ',	'ed',	'v' ],	# castillian
 [ 'isteis',	'er',	'(you) ',	'ed',	'v' ],	# castillian
 [ 'isteis',	'ir',	'(you) ',	'ed',	'v' ],	# castillian
 [ 'aron',	'ar',	'(they) ',	'ed',	'v' ],
 [ 'ieron',	'er',	'(they) ',	'ed',	'v' ],
 [ 'ieron',	'ir',	'(they) ',	'ed',	'v' ],
 # Verb imperfect
 [ 'aba',	'ar',	'(it) ',	'ed',	'v' ],	# i
 [ 'a',	'er',	'(it) ',	'ed',	'v' ],	# i
 [ 'a',	'ir',	'(it) ',	'ed',	'v' ],	# i
 [ 'abas',	'ar',	'(you) ',	'ed',	'v' ],
 [ 'as',	'er',	'(you) ',	'ed',	'v' ],
 [ 'as',	'ir',	'(you) ',	'ed',	'v' ],
 [ 'bamos',	'ar',	'(we) ',	'ed',	'v' ],
 [ 'amos',	'er',	'(we) ',	'ed',	'v' ],
 [ 'amos',	'ir',	'(we) ',	'ed',	'v' ],
 [ 'abais',	'ar',	'(you) ',	'ed',	'v' ],	# castillian
 [ 'ais',	'er',	'(you) ',	'ed',	'v' ],	# castillian
 [ 'ais',	'ir',	'(you) ',	'ed',	'v' ],	# castillian
 [ 'aban',	'ar',	'(they) ',	'ed',	'v' ],
 [ 'an',	'er',	'(they) ',	'ed',	'v' ],
 [ 'an',	'ir',	'(they) ',	'ed',	'v' ],
 # Verb future
 [ '',		'',	'(i) will ',	'',	'v' ],
 [ 's',	'',	'(you) will ',	'',	'v' ],
 [ '',		'',	'(it) will ',	'',	'v' ],
 [ 'emos',	'',	'(we) will ',	'',	'v' ],
 [ 'is',	'',	'(you) will ',	'',	'v' ],	# castillian
 [ 'n',	'',	'(they) will ',	'',	'v' ],
 # Verb conditional
 [ 'a',	'',	'(it) would ',		'',	'v' ],	# i
 [ 'as',	'',	'(you) would ',		'',	'v' ],
 [ 'amos',	'',	'(we) would ',		'',	'v' ],
 [ 'ais',	'',	'(you) would ',		'',	'v' ],	# castillian
 [ 'an',	'',	'(they) would ',	'',	'v' ],
 # Verb imperfect subjunctive
 [ 'ara',	'ar',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'ase',	'ar',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'aras',	'ar',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ases',	'ar',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ramos',	'ar',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'semos',	'ar',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'arais',	'ar',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'aseis',	'ar',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'aran',	'ar',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'asen',	'ar',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'iera',	'er',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'iese',	'er',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'ieras',	'er',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieses',	'er',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'iramos',	'er',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'isemos',	'er',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'ierais',	'er',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieseis',	'er',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieran',	'er',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'iesen',	'er',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'iera',	'ir',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'iese',	'ir',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'ieras',	'ir',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieses',	'ir',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'iramos',	'ir',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'isemos',	'ir',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'ierais',	'ir',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieseis',	'ir',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieran',	'ir',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'iesen',	'ir',	'(they) would\'ve ',	'ed',	'v' ],
 #  (For irregular verbs, derived from 3pp pret.)
 [ 'ara',	'aron',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'ase',	'aron',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'aras',	'aron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ases',	'aron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ramos',	'aron',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'semos',	'aron',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'arais',	'aron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'aseis',	'aron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'aran',	'aron',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'asen',	'aron',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'iera',	'ieron',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'iese',	'ieron',	'(he) would\'ve ',	'ed',	'v' ],	# i
 [ 'ieras',	'ieron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieses',	'ieron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'iramos',	'ieron',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'isemos',	'ieron',	'(we) would\'ve ', 	'ed',	'v' ],
 [ 'ierais',	'ieron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieseis',	'ieron',	'(you) would\'ve ',	'ed',	'v' ],
 [ 'ieran',	'ieron',	'(they) would\'ve ',	'ed',	'v' ],
 [ 'iesen',	'ieron',	'(they) would\'ve ',	'ed',	'v' ],
 # Adverb
 [ 'mente',	'',	'',		'ly',	'a' ],
 [ 'amente',	'o',	'',		'ly',	'a' ],
 # Appended pronouns (infinitive)
 [ 'lo',	'',	'',		' it',		'v' ],
 [ 'los',	'',	'',		' them',	'v' ],
 [ 'le',	'',	'',		' it',		'v' ],
 [ 'les',	'',	'',		' them',	'v' ],
 [ 'la',	'',	'',		' it',		'v' ],
 [ 'las',	'',	'',		' them',	'v' ],
 [ 'me',	'',	'',		' me',		'v' ],
 [ 'te',	'',	'',		' you',		'v' ],
 [ 'se',	'',	'',		' (self)',	'v' ],
 # Appended pronouns (imperative)
 [ 'alo',	'ar',	'',		' it',		'v' ],
 [ 'alos',	'ar',	'',		' them',	'v' ],
 [ 'ale',	'ar',	'',		' it',		'v' ],
 [ 'ales',	'ar',	'',		' them',	'v' ],
 [ 'ala',	'ar',	'',		' it',		'v' ],
 [ 'alas',	'ar',	'',		' them',	'v' ],
 [ 'ame',	'ar',	'',		' me',		'v' ],
 [ 'ate',	'ar',	'',		' you',		'v' ],
 [ 'ase',	'ar',	'',		' (self)',	'v' ],
 #
 [ 'elo',	'er',	'',		' it',		'v' ],
 [ 'elos',	'er',	'',		' them',	'v' ],
 [ 'ele',	'er',	'',		' it',		'v' ],
 [ 'eles',	'er',	'',		' them',	'v' ],
 [ 'ela',	'er',	'',		' it',		'v' ],
 [ 'elas',	'er',	'',		' them',	'v' ],
 [ 'eme',	'er',	'',		' me',		'v' ],
 [ 'ete',	'er',	'',		' you',		'v' ],
 [ 'ese',	'er',	'',		' (self)',	'v' ],
 #
 [ 'elo',	'ir',	'',		' it',		'v' ],
 [ 'elos',	'ir',	'',		' them',	'v' ],
 [ 'ele',	'ir',	'',		' it',		'v' ],
 [ 'eles',	'ir',	'',		' them',	'v' ],
 [ 'ela',	'ir',	'',		' it',		'v' ],
 [ 'elas',	'ir',	'',		' them',	'v' ],
 [ 'eme',	'ir',	'',		' me',		'v' ],
 [ 'ete',	'ir',	'',		' you',		'v' ],
 [ 'ese',	'ir',	'',		' (self)',	'v' ],

 # Numbers
 [ 'ids',	'e',	'',		' two' ],
 [ 'isiete',	'e',	'',		' seven' ],

 # Higher level transformations
 # Noun to noun
 [ 'ito',	'o',	'little ',	'',	'n' ],	# dimin.
 [ 'ita',	'a',	'little ',	'',	'n' ],	# dimin. fem.
 [ 'guito',	'go',	'little ',	'',	'n' ],	# dimin.
 [ 'guita',	'ga',	'little ',	'',	'n' ],	# dimin. fem.
 # Noun to verb
 [ 'ador',	'ar',	'',		'er',	'v' ],
 [ 'edor',	'er',	'',		'er',	'v' ],
 [ 'idor',	'ir',	'',		'er',	'v' ],
 # Verb to noun
 [ 'acin',	'ar',	'',		'ing',	'v' ],
];

#Fields:
# x -> translation		all words
# r -> root			irregular declensions
# t -> type:			all words
#	n -> noun
#	v -> verb
#	a -> adjective
#	p -> preposition
#	art -> article
#	pro -> pronoun
#	conj -> conjunction
#	adv -> adverb
# g -> gender:			nouns, pronouns, adjectives
#	m -> masculine		default
#	f -> feminine		default for words ending in a
# n -> number:			articles, pronouns, verbs, adjectives
#	s -> singular		default
#	p -> plural		default for words ending in s
# p -> person:			pronouns, verbs
#	1 -> 1st person
#	2 -> 2nd person
#	3 -> 3rd person		default
# c -> conjugation (verbs):
#	p -> present
#	i -> imperfect
#	f -> future
#	c -> conditional
#	t -> preterite
#	ps -> present subjunctive
#	is -> imperfect subjunctive
#	inf -> infinitive		default
#	pp -> past participle
#	g -> gerund
#	imp -> imperative
# c -> conjugation (pronouns):
#	s -> subject
#	do -> direct object
#	io -> indirect object

# Spanish to English dictionary
%dictionary = (
 # Functional words
 #  Articles
 #   Definite articles
 #    Masculine
 #     Singular
 'el'	=> { 'x' => 'the',
	     '#' => 'he',
             't' => 'art' },
 #    Feminine
 #     Singular
 'la'	=> { 'x' => 'the',
	     't' => 'art',
	     'g' => 'f' },
 #     Plural
 'las'	=> { 'x' => 'the',
	     't' => 'art',
	     'g' => 'f',
	     'n' => 'p' },
 'lo'	=> { 'x' => '(the)',
	     '#' => 'it' },
 #   Indefinite articles
 'los'	=> { 'x' => 'the',
	     't' => 'art',
	     'n' => 'p' },
 #     Plural
 'un'	=> { 'x' => 'a',
	     '#' => 'an, one',
	     't' => 'art',
	     'g' => 'm' },
 'una'	=> { 'x' => 'a',
	     '#' => 'an, one',
	     't' => 'art',
	     'g' => 'f' },
 'uno'	=> { 'x' => 'one',
	     '#' => 'a, an',
 	     'g' => 'm' },
 'unas'	=> { 'x' => 'some',
 	     'g' => 'f' },
 'unos'	=> { 'x' => 'some',
 	     'g' => 'm' },

 #  Pronouns & possessive adjectives
 #   1st person
 'yo'		=> { 'x' => 'i',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'me'		=> { 'x' => 'me',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'm'		=> { 'x' => 'me',
		     '#' => 'pronoun',
		     't' => 'pro',
		     'p' => '1',
		     'n' => 's' },
 'mi'		=> { 'x' => 'my',
		     '#' => 'poss. adjective',
		     't' => 'a' },
 'mo'		=> { 'x' => 'mine',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'g' => 'm' },
 'ma'		=> { 'x' => 'mine',
 		     't' => 'pro',
		     'p' => '1',
		     'n' => 's',
		     'g' => 'f' },
 'nosotros'	=> { 'x' => 'we' },
 'nos'		=> { 'x' => 'us',
		     '#' => '(also reflexive)' },
 'nuestro'	=> { 'x' => 'our' },
 #   2nd person
 't'		=> { 'x' => 'you',
		     '#' => 'familiar',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 's' },
 'tu'		=> { 'x' => 'your',
		     't' => 'a' },
 'te'		=> { 'x' => 'you',
		     't' => 'pro' },
 'ti'		=> { 'x' => 'you',
		     '#' => 'used after prepositions only',
		     't' => 'pro' },
 'tuyo'		=> { 'x' => 'yours',
		     '#' => 'possessive adjective (yours)' },
 'vosotros'	=> { 'x' => 'you',
 		     't' => 'pro',
		     'p' => '2',
		     'n' => 'p' },
 #   3rd person
 'usted'	=> { 'x' => 'you',
		     '#' => 'formal',
		     't' => 'pro',
		     'p' => '3',
		     '#' => 'grammatical 3rd, logical 2nd',
		     'n' => 's' },
 'ustedes'	=> { 'x' => 'you',
		     '#' => 'formal',
		     't' => 'pro',
		     'p' => '3',
		     '#' => 'grammatical 3rd, logical 2nd',
		     'n' => 'p' },
 'l'		=> { 'x' => 'he',
		     '#' => 'after prep. him',
 		     't' => 'pro' },
 'le'		=> { 'x' => 'him/her',
 		     't' => 'pro' },
 'les'		=> { 'x' => 'them' },
 'ella'		=> { 'x' => 'her',
		     '#' => 'she, it',
 		     't' => 'pro' },
 'ello'		=> { 'x' => 'it',
		     't' => 'pro' },
 'ellos'	=> { 'x' => 'they',
		     '#' => 'them, theirs',
		     't' => 'pro',
		     'g' => 'm',
		     'n' => 'p' },
 'ellas'	=> { 'x' => 'they',
		     '#' => 'them, theirs',
		     't' => 'pro',
		     'g' => 'f',
		     'n' => 'p' },
 'se'		=> { 'x' => '(self)',
		     '#' => 'refl. pron. (no translation)',
		     't' => 'pro' },
 'su'		=> { 'x' => 'its',
		     '#' => 'poss. adj. his, her, their' },
 'sus'		=> { 'x' => 'its',
		     '#' => 'poss. adj. his, her, their',
		     'n' => 'p' },
 'suyo'		=> { 'x' => 'theirs',
		     '#' => 'his, hers' },

 #   Indefinite pronouns
 'algo'		=> { 'x' => 'something',
		     '#' => 'anything',
		     't' => 'n' },
 'alguien'	=> { 'x' => 'anybody',
		     '#' => 'somebody',
		     't' => 'n',
		     '#' => 'noun or pronoun?' },
 'nada'		=> { 'x' => 'nothing',
		     '#' => 'pron.?' },
 'nadie'	=> { 'x' => 'nobody',
		     '#' => 'pronoun?',
 		     't' => 'n' },
 'ningn'	=> { 'x' => 'no',
		     '#' => 'nodoby, none' },
 'ninguno'	=> { 'x' => 'no',
		     '#' => 'nodoby, none' },

 #  Interrogatives
 'cmo'		=> { 'x' => 'how',
		     '#' => 'interr. why',
 		     't' => 'adv' },
 'cual'		=> { 'x' => 'which',
		     '#' => 'interr. and adv.',
 		     't' => 'pro',
		     'n' => 's' },
 'cuales'	=> { 'x' => 'which',
		     '#' => 'interr. and adv.',
 		     't' => 'pro',
		     'n' => 'p' },
 'cul'		=> { 'x' => 'which',
		     '#' => 'interr.',
 		     't' => 'pro' },
 'cundo'	=> { 'x' => 'when',
		     '#' => 'interr.',
 		     't' => 'adv' },
 'cunto'	=> { 'x' => 'how much',
		     '#' => 'interr.',
 		     't' => 'a' },
 'dnde'	=> { 'x' => 'where',
		     '#' => 'interr.',
 		     't' => 'adv' },
 'qu'		=> { 'x' => 'what',
		     '#' => 'interr. pronoun',
 		     't' => 'pro' },
 'quin'	=> { 'x' => 'who',
		     '#' => 'interr. pronoun',
 		     't' => 'pro',
		     'n' => 's' },
 'quines'	=> { 'x' => 'who',
		     '#' => 'interr. pronoun',
 		     't' => 'pro',
		     'n' => 'p' },

 #  Other functional words
 'a'		=> { 'x' => '(to)',
		     '#' => 'at, (often not translated)',
		     't' => 'p' },
 'ac'		=> { 'x' => 'here',
		     '#' => 'adv. of place; cf aqui',
 		     't' => 'adv' },
 'adems'	=> { 'x' => 'besides' },
 'adis'	=> { 'x' => 'goodbye',
 		     't' => 'interj' },
 'afuera'	=> { 'x' => 'outside',
		     '#' => 'adv.?' },
 'ah'		=> { 'x' => 'there',
		     '#' => 'adverb of place',
 		     't' => 'adv' },
 'ahora'	=> { 'x' => 'now',
		     '#' => 'temporal adverb',
 		     't' => 'adv' },
 'algn'	=> { 'x' => 'some',
		     '#' => 'any' },
 'alguno'	=> { 'x' => 'some',
		     '#' => 'any' },
 'all'		=> { 'x' => '(over) there',
		     '#' => 'adverb of place',
 		     't' => 'adv' },
 'all'		=> { 'x' => '(over) there',
		     '#' => 'adverb of place',
 		     't' => 'adv' },
 'alrededor'	=> { 'x' => 'around',
 		     't' => 'p' },
 'ambos'	=> { 'x' => 'both',
		     't' => 'a',
		     'n' => 'p' },
 'antes'	=> { 'x' => 'before',
		     '#' => 'temporal adverb',
 		     't' => 'adv' },
 'apenas'	=> { 'x' => 'hardly',
		     '#' => 'adv. scarcely; conj. as soon as' },
 'aquel'	=> { 'x' => 'that',
		     '#' => 'demonstrative' },
 'aquello'	=> { 'x' => 'that',
		     '#' => 'demonstrative' },
 'aqu'		=> { 'x' => 'here',
		     '#' => 'adv. of place; cf aca' },
 'arriba'	=> { 'x' => 'above',
		     '#' => 'up' },
 'as'		=> { 'x' => 'thus',
		     '#' => 'this/that way, like this/that',
 		     't' => 'adv' },
 'aun'		=> { 'x' => 'even' },
 'an'		=> { 'x' => 'still' },
 'aunque'	=> { 'x' => 'though',
		     '#' => 'although' },
 'ayer'		=> { 'x' => 'yesterday',
		     '#' => 'noun and temporal adverb',
 		     't' => 'n',
		     'g' => 'm' },
 'bajo'		=> { 'x' => 'under' },
 'bien'		=> { 'x' => 'well',
 		     't' => 'adv' },
 'cada'		=> { 'x' => 'each',
		     '#' => 'dict. says adv.' },
 'casi'		=> { 'x' => 'almost',
 		     't' => 'adv' },
 'cerca'	=> { 'x' => 'close',
		     '#' => 'near, nearby',
 		     't' => 'adv' },
 'como'		=> { 'x' => 'like',
		     '#' => 'as, how',
 		     't' => 'adv',
		     '#' => 'also form of comer' },
 'con'		=> { 'x' => 'with',
		     't' => 'p' },
 'contra'	=> { 'x' => 'against',
 		     't' => 'p' },
 'cualquier'	=> { 'x' => 'whichever',
		     'g' => 'm' },
 'cualquiera'	=> { 'x' => 'whichever',
		     'g' => 'f' },
 'cuando'	=> { 'x' => 'when',
		     '#' => 'and conj.',
 		     't' => 'adv' },
 'cuanto'	=> { 'x' => 'how much',
		     '#' => 'how many, etc.',
 		     't' => 'a' },
 'cuyo'		=> { 'x' => 'whose',
		     '#' => 'pronoun?' },
 'de'		=> { 'x' => 'of',
		     '#' => 'from',
		     't' => 'p' },
 'delante'	=> { 'x' => 'ahead',
		     '#' => 'in front, opposite',
 		     't' => 'adv' },
 'dems'	=> { 'x' => 'rest',
		     '#' => 'other',
 		     't' => 'a' },
 'demasiado'	=> { 'x' => 'too',
		     '#' => 'too much; g. and n. agreement?' },
 'demostrar'	=> { 'x' => 'demonstrate',
		     '#' => 'show, prove',
 		     't' => 'v' },
 'dentro'	=> { 'x' => 'inside',
		     '#' => 'within; adj. and adv.' },
 'desde'	=> { 'x' => 'from',
		     '#' => 'since',
		     't' => 'p' },
 'despus'	=> { 'x' => 'after',
		     '#' => 'dict. says adv.',
 		     't' => 'adv' },	
 'detrs'	=> { 'x' => 'behind',
		     '#' => 'dict. says adverb',
 		     't' => 'adv' },
 'donde'	=> { 'x' => 'where',
 		     't' => 'adv' },
 'durante'	=> { 'x' => 'during',
 		     't' => 'p' },
 'e'		=> { 'x' => 'and',
		     '#' => 'before vowel; cf y',
 		     't' => 'conj' },
 'en'		=> { 'x' => 'in',
		     '#' => 'on, about (pensar)',
        	     't' => 'p' },
 'encima'	=> { 'x' => 'above',
		     '#' => 'over; dict. says adv.',
 		     't' => 'adv' },
 'entonces'	=> { 'x' => 'so',
		     '#' => 'conj.?' },
 'entre'	=> { 'x' => 'between',
 		     't' => 'p' },
 'esa'		=> { 'x' => 'that' },
 'esas'		=> { 'x' => 'those' },
 'ese'		=> { 'x' => 'that' },
 'se'		=> { 'x' => 'that' },
 'eso'		=> { 'x' => 'that' },
 'esos'		=> { 'x' => 'those' },
 'esta'		=> { 'x' => 'this',
		     't' => 'a' },
 'sta'		=> { 'x' => 'this',
		     '#' => 'demonstrative' },
 'estas'	=> { 'x' => 'these',
		     't' => 'a' },
 'este'		=> { 'x' => 'this',
		     't' => 'a' },
 'ste'		=> { 'x' => 'this',
		     '#' => 'demonstrative' },
 'estes'	=> { 'x' => 'these',
		     't' => 'a' },
 'esto'		=> { 'x' => 'this',
		     '#' => 'pronoun?' },
 'hacia'	=> { 'x' => 'towards',
		     '#' => 'near' },
 'hasta'	=> { 'x' => 'until',
 		     't' => 'p' },
 'hoy'		=> { 'x' => 'today',
		     '#' => 'noun and temporal adv.?' },
 'jams'	=> { 'x' => 'never' },
 'juntos'	=> { 'x' => 'together',
		     '#' => 'adj. or adv.?' },
 'luego'	=> { 'x' => 'later',
		     '#' => '"hasta luego" = "till later"' },
 'ms'		=> { 'x' => 'more',
		     '#' => 'adverb?' },
 'menos'	=> { 'x' => 'less',
		     '#' => 'cf ms',
 		     't' => 'adv' },
 'mientras'	=> { 'x' => 'while',
 		     't' => 'p' },
 'muy'		=> { 'x' => 'very',
 		     't' => 'adv' },
 'ni'		=> { 'x' => 'nor',
		     '#' => 'conjunction?' },
 'no'		=> { 'x' => 'not',
		     '#' => 'key word',
		     't' => 'adv',
		     '#' => 'prevent conjugations etc.' },
 'nunca'	=> { 'x' => 'never' },
 'o'		=> { 'x' => 'or' },
 'obstante'	=> { 'x' => 'nevertheless',
		     '#' => 'always "no ~"',
 		     't' => 'adv' },
 'otro'		=> { 'x' => 'other',
		     '#' => 'another',
 		     't' => 'a' },
 'para'		=> { 'x' => 'for',
		     '#' => 'to',
 		     't' => 'p' },
 'pero'		=> { 'x' => 'but',
 		     't' => 'conj' },
 'por'		=> { 'x' => 'for',
		     '#' => 'by, per',
		     't' => 'p' },
 'porque'	=> { 'x' => 'because' },
 'pronto'	=> { 'x' => 'soon',
		     '#' => 'temporal adverb' },
 'propio'	=> { 'x' => 'own',
		     '#' => 'eg. "mi propia cosa" is "my own thing"',
 		     't' => 'a' },
 'pues'		=> { 'x' => 'then',
 		     't' => 'adv' },
 'que'		=> { 'x' => 'that',
		     '#' => 'than, what',
		     't' => 'pro.rel' },
 'quien'	=> { 'x' => 'who',
		     '#' => 'pronoun',
 		     't' => 'pro',
		     'n' => 's' },
 'quienes'	=> { 'x' => 'who',
		     '#' => 'pronoun',
 		     't' => 'pro',
		     'n' => 's' },
 'quiz'	=> { 'x' => 'maybe',
		     '#' => 'perhaps' },
 'quizs'	=> { 'x' => 'maybe',
		     '#' => 'perhaps' },
 'segn'	=> { 'x' => 'as',
		     '#' => 'while conj.; prep. according to' },
 'si'		=> { 'x' => 'if',
		     '#' => 'whether' },
 's'		=> { 'x' => 'yes',
		     '#' => 'interjection' },
 'siempre'	=> { 'x' => 'always',
 		     't' => 'adv' },
 'sino'		=> { 'x' => 'except',
		     '#' => 'but' },
 'siquiera'	=> { 'x' => 'at least',
		     '#' => 'even if, even though' },
 'sobre'	=> { 'x' => 'over',
		     '#' => 'on, about' },
 'tal'		=> { 'x' => 'such' },
 'tambin'	=> { 'x' => 'also',
		     '#' => 'too' },
 'tampoco'	=> { 'x' => 'neither' },
 'tan'		=> { 'x' => 'so',
		     '#' => 'modifies adjectives' },
 'tanto'	=> { 'x' => 'so much',
		     '#' => '(2 words)' },
 'todava'	=> { 'x' => 'still',
		     '#' => '(neg. is not yet)' },
 'todo'		=> { 'x' => 'all',
		     '#' => 'everything, everybody',
 		     't' => 'a',
		     '#' => 'at least conjugates like an adj.' },
 'tras'		=> { 'x' => 'behind',
		     '#' => 'after',
 		     't' => 'p' },
 'travs'	=> { 'x' => 'across',
		     '#' => 'over, through' },
 'nico'	=> { 'x' => 'only' },
 'y'		=> { 'x' => 'and',
		     't' => 'conj' },
 'ya'		=> { 'x' => 'already',
		     '#' => 'now' },

 #   Partitive / Contractions
 'al'		=> { 'x' => 'to the',
		     '#' => '(2 words)',
		     'g' => 'm' },
 'conmigo'	=> { 'x' => 'with me',
		     '#' => 'two words' },
 'contigo'	=> { 'x' => 'with you',
		     '#' => 'two words' },
 'del'		=> { 'x' => 'of the',
		     '#' => '(two words)',
		     'g' => 'm' },

 # Numbers: cardinal and ordinal
 'cero'		=> { 'x' => 'zero' },
 # 'uno'
 'primer'	=> { 'x' => 'first',
		     't' => 'a' },
 'primero'	=> { 'x' => 'first',
		     't' => 'a' },
 'dos'		=> { 'x' => 'two' },
 'segundo'	=> { 'x' => 'second',
		     '#' => 'noun, ordinal number' },
 'tres'		=> { 'x' => 'three' },
 'tercer'	=> { 'x' => 'third',
		     '#' => 'ordinal',
		     't' => 'a' },
 'tercero'	=> { 'x' => 'third',
		     '#' => 'ordinal',
		     't' => 'a' },
 'cuatro'	=> { 'x' => 'four' },
 'cinco'	=> { 'x' => 'five' },
 'seis'		=> { 'x' => 'six' },
 'siete'	=> { 'x' => 'seven' },
 'ocho'		=> { 'x' => 'eight' },
 'octavo'	=> { 'x' => 'eighth' },
 'nueve'	=> { 'x' => 'nine' },
 'diez'		=> { 'x' => 'ten' },
 'decena'	=> { 'x' => 'ten',
		     '#' => '(or so), una ~',
 		     't' => 'n' },
 'once'		=> { 'x' => 'eleven' },
 'doce'		=> { 'x' => 'twelve' },
 'docena'	=> { 'x' => 'dozen',
 		     't' => 'n' },
 'veinte'	=> { 'x' => 'twenty' },
 'treinta'	=> { 'x' => 'thirty' },
 'cuarenta'	=> { 'x' => 'forty' },
 'cincuenta'	=> { 'x' => 'fifty' },
 'sesenta'	=> { 'x' => 'sixty' },
 'setenta'	=> { 'x' => 'seventy' },
 'ochenta'	=> { 'x' => 'eighty' },
 'noventa'	=> { 'x' => 'ninety' },
 'cien'		=> { 'x' => 'hundred' },
 'ciento'	=> { 'x' => 'hundred' },
 'mil'		=> { 'x' => 'thousand' },
 'milln'	=> { 'x' => 'million' },

 # Days and months
 'domingo'	=> { 'x' => 'sunday' },
 'lunes'	=> { 'x' => 'monday' },
 'martes'	=> { 'x' => 'tuesday' },
 'mircoles'	=> { 'x' => 'wednesday' },
 'jueves'	=> { 'x' => 'thursday',
		     '#' => 'inv.' },
 'viernes'	=> { 'x' => 'friday',
		     't' => 'n' },
 'sbado'	=> { 'x' => 'saturday',
		     't' => 'n' },
 'enero'	=> { 'x' => 'january' },
 'febrero'	=> { 'x' => 'february' },
 'marzo'	=> { 'x' => 'march' },
 'abril'	=> { 'x' => 'april' },
 'mayo'		=> { 'x' => 'may' },
 'junio'	=> { 'x' => 'june' },
 'julio'	=> { 'x' => 'july' },
 'agosto'	=> { 'x' => 'august' },
 'septiembre'	=> { 'x' => 'september' },
 'octubre'	=> { 'x' => 'october' },
 'noviembre'	=> { 'x' => 'november' },
 'diciembre'	=> { 'x' => 'december' },

 # Key verbs
 #  ser - be
 'ser'		=> { 'x' => 'be',
		     '#' => '"to be"',
		     't' => 'v',
		     '#' => 'also noun - being' },
 #   ser present
 'soy'		=> { 'x' => '(i) am',
		     'r' => 'ser',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
 'eres'		=> { 'x' => '(you) are',
 		     'r' => 'ser',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'p' },
 'es'		=> { 'x' => '(it) is',
		     '#' => 'he/she',
		     'r' => 'ser',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 'p' },
 'somos'	=> { 'x' => '(we) are',
		     'r' => 'ser',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'p' },
 'sois'		=> { 'x' => '(you) are',
		     'r' => 'ser',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'p' },
 'son'		=> { 'x' => '(they) are',
		     'r' => 'ser',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'p' },
 #   ser imperfect
 'era'		=> { 'x' => '(it) was',
		     '#' => '(i) was',
		     'r' => 'ser',
		     't' => 'v',
		     'n' => 's',
		     'c' => 'i' },
 'eras'		=> { 'x' => '(you) were',
		     'r' => 'ser',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'i' },
 'ramos'	=> { 'x' => '(we) were',
 		     'r' => 'ser',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'i' },
 'erais'	=> { 'x' => '(you) were',
 		     'r' => 'ser',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'i' },
 'eran'		=> { 'x' => '(they) were',
 		     'r' => 'ser',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'i' },
 #   ser present subjunctive
 'sear'		=> { 'x' => 'be',
		     '#' => 'virtual base pres. subj.',
		     'r' => 'ser',
		     't' => 'v' },
 #   ser preterite; same as ir preterite!
 'fuir'		=> { 'x' => 'be',
		     '#' => 'virtual base pret.',
 		     'r' => 'ser',
		     '#' => 'also ir',
		     't' => 'v' },
  'fui'		=> { 'x' => '(i) was',
		     '#' => 'not "fu"; went',
 		     'r' => 'ser',
		     '#' => 'also ir',
		     't' => 'v' },
  'fue'		=> { 'x' => '(it) was',
		     '#' => 'not "fui"; went',
 		     'r' => 'ser',
		     '#' => 'also ir',
		     't' => 'v' },
  'fueron'	=> { 'x' => '(they) were',
		     '#' => 'not "fuieron"',
 		     'r' => 'ser',
		     '#' => 'also ir',
		     't' => 'v' },
 #   ser imperfect subjunctive?
 'fuera'	=> { 'x' => '(it) were',
		     '#' => '(i) were',
 		     'r' => 'ser',
		     '#' => 'also ir',
		     't' => 'v' },
 #  estar - be
 'estar'	=> { 'x' => 'be',
		     '#' => 'compare with "ser"',
		     't' => 'v' },
 'estoy'	=> { 'x' => '(i) am',
		     't' => 'v',
		     'r' => 'estar' },
 'ests'	=> { 'x' => '(you) are',
		     't' => 'v',
		     'r' => 'estar' },
 'est'		=> { 'x' => '(it) is',
 		     'r' => 'estar',
		     't' => 'v' },
 'estado'	=> { 'x' => 'state',
		     '#' => 'also pp. of estar; been',
 		     't' => 'n' },
 'estn'	=> { 'x' => '(they) are',
 		     'r' => 'estar',
		     't' => 'v' },
 'estuve'	=> { 'x' => '(i) was',
 		     'r' => 'estar',
 		     't' => 'v' },
 'estuvieron'	=> { 'x' => '(they) were',
 		     'r' => 'estar',
 		     't' => 'v' },
 'estuvo'	=> { 'x' => '(it) was',
 		     'r' => 'estar',
 		     't' => 'v' },
 # haber - have
 'haber'	=> { 'x' => 'have',
		     't' => 'v' },
 #  haber pres.
 'he'		=> { 'x' => '(i) have',
 		     'r' => 'haber',
		     't' => 'v' },
 'has'		=> { 'x' => '(you) have',
 		     'r' => 'haber',
		     't' => 'v' },
 'ha'		=> { 'x' => '(it) has',
 		     'r' => 'haber',
		     't' => 'v' },
 'hemos'	=> { 'x' => '(we) have' },
 'han'		=> { 'x' => '(they) had' },
 #  haber pres. subj.
 'hayar'	=> { 'x' => 'have',
		     '#' => 'virtual base',
 		     'r' => 'haber',
		     't' => 'v' },
 #  haber fut. and condit.
 'habrar'	=> { 'x' => 'have',
		     '#' => 'virtual base',
 		     'r' => 'haber',
		     't' => 'v' },
 #  haber pret.
 'huber'	=> { 'x' => 'have',
		     '#' => 'virtual base',
 		     'r' => 'haber',
		     't' => 'v' },
 'hube'		=> { 'x' => '(i) had',
 		     'r' => 'haber',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
 'hubo'		=> { 'x' => '(it) had',
 		     'r' => 'haber',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
 #  haber imp. subj.
 'hubiera'	=> { 'x' => '(it) had',
		     '#' => 'I had' },
 #  impersonal!
 'hay'		=> { 'x' => 'there is',
		     '#' => 'there are (two words)' },
 #  hacer - to do; to make
 'hacer'	=> { 'x' => 'make',
		     '#' => 'do',
 		     't' => 'v' },
 'haga'		=> { 'x' => '(it) makes',
		     't' => 'v',
		     'r' => 'hacer' },
 'hagamos'	=> { 'x' => '(we) make' },
 'hagan'	=> { 'x' => '(they) make' },
 'hagas'	=> { 'x' => '(you) make' },
 'hago'		=> { 'x' => '(i) make' },
 'hizo'		=> { 'x' => '(it) made',
		     '#' => '(it) did',
 		     'r' => 'hacer',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
 'hecho'	=> { 'x' => 'made' },
 #  gustar - to please; translates to 'to like' in English
 'gustar'	=> { 'x' => 'please',
		     '#' => '"like" but with subj & obj swapped',
		     't' => 'v' },
 #  ir - to go
 'ir'		=> { 'x' => 'go',
		     't' => 'v' },
  'var'		=> { 'x' => 'go',
		     '#' => 'virtual base pres.',
  		     'r' => 'ir',
		     't' => 'v' },
   'voy'	=> { 'x' => '(i) go',
		     '#' => 'not "vo"',
		     't' => 'v',
		     'r' => 'ir',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
   'vais'	=> { 'x' => '(you) go',
		     '#' => 'not "vis"',
		     't' => 'v',
		     'r' => 'ir',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'p' },
  'vayar'	=> { 'x' => 'go',
		     '#' => 'virtual base pres. subj.',
  		     'r' => 'ir',
		     't' => 'v' },
  'iba'		=> { 'x' => '(it) went',
		     '#' => 'i, he/she/it',
		     't' => 'v',
		     'r' => 'ir',
		     'n' => 's',
		     'c' => 'i' },
  'ibas'	=> { 'x' => '(you) went',
		     't' => 'v',
		     'r' => 'ir',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'i' },
  'ibamos'	=> { 'x' => '(we) went',
		     't' => 'v',
		     'r' => 'ir',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 'i' },
  'ibais'	=> { 'x' => '(you) went',
		     't' => 'v',
		     'r' => 'ir',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'i' },
  'iban'	=> { 'x' => '(they) went',
		     't' => 'v',
		     'r' => 'ir',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'i' },
 #  poder - can (modal)
 'poder'	=> { 'x' => 'can',
		     '#' => 'be able to',
		     't' => 'v',
		     '#' => 'also noun: power' },
 'pude'		=> { 'x' => '(i) could',
 		     'r' => 'poder',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
 #   virtual base for pres., subj.
 'pueder'	=> { 'x' => 'can',
		     '#' => 'be able to',
 		     'r' => 'poder',
		     't' => 'v' },
 #   virtual base for preterite
 'puder'	=> { 'x' => 'be able to',
		     '#' => 'could',
 		     'r' => 'poder',
		     't' => 'v' },
 #   virtual base for fut., cond.
 'podr'		=> { 'x' => 'be able to',
		     'r' => 'poder',
		     't' => 'v' },
 #  deber - should (modal)
 'deber'	=> { 'x' => 'should',
		     't' => 'v' },
 #  tener - have
 'tener'	=> { 'x' => 'have',
		     't' => 'v' },
  'ten'		=> { 'x' => 'have',
		     '#' => 'imperative',
  		     'r' => 'tener',
		     't' => 'v',
		     'n' => 's',
		     'c' => 'imp' },
  'tenger'	=> { 'x' => 'have',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'tener',
		     't' => 'v' },
  'tiener'	=> { 'x' => 'have',
		     '#' => 'virtual base pres.',
  		     'r' => 'tener',
		     't' => 'v' },
  'tendr'	=> { 'x' => 'have',
		     '#' => 'virtual base fut., condit.',
  		     'r' => 'tener',
		     't' => 'v' },
  'tuver'	=> { 'x' => 'have',
		     '#' => 'virtual base pret.',
  		     'r' => 'tener',
		     't' => 'v' },
  'tuve'	=> { 'x' => '(it) had',
		     'r' => 'tener',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
  'tuvo'	=> { 'x' => '(i) had',
		     'r' => 'tener',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
 # Vocabulary
 'abada'	=> { 'x' => 'abbey',
		     't' => 'n' },
 'abajo'	=> { 'x' => 'below',
		     '#' => 'under, beneath',
 		     't' => 'a' },
 'abandonar'	=> { 'x' => 'abandon',
 		     't' => 'v' },
 'abecedario'	=> { 'x' => 'alphabet',
 		     't' => 'n' },
 'abierto'	=> { 'x' => 'open',
		     't' => 'a' },
 'abogado'	=> { 'x' => 'lawyer',
		     '#' => 'solicitor; person',
 		     't' => 'n' },
 'abolir'	=> { 'x' => 'abolish',
		     't' => 'v' },
 'abordar'	=> { 'x' => 'board',
		     't' => 'v' },
 'abrazar'	=> { 'x' => 'hug',
		     't' => 'v' },
 'abrazo'	=> { 'x' => 'hug',
		     't' => 'n' },
 'abrir'	=> { 'x' => 'open',
		     't' => 'v' },
 'abrupto'	=> { 'x' => 'abrupt',
		     '#' => 'steep',
		     't' => 'a' },
 'abuela'	=> { 'x' => 'grandmother',
		     't' => 'n' },
 'abuelo'	=> { 'x' => 'grandfather',
		     't' => 'n' },
 'abuelos'	=> { 'x' => 'grandparents',
		     't' => 'n',
		     'n' => 'p' },
 'abundar'	=> { 'x' => 'abound',
		     '#' => 'be plentiful',
 		     't' => 'v' },
 'aburrido'	=> { 'x' => 'boring',
 		     't' => 'a' },
 'abusar'	=> { 'x' => 'abuse',
		     't' => 'v' },
 'acabar'	=> { 'x' => 'finish',
		     't' => 'v' },
 'academia'	=> { 'x' => 'academy',
 		     't' => 'n' },
 'acadmico'	=> { 'x' => 'academic',
		     't' => 'a' },
 'acceso'	=> { 'x' => 'access',
		     't' => 'n' },
 'acento'	=> { 'x' => 'accent',
		     '#' => 'stress',
		     't' => 'n' },
 'acera'	=> { 'x' => 'pavement',
		     '#' => 'sidewalk, footpath',
		     't' => 'n' },
 'acercar'	=> { 'x' => 'bring nearer',
		     '#' => 'move nearer',
		     't' => 'v' },
 'acercarse'	=> { 'x' => 'approach',
		     '#' => 'come near',
		     't' => 'v' },
 'acero'	=> { 'x' => 'steel',
		     't' => 'n' },
 'acolchar'	=> { 'x' => 'pad',
 		     't' => 'v' },
 'acompaar'	=> { 'x' => 'accompany',
 		     't' => 'v' },
 'acordar'	=> { 'x' => 'agree',
		     '#' => 'resolve, remind',
 		     't' => 'v',
		     '#' => 'refl. agree; remember' },
 'activar'	=> { 'x' => 'activate',
 		     't' => 'v' },
 'actriz'	=> { 'x' => 'actress',
		     't' => 'n' },
 'actual'	=> { 'x' => 'current',
 		     't' => 'a' },
 'actualizar'	=> { 'x' => 'update',
		     '#' => 'modernize',
		     't' => 'v' },
 'actuar'	=> { 'x' => 'act',
		     't' => 'v' },
 'acuerdo'	=> { 'x' => 'agreement',
		     '#' => 'also form of "acordar"',
 		     't' => 'n' },
 'acusacin'	=> { 'x' => 'accusation',
 		     't' => 'n',
		     'g' => 'f' },
 'acusar'	=> { 'x' => 'accuse',
		     '#' => 'reveal; denounce',
 		     't' => 'v' },
 'adaptar'	=> { 'x' => 'adapt',
 		     't' => 'v' },
 'adjetivo'	=> { 'x' => 'adjective',
 		     't' => 'n' },
 'administracin'	=> { 'x' => 'administration',
 			     't' => 'n' },
 'administrar'	=> { 'x' => 'administrate',
		     '#' => 'administer',
		     't' => 'v' },
 'adolescencia'	=> { 'x' => 'adolescence',
		     't' => 'n' },
 'adoptar'	=> { 'x' => 'adopt',
 		     't' => 'v' },
 'adornar'	=> { 'x' => 'adorn',
		     't' => 'v' },
 'adquirir'	=> { 'x' => 'acquire',
		     't' => 'v' },
 'aduear'	=> { 'x' => 'take possession of',
		     '#' => 'multiword; refl. only!',
 		     't' => 'v' },
 'advertencia'	=> { 'x' => 'warning',
		     '#' => 'preface, foreword',
 		     't' => 'n' },
 'advertir'	=> { 'x' => 'notice',
		     't' => 'v' },
 'africano'	=> { 'x' => 'african',
		     '#' => 'adj. and noun' },
 'agua'		=> { 'x' => 'water',
		     '#' => 'fem. but takes masc. art.',
		     't' => 'f' },
 'aguardiente'	=> { 'x' => 'firewater',
		     '#' => 'brandy, liquor',
 		     't' => 'n',
		     'g' => 'm' },
 'aeropuerto'	=> { 'x' => 'airport',
		     't' => 'n' },
 'agotar'	=> { 'x' => 'exhaust',
		     '#' => 'drain, use up, deplete',
		     't' => 'v' },
 'agradecimiento'	=> { 'x' => 'thanks',
			     't' => 'n' },
 'agradecimientos'	=> { 'x' => 'gratitude',
			     't' => 'n' },
 'ahogar'	=> { 'x' => 'drown',
		     '#' => 'suffocate',
 		     't' => 'v' },
 'ahorrar'	=> { 'x' => 'save',
		     't' => 'v' },
 'aire'		=> { 'x' => 'air',
		     't' => 'n' },
 'aislado'	=> { 'x' => 'isolated',
		     '#' => 'insulated',
 		     't' => 'a' },
 'ajo'		=> { 'x' => 'garlic',
		     't' => 'n' },
 'ala'		=> { 'x' => 'wing',
		     't' => 'n' },
 'alcanzar'	=> { 'x' => 'reach',
		     't' => 'v' },
 'alcohol'	=> { 'x' => 'alcohol',
		     't' => 'n' },
 'aldea'	=> { 'x' => 'village',
		     't' => 'n' },
 'alegre'	=> { 'x' => 'happy',
		     '#' => 'cheerful, merry, tight;',
 		     't' => 'a',
		     '#' => 'risque, blue' },
 'alegra'	=> { 'x' => 'happiness',
		     '#' => 'merriment',
 		     't' => 'n' },
 'alejar'	=> { 'x' => 'remove',
		     '#' => 'estrange; refl. move away',
 		     't' => 'v' },
 'alemn'	=> { 'x' => 'german',
		     '#' => 'noun and adj.?' },
 'alemania'	=> { 'x' => 'germany',
		     '#' => 'capitalize',
		     't' => 'n' },
 'alfabtico'	=> { 'x' => 'alphabetic',
 		     't' => 'a' },
 'alfabetizar'	=> { 'x' => 'alphabetize',
 		     't' => 'v' },
 'alfabeto'	=> { 'x' => 'alphabet',
 		     't' => 'n' },
 'alianza'	=> { 'x' => 'alliance',
		     '#' => 'wedding ring',
 		     't' => 'n' },
 'aliento'	=> { 'x' => 'breath',
		     '#' => 'breathing',
 		     't' => 'n' },
 'alimentar'	=> { 'x' => 'feed',
		     '#' => 'nourish',
 		     't' => 'v' },
 'aliviar'	=> { 'x' => 'relieve',
		     '#' => 'lighten',
 		     't' => 'v' },
 'alma'		=> { 'x' => 'soul',
		     '#' => 'person',
 		     't' => 'n' },
 'almendra'	=> { 'x' => 'almond',
		     't' => 'n' },
 'almidonar'	=> { 'x' => 'starch' },
 'almohada'	=> { 'x' => 'pillow',
		     't' => 'n' },
 'alquilar'	=> { 'x' => 'rent',
		     't' => 'v' },
 'alquiler'	=> { 'x' => 'hire',
		     '#' => 'renting',
 		     't' => 'v' },
 'alrededores'	=> { 'x' => 'surroundings',
 		     't' => 'n',
		     'g' => 'm',
		     'n' => 'p' },
 'alto'		=> { 'x' => 'tall',
 		     't' => 'a' },
 'altura'	=> { 'x' => 'height',
		     '#' => 'depth, latitude',
		     't' => 'n' },
 'amanecer'	=> { 'x' => 'dawn',
 		     't' => 'n' },
 'amar'		=> { 'x' => 'love',
		     't' => 'v' },
 'amarillo'	=> { 'x' => 'yellow',
		     't' => 'a' },
 'amarrar'	=> { 'x' => 'tie up',
		     '#' => 'moor',
		     't' => 'v' },
 'mbito'	=> { 'x' => 'field',
		     '#' => 'area',
 		     't' => 'n' },
 'amrica'	=> { 'x' => 'america',
		     '#' => 'capitalize',
 		     't' => 'n' },
 'americano'	=> { 'x' => 'american',
		     '#' => 'adj. and noun' },
 'amiga'	=> { 'x' => 'friend',
		     't' => 'n' },
 'amigo'	=> { 'x' => 'friend',
		     't' => 'n' },
 'amistad'	=> { 'x' => 'friendship',
 		     't' => 'n',
		     'g' => 'f' },
 'amor'		=> { 'x' => 'love',
		     't' => 'n',
		     'g' => 'm' },
 'amplio'	=> { 'x' => 'spacious' },
 'anlisis'	=> { 'x' => 'analysis',
 		     't' => 'n' },
 'anatoma'	=> { 'x' => 'anatomy',
 		     't' => 'n' },
 'ancho'	=> { 'x' => 'wide',
 		     't' => 'a' },
 'andar'	=> { 'x' => 'walk',
		     't' => 'v' },
 'anestesiar'	=> { 'x' => 'anaesthetize',
 		     't' => 'v' },
 'angosto'	=> { 'x' => 'narrow',
 		     't' => 'a' },
 'ngulo'	=> { 'x' => 'angle',
 		     't' => 'n' },
 'anhelar'	=> { 'x' => 'desire',
		     '#' => 'long for',
 		     't' => 'v' },
 'anhelo'	=> { 'x' => 'desire',
 		     't' => 'n' },
 'anillo'	=> { 'x' => 'ring',
 		     't' => 'n' },
 'ano'		=> { 'x' => 'anus',
		     '#' => 'not ao = year!',
 		     't' => 'n' },
 'anoche'	=> { 'x' => 'last night',
		     '#' => 'multiword',
 		     't' => 'adv' },
 'ansioso'	=> { 'x' => 'anxious',
		     't' => 'a' },
 'antigedad'	=> { 'x' => 'antiquity',
		     't' => 'n' },
 'antiguo'	=> { 'x' => 'ancient',
		     't' => 'a' },
 'antologa'	=> { 'x' => 'anthology',
 		     't' => 'n' },
 'aadir'	=> { 'x' => 'add',
		     't' => 'v' },
 'ao'		=> { 'x' => 'year',
		     't' => 'n' },
 'anunciar'	=> { 'x' => 'advertise',
		     '#' => 'announce, proclaim',
		     't' => 'v' },
 'aparecer'	=> { 'x' => 'appear',
		     '#' => 'irreg.',
		     't' => 'v' },
  'aparezcer'	=> { 'x' => 'appear',
		     '#' => 'virtual base 1ps pres.; pres. subj.',
  		     'r' => 'aparecer',
		     't' => 'v' },
 'aparicin'	=> { 'x' => 'apparition',
		     '#' => 'appearance',
		     't' => 'n' },
 'aparte'	=> { 'x' => 'separately',
		     '#' => 'besides',
 		     't' => 'adv' },
 'apegar'	=> { 'x' => 'become attached to',
		     '#' => 'multiword',
 		     't' => 'v',
		     '#' => 'actually must be "apagarse a"' },
 'apego'	=> { 'x' => 'attachment',
		     '#' => 'devotion',
 		     't' => 'n' },
 'aplicacin'	=> { 'x' => 'application',
		     't' => 'n',
		     'g' => 'f' },
 'aplicar'	=> { 'x' => 'apply',
		     't' => 'v' },
 'aportar'	=> { 'x' => 'contribute',
		     't' => 'v' },
 'apoyar'	=> { 'x' => 'support',
		     '#' => 'lean, rest',
		     't' => 'v' },
 'aprender'	=> { 'x' => 'learn',
		     't' => 'v' },
 'apretar'	=> { 'x' => 'squeeze',
		     '#' => 'irregular',
		     't' => 'v' },
  'aprieta'	=> { 'x' => '(it) squeezes',
  		     'r' => 'apretar',
		     't' => 'v' },
 'aprox'	=> { 'x' => 'approx',
		     '#' => 'abbrev.' },
 'archivo'	=> { 'x' => 'file',
		     '#' => '(computer)',
 		     't' => 'n' },
 'arder'	=> { 'x' => 'burn',
 		     't' => 'v' },
 'rea'		=> { 'x' => 'area',
		     't' => 'n' },
 'rbol'	=> { 'x' => 'tree',
 		     't' => 'n' },
 'argentina'	=> { 'x' => 'argentina',
		     '#' => 'proper noun',
		     't' => 'n' },
 'argentino'	=> { 'x' => 'argentinian',
		     '#' => 'noun and adj.' },
 'argolla'	=> { 'x' => 'ring',
		     '#' => '(large) ring',
		     't' => 'n' },
 'argumento'	=> { 'x' => 'argument',
		     '#' => 'plot',
 		     't' => 'n' },
 'arma'		=> { 'x' => 'arm',
		     '#' => 'as in weapon',
 		     't' => 'n' },
 'armar'	=> { 'x' => 'arm',
		     '#' => 'assemble',
 		     't' => 'v' },
 'arraigar'	=> { 'x' => 'establish',
		     '#' => 'take root',
		     't' => 'v' },
 'arrancar'	=> { 'x' => 'extract',
		     '#' => 'pull out, boot',
		     't' => 'v' },
 'arranque'	=> { 'x' => '(it) extracts',
		     '#' => 'pulls out, boot',
		     'r' => 'arrancar',
		     't' => 'v' },
 'arreglar'	=> { 'x' => 'fix',
		     '#' => 'tidy up, repair',
		     't' => 'v' },
 'arruinar'	=> { 'x' => 'ruin',
		     't' => 'v' },
 'arte'		=> { 'x' => 'art',
		     't' => 'n' },
 'artculo'	=> { 'x' => 'article',
 		     't' => 'n' },
 'artista'	=> { 'x' => 'artist',
		     '#' => 'person',
 		     't' => 'n' },
 'asegurar'	=> { 'x' => 'secure',
		     '#' => 'fasten, guarantee',
 		     't' => 'v' },
 'asesinar'	=> { 'x' => 'assassinate',
 		     't' => 'v' },
 'asesinato'	=> { 'x' => 'assassination',
 		     't' => 'v' },
 'asesino'	=> { 'x' => 'assassin',
		     '#' => 'person',
 		     't' => 'n' },
 'asiento'	=> { 'x' => 'seat',
		     't' => 'n' },
 'asignar'	=> { 'x' => 'assign',
		     't' => 'v' },
 'asilo'	=> { 'x' => 'asylum',
		     '#' => 'refuge, home',
 		     't' => 'n' },
 'asociacin'	=> { 'x' => 'association',
 		     't' => 'n' },
 'aspecto'	=> { 'x' => 'look',
		     't' => 'n' },
 'aspiracin'	=> { 'x' => 'aspiration',
 		     't' => 'n',
		     'g' => 'f' },
 'astronauta'	=> { 'x' => 'astronaut',
 		     't' => 'n' },
 'atar'		=> { 'x' => 'tie',
		     '#' => 'tie up',
 		     't' => 'v' },
 'atardecer'	=> { 'x' => 'dusk',
		     '#' => 'to get dark',
 		     't' => 'n' },
 'atad'	=> { 'x' => 'coffin',
		     't' => 'n' },
 'atencin'	=> { 'x' => 'attention',
 		     't' => 'n' },
 'aterrizar'	=> { 'x' => 'land',
 		     't' => 'v' },
 'atractivo'	=> { 'x' => 'attractive',
 		     't' => 'a' },
 'atrasar'	=> { 'x' => 'be slow',
		     '#' => 'multiword; remain behind',
 		     't' => 'v' },
 'atraso'	=> { 'x' => 'slowness',
 		     't' => 'n' },
 'atrevido'	=> { 'x' => 'daring' },
 'atribucin'	=> { 'x' => 'powers',
		     '#' => 'responsibilities',
 		     't' => 'n',
		     'g' => 'f' },
 'atribuir'	=> { 'x' => 'attribute',
 		     't' => 'v' },
 'atributo'	=> { 'x' => 'attribute',
 		     't' => 'n' },
 'ausencia'	=> { 'x' => 'absence',
 		     't' => 'n' },
 'australia'	=> { 'x' => 'australia',
		     '#' => 'capitalize',
		     't' => 'n' },
 'australiano'	=> { 'x' => 'australian',
		     '#' => 'noun and adj.',
		     't' => 'n' },
 'autobs'	=> { 'x' => 'bus',
		     't' => 'n' },
 'automtico'	=> { 'x' => 'automatic',
		     't' => 'a' },
 'automvil'	=> { 'x' => 'automobile',
		     't' => 'n' },
 'autor'	=> { 'x' => 'author',
		     't' => 'n' },
 'autoridad'	=> { 'x' => 'authority',
 		     't' => 'n' },
 'avanzar'	=> { 'x' => 'advance',
		     't' => 'v' },
 'avenida'	=> { 'x' => 'avenue',
		     't' => 'n' },
 'aventura'	=> { 'x' => 'adventure',
		     't' => 'n' },
 'averiado'	=> { 'x' => 'broken down',
		     '#' => 'multiword;',
 		     't' => 'a',
		     '#' => 'out of order' },
 'avera'	=> { 'x' => 'breakdown',
		     '#' => 'fault',
 		     't' => 'n' },
 'avin'	=> { 'x' => 'plane',
		     't' => 'n' },
 'ayudar'	=> { 'x' => 'help',
		     't' => 'v' },
 'azul'		=> { 'x' => 'blue',
 		     't' => 'a' },
 'bailar'	=> { 'x' => 'dance',
 		     't' => 'v' },
 'bajar'	=> { 'x' => 'drop',
		     '#' => 'go down, come down, fall, lower',
 		     't' => 'v' },
 'banco'	=> { 'x' => 'bank',
		     '#' => 'bench, desk',
 		     't' => 'n' },
 'banda'	=> { 'x' => 'band',
		     't' => 'n' },
 'bandera'	=> { 'x' => 'flag',
		     't' => 'n' },
 'baar'	=> { 'x' => 'bathe',
 		     't' => 'v' },
 'bar'		=> { 'x' => 'bar',
 		     't' => 'n',
		     'g' => 'm' },
 'barato'	=> { 'x' => 'cheap',
		     '#' => 'inexpensive',
		     't' => 'a' },
 'barbilla'	=> { 'x' => 'chin',
		     '#' => 'tip of chin',
 		     't' => 'n' },
 'barca'	=> { 'x' => 'boat',
		     '#' => 'small boat',
 		     't' => 'n' },
 'barco'	=> { 'x' => 'boat',
		     '#' => 'ship',
 		     't' => 'n' },
 'barrio'	=> { 'x' => 'neighbourhood',
		     '#' => 'area, suburb',
 		     't' => 'n' },
 'basar'	=> { 'x' => 'base',
		     't' => 'v' },
 'bsico'	=> { 'x' => 'basic',
		     't' => 'a' },
 'baso'		=> { 'x' => 'base',
 		     't' => 'n' },
 'basta'	=> { 'x' => 'enough',
		     '#' => 'interj.?' },
 'bastante'	=> { 'x' => 'enough',
		     '#' => 'pretty much',
 		     't' => 'a' },
 'bastardo'	=> { 'x' => 'bastard',
 		     't' => 'n' },
 'bastn'	=> { 'x' => 'stick',
		     't' => 'n' },
 'basura'	=> { 'x' => 'rubbish',
		     '#' => 'garbage',
		     't' => 'n' },
 'bal'		=> { 'x' => 'trunk',
		     '#' => 'boot',
		     't' => 'n' },
 'beber'	=> { 'x' => 'drink',
 		     't' => 'v' },
 'bebida'	=> { 'x' => 'drink',
 		     't' => 'n' },
 'blgica'	=> { 'x' => 'belgium',
		     '#' => 'proper, place',
		     't' => 'n' },
 'belice'	=> { 'x' => 'belize',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'belleza'	=> { 'x' => 'beauty',
 		     't' => 'n' },
 'bello'	=> { 'x' => 'beautiful',
		     't' => 'a' },
 'beneficio'	=> { 'x' => 'beneficiary',
 		     't' => 'n' },
 'besar'	=> { 'x' => 'kiss',
 		     't' => 'v' },
 'beso'		=> { 'x' => 'kiss',
		     't' => 'n' },
 'bibliografa'	=> { 'x' => 'bibliography',
		     't' => 'n' },
 'biblioteca'	=> { 'x' => 'library',
		     't' => 'n' },
 'bicicleta'	=> { 'x' => 'bicycle',
 		     't' => 'n' },
 'bienvenido'	=> { 'x' => 'welcome' },
 'bigote'	=> { 'x' => 'moustache',
 		     't' => 'n',
		     'g' => 'm' },
 'blanco'	=> { 'x' => 'white',
 		     't' => 'a' },
 'blusa'	=> { 'x' => 'blouse',
		     't' => 'n' },
 'bobo'		=> { 'x' => 'fool',
		     '#' => 'clown, daft, silly',
		     't' => 'n' },
 'boca'		=> { 'x' => 'mouth',
		     't' => 'n' },
 'bocadillo'	=> { 'x' => 'sandwich',
		     't' => 'n' },
 'boina'	=> { 'x' => 'beret',
 		     't' => 'n' },
 'boleto'	=> { 'x' => 'ticket',
		     't' => 'n' },
 'bolivia'	=> { 'x' => 'bolivia',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'bola'		=> { 'x' => 'bag',
 		     't' => 'n' },
 'bolsillo'	=> { 'x' => 'pocket',
		     't' => 'n' },
 'bonito'	=> { 'x' => 'pretty',
		     't' => 'a' },
 'borracho'	=> { 'x' => 'drunk',
		     '#' => 'noun and adj.?' },
 'borrar'	=> { 'x' => 'erase',
		     '#' => 'rub out',
 		     't' => 'v' },
 'bota'		=> { 'x' => 'boot',
 		     't' => 'n' },
 'botella'	=> { 'x' => 'bottle',
 		     't' => 'n' },
 'botn'	=> { 'x' => 'button',
		     't' => 'n' },
 'brasa'	=> { 'x' => 'hot coal',
		     '#' => 'multiword',
 		     't' => 'n' },
 'brasil'	=> { 'x' => 'brazil',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'brazo'	=> { 'x' => 'arm',
		     't' => 'n' },
 'brillar'	=> { 'x' => 'shine',
 		     't' => 'v' },
 'brisa'	=> { 'x' => 'breeze',
		     't' => 'n' },
 'brotar'	=> { 'x' => 'spout',
		     '#' => 'gush, break out',
 		     't' => 'v' },
 'bruja'	=> { 'x' => 'witch',
		     '#' => 'person',
 		     't' => 'n' },
 'bruto'	=> { 'x' => 'stupid',
		     '#' => 'brutish, gross',
 		     't' => 'a' },
 'buen'		=> { 'x' => 'good',
		     't' => 'a' },
 'bueno'	=> { 'x' => 'good',
		     't' => 'a' },
 'burlar'	=> { 'x' => 'deceive',
		     '#' => 'joke, make fun of',
 		     't' => 'v' },
 'buscar'	=> { 'x' => 'look for',
		     '#' => 'seek, search',
		     't' => 'v' },
 'bsqueda'	=> { 'x' => 'search',
		     't' => 'n' },
 'caballero'	=> { 'x' => 'gentleman',
		     '#' => 'knight, sir',
		     't' => 'n' },
 'caballo'	=> { 'x' => 'horse',
		     't' => 'n' },
 'cabellera'	=> { 'x' => 'hair',
		     '#' => '(head of)',
 		     't' => 'n' },
 'cabello'	=> { 'x' => 'hair',
		     '#' => 'TODO uncountable',
		     't' => 'n' },
 'caber'	=> { 'x' => 'fit',
		     't' => 'v' },
 'cabeza'	=> { 'x' => 'head',
		     't' => 'n' },
 'cacahuate'	=> { 'x' => 'peanut',
		     '#' => 'latin america only',
		     't' => 'n' },
 'cach'	=> { 'x' => 'cache',
 		     't' => 'n' },
 'cachondeo'	=> { 'x' => 'farce',
		     '#' => 'joke',
 		     't' => 'n' },
 'cadver'	=> { 'x' => 'cadaver',
		     't' => 'n' },
 'caer'		=> { 'x' => 'fall',
 		     't' => 'v' },
 'caf'		=> { 'x' => 'coffee',
		     '#' => 'TODO uncountable',
		     't' => 'n' },
 'caja'		=> { 'x' => 'box',
		     '#' => 'and everything else!',
		     't' => 'n' },
 'calcular'	=> { 'x' => 'calculate',
 		     't' => 'v' },
 'calidad'	=> { 'x' => 'quality',
 		     't' => 'n' },
 'caliente'	=> { 'x' => 'hot',
		     '#' => 'warm',
 		     't' => 'a' },
 'calor'	=> { 'x' => 'heat',
		     't' => 'n' },
 'calma'	=> { 'x' => 'calm',
 		     't' => 'n' },
 'calmar'	=> { 'x' => 'calm',
		     '#' => 'calm down',
 		     't' => 'v' },
 'calle'	=> { 'x' => 'street',
		     't' => 'n' },
 'callejn'	=> { 'x' => 'alley',
 		     't' => 'n' },
 'cama'		=> { 'x' => 'bed',
		     't' => 'n' },
 'cmara'	=> { 'x' => 'camera',
 		     't' => 'n' },
 'cambiar'	=> { 'x' => 'change',
		     't' => 'v' },
 'cambio'	=> { 'x' => 'change',
		     '#' => 'also form of cambiar',
		     't' => 'n' },
 'camello'	=> { 'x' => 'camel',
		     '#' => 'animate; pusher',
 		     't' => 'n' },
 'caminar'	=> { 'x' => 'walk',
		     '#' => 'go',
 		     't' => 'v' },
 'camino'	=> { 'x' => 'way',
		     '#' => 'road, track',
		     't' => 'n' },
 'camin'	=> { 'x' => 'truck',
		     '#' => 'bus',
		     't' => 'n' },
 'campo'	=> { 'x' => 'countryside',
		     '#' => 'country',
 		     't' => 'n' },
 'canal'	=> { 'x' => 'channel',
		     't' => 'n' },
 'cancin'	=> { 'x' => 'song',
 		     't' => 'n',
		     'g' => 'f' },
 'canguro'	=> { 'x' => 'kangaroo',
		     't' => 'n' },
 'cansar'	=> { 'x' => 'tire',
		     '#' => 'bore',
		     't' => 'v' },
 'cantar'	=> { 'x' => 'sing',
		     't' => 'v' },
 'cantidad'	=> { 'x' => 'quantity',
		     't' => 'n' },
 'camo'	=> { 'x' => 'hemp',
		     't' => 'n' },
 'capa'		=> { 'x' => 'cloak',
		     '#' => 'cape',
		     't' => 'n' },
 'capaz'	=> { 'x' => 'able',
		     '#' => 'capable; roomy',
 		     't' => 'a' },
 'cara'		=> { 'x' => 'face',
		     't' => 'n' },
 'carcter'	=> { 'x' => 'character',
		     't' => 'n' },
 'caracteres'	=> { 'x' => 'characters',
		     't' => 'n',
		     'n' => 'p' },
 'caracterstica'	=> { 'x' => 'characteristic',
 			     't' => 'n' },
 'caracterizar'	=> { 'x' => 'characterize',
		     't' => 'v' },
 'carga'	=> { 'x' => 'load',
		     '#' => 'cargo, charge',
		     't' => 'n' },
 'cargar'	=> { 'x' => 'load',
		     '#' => 'charge',
		     't' => 'v' },
 'carne'	=> { 'x' => 'meat',
		     '#' => 'flesh',
 		     't' => 'n' },
 'caro'		=> { 'x' => 'expensive',
		     '#' => 'dear',
		     't' => 'a' },
 'carretera'	=> { 'x' => 'highway',
 		     't' => 'n' },
 'carta'	=> { 'x' => 'letter',
 		     't' => 'n' },
 'casa'		=> { 'x' => 'house',
		     '#' => 'home',
		     't' => 'n' },
 'casar'	=> { 'x' => 'marry',
		     't' => 'v' },
 'caso'		=> { 'x' => 'case',
		     't' => 'n' },
 'castellano'	=> { 'x' => 'castilian',
		     '#' => 'adj. and noun' },
 'castillo'	=> { 'x' => 'castle',
		     't' => 'n' },
 'caver'	=> { 'x' => 'dig',
 		     't' => 'v' },
 'caverna'	=> { 'x' => 'cavern',
		     '#' => 'cave',
 		     't' => 'n' },
 'cavernoso'	=> { 'x' => 'cavernous',
 		     't' => 'a' },
 'celebrar'	=> { 'x' => 'celebrate',
		     '#' => 'praise; be glad',
 		     't' => 'v' },
 'celular'	=> { 'x' => 'cellular',
		     '#' => 'adj. or noun?' },
 'cena'		=> { 'x' => 'dinner',
		     '#' => 'eveining meal',
 		     't' => 'n' },
 'cenicero'	=> { 'x' => 'ashtray',
		     't' => 'n' },
 'centavo'	=> { 'x' => 'cent',
		     't' => 'n' },
 'cerdo'	=> { 'x' => 'pig',
		     't' => 'n' },
 'cerrar'	=> { 'x' => 'close',
		     't' => 'v' },
 'cerro'	=> { 'x' => 'hill',
 		     't' => 'n' },
 'cerveza'	=> { 'x' => 'beer',
 		     't' => 'n' },
 'ciego'	=> { 'x' => 'blind',
		     '#' => 'adj. and noun' },
 'cielo'	=> { 'x' => 'sky',
		     '#' => 'heaven',
		     't' => 'n' },
 'ciencia'	=> { 'x' => 'science',
		     't' => 'n' },
 'cierto'	=> { 'x' => 'certain',
		     't' => 'a' },
 'cifra'	=> { 'x' => 'number',
		     '#' => 'quantity, code',
 		     't' => 'n' },
 'cilindro'	=> { 'x' => 'cylinder',
		     't' => 'n' },
 'cine'		=> { 'x' => 'cinema',
		     't' => 'n' },
 'cinematografa'	=> { 'x' => 'cinematography',
 			     't' => 'n' },
 'cinematogrfica'	=> { 'x' => 'film',
		     '#' => 'cine-',
 			     't' => 'a' },
 'cinta'	=> { 'x' => 'tape',
		     '#' => 'band, strip, ribbon, reel',
 		     't' => 'n' },
 'cintura'	=> { 'x' => 'waist',
 		     't' => 'n' },
 'cinturn'	=> { 'x' => 'belt',
 		     't' => 'n' },
 'cisne'	=> { 'x' => 'swan',
		     't' => 'n' },
 'ciudad'	=> { 'x' => 'city',
		     't' => 'n' },
 'clandestino'	=> { 'x' => 'underground',
		     '#' => 'clandestine',
 		     't' => 'a' },
 'clase'	=> { 'x' => 'kind',
		     '#' => 'class',
		     't' => 'n' },
 'clausurar'	=> { 'x' => 'shut down',
		     '#' => 'close, bring to a close',
		     't' => 'v' },
 'claro'	=> { 'x' => 'clear',
		     '#' => 'also interj.',
 		     't' => 'a' },
 'clave'	=> { 'x' => 'key',
		     '#' => 'code?',
		     't' => 'n' },
 'clic'		=> { 'x' => 'click',
		     '#' => 'also verb?',
 		     't' => 'n' },
 'cliente'	=> { 'x' => 'client',
		     't' => 'n' },
 'cocer'	=> { 'x' => 'cook',
		     '#' => 'irregular',
 		     't' => 'v' },
 'cocina'	=> { 'x' => 'kitchen',
		     't' => 'n' },
 'cctel'	=> { 'x' => 'cocktail',
 		     't' => 'n',
		     'g' => 'm' },
 'coche'	=> { 'x' => 'car',
		     '#' => 'coach',
		     't' => 'n' },
 'cdigo'	=> { 'x' => 'code',
		     't' => 'n' },
 'codificacin'	=> { 'x' => 'codification',
		     't' => 'n' },
 'coincidencia'	=> { 'x' => 'coincidence',
 		     't' => 'n',
		     'g' => 'f' },
 'cojear'	=> { 'x' => 'limp',
		     't' => 'v' },
 'cola'		=> { 'x' => 'tail',
		     '#' => 'queue',
		     't' => 'n' },
 'colaborar'	=> { 'x' => 'collaborate',
		     't' => 'v' },
 'coleccin'	=> { 'x' => 'collection',
		     't' => 'n' },
 'colgar'	=> { 'x' => 'hang',
		     '#' => 'hang up',
		     't' => 'v' },
 'color'	=> { 'x' => 'colour',
		     '#' => '(am = color)',
		     't' => 'n' },
 'columna'	=> { 'x' => 'column',
		     't' => 'n' },
 'comadre'	=> { 'x' => 'gossip',
		     '#' => 'godmother, neighbour',
		     't' => 'n' },
 'comando'	=> { 'x' => 'command',
 		     't' => 'n' },
 'comentario'	=> { 'x' => 'comment',
 		     't' => 'n' },
 'comenzar'	=> { 'x' => 'commence',
		     't' => 'v' },
 'comer'	=> { 'x' => 'eat',
		     't' => 'v' },
 'comercial'	=> { 'x' => 'commercial',
 		     't' => 'a' },
 'cometa'	=> { 'x' => 'comet',
		     '#' => 'also nf kite',
 		     't' => 'n',
		     'g' => 'm' },
 'comida'	=> { 'x' => 'food',
		     't' => 'n' },
 'comillas'	=> { 'x' => 'quote marks',
		     '#' => 'multiword',
 		     't' => 'n',
		     'n' => 'p' },
 'compadecer'	=> { 'x' => 'pity',
		     't' => 'v' },
 'compaa'	=> { 'x' => 'company',
		     't' => 'n' },
 'comparar'	=> { 'x' => 'compare',
 		     't' => 'v' },
 'compartir'	=> { 'x' => 'share',
 		     't' => 'v' },
 'compatible'	=> { 'x' => 'compatible',
		     't' => 'a' },
 'compilar'	=> { 'x' => 'compile',
 		     't' => 'v' },
 'complejo'	=> { 'x' => 'complex',
 		     't' => 'a' },
 'completo'	=> { 'x' => 'complete',
		     't' => 'a' },
 'complicacin'	=> { 'x' => 'complication',
		     't' => 'n' },
 'componer'	=> { 'x' => 'arrange',
		     '#' => 'put together',
		     't' => 'v' },
 'comprar'	=> { 'x' => 'buy',
		     't' => 'v' },
 'compresin'	=> { 'x' => 'compression',
		     't' => 'n' },
 'computacin'	=> { 'x' => 'computacion',
 		     't' => 'n' },
 'computadora'	=> { 'x' => 'computer',
		     't' => 'n' },
 'comn'	=> { 'x' => 'common',
 		     't' => 'a' },
 'concierto'	=> { 'x' => 'concert',
		     '#' => 'concerto',
 		     't' => 'n' },
 'concreto'	=> { 'x' => 'concrete',
		     '#' => 'also adj.?',
 		     't' => 'n' },
 'condecoracin'	=> { 'x' => 'medal',
 			     't' => 'n',
			     'g' => 'f' },
 'condena'	=> { 'x' => 'chain',
 		     't' => 'n' },
 'condenar'	=> { 'x' => 'condemn',
		     '#' => 'convict',
 		     't' => 'v' },
 'conducir'	=> { 'x' => 'drive',
		     '#' => 'take, convey, lead',
		     't' => 'v',
		     '#' => 'castillian; cf manejar' },
 'conductor'	=> { 'x' => 'driver',
		     '#' => 'conductor',
		     't' => 'n' },
 'confeccin'	=> { 'x' => 'preparation',
 		     't' => 'n',
		     'g' => 'f' },
 'confesar'	=> { 'x' => 'confess',
		     '#' => 'admit',
 		     't' => 'v' },
 'confianza'	=> { 'x' => 'confidence',
		     't' => 'n' },
 'configuracin'	=> { 'x' => 'configuration',
			     't' => 'n',
			     'g' => 'f' },
 'conforme'	=> { 'x' => 'conformant' },
 'confortable'	=> { 'x' => 'comfortable',
 		     't' => 'a' },
 'confusin'	=> { 'x' => 'confusion',
 		     't' => 'n' },
 'congreso'	=> { 'x' => 'congress',
 		     't' => 'n' },
 'conocer'	=> { 'x' => 'know',
		     '#' => 'irreg. cf saber',
		     't' => 'v' },
 'conocimiento'	=> { 'x' => 'knowledge',
 		     't' => 'n' },
 'consecuencia'	=> { 'x' => 'consquence',
 		     't' => 'n' },
 'conseguir'	=> { 'x' => 'get',
		     '#' => 'obtain',
		     't' => 'v' },
 'consejo'	=> { 'x' => 'advice',
		     '#' => 'council',
		     't' => 'n' },
 'conservar'	=> { 'x' => 'conserve',
		     '#' => 'keep, preserve',
		     't' => 'v' },
 'conservarse'	=> { 'x' => 'survive',
		     't' => 'v' },
 'considerar'	=> { 'x' => 'consider',
 		     't' => 'v' },
 'consolar'	=> { 'x' => 'console',
		     't' => 'v' },
 'consorcio'	=> { 'x' => 'consortium',
		     't' => 'n' },
 'constante'	=> { 'x' => 'constant',
		     '#' => 'adjective and noun' },
 'construccin'	=> { 'x' => 'construction',
		     't' => 'n' },
 'consuelan'	=> { 'x' => '(they) console',
		     '#' => 'verb type 98',
		     't' => 'v',
		     'r' => 'consolar' },
 'consulta'	=> { 'x' => 'consultation',
 		     't' => 'n' },
 'consultar'	=> { 'x' => 'consult',
		     't' => 'v' },
 'contacto'	=> { 'x' => 'contact',
		     't' => 'n' },
 'contar'	=> { 'x' => 'tell',
		     '#' => 'irreg.; of a story; count',
 		     't' => 'v' },
 'contemplar'	=> { 'x' => 'contemplate',
		     '#' => 'look at',
		     't' => 'v' },
 'contener'	=> { 'x' => 'contain',
		     't' => 'v' },
 'contestar'	=> { 'x' => 'answer',
		     '#' => 'reply',
		     't' => 'v' },
 'continuar'	=> { 'x' => 'continue',
 		     't' => 'v' },
 'contrario'	=> { 'x' => 'contrary',
		     '#' => 'adj. or noun?',
 		     't' => 'a' },
 'contraste'	=> { 'x' => 'contrast',
 		     't' => 'n' },
 'contratiempo'	=> { 'x' => 'setback',
 		     't' => 'n' },
 'controlar'	=> { 'x' => 'control',
		     '#' => 'inspect, check',
		     't' => 'v' },
 'convencer'	=> { 'x' => 'convince',
 		     't' => 'v' },
 'conversacin'	=> { 'x' => 'conversation',
		     't' => 'n' },
 'conversar'	=> { 'x' => 'converse',
 		     't' => 'v' },
 'convertir'	=> { 'x' => 'convert',
		     't' => 'v' },
 'convivir'	=> { 'x' => 'live together',
		     '#' => 'multiword',
 		     't' => 'v' },
 'coordinar'	=> { 'x' => 'coordinate',
 		     't' => 'v' },
 'copa'		=> { 'x' => 'cup',
 		     't' => 'n' },
 'copiar'	=> { 'x' => 'copy',
		     't' => 'v' },
 'corazn'	=> { 'x' => 'heart',
		     't' => 'n' },
 'corbata'	=> { 'x' => 'tie',
		     '#' => 'neck tie',
 		     't' => 'n' },
 'cordn'	=> { 'x' => 'string',
 		     't' => 'n',
		     'g' => 'm' },
 'cordura'	=> { 'x' => 'wisdom',
		     '#' => 'care, need better translation',
 		     't' => 'n' },
 'corea'	=> { 'x' => 'korea',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'coreano'	=> { 'x' => 'korean',
		     '#' => 'noun and adj.?' },
 'coro'		=> { 'x' => 'chorus',
 		     't' => 'n' },
 'corona'	=> { 'x' => 'crown',
		     '#' => 'mexican beer',
 		     't' => 'n' },
 'coronacin'	=> { 'x' => 'coronation',
 		     't' => 'n',
		     'g' => 'f' },
 'coronar'	=> { 'x' => 'crown',
 		     't' => 'v' },
 'coronel'	=> { 'x' => 'colonel',
		     't' => 'n',
		     'g' => 'm',
		     '#' => 'person' },
 'corporacin'	=> { 'x' => 'corporation',
		     't' => 'n' },
 'correcto'	=> { 'x' => 'correct',
 		     't' => 'a' },
 'corregir'	=> { 'x' => 'correct',
		     '#' => 'refl. reform',
 		     't' => 'v' },
 'correo'	=> { 'x' => 'mail',
		     '#' => 'post',
		     't' => 'n' },
 'correos'	=> { 'x' => 'post office',
		     't' => 'n' },
 'correr'	=> { 'x' => 'run',
		     't' => 'v' },
 'corresponder'	=> { 'x' => 'correspond',
 		     't' => 'v' },
 'cortar'	=> { 'x' => 'cut',
 		     't' => 'v' },
 'cortina'	=> { 'x' => 'curtain',
		     't' => 'n' },
 'cosa'		=> { 'x' => 'thing',
		     '#' => '(plural often "stuff")',
		     't' => 'n' },
 'costar'	=> { 'x' => 'cost',
		     '#' => 'require, find it hard',
		     't' => 'v' },
  'cuestar'	=> { 'x' => 'cost',
		     '#' => 'virtual base',
  		     't' => 'v' },
 'coyote'	=> { 'x' => 'coyote',
		     '#' => 'animal',
 		     't' => 'n' },
 'crear'	=> { 'x' => 'create',
		     't' => 'v' },
 'crecer'	=> { 'x' => 'grow',
		     't' => 'v' },
 'creer'	=> { 'x' => 'believe',
		     '#' => 'irreg. as leer; no virtual bases',
		     't' => 'v' },
  'creo'	=> { 'x' => '(i) believe',
		     '#' => 'same as "i create"!',
 		     'r' => 'creer',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
  'crey'	=> { 'x' => '(it) believed',
		     '#' => '1ps pret.',
  		     'r' => 'creer',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
  'creyeron'	=> { 'x' => '(they) believed',
		     '#' => '3pp pret.',
  		     'r' => 'creer',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 't' },
 'crepsculo'	=> { 'x' => 'twilight',
		     '#' => 'dusk',
 		     't' => 'n' },
 'criatura'	=> { 'x' => 'creature',
		     '#' => 'animate',
 		     't' => 'n' },
 'cristiano'	=> { 'x' => 'christian',
		     '#' => 'adj. and noun?' },
 'cruel'	=> { 'x' => 'cruel',
 		     't' => 'a' },
 'crueldad'	=> { 'x' => 'cruelty',
 		     't' => 'n',
		     'g' => 'f' },
 'cubierta'	=> { 'x' => 'cover',
 		     't' => 'n' },
 'cubierto'	=> { 'x' => 'covered',
		     '#' => 'also noun = cover',
 		     't' => 'a' },
 'cubrir'	=> { 'x' => 'cover',
		     't' => 'v' },
 'cuello'	=> { 'x' => 'neck',
		     't' => 'n' },
 'crudo'	=> { 'x' => 'raw',
 		     't' => 'a' },
 'cruz'		=> { 'x' => 'cross',
 		     't' => 'n' },
 'cruzar'	=> { 'x' => 'cross',
 		     't' => 'v' },
 'cuadrado'	=> { 'x' => 'square',
		     't' => 'a' },
 'cuarto'	=> { 'x' => 'room',
		     't' => 'n' },
 'cuclilla'	=> { 'x' => 'haunch',
		     '#' => 'en/de cuclillas = squatting',
 		     't' => 'n' },
 'cuchara'	=> { 'x' => 'spoon',
 		     't' => 'n' },
 'cuchillo'	=> { 'x' => 'knife',
 		     't' => 'n' },
 'cuenta'	=> { 'x' => 'account',
		     '#' => 'also imperative of contar',
		     't' => 'n' },
 'cuento'	=> { 'x' => 'story',
		     '#' => 'also form "contar"',
 		     't' => 'n' },
 'cuerno'	=> { 'x' => 'horn',
		     '#' => 'croissant',
 		     't' => 'n' },
 'cuerpo'	=> { 'x' => 'body',
		     't' => 'n' },
 'cuervo'	=> { 'x' => 'curve',
 		     't' => 'n' },
 'cuestin'	=> { 'x' => 'matter',
		     '#' => 'question, issue',
 		     't' => 'n' },
 'cuidar'	=> { 'x' => 'look after',
		     't' => 'v' },
 'culpa'	=> { 'x' => 'fault',
		     '#' => 'guilt',
 		     't' => 'n' },
 'culpar'	=> { 'x' => 'blame',
		     '#' => 'accuse',
 		     't' => 'v' },
 'cultura'	=> { 'x' => 'culture',
 		     't' => 'n' },
 'cumpleaos'	=> { 'x' => 'birthday',
		     '#' => 'inv. sing. & plural',
 		     't' => 'n' },
 'curar'	=> { 'x' => 'cure',
 		     't' => 'v' },
 'curioso'	=> { 'x' => 'curious',
 		     't' => 'a' },
 'curso'	=> { 'x' => 'course',
		     't' => 'n' },
 'charco'	=> { 'x' => 'puddle',
		     '#' => 'pool',
 		     't' => 'n' },
 'chaqueta'	=> { 'x' => 'jacket',
		     't' => 'n' },
 'chava'	=> { 'x' => 'girl',
		     '#' => 'person',
 		     't' => 'n' },
 'chavo'	=> { 'x' => 'boy',
		     '#' => 'person',
 		     't' => 'v' },
 'checo'	=> { 'x' => 'czech',
		     '#' => 'adj. and noun?' },
 'chica'	=> { 'x' => 'girl',
		     't' => 'n' },
 'chico'	=> { 'x' => 'boy',
		     't' => 'n' },
 'chile'	=> { 'x' => 'chile',
		     '#' => 'proper noun',
		     't' => 'n' },
 'chileno'	=> { 'x' => 'chilean',
		     '#' => 'adj. and noun' },
 'chingar'	=> { 'x' => 'fuck',
		     't' => 'v' },
 'dama'		=> { 'x' => 'lady',
		     '#' => 'queen',
		     't' => 'n' },
 'danza'	=> { 'x' => 'dance',
		     't' => 'n' },
 'danzar'	=> { 'x' => 'dance',
		     't' => 'v' },
 'daar'	=> { 'x' => 'damage',
		     '#' => 'hurt',
		     't' => 'v' },
 'dao'		=> { 'x' => 'damage',
 		     't' => 'n' },
 'dar'		=> { 'x' => 'give',
		     't' => 'v' },
  # dar pres.
  'doy'		=> { 'x' => '(i) give',
  		     'r' => 'dar',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
  'dais'	=> { 'x' => '(you) give',
  		     'r' => 'dar',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'p' },
  # dar pres. subj.
  'd'		=> { 'x' => '(it) gives',
		     '#' => 'i',
  		     'r' => 'dar',
		     't' => 'v',
		     'n' => 's',
		     'c' => 'ps' },
  'ds'		=> { 'x' => '(you) give',
  		     'r' => 'dar',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 'ps' },
  'deis'	=> { 'x' => '(you) give',
  		     'r' => 'dar',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 'ps' },
  'den'		=> { 'x' => '(they) give',
  		     'r' => 'dar',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 'ps' },
  # dar pret.
  'di'		=> { 'x' => '(i) gave',
		     '#' => 'also sing. imp. decir',
		     'r' => 'dar',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
  'diste'	=> { 'x' => '(you) gave',
		     'r' => 'dar',
		     't' => 'v',
		     'p' => '2',
		     'n' => 's',
		     'c' => 't' },
  'dio'		=> { 'x' => '(it) gave',
  		     'r' => 'dar',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
  'dimos'	=> { 'x' => '(we) gave',
		     'r' => 'dar',
		     't' => 'v',
		     'p' => '1',
		     'n' => 'p',
		     'c' => 't' },
  'disteis'	=> { 'x' => '(you) gave',
		     'r' => 'dar',
		     't' => 'v',
		     'p' => '2',
		     'n' => 'p',
		     'c' => 't' },
  'dieron'	=> { 'x' => '(they) gave',
		     'r' => 'dar',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 't' },
 'dato'		=> { 'x' => 'fact',
		     '#' => 'piece of information',
		     't' => 'n' },
 'debajo'	=> { 'x' => 'underneath',
		     '#' => 'below, under',
 		     't' => 'adv',
		     '#' => 'dict. says adv.' },
 'debate'	=> { 'x' => 'debate',
 		     't' => 'n',
		     'g' => 'm' },
 'debatir'	=> { 'x' => 'debate',
 		     't' => 'v' },
 'dcada'	=> { 'x' => 'decade',
 		     't' => 'n' },
 'decidir'	=> { 'x' => 'decide',
		     't' => 'v' },
 'decir'	=> { 'x' => 'say',
		     '#' => 'tell',
		     't' => 'v' },
  'dicho'	=> { 'x' => 'said',
  		     'r' => 'decir',
		     't' => 'v',
		     'c' => 'pp' },
  'diciendo'	=> { 'x' => 'saying',
  		     'r' => 'decir',
		     't' => 'v',
		     'c' => 'g' },
  # decir imperative
#  'di'		=> { 'x' => 'say',
#		     '#' => 'tell; also 1p pret. dar',
#  		     'r' => 'decir',
#		     't' => 'v',
#		     'p' => '2',
#		     'n' => 'p',
#		     'c' => 'imp' },
  # decir pres., pres. subj.
  'digir'	=> { 'x' => 'say',
		     '#' => 'virtual base pres. pres. subj.',
  		     'r' => 'decir',
		     '#' => 'not pres. 1pp/2pp (confl. dar)',
		     't' => 'v' },
  # decir pres.
  'dicir'	=> { 'x' => 'say',
		     '#' => 'virtual base',
  		     'r' => 'decir',
		     't' => 'v',
		     'c' => 'p' },
  # decir fut., condit.
  'dir'		=> { 'x' => 'say',
  		     'r' => 'decir',
		     't' => 'v' },
  # decir pret.
  'dijir'	=> { 'x' => 'say',
		     '#' => 'virtual base',
  		     'r' => 'decir',
		     't' => 'v' },
  'dijeron'	=> { 'x' => '(they) said',
  		     'r' => 'decir',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 't' },
 'decisivo'	=> { 'x' => 'decisive',
 		     't' => 'a' },
 'declaracin'	=> { 'x' => 'declaration',
 		     't' => 'n' },
 'dedicar'	=> { 'x' => 'dedicate',
		     't' => 'v' },
 'dedo'		=> { 'x' => 'finger',
		     't' => 'n' },
 'defecto'	=> { 'x' => 'defect',
		     '#' => 'flaw',
		     't' => 'n' },
 'defensa'	=> { 'x' => 'defense',
 		     't' => 'n' },
 'defensivo'	=> { 'x' => 'defensive',
 		     't' => 'a' },
 'definir'	=> { 'x' => 'define',
		     '#' => 'determine, establish, clarify',
		     't' => 'v' },
 'dejar'	=> { 'x' => 'leave',
		     't' => 'v' },
 'delantero'	=> { 'x' => 'front',
		     't' => 'a' },
 'delgado'	=> { 'x' => 'thin',
		     't' => 'a' },
 'departamento'	=> { 'x' => 'department',
		     't' => 'n' },
 'deporte'	=> { 'x' => 'sport',
 		     't' => 'n',
		     'g' => 'm' },
 'deprisa'	=> { 'x' => 'quickly',
		     '#' => 'hurriedly',
 		     't' => 'adv' },
 'derecho'	=> { 'x' => 'right',
		     '#' => 'as in "human rights"',
 		     't' => 'n' },
 'desanimar'	=> { 'x' => 'discourage',
		     '#' => 'depress',
 		     't' => 'v' },
 'desaparecer'	=> { 'x' => 'disappear',
		     't' => 'v' },
 'desarrollar'	=> { 'x' => 'develop',
		     't' => 'v' },
 'desastre'	=> { 'x' => 'disaster',
		     't' => 'n' },
 'desayunar'	=> { 'x' => 'breakfast',
		     't' => 'v' },
 'descalzar'	=> { 'x' => 'remove',
		     '#' => 'take off; only of shoes',
 		     't' => 'v' },
 'descalzo'	=> { 'x' => 'barefoot',
		     '#' => 'bare - only of feet',
 		     't' => 'a' },
 'descansar'	=> { 'x' => 'rest',
 		     't' => 'v' },
 'desconocido'	=> { 'x' => 'unknown',
		     't' => 'a' },
 'descriptivo'	=> { 'x' => 'descriptive',
		     't' => 'a' },
 'descubrir'	=> { 'x' => 'discover',
 		     't' => 'v' },
 'desear'	=> { 'x' => 'wish',
		     't' => 'v' },
 'desengaar'	=> { 'x' => 'disillusion',
 		     't' => 'v' },
 'desengao'	=> { 'x' => 'disillusionment',
		     '#' => 'disappointment',
 		     't' => 'n' },
 'deseo'	=> { 'x' => 'desire',
		     '#' => 'wish',
 		     't' => 'n' },
 'desesperar'	=> { 'x' => 'exasperate',
		     't' => 'v' },
 'desquiciado'	=> { 'x' => 'deranged',
 		     't' => 'a' },
 'desechar'	=> { 'x' => 'discard',
		     't' => 'v' },
 'desierto'	=> { 'x' => 'desert',
		     't' => 'n' },
 'desmentir'	=> { 'x' => 'contradict',
		     '#' => 'irreg; deny',
 		     't' => 'v' },
 'desnudo'	=> { 'x' => 'naked',
		     't' => 'a' },
 'desorbitado'	=> { 'x' => 'boundless',
		     't' => 'a' },
 'despacho'	=> { 'x' => 'office',
		     't' => 'n' },
 'despedir'	=> { 'x' => 'dismiss',
		     '#' => 'see off, show out',
		     't' => 'v' },
 'despeadero'	=> { 'x' => 'cliff',
		     '#' => 'precipice',
		     't' => 'n' },
 'desperdicios'	=> { 'x' => 'rubbish',
		     't' => 'n' },
 'despido'	=> { 'x' => 'dismissed',
		     '#' => 'p.p.',
		     't' => 'v',
		     'r' => 'despedir' },
 'despierto'	=> { 'x' => 'awake',
 		     't' => 'a' },
 'destartalado'	=> { 'x' => 'untidy',
		     '#' => 'tumbledown',
 		     't' => 'a' },
 'desteir'	=> { 'x' => 'fade',
 		     't' => 'v' },
 'destino'	=> { 'x' => 'destination',
		     '#' => 'destiny',
 		     't' => 'n' },
 'detalle'	=> { 'x' => 'detail',
		     't' => 'n' },
 'detener'	=> { 'x' => 'stop',
		     '#' => 'arrest',
 		     't' => 'v' },
  # modelled on 'tener'
  'deten'	=> { 'x' => 'stop',
		     '#' => 'imperative',
  		     'r' => 'detener',
		     't' => 'v',
		     'n' => 's',
		     'c' => 'imp' },
  'detenger'	=> { 'x' => 'stop',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'detener',
		     't' => 'v' },
  'detiener'	=> { 'x' => 'stop',
		     '#' => 'virtual base pres.',
  		     'r' => 'detener',
		     't' => 'v' },
  'detendr'	=> { 'x' => 'stop',
		     '#' => 'virtual base fut., condit.',
  		     'r' => 'detener',
		     't' => 'v' },
  'detuver'	=> { 'x' => 'stop',
		     '#' => 'virtual base pret.',
  		     'r' => 'detener',
		     't' => 'v' },
  'detuve'	=> { 'x' => '(it) stopped',
		     'r' => 'detener',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
  'detuvo'	=> { 'x' => '(i) stopped',
		     'r' => 'detener',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
 'devorar'	=> { 'x' => 'devour',
 		     't' => 'v' },
 # di - see dar
 'da'		=> { 'x' => 'day',
		     't' => 'n',
		     'g' => 'm' },
 'diablo'	=> { 'x' => 'devil',
		     '#' => 'animate',
 		     't' => 'n' },
 'diagrama'	=> { 'x' => 'diagram',
 		     't' => 'n',
		     'g' => 'm' },
 'dimetro'	=> { 'x' => 'diameter',
		     't' => 'n' },
 'diario'	=> { 'x' => 'diary',
 		     't' => 'n' },
 'dibujo'	=> { 'x' => 'drawing',
 		     't' => 'n' },
 'dibujar'	=> { 'x' => 'draw',
		     '#' => 'sketch',
 		     't' => 'v' },
 'diccionario'	=> { 'x' => 'dictionary',
 		     't' => 'n' },
 # dice, dices, dicen - see decir
 'diente'	=> { 'x' => 'tooth',
		     't' => 'n' },
 'diferencia'	=> { 'x' => 'difference',
 		     't' => 'n' },
 'diferente'	=> { 'x' => 'different',
 		     't' => 'a' },
 'difcil'	=> { 'x' => 'difficult',
 		     't' => 'a' },
 'dignidad'	=> { 'x' => 'dignity',
 		     't' => 'n' },
 # digo - see decir
 'dinamarca'	=> { 'x' => 'denmark',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'dinamarqus'	=> { 'x' => 'danish',
		     '#' => 'noun and adj.?' },
 'dinmico'	=> { 'x' => 'dynamic',
 		     't' => 'a' },
 'dinamita'	=> { 'x' => 'dynamite',
		     't' => 'n' },
 'dinero'	=> { 'x' => 'money',
		     't' => 'n' },
 'dios'		=> { 'x' => 'god',
		     't' => 'n' },
 'direccin'	=> { 'x' => 'direction',
		     't' => 'n' },
 'directo'	=> { 'x' => 'direct',
 		     't' => 'a' },
 'director'	=> { 'x' => 'director',
 		     't' => 'n' },
 'directorio'	=> { 'x' => 'directory',
 		     't' => 'n' },
 'dirigir'	=> { 'x' => 'direct',
 		     't' => 'v' },
 'disco'	=> { 'x' => 'disk',
		     't' => 'n' },
 'disconforme'	=> { 'x' => 'differing',
		     't' => 'a' },
 'disculpar'	=> { 'x' => 'excuse',
		     '#' => 'pardon',
		     't' => 'v' },
 'discutir'	=> { 'x' => 'discuss',
		     '#' => 'argue',
 		     't' => 'v' },
 'disear'	=> { 'x' => 'design',
		     't' => 'v' },
 'diseo'	=> { 'x' => 'design',
		     't' => 'n' },
 'disfrutar'	=> { 'x' => 'enjoy',
 		     't' => 'v' },
 'disponer'	=> { 'x' => 'arrange',
		     '#' => 'put in order, prepare',
		     't' => 'v' },
 'disponibilidad'	=> { 'x' => 'availability',
			     't' => 'n' },
 'disponible'	=> { 'x' => 'available',
 		     't' => 'a' },
 'distancia'	=> { 'x' => 'distance',
 		     't' => 'n' },
 'distante'	=> { 'x' => 'distant',
 		     't' => 'a' },
 'distincin'	=> { 'x' => 'distinction',
 		     't' => 'n' },
 'distinto'	=> { 'x' => 'distinct',
		     '#' => 'different',
		     't' => 'a' },
 'distribucin'	=> { 'x' => 'distribution',
		     't' => 'n' },
 'distribuidor'	=> { 'x' => 'distributor',
		     't' => 'v' },
 'distrito'	=> { 'x' => 'district',
 		     't' => 'n' },
 'diverso'	=> { 'x' => 'diverse',
 		     't' => 'a' },
 'divulgar'	=> { 'x' => 'spread',
		     '#' => 'divulge, circulate',
		     't' => 'v' },
 'doblar'	=> { 'x' => 'double',
 		     't' => 'v' },
 'doble'	=> { 'x' => 'double',
		     '#' => 'also noun',
 		     't' => 'a' },
 'doctor'	=> { 'x' => 'doctor',
		     '#' => 'person',
		     't' => 'n',
		     'g' => 'm' },
 'docu'		=> { 'x' => 'doco',
		     '#' => 'documentary (slang, abbreviation)',
 		     't' => 'n',
		     'g' => 'm' },
 'documental'	=> { 'x' => 'documentary',
		     '#' => 'adj & nm' },
 'documento'	=> { 'x' => 'document',
		     't' => 'n' },
 'dlar'	=> { 'x' => 'dollar',
 		     't' => 'n',
		     'g' => 'm' },
 'doler'	=> { 'x' => 'hurt',
		     '#' => 'irreg.; grieve',
 		     't' => 'v' },
  'dueler'	=> { 'x' => 'hurt',
  		     'r' => 'doler',
		     't' => 'v' },
 'dolor'	=> { 'x' => 'pain',
		     '#' => 'grief, sorrow',
 		     't' => 'n',
		     'g' => 'm' },
 'domar'	=> { 'x' => 'tame',
 		     't' => 'v' },
 'dormir'	=> { 'x' => 'sleep',
		     '#' => 'irreg.',
		     't' => 'v' },
 'duda'		=> { 'x' => 'doubt',
		     '#' => '"sin duda(s)" is common',
		     't' => 'n' },
 'dueo'	=> { 'x' => 'owner',
		     '#' => 'landlord, owner',
 		     't' => 'n' },
 'dulce'	=> { 'x' => 'sweet',
 		     't' => 'a' },
 'do'		=> { 'x' => 'duo',
 		     't' => 'n' },
 'duracin'	=> { 'x' => 'duration',
 		     't' => 'n' },
 'duro'		=> { 'x' => 'hard',
		     't' => 'a' },
 'economa'	=> { 'x' => 'economy',
		     't' => 'n' },
 'echar'	=> { 'x' => 'throw',
		     't' => 'v' },
 'edad'		=> { 'x' => 'age',
 		     't' => 'n',
		     'g' => 'f' },
 'edicin'	=> { 'x' => 'edition',
		     't' => 'n' },
 'edificio'	=> { 'x' => 'building',
		     't' => 'n' },
 'editar'	=> { 'x' => 'edit',
 		     't' => 'v' },
 'educacin'	=> { 'x' => 'education',
 		     't' => 'n' },
 'efectivo'	=> { 'x' => 'cash',
 		     't' => 'n' },
 'efecto'	=> { 'x' => 'effect',
 		     't' => 'n' },
 'egresar'	=> { 'x' => 'withdraw',
 		     't' => 'v' },
 'eje'		=> { 'x' => 'axle',
		     '#' => 'axis',
		     't' => 'n' },
 'ejecucin'	=> { 'x' => 'execution',
 		     't' => 'n' },
 'ejecutar'	=> { 'x' => 'execute',
 		     't' => 'v' },
 'ejemplo'	=> { 'x' => 'example',
		     't' => 'n' },
 'ejercicio'	=> { 'x' => 'exercise',
		     '#' => 'tenure',
 		     't' => 'n' },
 'ejrcito'	=> { 'x' => 'army',
 		     't' => 'n' },
 'elaborar'	=> { 'x' => 'make',
		     't' => 'v' },
 'elstica'	=> { 'x' => 'flexible',
 		     't' => 'a' },
 'electoral'	=> { 'x' => 'electoral',
 		     't' => 'a' },
 'electricidad'	=> { 'x' => 'electricity',
 		     't' => 'n' },
 'electrnico'	=> { 'x' => 'electronic',
		     't' => 'a' },
 'elegante'	=> { 'x' => 'elegant',
		     '#' => 'graceful, stylish, fashionable',
		     't' => 'a' },
 'elegir'	=> { 'x' => 'choose',
		     '#' => 'select, elect',
 		     't' => 'v' },
 'elemento'	=> { 'x' => 'element',
		     't' => 'n' },
 'elevacin'	=> { 'x' => 'elevation',
		     '#' => 'raising, lifting, rise, height',
		     't' => 'n',
		     'g' => 'f' },
 'embargo'	=> { 'x' => 'embargo',
		     '#' => 'seizure; sin embargo = however, still',
 		     't' => 'n' },
 'emborrachar'	=> { 'x' => 'intoxicate',
		     '#' => 'make drunk',
 		     't' => 'v' },
 'emigrar'	=> { 'x' => 'migrate',
		     '#' => 'emigrate',
 		     't' => 'v' },
 'empezar'	=> { 'x' => 'begin',
		     '#' => 'irregular',
		     't' => 'v' },
 'empresa'	=> { 'x' => 'enterprise',
		     '#' => 'company',
		     't' => 'n' },
 'emular'	=> { 'x' => 'emulate',
		     't' => 'v' },
 'enamorado'	=> { 'x' => 'in love',
		     '#' => 'enamoured',
 		     't' => 'a' },
 'enamorar'	=> { 'x' => 'fall in love',
		     '#' => 'enamour',
		     't' => 'v' },
 'enano'	=> { 'x' => 'dwarf',
		     '#' => 'tiny; noun and adj.' },
 'encantar'	=> { 'x' => 'charm',
		     '#' => 'delight',
		     't' => 'v' },
 'encontrar'	=> { 'x' => 'find',
		     't' => 'v' },
  'encuentrar'	=> { 'x' => '(you) find',
		     '#' => 'virtual base',
		     't' => 'v',
		     'r' => 'encontrar' },
 'endulzar'	=> { 'x' => 'sweeten',
 		     't' => 'v' },
 'enfermo'	=> { 'x' => 'sick',
		     '#' => 'ill',
		     't' => 'a' },
 'enfrentar'	=> { 'x' => 'face',
		     '#' => 'confront',
 		     't' => 'v' },
 'engaar'	=> { 'x' => 'deceive',
		     '#' => 'cheat, swindle',
 		     't' => 'v' },
 'enjuiciar'	=> { 'x' => 'judge',
		     't' => 'v' },
 'enlace'	=> { 'x' => 'link',
 		     't' => 'n',
		     'g' => 'm' },
 'enlazar'	=> { 'x' => 'link',
		     '#' => 'tie, bind, connect',
 		     't' => 'v' },
 'enojar'	=> { 'x' => 'anger',
		     '#' => 'annoy, upset',
		     't' => 'v' },
 'enorme'	=> { 'x' => 'enormous',
 		     't' => 'a' },
 'encuesta'	=> { 'x' => 'inquiry',
		     '#' => 'investigation, opinion poll',
 		     't' => 'n' },
 'ensear'	=> { 'x' => 'teach',
		     't' => 'v' },
 'entender'	=> { 'x' => 'understand',
 		     't' => 'v' },
  'entiender'	=> { 'x' => 'understand',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'entender',
		     't' => 'v' },
 'enterar'	=> { 'x' => 'inform',
		     '#' => 'tell',
		     't' => 'v' },
 'enterarse'	=> { 'x' => 'find out',
		     '#' => '(2 words), get to know' },
 'entero'	=> { 'x' => 'entire',
 		     't' => 'a' },
 'enterrar'	=> { 'x' => 'bury',
 		     't' => 'v' },
 'entrada'	=> { 'x' => 'entrance',
		     '#' => 'entered' },
 'entrar'	=> { 'x' => 'enter',
		     't' => 'v' },
 'entrevista'	=> { 'x' => 'interview',
 		     't' => 'n' },
 'entusiasmo'	=> { 'x' => 'enthusiasm',
 		     't' => 'n' },
 'enviar'	=> { 'x' => 'send',
		     't' => 'v' },
 'envidia'	=> { 'x' => 'envy',
		     '#' => 'jealousy',
 		     't' => 'n' },
 'envidioso'	=> { 'x' => 'envious',
		     '#' => 'jealous',
 		     't' => 'a' },
 'episodio'	=> { 'x' => 'episode',
 		     't' => 'n' },
 'equilibrio'	=> { 'x' => 'equilibrium',
 		     't' => 'n' },
 'erguir'	=> { 'x' => 'raise',
		     '#' => 'lift, straighten',
		     't' => 'v' },
 'error'	=> { 'x' => 'error',
		     't' => 'n' },
 'escalera'	=> { 'x' => 'stair',
		     '#' => 'step',
 		     't' => 'n' },
 'escao'	=> { 'x' => 'bench',
		     't' => 'n' },
 'escapar'	=> { 'x' => 'escape',
 		     't' => 'v' },
 'escaso'	=> { 'x' => 'scarce',
		     '#' => 'thin, sparse',
 		     't' => 'a' },
 'escatimar'	=> { 'x' => 'skimp',
		     't' => 'v' },
 'esclava'	=> { 'x' => 'slave',
		     '#' => 'person',
 		     't' => 'n' },
 'esclavo'	=> { 'x' => 'slave',
		     '#' => 'person',
 		     't' => 'n' },
 'escolar'	=> { 'x' => 'school',
		     '#' => 'also n schoolboy, pupil',
 		     't' => 'a' },
 'esconder'	=> { 'x' => 'hide',
		     't' => 'v' },
 'escuchar'	=> { 'x' => 'listen',
		     't' => 'v' },
 'escudo'	=> { 'x' => 'shield',
 		     't' => 'n' },
 'escuela'	=> { 'x' => 'school',
		     't' => 'n' },
 'escribir'	=> { 'x' => 'write',
		     't' => 'v' },
 'escrito'	=> { 'x' => 'written',
		     '#' => 'form of escribir',
		     't' => 'a' },
 'escritor'	=> { 'x' => 'writer',
		     't' => 'n' },
 'escritorio'	=> { 'x' => 'desk',
		     '#' => 'desktop',
 		     't' => 'n' },
 'escritura'	=> { 'x' => 'writing',
		     '#' => 'handwriting, deed',
 		     't' => 'n',
		     'g' => 'f' },
 'esfuerzo'	=> { 'x' => 'force',
		     't' => 'n' },
 'espacio'	=> { 'x' => 'space',
		     't' => 'n' },
 'espada'	=> { 'x' => 'sword',
 		     't' => 'n' },
 'espalda'	=> { 'x' => 'back',
		     't' => 'n' },
 'espaa'	=> { 'x' => 'spain',
		     '#' => 'capitalize',
		     't' => 'n' },
 'espaol'	=> { 'x' => 'spanish',
		     '#' => 'noun and adj.?' },
 'espaola'	=> { 'x' => 'spanish',
 		     't' => 'a' },
 'especial'	=> { 'x' => 'special',
		     't' => 'a' },
 'especificar'	=> { 'x' => 'specify',
		     't' => 'v' },
 'especfico'	=> { 'x' => 'specific',
		     't' => 'a' },
 'espejo'	=> { 'x' => 'mirror',
		     't' => 'n' },
 'espera'	=> { 'x' => 'wait',
		     '#' => 'waiting' },
 'esperanto'	=> { 'x' => 'esperanto',
 		     't' => 'n' },
 'esperar'	=> { 'x' => 'wait',
		     '#' => 'wait for',
		     't' => 'v' },
 'esperanza'	=> { 'x' => 'hope',
		     '#' => 'expectation',
 		     't' => 'n' },
 'espesor'	=> { 'x' => 'thickness',
		     't' => 'n' },
 'espiritual'	=> { 'x' => 'spiritual',
		     't' => 'a' },
 'esponjoso'	=> { 'x' => 'spongy',
 		     't' => 'a' },
 'esquina'	=> { 'x' => 'corner',
 		     't' => 'n' },
 'estable'	=> { 'x' => 'stable',
 		     't' => 'a' },
 'estacionar'	=> { 'x' => 'park',
 		     't' => 'v' },
 'estadstica'	=> { 'x' => 'statistic',
 		     't' => 'n' },
 'estancar'	=> { 'x' => 'block',
		     '#' => 'hold up',
 		     't' => 'v' },
 'estndar'	=> { 'x' => 'standard',
		     't' => 'n' },
 'estanque'	=> { 'x' => 'pool',
		     '#' => 'pond, reservoir',
 		     't' => 'n',
		     'g' => 'm' },
 'estilo'	=> { 'x' => 'style',
		     't' => 'n' },
 'estrago'	=> { 'x' => 'ruin',
		     '#' => 'destruction',
 		     't' => 'n' },
 'estrecho'	=> { 'x' => 'narrow',
 		     't' => 'a' },
 'estrella'	=> { 'x' => 'star',
		     't' => 'n' },
 'estrenar'	=> { 'x' => 'debut',
		     '#' => 'wear for the first time, move into (house)',
 		     't' => 'v',
		     '#' => 'premiere (movie)' },
 'estructura'	=> { 'x' => 'structure',
 		     't' => 'n' },
 'estudiar'	=> { 'x' => 'study',
		     't' => 'v' },
 'estupor'	=> { 'x' => 'stupor',
		     '#' => 'astonishment, amazement',
 		     't' => 'n',
		     'g' => 'm' },
 'etc'		=> { 'x' => 'etc' },
 'europa'	=> { 'x' => 'europe',
		     '#' => 'proper; place',
		     't' => 'n' },
 'europeo'	=> { 'x' => 'european',
		     't' => 'a' },
 'evitar'	=> { 'x' => 'avoid',
 		     't' => 'v' },
 'evolucin'	=> { 'x' => 'evolution',
 		     't' => 'n' },
 'exacto'	=> { 'x' => 'exact',
		     't' => 'a' },
 'exagerar'	=> { 'x' => 'exaggerate',
 		     't' => 'v' },
 'examinar'	=> { 'x' => 'examine',
		     '#' => 'refl. be examined, have an examination',
 		     't' => 'v' },
 'excluir'	=> { 'x' => 'exclude',
		     't' => 'v' },
 'exclusiva'	=> { 'x' => 'exclusive',
 		     't' => 'n' },
 'exclusivo'	=> { 'x' => 'exclusive',
 		     't' => 'a' },
 'xito'	=> { 'x' => 'result',
		     '#' => 'outcome, success',
		     't' => 'n' },
 'existir'	=> { 'x' => 'exist',
 		     't' => 'v' },
 'exorcizar'	=> { 'x' => 'exorcize',
 		     't' => 'v' },
 'experiencia'	=> { 'x' => 'experience',
		     't' => 'n',
		     'g' => 'f' },
 'experimento'	=> { 'x' => 'experiment',
 		     't' => 'n' },
 'explicacin'	=> { 'x' => 'explanation',
 		     't' => 'n',
		     'g' => 'f' },
 'explicar'	=> { 'x' => 'explain',
		     '#' => 'irreg.',
 		     't' => 'v' },
 'explosin'	=> { 'x' => 'explosion',
 		     't' => 'n' },
 'expresar'	=> { 'x' => 'express',
		     't' => 'v' },
 'expulsin'	=> { 'x' => 'expulsion',
 		     't' => 'n',
		     'g' => 'f' },
 'extender'	=> { 'x' => 'extend',
 		     't' => 'v' },
  'extiender'	=> { 'x' => 'extend',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'extender',
		     't' => 'v' },
 'extensible'	=> { 'x' => 'extensible',
 		     't' => 'a' },
 'extraccin'	=> { 'x' => 'extraction',
 		     't' => 'n' },
 'extraar'	=> { 'x' => 'miss',
		     't' => 'v' },
 'extranjero'	=> { 'x' => 'stranger',
		     't' => 'n' },
 'extrao'	=> { 'x' => 'strange',
		     't' => 'a' },
 'extremo'	=> { 'x' => 'extreme',
		     '#' => 'noun too?',
 		     't' => 'a' },
 'fabuloso'	=> { 'x' => 'fabulous',
 		     't' => 'a' },
 'fcil'	=> { 'x' => 'easy',
 		     't' => 'a' },
 'facultad'	=> { 'x' => 'faculty',
		     't' => 'n',
		     'g' => 'f' },
 'falda'	=> { 'x' => 'skirt',
 		     't' => 'n' },
 'fallar'	=> { 'x' => 'fail',
		     '#' => 'miss',
		     't' => 'v' },
 'faltar'	=> { 'x' => 'lack',
 		     't' => 'v' },
 'familia'	=> { 'x' => 'family',
 		     't' => 'n' },
 'familiar'	=> { 'x' => 'familiar',
		     't' => 'a' },
 'fantico'	=> { 'x' => 'fan',
		     '#' => 'fanatic; also adj. fanatical',
 		     't' => 'n' },
 'fanatismo'	=> { 'x' => 'fanaticism',
 		     't' => 'n' },
 'fantasma'	=> { 'x' => 'ghost',
 		     't' => 'n' },
 'faro'		=> { 'x' => 'headlamp',
		     '#' => 'lighthouse',
 		     't' => 'n' },
 'favorito'	=> { 'x' => 'favourite',
 		     't' => 'a' },
 'fe'		=> { 'x' => 'faith',
		     '#' => 'certificate',
 		     't' => 'n',
		     'g' => 'f' },
 'fecha'	=> { 'x' => 'date',
 		     't' => 'n' },
 'federacin'	=> { 'x' => 'federation',
 		     't' => 'n' },
 'federal'	=> { 'x' => 'federal',
 		     't' => 'a' },
 'feliz'	=> { 'x' => 'happy',
 		     't' => 'a' },
 'feo'		=> { 'x' => 'ugly',
 		     't' => 'a' },
 'ficha'	=> { 'x' => 'token',
 		     't' => 'n' },
 'fichero'	=> { 'x' => 'file',
 		     't' => 'n' },
 'fiel'		=> { 'x' => 'faithful',
		     '#' => 'loyal',
 		     't' => 'a' },
 'fiesta'	=> { 'x' => 'party',
 		     't' => 'n' },
 'figura'	=> { 'x' => 'figure',
		     '#' => 'shape, form',
		     't' => 'n' },
 'figurar'	=> { 'x' => 'figure',
		     '#' => 'represent',
 		     't' => 'v' },
 'fijar'	=> { 'x' => 'fix',
		     '#' => 'affix, stick',
 		     't' => 'v' },
 'fijo'		=> { 'x' => 'fixed',
		     '#' => 'firm, permanent',
 		     't' => 'a' },
 'filmar'	=> { 'x' => 'film',
		     '#' => 'shoot',
 		     't' => 'v' },
 'fin'		=> { 'x' => 'end',
 		     't' => 'n' },
 'final'	=> { 'x' => 'end',
		     't' => 'n' },
 'finalmente'	=> { 'x' => 'finally',
 		     't' => 'adv' },
 'fingir'	=> { 'x' => 'fake',
		     '#' => 'simulate',
 		     't' => 'v' },
 'finlands'	=> { 'x' => 'finnish',
		     '#' => 'noun and adj.?' },
 'firmar'	=> { 'x' => 'sign',
		     't' => 'v' },
 'firme'	=> { 'x' => 'firm',
 		     't' => 'a' },
 'fsico'	=> { 'x' => 'physical',
		     't' => 'a' },
 'flaco'	=> { 'x' => 'skinny',
		     '#' => 'thin, weak, feeble',
 		     't' => 'a' },
 'flecha'	=> { 'x' => 'arrow',
 		     't' => 'n' },
 'flojo'	=> { 'x' => 'lazy',
 		     't' => 'a' },
 'flor'		=> { 'x' => 'flower',
 		     't' => 'n' },
 'flotar'	=> { 'x' => 'float',
 		     't' => 'v' },
 'fondo'	=> { 'x' => 'bottom',
		     '#' => 'back; fund',
 		     't' => 'n' },
 'foto'		=> { 'x' => 'photo',
 		     't' => 'n' },
 'fotografa'	=> { 'x' => 'photography',
 		     't' => 'n' },
 'forma'	=> { 'x' => 'form',
 		     't' => 'n' },
 'formar'	=> { 'x' => 'form',
 		     't' => 'v' },
 'francs'	=> { 'x' => 'french',
		     '#' => 'noun and adj.?' },
 'francia'	=> { 'x' => 'france',
		     '#' => 'proper; place',
		     't' => 'n' },
 'frase'	=> { 'x' => 'phrase',
		     't' => 'n' },
 'fraude'	=> { 'x' => 'fraud',
		     '#' => 'dishonesty, also adj.',
 		     't' => 'n',
		     'g' => 'm' },
 'frecuencia'	=> { 'x' => 'frequency',
		     't' => 'n' },
 'frente'	=> { 'x' => 'front',
 		     't' => 'n' },
 'fresa'	=> { 'x' => 'strawberry',
 		     't' => 'n' },
 'fresco'	=> { 'x' => 'fresh',
 		     't' => 'a' },
 'frjol'	=> { 'x' => 'bean',
 		     't' => 'n' },
 'fro'		=> { 'x' => 'cold',
 		     't' => 'a' },
 'frontera'	=> { 'x' => 'border',
 		     't' => 'n' },
 'fruta'	=> { 'x' => 'fruit',
 		     't' => 'n' },
 'fuego'	=> { 'x' => 'fire',
 		     't' => 'n' },
 'fuente'	=> { 'x' => 'source',
 		     't' => 'n' },
 'fuerte'	=> { 'x' => 'strong',
 		     't' => 'a' },
 'fuerza'	=> { 'x' => 'force',
		     '#' => 'strength, power',
 		     't' => 'n' },
 'funcin'	=> { 'x' => 'function',
 		     't' => 'n' },
 'funcionar'	=> { 'x' => 'function',
		     't' => 'v' },
 'fundamento'	=> { 'x' => 'fundamental',
		     't' => 'n' },
 'ftbol'	=> { 'x' => 'football',
		     '#' => 'soccer',
 		     't' => 'n' },
 'futuro'	=> { 'x' => 'future',
 		     't' => 'n' },
 'galaxia'	=> { 'x' => 'galaxy',
 		     't' => 'n' },
 'ganar'	=> { 'x' => 'earn',
		     '#' => 'get, obtain, gain, win',
		     't' => 'v' },
 'garanta'	=> { 'x' => 'guarantee',
 		     't' => 'n' },
 'gasolina'	=> { 'x' => 'petrol',
		     '#' => '(gasoline u.s.)',
 		     't' => 'n' },
 'gastar'	=> { 'x' => 'spend',
		     '#' => 'use up, waste, wear',
 		     't' => 'v' },
 'gato'		=> { 'x' => 'cat',
		     '#' => 'animate',
 		     't' => 'n' },
 'gaviota'	=> { 'x' => 'seagull',
 		     't' => 'n' },
 'general'	=> { 'x' => 'general',
		     '#' => 'noun and adj.?',
 		     't' => 'n' },
 'generar'	=> { 'x' => 'generate',
 		     't' => 'v' },
 'genio'	=> { 'x' => 'nature',
		     '#' => 'temper, disposition; genius',
 		     't' => 'n' },
 'gente'	=> { 'x' => 'people',
 		     't' => 'n',
		     'n' => 'p' },
 'gesticular'	=> { 'x' => 'gesticulate',
 		     't' => 'v' },
 'gestin'	=> { 'x' => 'management',
		     '#' => 'negotiation',
 		     't' => 'n',
		     'g' => 'f' },
 'gestionar'	=> { 'x' => 'manage',
		     '#' => 'try to arrange',
 		     't' => 'v' },
 'gigante'	=> { 'x' => 'giant',
 		     't' => 'a' },
 'girar'	=> { 'x' => 'spin',
		     '#' => 'turn',
 		     't' => 'v' },
 'gobierno'	=> { 'x' => 'government',
		     't' => 'n' },
 'golpe'	=> { 'x' => 'blow',
		     't' => 'n' },
 'gordo'	=> { 'x' => 'fat',
 		     't' => 'a' },
 'gozo'		=> { 'x' => 'joy',
		     '#' => 'pleasure',
		     't' => 'n' },
 'gracias'	=> { 'x' => 'thankyou',
		     '#' => 'thanks; noun and interj.?' },
 'gran'		=> { 'x' => 'great',
 		     't' => 'a' },
 'grande'	=> { 'x' => 'great',
 		     't' => 'a' },
 'grandote'	=> { 'x' => 'giant',
		     '#' => '(argentina)',
 		     't' => 'n',
		     'g' => 'm' },
 'granizar'	=> { 'x' => 'hail',
 		     't' => 'v' },
 'gratis'	=> { 'x' => 'free',
		     '#' => 'cf libre; adj. and adv.',
		     't' => 'adv' },
 'gratuito'	=> { 'x' => 'free',
		     '#' => 'gratuitious',
		     't' => 'a' },
 'grave'	=> { 'x' => 'heavy',
		     '#' => 'grave, serious',
 		     't' => 'a' },
 'gravedad'	=> { 'x' => 'gravity',
 		     't' => 'n',
		     'g' => 'f' },
 'griego'	=> { 'x' => 'greek',
		     '#' => 'noun and adj.?' },
 'gris'		=> { 'x' => 'grey',
 		     't' => 'a' },
 'gritar'	=> { 'x' => 'yell',
		     't' => 'v' },
 'grueso'	=> { 'x' => 'thick',
 		     't' => 'a' },
 'grupo'	=> { 'x' => 'group',
		     't' => 'n' },
 'guardar'	=> { 'x' => 'keep',
		     '#' => 'guard, watch, save',
 		     't' => 'v' },
 'guatemala'	=> { 'x' => 'guatemala',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'guerrilla'	=> { 'x' => 'guerrilla warfare',
		     '#' => 'guerrilla band',
 		     't' => 'n' },
 'gua'		=> { 'x' => 'guide',
		     't' => 'n' },
 'guitarra'	=> { 'x' => 'guitar',
		     't' => 'n' },
 'gusano'	=> { 'x' => 'worm',
		     '#' => 'animate',
 		     't' => 'n' },
 'gusto'	=> { 'x' => 'taste',
		     '#' => 'pleasure',
 		     't' => 'n' },
 'habituar'	=> { 'x' => 'accustom',
		     't' => 'v' },
 'hablar'	=> { 'x' => 'speak',
		     't' => 'v' },
 'habitacin'	=> { 'x' => 'room',
		     't' => 'n' },
 'hamaca'	=> { 'x' => 'hammock',
 		     't' => 'n' },
 'hambre'	=> { 'x' => 'hunger',
		     '#' => 'related to adj. hungry in English',
 		     't' => 'n' },
 'hebra'	=> { 'x' => 'thread',
 		     't' => 'n' },
 'herir'	=> { 'x' => 'injure',
 		     't' => 'n' },
 'hermana'	=> { 'x' => 'sister',
		     '#' => 'person',
 		     't' => 'n' },
 'hermanastra'	=> { 'x' => 'stepsister',
		     '#' => 'person',
 		     't' => 'n' },
 'hermanastro'	=> { 'x' => 'stepbrother',
		     '#' => 'person',
 		     't' => 'n' },
 'hermano'	=> { 'x' => 'brother',
		     '#' => 'person',
 		     't' => 'n' },
 'hermoso'	=> { 'x' => 'pretty',
 		     't' => 'a' },
 'herramienta'	=> { 'x' => 'tool',
 		     't' => 'n' },
 'hierro'	=> { 'x' => 'iron',
 		     't' => 'n' },
 'hija'		=> { 'x' => 'daughter',
 		     't' => 'n' },
 'hijo'		=> { 'x' => 'son',
 		     't' => 'n' },
 'historia'	=> { 'x' => 'story',
		     '#' => 'history',
 		     't' => 'n' },
 'hogar'	=> { 'x' => 'home',
		     '#' => 'noun and adv.?' },
 'hoguera'	=> { 'x' => 'bonfire',
 		     't' => 'n' },
 'hoja'		=> { 'x' => 'leaf',
 		     't' => 'n' },
 'hojarasca'	=> { 'x' => 'leaf storm',
		     't' => 'n' },
 'hola'		=> { 'x' => 'hello',
		     '#' => 'hi',
 		     't' => 'interj' },
 'holanda'	=> { 'x' => 'holland',
		     '#' => 'proper, place; the netherlands',
 		     't' => 'n' },
 'holands'	=> { 'x' => 'dutch',
		     '#' => 'noun and adj.?' },
 'hombre'	=> { 'x' => 'man',
		     '#' => 'person',
 		     't' => 'n' },
 'hombro'	=> { 'x' => 'shoulder',
 		     't' => 'n' },
 'hongo'	=> { 'x' => 'mushroom',
 		     't' => 'n' },
 'honrar'	=> { 'x' => 'honour',
 		     't' => 'v' },
 'hora'		=> { 'x' => 'hour',
 		     't' => 'n' },
 'hormiga'	=> { 'x' => 'ant',
		     't' => 'n' },
 'horizontal'	=> { 'x' => 'horizontal',
 		     't' => 'a' },
 'hotel'	=> { 'x' => 'hotel',
 		     't' => 'n' },
 'huele'	=> { 'x' => '(it) smells',
		     'r' => 'oler',
 		     't' => 'v' },
 'hueso'	=> { 'x' => 'bone',
 		     't' => 'n' },
 'huevo'	=> { 'x' => 'egg',
 		     't' => 'n' },
 'humano'	=> { 'x' => 'human',
 		     't' => 'n' },
 'hmedo'	=> { 'x' => 'damp',
		     '#' => 'wet',
 		     't' => 'n' },
 'humo'		=> { 'x' => 'smoke',
		     '#' => 'fumes, steam, vapour',
 		     't' => 'n' },
 'humor'	=> { 'x' => 'mood',
		     '#' => 'temper, humour',
 		     't' => 'n',
		     'g' => 'm' },
 'hngaro'	=> { 'x' => 'hungarian',
		     '#' => 'noun and adj.?' },
 'idea'		=> { 'x' => 'idea',
		     't' => 'n' },
 'idioma'	=> { 'x' => 'language',
 		     't' => 'n' },
 'dolo'	=> { 'x' => 'idol',
		     't' => 'n' },
 'iglesia'	=> { 'x' => 'church',
 		     't' => 'n' },
 'igual'	=> { 'x' => 'equal',
 		     't' => 'a' },
 'ilegal'	=> { 'x' => 'illegal',
 		     't' => 'a' },
 'impactar'	=> { 'x' => 'impact',
 		     't' => 'v' },
 'impacto'	=> { 'x' => 'impact',
 		     't' => 'n' },
 'impedir'	=> { 'x' => 'impede',
		     '#' => 'obstruct, prevent',
		     't' => 'v' },
 'implicar'	=> { 'x' => 'imply',
		     '#' => 'involve',
 		     't' => 'v' },
 'importante'	=> { 'x' => 'important',
		     't' => 'a' },
 'importar'	=> { 'x' => 'matter',
		     '#' => 'be important',
		     't' => 'v' },
 'imposible'	=> { 'x' => 'impossible',
 		     't' => 'a' },
 'impresin'	=> { 'x' => 'impression',
 		     't' => 'n' },
 'impresionar'	=> { 'x' => 'impress',
		     '#' => 'move, strike',
		     't' => 'v' },
 'imprimir'	=> { 'x' => 'print',
 		     't' => 'v' },
 'impuesto'	=> { 'x' => 'tax',
 		     't' => 'n' },
 'incluir'	=> { 'x' => 'include',
 		     't' => 'v' },
 'incomprensin'	=> { 'x' => 'incomprehension',
			     't' => 'n' },
 'incrustar'	=> { 'x' => 'encrust',
 		     't' => 'v' },
 'independiente'	=> { 'x' => 'independent',
 			     't' => 'a' },
 'indicar'	=> { 'x' => 'indicate',
		     't' => 'v' },
 'individuo'	=> { 'x' => 'individual',
		     't' => 'n' },
 'indonesia'	=> { 'x' => 'indonesia',
		     '#' => 'proper; place',
		     't' => 'n' },
 'industria'	=> { 'x' => 'industry',
		     't' => 'n' },
 'inevitable'	=> { 'x' => 'inevitable',
 		     't' => 'a' },
 'infiel'	=> { 'x' => 'unfaithful',
		     '#' => 'disloyal',
 		     't' => 'a' },
 'informacin'	=> { 'x' => 'information',
		     't' => 'n',
		     'g' => 'f' },
 'informar'	=> { 'x' => 'inform',
 		     't' => 'v' },
 'informtica'	=> { 'x' => 'computer science',
		     '#' => 'information technology',
 		     't' => 'n' },
 'informe'	=> { 'x' => 'report',
 		     't' => 'n' },
 'ingeniero'	=> { 'x' => 'engineer',
 		     't' => 'n' },
 'inglaterra'	=> { 'x' => 'england',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'ingls'	=> { 'x' => 'english',
		     '#' => 'noun & adjective' },
 'ingresar'	=> { 'x' => 'enter',
		     '#' => 'join; deposit',
 		     't' => 'v' },
 'ingreso'	=> { 'x' => 'entry',
		     '#' => 'income, takings',
 		     't' => 'n' },
 'inspiracin'	=> { 'x' => 'inspiration',
 		     't' => 'n' },
 'instalacin'	=> { 'x' => 'installation',
		     't' => 'n',
		     'g' => 'f' },
 'instalar'	=> { 'x' => 'install',
		     't' => 'v' },
 'instantneo'	=> { 'x' => 'instantaneous',
 		     't' => 'a' },
 'instante'	=> { 'x' => 'instant',
		     '#' => 'noun or adjective?' },
 'instruccin'	=> { 'x' => 'instruction',
 		     't' => 'n' },
 'insultar'	=> { 'x' => 'insult',
		     't' => 'v' },
 'insulto'	=> { 'x' => 'insult',
		     '#' => 'vers. insultar',
 		     't' => 'n' },
 'integrar'	=> { 'x' => 'integrate',
		     't' => 'v' },
 'integridad'	=> { 'x' => 'integrity',
		     '#' => 'wholesome',
		     't' => 'n' },
 'intelectual'	=> { 'x' => 'intellectual',
		     '#' => 'adj. and invar. noun' },
 'intencin'	=> { 'x' => 'intention',
		     '#' => 'purpose',
 		     't' => 'n',
		     'g' => 'f' },
 'intentar'	=> { 'x' => 'try',
		     't' => 'v' },
 'interesante'	=> { 'x' => 'interesting',
		     't' => 'a' },
 'interesar'	=> { 'x' => 'interest',
 		     't' => 'v' },
 'interior'	=> { 'x' => 'interior',
		     't' => 'a' },
 'internacional'	=> { 'x' => 'international',
 			     't' => 'a' },
 'interpretar'	=> { 'x' => 'interpret',
		     't' => 'v' },
 'interrogar'	=> { 'x' => 'interrogate',
 		     't' => 'v' },
 'interrumpir'	=> { 'x' => 'interrupt',
 		     't' => 'v' },
 'introducir'	=> { 'x' => 'introduce',
		     't' => 'v' },
 'intil'	=> { 'x' => 'useless',
 		     't' => 'a' },
 'inventar'	=> { 'x' => 'invent',
		     't' => 'v' },
 'invertir'	=> { 'x' => 'invert',
		     '#' => 'invest, turn upside down',
 		     't' => 'v',
		     '#' => 'spend (time)' },
 'invitar'	=> { 'x' => 'invite',
		     't' => 'v' },
 'involucrar'	=> { 'x' => 'involve',
 		     't' => 'v' },
 'irlanda'	=> { 'x' => 'ireland',
		     '#' => 'proper, place',
		     't' => 'n' },
 'irregular'	=> { 'x' => 'irregular',
		     't' => 'a' },
 'isla'		=> { 'x' => 'island',
		     't' => 'n' },
 'islands'	=> { 'x' => 'icelandic',
		     '#' => 'noun and adj.?' },
 'italia'	=> { 'x' => 'italy',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'italiano'	=> { 'x' => 'italian',
		     '#' => 'noun and adj.?' },
 'izquierdo'	=> { 'x' => 'left',
		     '#' => 'opp. right',
 		     't' => 'a' },
 'jacinto'	=> { 'x' => 'hyacinth',
 		     't' => 'n' },
 'jaleo'	=> { 'x' => 'racket',
		     '#' => 'uproar',
 		     't' => 'n' },
 'japn'	=> { 'x' => 'japan',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'jardn'	=> { 'x' => 'garden',
 		     't' => 'n' },
 'jefe'		=> { 'x' => 'chief',
		     '#' => 'boss - person',
 		     't' => 'n' },
 'jerez'	=> { 'x' => 'sherry',
 		     't' => 'n',
		     'g' => 'm' },
 'jinetear'	=> { 'x' => 'ride',
		     '#' => 'of a horse',
 		     't' => 'v' },
 'joder'	=> { 'x' => 'fuck',
 		     't' => 'v' },
 'joven'	=> { 'x' => 'young',
		     '#' => 'youth, young man/woman, boy/girl',
 		     't' => 'a' },
 'juego'	=> { 'x' => 'game',
 		     't' => 'n' },
 'jugar'	=> { 'x' => 'play',
 		     't' => 'v' },
 'juntar'	=> { 'x' => 'join',
 		     't' => 'v' },
 'junto'	=> { 'x' => 'joined',
		     '#' => 'together',
 		     't' => 'a' },
 'justamente'	=> { 'x' => 'just',
		     '#' => 'justly, fairly, exactly',
 		     't' => 'adv' },
 'justificacin'	=> { 'x' => 'justification',
 			     't' => 'n',
			     'g' => 'f' },
 'justificar'	=> { 'x' => 'justify',
 		     't' => 'v' },
 'juventud'	=> { 'x' => 'youth',
 		     't' => 'n',
		     'g' => 'f' },
 'juzgar'	=> { 'x' => 'judge',
 		     't' => 'v' },
 'labio'	=> { 'x' => 'lip',
 		     't' => 'n' },
 'lado'		=> { 'x' => 'side',
 		     't' => 'n' },
 'ladrn'	=> { 'x' => 'thief',
		     '#' => 'person',
 		     't' => 'n',
		     'g' => 'm' },
 'lago'		=> { 'x' => 'lake',
 		     't' => 'n' },
 'lgrima'	=> { 'x' => 'tear',
		     '#' => 'teardrop',
 		     't' => 'n' },
 'lmina'	=> { 'x' => 'sheet',
 		     't' => 'n' },
 'lanzar'	=> { 'x' => 'throw',
 		     't' => 'v' },
 'largo'	=> { 'x' => 'long',
 		     't' => 'a' },
 'lata'		=> { 'x' => 'can',
 		     't' => 'n' },
 'latino'	=> { 'x' => 'latin',
		     '#' => 'adj. and noun?' },
 'latinoamericano'	=> { 'x' => 'latin american',
			     't' => 'a' },
 'lad'		=> { 'x' => 'lute',
		     't' => 'n' },
 'lavar'	=> { 'x' => 'wash',
 		     't' => 'v' },
 'lazo'		=> { 'x' => 'tie',
 		     't' => 'n' },
 'leccin'	=> { 'x' => 'lesson',
 		     't' => 'n' },
 'lectura'	=> { 'x' => 'reading',
 		     't' => 'n' },
 'lecho'	=> { 'x' => 'bed',
 		     't' => 'n' },
 'len'		=> { 'x' => 'lion',
 		     't' => 'n' },
 'leona'	=> { 'x' => 'lioness',
 		     't' => 'n' },
 'leer'		=> { 'x' => 'read',
 		     't' => 'v' },
  'leo'		=> { 'x' => '(i) read',
 		     'r' => 'creer',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
  'ley'	=> { 'x' => '(it) read',
  		     'r' => 'creer',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
  'leyeron'	=> { 'x' => '(they) read',
  		     'r' => 'creer',
		     't' => 'v',
		     'p' => '3',
		     'n' => 'p',
		     'c' => 't' },
  'leyendo'	=> { 'x' => 'reading',
  		     'r' => 'leer',
		     't' => 'v',
		     'c' => 'g' },
 'ledo'	=> { 'x' => 'well-read',
 		     't' => 'a' },
 'lejano'	=> { 'x' => 'distant',
 		     't' => 'a' },
 'lejos'	=> { 'x' => 'far',
 		     't' => 'a' },
 'lengua'	=> { 'x' => 'tongue',
		     '#' => 'language',
 		     't' => 'n' },
 'lenguaje'	=> { 'x' => 'language',
 		     't' => 'n',
		     'g' => 'm' },
 'lento'	=> { 'x' => 'slow',
 		     't' => 'a' },
 'letra'	=> { 'x' => 'letter',
		     '#' => 'lyrics',
 		     't' => 'n' },
 'levantar'	=> { 'x' => 'get up',
		     '#' => 'rise',
 		     't' => 'v' },
 'ley'		=> { 'x' => 'law',
 		     't' => 'n',
		     'g' => 'f' },
 'liberar'	=> { 'x' => 'free',
 		     't' => 'v' },
 'libertad'	=> { 'x' => 'freedom',
 		     't' => 'n' },
 'libertar'	=> { 'x' => 'free',
		     '#' => 'release, set free',
 		     't' => 'v' },
 'librar'	=> { 'x' => 'save',
		     '#' => 'wage (war), exempt, escape',
 		     't' => 'v' },
 'libre'	=> { 'x' => 'free',
 		     't' => 'a' },
 'librera'	=> { 'x' => 'bookshop',
 		     't' => 'n' },
 'libro'	=> { 'x' => 'book',
 		     't' => 'n' },
 'licencia'	=> { 'x' => 'licence',
 		     't' => 'n' },
 'licor'	=> { 'x' => 'liquor',
		     '#' => 'spirits',
 		     't' => 'n' },
 'lder'	=> { 'x' => 'leader',
		     '#' => 'person',
 		     't' => 'n',
		     'g' => 'm' },
 'ligar'	=> { 'x' => 'tie',
 		     't' => 'v' },
 'lima'		=> { 'x' => 'lime',
 		     't' => 'n' },
 'limitar'	=> { 'x' => 'limit',
		     't' => 'v' },
 'limn'	=> { 'x' => 'lemon',
		     '#' => 'lime',
 		     't' => 'n' },
 'limpiar'	=> { 'x' => 'clean',
 		     't' => 'v' },
 'limpio'	=> { 'x' => 'clean',
 		     't' => 'a' },
 'lince'	=> { 'x' => 'lynx',
 		     't' => 'n' },
 'lindo'	=> { 'x' => 'pretty',
		     't' => 'a' },
 'lnea'	=> { 'x' => 'line',
		     't' => 'n' },
 'lingstica'	=> { 'x' => 'linguistics',
		     't' => 'n' },
 'lino'		=> { 'x' => 'linen',
 		     't' => 'n' },
 'liso'		=> { 'x' => 'straight',
		     '#' => 'flat, even',
		     't' => 'a' },
 'lista'	=> { 'x' => 'list',
		     '#' => 'ready, smart',
		     't' => 'n' },
 'listo'	=> { 'x' => 'ready',
		     '#' => 'smart',
		     't' => 'a' },
 'literatura'	=> { 'x' => 'literature',
 		     't' => 'n' },
 'litro'	=> { 'x' => 'litre',
		     't' => 'n' },
 'loco'		=> { 'x' => 'crazy',
		     't' => 'a' },
 'locura'	=> { 'x' => 'madness',
		     '#' => 'crazy act',
 		     't' => 'n' },
 'lograr'	=> { 'x' => 'achieve',
		     '#' => 'get, obtain',
 		     't' => 'v' },
 'lucha'	=> { 'x' => 'fight',
 		     't' => 'n' },
 'luchar'	=> { 'x' => 'fight',
 		     't' => 'v' },
 'lugar'	=> { 'x' => 'place',
		     't' => 'n' },
 'lgubre'	=> { 'x' => 'mournful',
 		     't' => 'a' },
 'lujoso'	=> { 'x' => 'luxurious',
 		     't' => 'a' },
 'luna'		=> { 'x' => 'moon',
 		     't' => 'n' },
 'lunar'	=> { 'x' => 'mole',
		     '#' => 'anat.; also adj. = lunar',
 		     't' => 'n',
		     'g' => 'm' },
 'luxemburgo'	=> { 'x' => 'luxembourg',
		     '#' => 'proper; place',
		     't' => 'n' },
 'luz'		=> { 'x' => 'light',
 		     't' => 'n',
		     'g' => 'f' },
 'llamar'	=> { 'x' => 'call',
		     't' => 'v' },
 'llanta'	=> { 'x' => 'tyre',
		     '#' => '(tire in u.s.)',
		     't' => 'n' },
 'llanto'	=> { 'x' => 'weeping',
 		     't' => 'n' },
 'llave'	=> { 'x' => 'key',
		     't' => 'n' },
 'llegar'	=> { 'x' => 'arrive',
		     '#' => 'irreg.',
		     't' => 'v' },
 'llenar'	=> { 'x' => 'fill',
 		     't' => 'v' },
 'lleno'	=> { 'x' => 'full',
		     '#' => 'filled',
 		     't' => 'a' },
 'llevar'	=> { 'x' => 'take',
		     't' => 'v' },
 'lluvia'	=> { 'x' => 'rain',
		     't' => 'n' },
 'llorar'	=> { 'x' => 'cry',
		     't' => 'v' },
 'llover'	=> { 'x' => 'rain',
		     '#' => 'irreg; impers.',
 		     't' => 'v' },
  'lluever'	=> { 'x' => 'rain',
  		     'r' => 'llover',
		     't' => 'v' },
 'madera'	=> { 'x' => 'wood',
 		     't' => 'n' },
 'madre'	=> { 'x' => 'mother',
 		     't' => 'n' },
 'mal'		=> { 'x' => 'bad',
 		     't' => 'a' },
 'maldicin'	=> { 'x' => 'curse',
 		     't' => 'n' },
 'malo'		=> { 'x' => 'bad',
 		     't' => 'a' },
 'maltratar'	=> { 'x' => 'mistreat',
		     '#' => 'ill-treat',
		     't' => 'v' },
 'mam'		=> { 'x' => 'mum',
 		     't' => 'n' },
 'manchar'	=> { 'x' => 'stain',
 		     't' => 'v' },
 'mandar'	=> { 'x' => 'lead',
		     '#' => 'order, command, send',
		     't' => 'v' },
 'mandbula'	=> { 'x' => 'jaw',
 		     't' => 'n' },
 'manejar'	=> { 'x' => 'drive',
		     '#' => 'mexico; cf conducir',
 		     't' => 'v' },
 'manera'	=> { 'x' => 'way',
		     '#' => 'manner',
 		     't' => 'n' },
 'manga'	=> { 'x' => 'sleeve',
 		     't' => 'n' },
 'mano'		=> { 'x' => 'hand',
 		     't' => 'n',
		     'g' => 'f' },
 'manso'	=> { 'x' => 'tame',
		     '#' => 'gentle, mild',
		     't' => 'a' },
 'mantener'	=> { 'x' => 'maintain',
 		     't' => 'v' },
 'manzana'	=> { 'x' => 'apple',
 		     't' => 'n' },
 'maana'	=> { 'x' => 'morning',
		     '#' => 'tomorrow',
 		     't' => 'n' },
 'mquina'	=> { 'x' => 'machine',
 		     't' => 'n' },
 'mar'		=> { 'x' => 'sea',
		     't' => 'n',
		     'g' => 'f' },
 'marcar'	=> { 'x' => 'mark',
		     't' => 'v' },
 'marica'	=> { 'x' => 'sissy',
		     '#' => 'adj. or noun?',
 		     't' => 'n',
		     'g' => 'm' },
 'maricn'	=> { 'x' => 'queer',
 		     't' => 'n',
		     'g' => 'm' },
 'mariguana'	=> { 'x' => 'marijuana',
 		     't' => 'n' },
 'marihuana'	=> { 'x' => 'marijuana',
 		     't' => 'n' },
 'marido'	=> { 'x' => 'husband',
 		     't' => 'n' },
 'mariposa'	=> { 'x' => 'butterfly',
 		     't' => 'n' },
 'matar'	=> { 'x' => 'kill',
 		     't' => 'v' },
 'materia'	=> { 'x' => 'matter',
		     '#' => 'material',
		     't' => 'n' },
 'material'	=> { 'x' => 'material',
		     '#' => 'adj. and noun' },
 'matn'	=> { 'x' => 'bully',
 		     't' => 'n' },
 'matrimonio'	=> { 'x' => 'marriage',
		     '#' => '(married) couple',
 		     't' => 'n' },
 'mausoleo'	=> { 'x' => 'mausoleum',
 		     't' => 'n' },
 'mximo'	=> { 'x' => 'maximum',
		     '#' => 'adj. and noun' },
 'mayor'	=> { 'x' => 'main',
 		     't' => 'a' },
 'mayora'	=> { 'x' => 'majority',
 		     't' => 'n' },
 'mayscula'	=> { 'x' => 'capital',
		     '#' => 'capital/upper case letter',
		     't' => 'n' },
 'medida'	=> { 'x' => 'measure',
		     '#' => 'measurement, prudence',
 		     't' => 'n' },
 'medio'	=> { 'x' => 'half',
		     '#' => 'also noun?',
 		     't' => 'a' },
 'medir'	=> { 'x' => 'measure',
 		     't' => 'v' },
 'mejor'	=> { 'x' => 'better',
		     '#' => 'compar. of bueno?',
 		     't' => 'a' },
 'mejora'	=> { 'x' => 'better',
 		     't' => 'a' },
 'mejorar'	=> { 'x' => 'improve',
 		     't' => 'v' },
 'melanclico'	=> { 'x' => 'melancholy',
		     '#' => 'sad, dreamy',
		     't' => 'a' },
 'meln'	=> { 'x' => 'melon',
 		     't' => 'n' },
 'memoria'	=> { 'x' => 'memory',
		     't' => 'n' },
 'mensaje'	=> { 'x' => 'message',
		     't' => 'n' },
 'mentar'	=> { 'x' => 'mention',
 		     't' => 'v' },
 'mentir'	=> { 'x' => 'lie',
 		     't' => 'v' },
 'mentira'	=> { 'x' => 'lie',
 		     't' => 'n' },
 'menudo'	=> { 'x' => 'small',
		     '#' => 'tiny',
 		     't' => 'a' },
 'mercado'	=> { 'x' => 'market',
		     't' => 'n' },
 'merecer'	=> { 'x' => 'deserve',
		     '#' => 'merit',
 		     't' => 'v' },
 'mes'		=> { 'x' => 'month',
		     't' => 'n' },
 'meseta'	=> { 'x' => 'landing',
		     '#' => 'meseta, tableland',
		     't' => 'n' },
 'meter'	=> { 'x' => 'insert',
		     '#' => 'put',
 		     't' => 'v' },
 'metodologa'	=> { 'x' => 'methodology',
 		     't' => 'n' },
 'mxico'	=> { 'x' => 'mexico',
		     '#' => 'proper place',
 		     't' => 'n' },
 'mexicano'	=> { 'x' => 'mexican',
		     '#' => 'noun and adj.?' },
 'mezclar'	=> { 'x' => 'mix',
 		     't' => 'v' },
 'miedo'	=> { 'x' => 'fear',
		     't' => 'n' },
 'miel'		=> { 'x' => 'honey',
 		     't' => 'n' },
 'mierda'	=> { 'x' => 'shit',
 		     't' => 'n' },
 'milagro'	=> { 'x' => 'miracle',
 		     't' => 'n' },
 'militante'	=> { 'x' => 'militant',
		     '#' => 'noun and adj.' },
 'mnimo'	=> { 'x' => 'minimum',
		     '#' => 'noun and adj.?',
 		     't' => 'a' },
 'ministerio'	=> { 'x' => 'ministry',
 		     't' => 'n' },
 'minscula'	=> { 'x' => 'small letter',
		     '#' => 'multiword; lower case',
 		     't' => 'n' },
 'minuto'	=> { 'x' => 'minute',
 		     't' => 'n' },
 'mirar'	=> { 'x' => 'look at',
		     '#' => 'examine',
		     't' => 'v' },
 'misa'		=> { 'x' => 'mass',
		     't' => 'n' },
 'mismo'	=> { 'x' => 'same',
		     '#' => 'also adj.?',
 		     't' => 'n' },
 'mitad'	=> { 'x' => 'half',
		     't' => 'n',
		     'g' => 'm' },
 'misterio'	=> { 'x' => 'mystery',
 		     't' => 'n' },
 'misterioso'	=> { 'x' => 'mysterious',
 		     't' => 'a' },
 'moda'		=> { 'x' => 'fashion',
		     '#' => 'style',
		     't' => 'n' },
 'modelo'	=> { 'x' => 'model',
		     '#' => 'also a mexican beer',
 		     't' => 'n' },
 'mdem'	=> { 'x' => 'modem',
		     't' => 'n' },
 'moderno'	=> { 'x' => 'modern',
		     't' => 'a' },
 'modificar'	=> { 'x' => 'modify',
		     't' => 'v' },
 'modo'		=> { 'x' => 'way',
		     '#' => 'manner, mode',
		     't' => 'n' },
 'mole'		=> { 'x' => 'mass',
		     '#' => 'bulk, pile',
		     't' => 'n',
		     'g' => 'f' },
 'molestar'	=> { 'x' => 'annoy',
 		     't' => 'v' },
 'momentneo'	=> { 'x' => 'momentary',
		     't' => 'a' },
 'momento'	=> { 'x' => 'moment',
		     't' => 'n' },
 'mono'		=> { 'x' => 'monkey',
		     '#' => 'animate',
 		     't' => 'n' },
 'monopolio'	=> { 'x' => 'monopoly',
 		     't' => 'n' },
 'monopolizar'	=> { 'x' => 'monopolize',
 		     't' => 'v' },
 'monstruo'	=> { 'x' => 'monster',
		     '#' => 'animate',
 		     't' => 'n' },
 'montaa'	=> { 'x' => 'mountain',
		     't' => 'n' },
 'montar'	=> { 'x' => 'mount',
 		     't' => 'v' },
 'monte'	=> { 'x' => 'mountain',
		     '#' => 'woodland, wild area',
		     't' => 'n' },
 'morado'	=> { 'x' => 'purple',
		     't' => 'a' },
 'morder'	=> { 'x' => 'bite',
		     't' => 'v' },
 'moreno'	=> { 'x' => 'brown',
		     '#' => 'also noun',
 		     't' => 'a' },
 'morir'	=> { 'x' => 'die',
		     '#' => 'irregular',
		     't' => 'v' },
  'muerir'	=> { 'x' => 'die',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'morir',
		     't' => 'v' },
  'murir'	=> { 'x' => 'die',
		     '#' => 'virtual base pret.',
  		     'r' => 'morir',
		     't' => 'v' },
 'mosca'	=> { 'x' => 'fly',
 		     't' => 'n' },
 'mostrar'	=> { 'x' => 'show',
 		     't' => 'v' },
 'motivar'	=> { 'x' => 'cause',
		     '#' => 'motivate, explain, justify',
 		     't' => 'v' },
 'motivo'	=> { 'x' => 'motive',
 		     't' => 'n' },
 'motor'	=> { 'x' => 'motor',
		     't' => 'n' },
 'mover'	=> { 'x' => 'move',
		     't' => 'v' },
  'muever'	=> { 'x' => 'move',
		     '#' => 'virtual base',
  		     'r' => 'mover',
		     't' => 'v' },
 'mvil'	=> { 'x' => 'mobile',
 		     't' => 'a' },
 'muchacha'	=> { 'x' => 'girl',
		     '#' => 'person',
 		     't' => 'n' },
 'muchacho'	=> { 'x' => 'boy',
		     '#' => 'person',
 		     't' => 'n' },
 'muchedumbre'	=> { 'x' => 'crowd',
		     't' => 'n' },
 'mucho'	=> { 'x' => 'much',
		     '#' => 'many',
 		     't' => 'a' },
 'mudar'	=> { 'x' => 'change',
		     '#' => 'clothes etc.',
 		     't' => 'v' },
 'mudo'		=> { 'x' => 'mute',
		     '#' => 'dumb; adj. and noun' },
 'muebles'	=> { 'x' => 'furniture',
 		     't' => 'n',
		     'n' => 'p' },
 'muelle'	=> { 'x' => 'spring',
		     '#' => 'wharf, pier',
 		     't' => 'n' },
 'muerte'	=> { 'x' => 'death',
 		     't' => 'n',
		     'g' => 'f' },
 'muerto'	=> { 'x' => 'dead',
		     '#' => 'also noun?',
 		     't' => 'a' },
 'muestra'	=> { 'x' => 'indication',
		     '#' => 'sign, indication, demonstration, proof, sample, model, pattern, token',
		     't' => 'n' },
 'mujer'	=> { 'x' => 'woman',
 		     't' => 'n' },
 'muleta'	=> { 'x' => 'crutch',
		     't' => 'n' },
 'mullido'	=> { 'x' => 'soft',
 		     't' => 'a' },
 'mltiple'	=> { 'x' => 'multiple',
		     't' => 'a' },
 'mundo'	=> { 'x' => 'world',
 		     't' => 'n' },
 'muralla'	=> { 'x' => 'wall',
		     '#' => '(city wall)',
 		     't' => 'n' },
 'msica'	=> { 'x' => 'music',
 		     't' => 'n' },
 'nacer'	=> { 'x' => 'be born',
		     '#' => '(2 words); transl. to passive in English',
 		     't' => 'v' },
 'nacimiento'	=> { 'x' => 'birth',
 		     't' => 'n' },
 'nacin'	=> { 'x' => 'nation',
 		     't' => 'n',
		     'g' => 'f' },
 'nacional'	=> { 'x' => 'national',
 		     't' => 'a' },
 'nacionalidad'	=> { 'x' => 'nationality',
 		     't' => 'n' },
 'nadar'	=> { 'x' => 'swim',
		     't' => 'v' },
 'nalgas'	=> { 'x' => 'buttocks',
		     '#' => 'only exists in plural?',
 		     't' => 'n',
		     'n' => 'p' },
 'narices'	=> { 'x' => 'nostrils',
		     '#' => 'only exists in plural?',
 		     't' => 'n',
		     'n' => 'p' },
 'nariz'	=> { 'x' => 'nose',
		     't' => 'n' },
 'natural'	=> { 'x' => 'natural',
		     't' => 'a' },
 'navegar'	=> { 'x' => 'navigate',
		     '#' => 'sail, fly',
		     't' => 'v' },
 'navidad'	=> { 'x' => 'christmas',
 		     't' => 'n' },
 'navo'	=> { 'x' => 'ship',
 		     't' => 'n' },
 'necesario'	=> { 'x' => 'necessary',
		     't' => 'a' },
 'necesidad'	=> { 'x' => 'necessity',
		     't' => 'n' },
 'necesitar'	=> { 'x' => 'need',
		     't' => 'v' },
 'necio'	=> { 'x' => 'foolish',
 		     't' => 'a' },
 'negocio'	=> { 'x' => 'business',
		     '#' => 'affair, deal, transaction',
		     't' => 'n' },
 'negro'	=> { 'x' => 'black',
		     '#' => 'adj. and noun' },
 'nena'		=> { 'x' => 'baby',
 		     't' => 'n' },
 'nene'		=> { 'x' => 'baby',
 		     't' => 'n' },
 'neurona'	=> { 'x' => 'neuron',
 		     't' => 'n' },
 'nevar'	=> { 'x' => 'snow',
		     't' => 'v' },
 'nido'		=> { 'x' => 'nest',
 		     't' => 'n' },
 'nieve'	=> { 'x' => 'snow',
 		     't' => 'n' },
 'niln'	=> { 'x' => 'nylon',
		     '#' => 'nonstandard spelling?',
		     't' => 'n' },
 'nia'		=> { 'x' => 'girl',
		     't' => 'n' },
 'nio'		=> { 'x' => 'boy',
 		     't' => 'n',
		     '#' => 'also adj. young?' },
 'nivel'	=> { 'x' => 'level',
		     't' => 'n' },
 'noche'	=> { 'x' => 'night',
 		     't' => 'n' },
 'nodo'		=> { 'x' => 'node',
 		     't' => 'n' },
 'nombre'	=> { 'x' => 'name',
 		     't' => 'n' },
 'nominacin'	=> { 'x' => 'nomination',
 		     't' => 'n',
		     'g' => 'f' },
 'nominar'	=> { 'x' => 'nominate',
 		     't' => 'v' },
 'norma'	=> { 'x' => 'standard',
		     '#' => 'rule',
 		     't' => 'n' },
 'normal'	=> { 'x' => 'normal',
 		     't' => 'a' },
 'norte'	=> { 'x' => 'north',
		     '#' => 'noun and adj./adv.?' },
 'noruega'	=> { 'x' => 'norway',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'noruego'	=> { 'x' => 'norwegian',
		     '#' => 'adj. and noun?' },
 'nota'		=> { 'x' => 'note',
		     '#' => 'mark',
 		     't' => 'n' },
 'notar'	=> { 'x' => 'notice',
		     '#' => 'note',
 		     't' => 'v' },
 'noticia'	=> { 'x' => 'news',
		     '#' => 'piece of news?',
 		     't' => 'n' },
 'novela'	=> { 'x' => 'novel',
 		     't' => 'n' },
 'novia'	=> { 'x' => 'girlfriend',
 		     't' => 'n' },
 'novio'	=> { 'x' => 'boyfriend',
 		     't' => 'n' },
 'nube'		=> { 'x' => 'cloud',
 		     't' => 'n',
		     'g' => 'f' },
 'nublar'	=> { 'x' => 'get dark',
		     '#' => 'grow dark; usu. refl.',
 		     't' => 'v' },
 'ncleo'	=> { 'x' => 'core',
		     't' => 'n' },
 'nudo'		=> { 'x' => 'knot',
 		     't' => 'n' },
 'nuevamente'	=> { 'x' => 'again',
		     '#' => 'anew',
 		     't' => 'adv' },
 'nuevo'	=> { 'x' => 'new',
 		     't' => 'a' },
 'nmero'	=> { 'x' => 'number',
 		     't' => 'n' },
 'numeroso'	=> { 'x' => 'numerous',
 		     't' => 'a' },
 'nylon'	=> { 'x' => 'nylon',
		     't' => 'n' },
 'objetivo'	=> { 'x' => 'object',
 		     't' => 'n' },
 'obra'		=> { 'x' => 'work',
 		     't' => 'n' },
 'obtener'	=> { 'x' => 'obtain',
 		     't' => 'v' },
 'ocano'	=> { 'x' => 'ocean',
 		     't' => 'n' },
 'ocupar'	=> { 'x' => 'occupy',
 		     't' => 'v' },
 'ocurrir'	=> { 'x' => 'happen',
 		     't' => 'v' },
 'odiar'	=> { 'x' => 'hate',
		     't' => 'v' },
 'oficial'	=> { 'x' => 'official',
		     '#' => 'noun and adj.?' },
 'oficina'	=> { 'x' => 'office',
 		     't' => 'n' },
 'ofrecer'	=> { 'x' => 'offer',
		     '#' => 'propose',
 		     't' => 'v' },
 'odo'		=> { 'x' => 'heard',
 		     'r' => 'or',
 		     't' => 'v' },
 'or'		=> { 'x' => 'hear',
 		     't' => 'v' },
 'ojeroso'	=> { 'x' => 'haggard',
 		     't' => 'a' },
 'ojo'		=> { 'x' => 'eye',
 		     't' => 'n' },
 'ola'		=> { 'x' => 'wave',
 		     't' => 'n' },
 'oler'		=> { 'x' => 'smell',
 		     't' => 'v' },
 'olor'		=> { 'x' => 'odor',
 		     't' => 'n' },
 'olvidar'	=> { 'x' => 'forget',
		     't' => 'v' },
 'omitir'	=> { 'x' => 'omit',
 		     't' => 'v' },
 'operacin'	=> { 'x' => 'operation',
 		     't' => 'n' },
 'operar'	=> { 'x' => 'operate',
 		     't' => 'v' },
 'operativo'	=> { 'x' => 'operator',
 		     't' => 'n' },
 'opinar'	=> { 'x' => 'think',
		     '#' => 'opine',
 		     't' => 'v' },
 'opinin'	=> { 'x' => 'opinion',
 		     't' => 'n' },
 'oportunidad'	=> { 'x' => 'opportunity',
		     't' => 'n' },
 'optimismo'	=> { 'x' => 'optimism',
 		     't' => 'n' },
 'optimista'	=> { 'x' => 'optimista',
		     '#' => 'gender invar.',
 		     't' => 'n' },
 'orden'	=> { 'x' => 'order',
		     '#' => 'command',
		     't' => 'n' },
 'ordenador'	=> { 'x' => 'computer',
		     't' => 'n' },
 'ordenar'	=> { 'x' => 'order',
		     't' => 'v' },
 'oreja'	=> { 'x' => 'ear',
		     't' => 'n' },
 'organismo'	=> { 'x' => 'organism',
		     '#' => 'organization',
 		     't' => 'n' },
 'organizacin'	=> { 'x' => 'organization',
 		     't' => 'n' },
 'orientar'	=> { 'x' => 'orient',
		     '#' => 'align?, point?',
		     't' => 'v' },
 'original'	=> { 'x' => 'original',
		     '#' => 'odd, strange',
		     't' => 'a' },
 'oro'		=> { 'x' => 'gold',
		     '#' => 'noun and adj.?' },
 'oscilar'	=> { 'x' => 'oscillate',
 		     't' => 'v' },
 'oscuridad'	=> { 'x' => 'obscurity',
		     '#' => 'darkness',
 		     't' => 'n' },
 'oscuro'	=> { 'x' => 'dark',
		     '#' => 'obscure',
		     't' => 'a' },
 'oso'		=> { 'x' => 'bear',
 		     't' => 'n' },
 'otoo'	=> { 'x' => 'autumn',
 		     't' => 'n' },
 'oye'		=> { 'x' => '(it) hears',
		     'r' => 'or',
		     't' => 'v' },
 'paciencia'	=> { 'x' => 'patience',
 		     't' => 'n' },
 'padre'	=> { 'x' => 'father',
		     't' => 'n' },
 'padrn'	=> { 'x' => 'census',
		     '#' => 'roll',
 		     't' => 'n' },
 'pagar'	=> { 'x' => 'pay',
		     't' => 'v' },
 'pgina'	=> { 'x' => 'page',
		     't' => 'n' },
 'pago'		=> { 'x' => 'payment',
 		     't' => 'n' },
 'pas'		=> { 'x' => 'country',
		     't' => 'n' },
 'pjaro'	=> { 'x' => 'bird',
		     '#' => 'animate',
 		     't' => 'n' },
 'palabra'	=> { 'x' => 'word',
		     't' => 'n' },
 'palillo'	=> { 'x' => 'chopstick',
 		     't' => 'n' },
 'palo'		=> { 'x' => 'stick',
 		     't' => 'n' },
 'pan'		=> { 'x' => 'bread',
 		     't' => 'n' },
 'pana'		=> { 'x' => 'corduroy',
 		     't' => 'n' },
 'pnico'	=> { 'x' => 'panic',
 		     't' => 'n' },
 'pantalones'	=> { 'x' => 'trousers',
 		     't' => 'n',
		     'n' => 'p' },
 'pantalla'	=> { 'x' => 'screen',
		     '#' => 'lampshade',
		     't' => 'n' },
 'pauelo'	=> { 'x' => 'scarf',
		     '#' => 'handkerchief',
 		     't' => 'n' },
 'papa'		=> { 'x' => 'potato',
		     '#' => 'pope',
		     '#' => 'potatos is mexican, pope is capitalized',
 		     't' => 'n' },
 'pap'		=> { 'x' => 'dad',
		     '#' => 'person',
 		     't' => 'n' },
 'papel'	=> { 'x' => 'paper',
 		     't' => 'n' },
 'par'		=> { 'x' => 'pair',
 		     't' => 'n',
		     'g' => 'm' },
 'paraguay'	=> { 'x' => 'paraguay',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'parar'	=> { 'x' => 'stop',
 		     't' => 'v' },
 'parchear'	=> { 'x' => 'patch',
 		     't' => 'v' },
 'parecer'	=> { 'x' => 'seem',
		     '#' => 'look',
 		     't' => 'v' },
  'parezcer'	=> { 'x' => 'seem',
		     '#' => 'virtual base 1ps pres.; pres. subj.',
  		     'r' => 'parecer',
		     't' => 'v' },
 'pareja'	=> { 'x' => 'couple',
		     '#' => 'pair',
 		     't' => 'n' },
 'pariente'	=> { 'x' => 'relative' },
 'pars'	=> { 'x' => 'paris',
 		     't' => 'n' },
 'parque'	=> { 'x' => 'park',
 		     't' => 'n' },
 'parrilla'	=> { 'x' => 'grill',
 		     't' => 'n' },
 'parte'	=> { 'x' => 'part',
 		     't' => 'n' },
 'particin'	=> { 'x' => 'partition',
		     't' => 'n',
		     'g' => 'f' },
 'participante'	=> { 'x' => 'participant',
		     '#' => 'invar.; person',
 		     't' => 'n' },
 'participar'	=> { 'x' => 'participate',
 		     't' => 'v' },
 'partido'	=> { 'x' => 'party',
 		     't' => 'n' },
 'partir'	=> { 'x' => 'split',
		     '#' => 'divide, share, distribute, break open, split open, cut, set off, st',
		     't' => 'v' },
 'pasado'	=> { 'x' => 'past',
 		     't' => 'n' },
 'pasar'	=> { 'x' => 'pass',
 		     't' => 'v' },
 'pasear'	=> { 'x' => 'walk',
		     '#' => 'go for a walk, take for a walk,',
 		     't' => 'v',
		     '#' => 'parade, show off' },
 'pasillo'	=> { 'x' => 'passage',
 		     't' => 'n' },
 'pasin'	=> { 'x' => 'passion',
 		     't' => 'n',
		     'g' => 'f' },
 'paso'		=> { 'x' => 'step',
		     '#' => 'pace, passage',
 		     't' => 'n' },
 'pastas'	=> { 'x' => 'cakes',
		     '#' => 'pastries',
 		     't' => 'n',
		     'n' => 'p' },
 'patinar'	=> { 'x' => 'skate',
 		     't' => 'v' },
 'patio'	=> { 'x' => 'patio',
		     '#' => 'courtyard',
 		     't' => 'n' },
 'patrn'	=> { 'x' => 'boss',
		     '#' => 'chief, master',
 		     't' => 'n' },
 'paz'		=> { 'x' => 'peace',
 		     't' => 'n' },
 'pd'		=> { 'x' => 'ps',
		     '#' => 'abbreviation - capitalized' },
 'pdt'		=> { 'x' => 'ps',
		     '#' => 'abbreviation - capitalized' },
 'pecho'	=> { 'x' => 'breast',
 		     't' => 'n' },
 'pedazo'	=> { 'x' => 'piece',
		     '#' => 'bit',
 		     't' => 'n' },
 'pedir'	=> { 'x' => 'ask for',
		     '#' => 'request; irreg.',
 		     't' => 'v' },
 'pelea'	=> { 'x' => 'fight',
 		     't' => 'n' },
 'pelear'	=> { 'x' => 'fight',
		     't' => 'v' },
 'pelcula'	=> { 'x' => 'movie',
 		     't' => 'n' },
 'peligro'	=> { 'x' => 'danger',
 		     't' => 'n' },
 'peligroso'	=> { 'x' => 'dangerous',
 		     't' => 'a' },
 'pelo'		=> { 'x' => 'hair',
		     '#' => 'countable; a strand',
 		     't' => 'n' },
 'pequeo'	=> { 'x' => 'small',
		     't' => 'a' },
 'pena'		=> { 'x' => 'grief',
		     '#' => 'sadness, regret, trouble, pain',
		     't' => 'n' },
 'pensamiento'	=> { 'x' => 'thought',
 		     't' => 'n' },
 'pensar'	=> { 'x' => 'think',
		     't' => 'v' },
  'piensar'	=> { 'x' => 'think',
		     '#' => 'virtual base for pres. and pres. subj.',
  		     't' => 'pensar',
		     't' => 'v' },
 'penumbra'	=> { 'x' => 'half-light',
		     't' => 'n' },
 'peor'		=> { 'x' => 'worse',
		     '#' => 'opp. mejor',
 		     't' => 'a' },
 'perder'	=> { 'x' => 'lose',
		     '#' => 'irreg.',
		     't' => 'v' },
 'perdonar'	=> { 'x' => 'forgive',
		     '#' => 'pardon, excuse',
 		     't' => 'v' },
 'perfecto'	=> { 'x' => 'perfect',
		     't' => 'a' },
 'peridico'	=> { 'x' => 'newspaper',
 		     't' => 'n' },
 'permitir'	=> { 'x' => 'permit',
		     't' => 'v' },
 'perro'	=> { 'x' => 'dog',
		     '#' => 'animal',
 		     't' => 'n' },
 'perseguir'	=> { 'x' => 'pursue',
		     '#' => 'hunt, chase, pester, annoy',
 		     't' => 'v' },
 'persistente'	=> { 'x' => 'persistent',
 		     't' => 'a' },
 'persona'	=> { 'x' => 'person',
		     't' => 'n' },
 'personaje'	=> { 'x' => 'character',
		     '#' => '(important) person',
 		     't' => 'n',
		     '#' => 'celebrity',
		     'g' => 'm' },
 'personalidad'	=> { 'x' => 'personality',
		     't' => 'n' },
 'per'		=> { 'x' => 'peru',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'peruano'	=> { 'x' => 'peruvian',
		     '#' => 'adj. and noun' },
 'pesar'	=> { 'x' => 'weigh',
		     '#' => 'be heavy',
 		     't' => 'v' },
 'peso'		=> { 'x' => 'peso',
		     '#' => 'currency',
 		     't' => 'n' },
 'peticin'	=> { 'x' => 'petition',
		     '#' => 'request, plea',
 		     't' => 'n',
		     'g' => 'f' },
 'pide'		=> { 'x' => '(it) requests',
		     't' => 'v',
		     'r' => 'pedir' },
 'piden'	=> { 'x' => '(they) request',
		     't' => 'v',
		     'r' => 'pedir' },
 'pides'	=> { 'x' => '(you) request',
		     't' => 'v',
		     'r' => 'pedir' },
 'pido'		=> { 'x' => '(i) request',
		     't' => 'v',
		     'r' => 'pedir' },
 'pie'		=> { 'x' => 'foot',
		     't' => 'n' },
 'piedra'	=> { 'x' => 'stone',
 		     't' => 'n' },
 'piel'		=> { 'x' => 'skin',
 		     't' => 'n' },
 'pierna'	=> { 'x' => 'leg',
 		     't' => 'n' },
 'pieza'	=> { 'x' => 'piece',
		     '#' => 'room',
 		     't' => 'n' },
 'pintar'	=> { 'x' => 'paint',
 		     't' => 'v' },
 'piso'		=> { 'x' => 'floor',
 		     't' => 'n' },
 'pistola'	=> { 'x' => 'pistol',
 		     't' => 'n' },
 'placer'	=> { 'x' => 'pleasure',
		     '#' => 'to please',
		     't' => 'n' },
 'planear'	=> { 'x' => 'plan',
		     '#' => 'glide',
 		     't' => 'v' },
 'planeta'	=> { 'x' => 'planet',
 		     't' => 'n' },
 'plstico'	=> { 'x' => 'plastic',
 		     't' => 'n' },
 'plata'	=> { 'x' => 'silver',
		     't' => 'n' },
 'plataforma'	=> { 'x' => 'platform',
		     't' => 'n' },
 'pltano'	=> { 'x' => 'banana',
 		     't' => 'n' },
 'playa'	=> { 'x' => 'beach',
		     't' => 'n' },
 'playera'	=> { 'x' => 't-shirt',
		     't' => 'n' },
 'pleno'	=> { 'x' => 'full',
		     '#' => 'complete',
		     't' => 'a' },
 'pobre'	=> { 'x' => 'poor',
 		     't' => 'a' },
 'pobreza'	=> { 'x' => 'poverty',
 		     't' => 'n' },
 'poco'		=> { 'x' => 'little',
		     '#' => 'few' },
 'poesa'	=> { 'x' => 'poetry',
 		     't' => 'n' },
 'polaco'	=> { 'x' => 'polish',
		     '#' => 'adj. and noun?' },
 'polica'	=> { 'x' => 'police',
		     '#' => 'person',
 		     't' => 'n' },
 'poltica'	=> { 'x' => 'politics',
 		     't' => 'n' },
 'poltico'	=> { 'x' => 'political',
		     '#' => 'noun and adj.; politician' },
 'polla'	=> { 'x' => 'hen',
		     '#' => 'animal',
 		     't' => 'n' },
 'pollo'	=> { 'x' => 'chicken',
		     '#' => 'animal',
 		     't' => 'n' },
 'polo'		=> { 'x' => 'pole',
		     '#' => 'as in north pole',
 		     't' => 'n' },
 'polonia'	=> { 'x' => 'poland',
		     '#' => 'proper, place',
		     't' => 'n' },
 'polvo'	=> { 'x' => 'powder',
		     't' => 'n' },
 'polvoriento'	=> { 'x' => 'dusty',
		     '#' => 'powdery',
		     't' => 'a' },
 'pomo'		=> { 'x' => 'knob',
		     '#' => 'doorknob',
		     't' => 'n' },
 'poner'	=> { 'x' => 'put',
		     't' => 'v' },
  'pongar'	=> { 'x' => 'put',
		     '#' => 'virtual base',
		     'r' => 'poner',
		     't' => 'v' },
 'portada'	=> { 'x' => 'cover',
 		     't' => 'n' },
 'portugal'	=> { 'x' => 'portugal',
		     '#' => 'proper, place',
		     't' => 'n' },
 'portugus'	=> { 'x' => 'portuguese',
		     '#' => 'noun and adj.' },
 'poquito'	=> { 'x' => 'little',
		     '#' => 'adj. and noun?' },
 'poseer'	=> { 'x' => 'possess',
		     't' => 'v' },
 'posibilidad'	=> { 'x' => 'possibility',
		     't' => 'n' },
 'posible'	=> { 'x' => 'possible',
		     't' => 'a' },
 'posicin'	=> { 'x' => 'position',
 		     't' => 'n' },
 'postal'	=> { 'x' => 'postcard',
		     't' => 'n' },
 'postergar'	=> { 'x' => 'postpone',
		     '#' => 'delay',
 		     't' => 'v' },
 'potencia'	=> { 'x' => 'power',
		     '#' => 'potential',
 		     't' => 'n' },
 'potencial'	=> { 'x' => 'potential',
		     '#' => 'adj. and noun masc.' },
 'potenciar'	=> { 'x' => 'boost',
 		     't' => 'v' },
 'prcticamente'	=> { 'x' => 'practically',
 			     't' => 'adv' },
 'precio'	=> { 'x' => 'price',
		     't' => 'n' },
 'predicar'	=> { 'x' => 'preach',
 		     't' => 'v' },
 'preferir'	=> { 'x' => 'prefer',
		     't' => 'v' },
 'pregunta'	=> { 'x' => 'question',
		     't' => 'n' },
 'preguntar'	=> { 'x' => 'ask',
		     't' => 'v' },
 'premio'	=> { 'x' => 'award',
		     '#' => 'prize; premium',
 		     't' => 'n' },
 'prensa'	=> { 'x' => 'press',
 		     't' => 'n' },
 'prensar'	=> { 'x' => 'press',
 		     't' => 'v' },
 'preocupar'	=> { 'x' => 'worry',
		     't' => 'v' },
 'preparar'	=> { 'x' => 'prepare',
		     '#' => 'get ready, process, teach, train',
		     't' => 'v' },
 'presentar'	=> { 'x' => 'present',
		     't' => 'v' },
 'prestar'	=> { 'x' => 'lend',
		     't' => 'v' },
 'pretender'	=> { 'x' => 'court',
		     '#' => 'woo; try, seek, claim',
 		     't' => 'v' },
 'pretexto'	=> { 'x' => 'excuse',
		     '#' => 'pretext',
 		     't' => 'n' },
 'previsto'	=> { 'x' => 'anticipated',
		     '#' => 'forecast',
 		     't' => 'a' },
 'primavera'	=> { 'x' => 'spring',
		     '#' => 'the season',
 		     't' => 'n' },
 'principal'	=> { 'x' => 'principal',
		     '#' => 'adjective and noun?',
 		     't' => 'a' },
 'prisin'	=> { 'x' => 'prison',
		     '#' => 'imprisonment',
 		     't' => 'n' },
 'probablemente'=> { 'x' => 'probably',
		     't' => 'adv' },
 'probar'	=> { 'x' => 'try',
		     '#' => 'irreg.',
		     't' => 'v' },
 'problema'	=> { 'x' => 'problem',
		     't' => 'n' },
 'procesamiento'	=> { 'x' => 'processing',
			     't' => 'n' },
 'proceso'	=> { 'x' => 'process',
		     '#' => 'trial',
 		     't' => 'n' },
 'produccin'	=> { 'x' => 'production',
 		     't' => 'n' },
 'producto'	=> { 'x' => 'product',
		     't' => 'n' },
 'productor'	=> { 'x' => 'producer',
 		     't' => 'n' },
 'profundo'	=> { 'x' => 'deep',
		     '#' => 'profound',
 		     't' => 'a' },
 'programa'	=> { 'x' => 'program',
		     't' => 'n' },
 'programador'	=> { 'x' => 'programmer',
		     't' => 'n' },
 'programar'	=> { 'x' => 'program',
		     't' => 'v' },
 'prohibido'	=> { 'x' => 'prohibited',
		     '#' => 'banned',
 		     't' => 'a' },
 'prohibir'	=> { 'x' => 'prohibit',
		     '#' => 'ban',
 		     't' => 'v' },
 'prolongar'	=> { 'x' => 'extend',
		     '#' => 'prolong',
		     't' => 'v' },
 'promesa'	=> { 'x' => 'promise',
 		     't' => 'n' },
 'prometer'	=> { 'x' => 'promise',
		     't' => 'v' },
 'propaganda'	=> { 'x' => 'advertising',
		     '#' => 'propaganda',
		     't' => 'n' },
 'propina'	=> { 'x' => 'tip',
		     't' => 'n' },
 'propuesta'	=> { 'x' => 'proposal',
 		     't' => 'n' },
 'proteccin'	=> { 'x' => 'protection',
 		     't' => 'n' },
 'proteger'	=> { 'x' => 'protect',
		     't' => 'v' },
 'proveer'	=> { 'x' => 'provide',
		     '#' => 'supply',
		     't' => 'v' },
 'provincia'	=> { 'x' => 'province',
 		     't' => 'n' },
 'prximo'	=> { 'x' => 'next',
 		     't' => 'a' },
 'proyectar'	=> { 'x' => 'project',
 		     't' => 'v' },
 'proyecto'	=> { 'x' => 'project',
		     '#' => 'plan',
 		     't' => 'n' },
 'prueba'	=> { 'x' => 'test',
		     '#' => 'proof, trial; tasing, sampling, fitting',
 		     't' => 'n',
		     '#' => 'also form of "probar"' },
 'publicar'	=> { 'x' => 'publish',
		     '#' => 'publicize',
		     't' => 'v' },
 'pblico'	=> { 'x' => 'public',
 		     't' => 'a' },
 'pueblo'	=> { 'x' => 'village',
		     '#' => 'town, people, nation',
 		     't' => 'n' },
 'puente'	=> { 'x' => 'bridge',
 		     't' => 'n' },
 'puerco'	=> { 'x' => 'pork',
 		     't' => 'n' },
 'puerta'	=> { 'x' => 'door',
 		     't' => 'n' },
 'puerto'	=> { 'x' => 'port',
 		     't' => 'n' },
 'puesto'	=> { 'x' => 'put' },
 'punta'	=> { 'x' => 'point',
		     't' => 'n' },
 'punto'	=> { 'x' => 'point',
		     't' => 'n' },
 'puntuacin'	=> { 'x' => 'punctuation',
		     't' => 'n' },
 'puo'		=> { 'x' => 'fist',
		     '#' => 'handful, cuff',
		     't' => 'n' },
 'puta'		=> { 'x' => 'bitch',
		     '#' => 'whore?',
		     '#' => 'vulgar and/or slang',
		     't' => 'n' },
 'puto'		=> { 'x' => 'faggot',
		     '#' => 'pejorative/vulgar and/or slang',
		     't' => 'n' },
 'quebrar'	=> { 'x' => 'break',
		     '#' => 'smash, bankrupt',
 		     't' => 'v' },
 'quedar'	=> { 'x' => 'stay',
		     '#' => 'remain',
		     't' => 'v' },
 'querer'	=> { 'x' => 'want',
		     '#' => 'love',
		     't' => 'v' },
  'quierer'	=> { 'x' => 'want',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'querer',
		     't' => 'v' },
  'querr'	=> { 'x' => 'want',
		     '#' => 'virtual base fut., condit.',
  		     'r' => 'querer',
		     't' => 'v' },
  'quiser'	=> { 'x' => 'want',
		     '#' => 'virtual base pret.',
  		     'r' => 'querer',
		     't' => 'v' },
  'quise'	=> { 'x' => '(i) wanted',
  		     'r' => 'querer',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
  'quiso'	=> { 'x' => '(he) wanted',
  		     'r' => 'querer',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
 'querido'	=> { 'x' => 'dear',
		     't' => 'a' },
 'quiebra'	=> { 'x' => 'break',
		     '#' => 'split, bankruptcy, slump',
 		     't' => 'n' },
 'quiebro'	=> { 'x' => 'swerve',
		     '#' => '(of the body)',
 		     't' => 'n' },
 'quieto'	=> { 'x' => 'placid',
		     '#' => 'still',
		     't' => 'a' },
 'quitar'	=> { 'x' => 'remove',
		     't' => 'v' },
 'rabia'	=> { 'x' => 'rage',
		     '#' => 'fury; rabies',
 		     't' => 'n' },
 'rabioso'	=> { 'x' => 'rabid',
		     '#' => 'furious',
		     't' => 'a' },
 'radical'	=> { 'x' => 'radical',
		     '#' => 'noun and adjective?' },
 'rpido'	=> { 'x' => 'quick',
		     '#' => 'fast',
		     't' => 'a' },
 'rapidez'	=> { 'x' => 'speed',
		     't' => 'n' },
 'raro'		=> { 'x' => 'strange',
		     '#' => 'rare, weird',
 		     't' => 'a' },
 'rato'		=> { 'x' => 'while',
		     '#' => 'short time',
		     't' => 'n' },
 'ratn'	=> { 'x' => 'mouse',
		     '#' => 'animate',
 		     't' => 'n' },
 'rayo'		=> { 'x' => 'ray',
		     '#' => 'as in x-ray, sun\'s rays',
 		     't' => 'n' },
 'raza'		=> { 'x' => 'race',
		     't' => 'v' },
 'razonar'	=> { 'x' => 'reason',
		     '#' => 'argue',
 		     't' => 'v' },
 'razn'	=> { 'x' => 'reason',
		     't' => 'n' },
 'real'		=> { 'x' => 'royal',
 		     't' => 'a' },
 'realidad'	=> { 'x' => 'reality',
 		     't' => 'n' },
 'realizar'	=> { 'x' => 'achieve',
		     't' => 'v' },
 'realmente'	=> { 'x' => 'really',
 		     't' => 'adv' },
 'realzar'	=> { 'x' => 'raise',
		     '#' => 'enhance',
		     't' => 'v' },
 'rebelin'	=> { 'x' => 'rebellion',
 		     't' => 'n' },
 'recaudacin'	=> { 'x' => 'collection',
		     '#' => 'takings',
 		     't' => 'n',
		     'g' => 'f' },
 'recibir'	=> { 'x' => 'receive',
		     't' => 'v' },
 'recoger'	=> { 'x' => 'collect',
		     '#' => 'harvest, gather, pick up',
 		     't' => 'v',
		     '#' => 'refl. retire' },
 'recopilar'	=> { 'x' => 'compile',
		     't' => 'v' },
 'recordar'	=> { 'x' => 'remember',
		     't' => 'v' },
 'recorrer'	=> { 'x' => 'search',
		     '#' => 'cross, travel through, cover',
 		     't' => 'v',
		     '#' => 'look over' },
 'recompilar'	=> { 'x' => 'recompile',
 		     't' => 'v' },
 'recuerdo'	=> { 'x' => 'souvenir',
 		     't' => 'n' },
 'recuerdos'	=> { 'x' => 'memories',
 		     't' => 'n',
		     'n' => 'p' },
 'recurrir'	=> { 'x' => 'appeal',
		     '#' => '~ a = resort to, turn to',
 		     't' => 'v' },
 'recurso'	=> { 'x' => 'resort',
		     '#' => 'resort',
 		     't' => 'n' },
 'recursos'	=> { 'x' => 'means',
		     '#' => 'resources',
 		     't' => 'n',
		     'n' => 'p' },
 'red'		=> { 'x' => 'net',
		     't' => 'n',
		     'g' => 'f' },
 'redondo'	=> { 'x' => 'round',
		     't' => 'a' },
 'reducir'	=> { 'x' => 'reduce',
		     '#' => 'limit',
		     't' => 'v' },
 'reembolsar'	=> { 'x' => 'refund',
		     't' => 'v' },
 'reembolso'	=> { 'x' => 'refund',
		     't' => 'n' },
 'reemplazar'	=> { 'x' => 'replace',
		     't' => 'v' },
 'reflexin'	=> { 'x' => 'reflection',
 		     't' => 'n',
		     'g' => 'f' },
 'reflexionar'	=> { 'x' => 'reflect',
		     '#' => 'reflect on, pause (to think)',
 		     't' => 'v' },
 'regalar'	=> { 'x' => 'give',
		     '#' => 'give away',
 		     't' => 'v' },
 'regalo'	=> { 'x' => 'present',
		     '#' => 'gift; also vers. regalar',
 		     't' => 'n' },
 'regaar'	=> { 'x' => 'chide',
		     't' => 'v' },
 'regar'	=> { 'x' => 'scatter',
		     '#' => 'water, sprinkle',
 		     't' => 'v' },
 'regla'	=> { 'x' => 'rule',
		     '#' => 'regulation',
 		     't' => 'n' },
 'reglamento'	=> { 'x' => 'rules',
		     '#' => 'regulations',
 		     't' => 'n' },
 'regresar'	=> { 'x' => 'return',
		     '#' => 'come back, go back',
 		     't' => 'v' },
 'reina'	=> { 'x' => 'quen',
		     't' => 'n' },
 'rer'		=> { 'x' => 'laugh',
		     '#' => 'irreg. accented infin. breaks grammar rules',
 		     't' => 'v' },
  'reir'	=> { 'x' => 'laugh',
		     '#' => 'virtual base fut. and condit.',
 		     't' => 'v' },
  'rir'	=> { 'x' => 'laugh',
		     '#' => 'virtual base pres. and pres. subj.',
 		     't' => 'v' },
 'relacin'	=> { 'x' => 'relationship',
 		     't' => 'n' },
 'relativo'	=> { 'x' => 'relative',
 		     't' => 'a' },
 'religin'	=> { 'x' => 'religion',
 		     't' => 'n' },
 'reloj'	=> { 'x' => 'clock',
		     '#' => 'watch',
 		     't' => 'n' },
 'remar'	=> { 'x' => 'row',
 		     't' => 'v' },
 'rendimiento'	=> { 'x' => 'efficiency',
		     '#' => 'output',
 		     't' => 'n' },
 'repartir'	=> { 'x' => 'distribute',
		     '#' => 'deliver',
 		     't' => 'v' },
 'repetir'	=> { 'x' => 'repeat',
 		     't' => 'v' },
 'reparto'	=> { 'x' => 'delivery',
		     '#' => 'distribution',
 		     't' => 'n' },
 'representar'	=> { 'x' => 'represent',
		     '#' => 'perform, look',
		     't' => 'v' },
 'reprochar'	=> { 'x' => 'reproach',
		     't' => 'v' },
 'res'		=> { 'x' => 'animal',
		     '#' => 'beast',
 		     't' => 'n',
		     'g' => 'm' },
 'reserva'	=> { 'x' => 'reserve',
		     '#' => 'reservation',
 		     't' => 'n' },
 'residir'	=> { 'x' => 'reside',
		     '#' => 'live',
 		     't' => 'v' },
 'resistir'	=> { 'x' => 'resist',
 		     't' => 'v' },
 'resolucin'	=> { 'x' => 'resolution',
 		     't' => 'n',
		     'g' => 'f' },
 'resolver'	=> { 'x' => 'resolve',
 		     't' => 'v' },
 'respectivamente'	=> { 'x' => 'respectively',
 			     't' => 'adv' },
 'respecto'	=> { 'x' => 'respect',
		     '#' => 'as in "with respect"',
 		     't' => 'n',
		     '#' => 'cf. respeto' },
 'respeto'	=> { 'x' => 'respect',
		     '#' => 'deference',
 		     't' => 'n' },
 'respirar'	=> { 'x' => 'breathe',
		     't' => 'v' },
 'responder'	=> { 'x' => 'respond',
		     '#' => 'answer',
 		     't' => 'v' },
 'respuesta'	=> { 'x' => 'reply',
		     '#' => 'answer',
 		     't' => 'n' },
 'resto'	=> { 'x' => 'rest',
		     '#' => 'remainder, el resto = the rest',
 		     't' => 'n' },
 'resumen'	=> { 'x' => 'rsum',
		     '#' => 'summary',
 		     't' => 'n',
		     'g' => 'm' },
 'resultado'	=> { 'x' => 'result',
		     't' => 'n' },
 'retener'	=> { 'x' => 'withhold',
		     't' => 'v' },
 'retenga'	=> { 'x' => '(it) withheld',
		     'r' => 'retener',
		     't' => 'v' },
 'retrete'	=> { 'x' => 'toilet',
 		     't' => 'n' },
 'reunir'	=> { 'x' => 'meet',
		     't' => 'v' },
 'revancha'	=> { 'x' => 'revenge',
 		     't' => 'n' },
 'revista'	=> { 'x' => 'magazine',
		     '#' => 'review',
 		     't' => 'n' },
 'revolcar'	=> { 'x' => 'roll',
		     '#' => 'only refl. revolcarse = roll about',
 		     't' => 'v' },
 'revolucin'	=> { 'x' => 'revolution',
 		     't' => 'n' },
 'revolucionario'	=> { 'x' => 'revolutionary',
		     '#' => 'noun or adjective?' },
 'rey'		=> { 'x' => 'king',
		     '#' => 'person',
 		     't' => 'n' },
 'rico'		=> { 'x' => 'rich',
		     't' => 'a' },
 'riesgo'	=> { 'x' => 'risk',
		     't' => 'n' },
 'rigor'	=> { 'x' => 'rigour',
 		     't' => 'n',
		     'g' => 'm' },
 'riguroso'	=> { 'x' => 'rigorous',
		     '#' => 'harsh, severe',
 		     't' => 'a' },
 'rincn'	=> { 'x' => 'corner',
		     't' => 'n' },
 'ro'		=> { 'x' => 'river',
		     '#' => 'also vers. "reir"',
 		     't' => 'n' },
 'risa'		=> { 'x' => 'laugh',
		     '#' => 'laughter',
 		     't' => 'n' },
 'ritmo'	=> { 'x' => 'rythm',
		     't' => 'n' },
 'robar'	=> { 'x' => 'steal',
		     '#' => 'rob',
 		     't' => 'v' },
 'roca'		=> { 'x' => 'rock',
		     't' => 'n' },
 'rodar'	=> { 'x' => 'roll',
		     '#' => 'wheel, travel, shoot (film)',
 		     't' => 'v' },
 'rodear'	=> { 'x' => 'surround',
		     't' => 'v' },
 'rodilla'	=> { 'x' => 'knee',
 		     't' => 'n' },
 'rojo'		=> { 'x' => 'red',
		     't' => 'a' },
 'rol'		=> { 'x' => 'list',
		     '#' => 'roll, role',
 		     't' => 'n',
		     'g' => 'm' },
 'roma'		=> { 'x' => 'rome',
		     't' => 'n' },
 'romano'	=> { 'x' => 'roman',
		     '#' => 'adj. and noun?' },
 'romper'	=> { 'x' => 'break',
		     't' => 'v' },
 'ropa'		=> { 'x' => 'clothes',
		     't' => 'n' },
 'rosa'		=> { 'x' => 'rose',
 		     't' => 'n' },
 'rosado'	=> { 'x' => 'pink',
		     '#' => 'also noun ros',
		     '#' => 'capitalized or spelled with z?',
 		     't' => 'a' },
 'roto'		=> { 'x' => 'broken',
		     't' => 'a' },
 'rotundo'	=> { 'x' => 'emphatic',
		     '#' => 'round',
 		     't' => 'a' },
 'rubio'	=> { 'x' => 'blonde',
		     '#' => 'adj. and noun?' },
 'rueda'	=> { 'x' => 'wheel',
		     't' => 'n' },
 'ruina'	=> { 'x' => 'ruin',
 		     't' => 'n' },
 'ruido'	=> { 'x' => 'noise',
		     '#' => 'noisy; noun or adjective' },
 'ruso'		=> { 'x' => 'russian',
		     '#' => 'adj. and noun?' },
 'ruta'		=> { 'x' => 'route',
 		     't' => 'n' },
 'saber'	=> { 'x' => 'know',
		     '#' => 'cf conocer',
		     't' => 'v' },
  's'		=> { 'x' => '(i) know',
		     '#' => 'cf "se" refl. pron.',
		     'r' => 'saber',
 		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 'p' },
  'seper'	=> { 'x' => 'know',
		     '#' => 'virtual base pres. subj.',
  		     'r' => 'saber',
		     't' => 'v' },
  'sabr'	=> { 'x' => 'know',
		     '#' => 'virtual base fut. and condit.',
  		     'r' => 'saber',
		     't' => 'v' },
  'super'	=> { 'x' => 'know',
		     '#' => 'virtual base pret.',
  		     'r' => 'saber',
		     't' => 'v' },
  'supe'	=> { 'x' => '(i) knew',
  		     'r' => 'saber',
		     't' => 'v',
		     'p' => '1',
		     'n' => 's',
		     'c' => 't' },
  'supo'	=> { 'x' => '(he) knew',
  		     'r' => 'saber',
		     't' => 'v',
		     'p' => '3',
		     'n' => 's',
		     'c' => 't' },
 'sabio'	=> { 'x' => 'wise',
		     '#' => 'learned',
 		     't' => 'a' },
 'sabor'	=> { 'x' => 'flavour',
 		     't' => 'n' },
 'sacar'	=> { 'x' => 'take out',
		     '#' => 'get, get out',
		     't' => 'v' },
 'sal'		=> { 'x' => 'salt',
 		     't' => 'n',
		     'g' => 'f' },
 'sala'		=> { 'x' => 'room',
		     '#' => '(big) room',
		     't' => 'n' },
 'salir'	=> { 'x' => 'leave',
		     't' => 'v' },
 'saln'	=> { 'x' => 'lounge',
		     't' => 'n' },
 'saltar'	=> { 'x' => 'jump',
		     't' => 'v' },
 'saludar'	=> { 'x' => 'greet',
		     '#' => 'salute',
		     't' => 'v' },
 'salvador'	=> { 'x' => 'saviour',
 		     't' => 'n' },
 'salvacin'	=> { 'x' => 'salvation',
 		     't' => 'n',
		     'g' => 'f' },
 'salvar'	=> { 'x' => 'save',
		     '#' => 'rescue, ...',
 		     't' => 'v' },
 'sangre'	=> { 'x' => 'blood',
 		     't' => 'n' },
 'sangra'	=> { 'x' => 'sangria',
 		     't' => 'n' },
 'santiago'	=> { 'x' => 'santiago',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'santo'	=> { 'x' => 'saint',
		     '#' => 'person',
 		     't' => 'n' },
 'secar'	=> { 'x' => 'dry',
 		     't' => 'v' },
 'seco'		=> { 'x' => 'dry',
 		     't' => 'a' },
 'sed'		=> { 'x' => 'thirst',
		     't' => 'n' },
 'seda'		=> { 'x' => 'silk',
		     't' => 'n' },
 'seguimiento'	=> { 'x' => 'continuation',
		     '#' => 'chase, pursuit',
 		     't' => 'n' },
 'seguir'	=> { 'x' => 'follow',
		     '#' => 'keep (doing)',
		     't' => 'v' },
  'siguir'	=> { 'x' => 'follow',
		     '#' => 'virtual base',
  		     'r' => 'seguir',
		     't' => 'v' },
 'seguro'	=> { 'x' => 'sure',
 		     't' => 'a' },
 'seleccionar'	=> { 'x' => 'select',
 		     't' => 'v' },
 'semana'	=> { 'x' => 'week',
		     't' => 'n' },
 'sembrar'	=> { 'x' => 'scatter',
		     '#' => 'sow, sprinkle, spread',
		     't' => 'v' },
 'semejante'	=> { 'x' => 'similar',
		     '#' => 'fellow man',
		     't' => 'a' },
 'semilla'	=> { 'x' => 'seed',
		     't' => 'n' },
 'sencillo'	=> { 'x' => 'natural',
		     '#' => 'simple, unaffected',
		     't' => 'a' },
 'sensibilidad'	=> { 'x' => 'sensibility',
 		     't' => 'n' },
 'sensato'	=> { 'x' => 'sensible',
		     't' => 'a' },
 'sentar'	=> { 'x' => 'sit',
		     't' => 'v' },
 'sentimental'	=> { 'x' => 'sentimental',
 		     't' => 'a' },
 'sentimiento'	=> { 'x' => 'feeling',
		     '#' => 'sentiment',
 		     't' => 'n' },
 'sentir'	=> { 'x' => 'feel',
		     't' => 'v' },
 'sea'		=> { 'x' => 'sign',
		     '#' => 'password (is this right - not contrasena?)',
		     't' => 'n' },
 'seal'	=> { 'x' => 'sign',
		     '#' => 'signal',
		     't' => 'n' },
 'seor'	=> { 'x' => 'gentleman',
		     '#' => 'person; mr., sir',
 		     't' => 'n' },
 'seora'	=> { 'x' => 'lady',
		     '#' => 'person; mrs.',
 		     't' => 'n' },
 'serie'	=> { 'x' => 'series',
		     '#' => 'sequence',
 		     't' => 'n',
		     'g' => 'f' },
 'serio'	=> { 'x' => 'serious',
		     't' => 'a' },
 'sereno'	=> { 'x' => 'calm',
		     '#' => 'serene',
		     't' => 'a' },
 'serpentear'	=> { 'x' => 'snake',
		     '#' => 'wriggle, wind',
		     't' => 'v' },
 'servidor'	=> { 'x' => 'server',
		     't' => 'n' },
 'servicio'	=> { 'x' => 'service',
 		     't' => 'n' },
 'servir'	=> { 'x' => 'serve',
 		     't' => 'v' },
 'seudnimo'	=> { 'x' => 'pseudonym',
		     't' => 'n' },
 'sexo'		=> { 'x' => 'sex',
 		     't' => 'n' },
 'sida'		=> { 'x' => 'aids',
		     't' => 'n' },
 'siento'	=> { 'x' => '(i) feel',
		     '#' => '"lo siento" is "I\'m sorry"',
		     't' => 'v',
		     'r' => 'sentir' },
 'siglo'	=> { 'x' => 'century',
		     't' => 'n' },
 'significar'	=> { 'x' => 'mean',
		     '#' => 'signify',
 		     't' => 'v' },
 'signo'	=> { 'x' => 'sign',
		     't' => 'n' },
 'sigo'		=> { 'x' => '(i) keep',
		     '#' => 'follow',
		     't' => 'v',
		     'r' => 'seguir' },
 'sigue'	=> { 'x' => '(it) keeps',
		     '#' => 'follows',
		     'r' => 'seguir',
		     't' => 'v' },
 'siguiendo'	=> { 'x' => 'following',
		     '#' => 'keeping, next; verb and adjective' },
 'siguiente'	=> { 'x' => 'next',
 		     't' => 'a' },
 'silencio'	=> { 'x' => 'silence',
		     't' => 'n' },
 'silvestre'	=> { 'x' => 'wild',
 		     't' => 'a' },
 'smbolo'	=> { 'x' => 'symbol',
		     't' => 'n' },
 'similar'	=> { 'x' => 'similar',
		     't' => 'a' },
 'simple'	=> { 'x' => 'simple',
		     't' => 'a' },
 'simplificar'	=> { 'x' => 'simplify',
		     't' => 'v' },
 'sin'		=> { 'x' => 'without',
		     '#' => '"sin embargo" = however, still',
		     't' => 'p',
		     '#' => 'preposition' },
 'sirena'	=> { 'x' => 'siren',
 		     't' => 'n' },
 'sistema'	=> { 'x' => 'system',
		     't' => 'n',
		     'g' => 'm' },
 'sitio'	=> { 'x' => 'site',
		     't' => 'n' },
 'sobrenatural'	=> { 'x' => 'supernatural',
 		     't' => 'a' },
 'sobrevivir'	=> { 'x' => 'survive',
 		     't' => 'v' },
 'sociedad'	=> { 'x' => 'society',
 		     't' => 'n' },
 'sofocar'	=> { 'x' => 'suffocate',
		     't' => 'v' },
 'software'	=> { 'x' => 'software',
		     't' => 'n' },
 'sol'		=> { 'x' => 'sun',
		     't' => 'n' },
 'soledad'	=> { 'x' => 'solitude',
		     '#' => 'loneliness',
 		     't' => 'n' },
 'solidaridad'	=> { 'x' => 'solidarity',
 		     't' => 'n' },
 'solidez'	=> { 'x' => 'solidity',
		     't' => 'v',
		     'g' => 'f' },
 'solitario'	=> { 'x' => 'lonely',
 		     't' => 'a' },
 'solo'		=> { 'x' => 'alone',
		     '#' => 'single' },
 'slo'		=> { 'x' => 'only' },
 'solucin'	=> { 'x' => 'solution',
 		     't' => 'n' },
 'sombra'	=> { 'x' => 'shadow',
		     '#' => 'shade',
 		     't' => 'n' },
 'sombrero'	=> { 'x' => 'hat',
		     't' => 'n' },
 'sombrilla'	=> { 'x' => 'parasol',
		     't' => 'n' },
 'sonar'	=> { 'x' => 'sound',
		     't' => 'v' },
  'suenar'	=> { 'x' => 'sound',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'sonar',
		     't' => 'v' },
 'sonrer'	=> { 'x' => 'smile',
		     '#' => 'irreg. accented infin. breaks grammar rules',
 		     't' => 'v' },
  'sonreir'	=> { 'x' => 'smile',
		     '#' => 'virtual base fut. and condit.',
 		     't' => 'v' },
  'sonrir'	=> { 'x' => 'smile',
		     '#' => 'virtual base pres. and pres. subj.',
 		     't' => 'v' },
 'sonrisa'	=> { 'x' => 'smile',
		     't' => 'n' },
 'soar'	=> { 'x' => 'dream',
 		     't' => 'v' },
  'suear'	=> { 'x' => 'dream',
		     '#' => 'virtual base pres., pres. subj.',
  		     'r' => 'soar',
		     't' => 'v' },
 'soportar'	=> { 'x' => 'support',
		     't' => 'v' },
 'soporte'	=> { 'x' => 'support',
		     't' => 'n',
		     'g' => 'm' },
 'sordo'	=> { 'x' => 'deaf',
		     '#' => 'adj. and noun' },
 'sordomudo'	=> { 'x' => 'deaf and dumb',
		     '#' => 'adj. and noun; multiword' },
 'sorprendente'	=> { 'x' => 'surprising',
 		     't' => 'a' },
 'sorprender'	=> { 'x' => 'surprise',
		     't' => 'v' },
 'sorpresa'	=> { 'x' => 'surprise',
		     't' => 'n' },
 'sostener'	=> { 'x' => 'support',
		     '#' => 'maintain',
 		     't' => 'v',
		     '#' => 'irreg.' },
 'suscribir'	=> { 'x' => 'subscribe',
 		     't' => 'v' },
 'suceder'	=> { 'x' => 'happen',
		     '#' => 'succeed, follow',
 		     't' => 'v' },
 'sucesin'	=> { 'x' => 'succession',
		     '#' => 'sequence, series',
 		     't' => 'n',
		     'g' => 'f' },
 'sucio'	=> { 'x' => 'dirty',
 		     't' => 'a' },
 'suecia'	=> { 'x' => 'sweden',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'sueco'	=> { 'x' => 'swedish',
		     '#' => 'adj. and noun?' },
 'suela'	=> { 'x' => 'sole',
 		     't' => 'n' },
 'suelo'	=> { 'x' => 'ground',
		     '#' => 'floor',
		     't' => 'n' },
 'sueo'	=> { 'x' => 'dream',
 		     't' => 'n' },
 'suerte'	=> { 'x' => 'luck',
		     '#' => 'chance, fate, destiny',
		     't' => 'n' },
 'suficiente'	=> { 'x' => 'enough',
		     '#' => 'sufficient',
		     't' => 'a' },
 'sufrir'	=> { 'x' => 'suffer',
		     '#' => 'bear',
		     't' => 'v' },
 'sugerencia'	=> { 'x' => 'suggestion',
		     't' => 'n' },
 'sugestivo'	=> { 'x' => 'fascinating',
		     '#' => 'stimulating',
 		     't' => 'a' },
 'suiza'	=> { 'x' => 'switzerland',
		     '#' => 'proper, place',
 		     't' => 'n' },
 'sujeto'	=> { 'x' => 'subject',
		     '#' => 'secure, fastened',
		     't' => 'n' },
 'supe'		=> { 'x' => '(i) knew',
		     't' => 'v',
		     'r' => 'saber' },
 'sper'	=> { 'x' => 'super',
 		     't' => 'a' },
 'superar'	=> { 'x' => 'surpass',
		     '#' => 'overcome',
		     't' => 'v' },
 'supermercado'	=> { 'x' => 'supermarket',
		     't' => 'n' },
 'suponer'	=> { 'x' => 'suppose',
		     't' => 'v' },
 'supongo'	=> { 'x' => '(i) suppose',
		     't' => 'v',
		     'r' => 'suponer' },
 'sur'		=> { 'x' => 'south',
		     '#' => 'noun and adj./adv.?',
		     't' => 'n' },
 'surgir'	=> { 'x' => 'arise',
		     't' => 'v' },
 'suspirar'	=> { 'x' => 'sigh',
 		     't' => 'v' },
 'suspiro'	=> { 'x' => 'sigh',
 		     't' => 'n' },
 'sustantivo'	=> { 'x' => 'noun',
		     '#' => 'nombre',
 		     't' => 'n' },
 'sutil'	=> { 'x' => 'subtle',
 		     't' => 'a' },
 'tabla'	=> { 'x' => 'panel',
		     '#' => 'plank, shelf, pleat, table?',
		     't' => 'n' },
 'tacto'	=> { 'x' => 'touch',
 		     't' => 'n' },
 'tailands'	=> { 'x' => 'thai',
		     '#' => 'noun and adj.?' },
 'tailandia'	=> { 'x' => 'thailand',
		     '#' => 'proper; place',
		     't' => 'n' },
 'taln'	=> { 'x' => 'heel',
		     '#' => 'cheque',
 		     't' => 'n' },
 'tallarines'	=> { 'x' => 'noodles',
		     't' => 'n' },
 'tamao'	=> { 'x' => 'size',
		     't' => 'n' },
 'tambor'	=> { 'x' => 'drum',
 		     't' => 'n',
		     'g' => 'm' },
 'tango'	=> { 'x' => 'tango',
 		     't' => 'n' },
 'tantear'	=> { 'x' => 'reckon',
		     '#' => 'try, test, weigh up',
		     't' => 'v' },
 'tapa'		=> { 'x' => 'top',
		     '#' => 'lid',
		     't' => 'n' },
 'tapicera'	=> { 'x' => 'upholstery',
		     '#' => 'tapestry',
		     't' => 'n' },
 'tardar'	=> { 'x' => 'delay',
		     '#' => 'be late',
 		     't' => 'v' },
 'tarde'	=> { 'x' => 'afternoon',
		     '#' => 'late (noun and adjective)' },
 'tarjeta'	=> { 'x' => 'card',
		     't' => 'n' },
 'tatuaje'	=> { 'x' => 'tattoo',
 		     't' => 'n' },
 'taza'		=> { 'x' => 'cup',
		     't' => 'n' },
 'tecla'	=> { 'x' => 'key',
		     't' => 'n' },
 'teclado'	=> { 'x' => 'keyboard',
		     't' => 'n' },
 'tecnologa'	=> { 'x' => 'technology',
		     't' => 'n' },
 'techo'	=> { 'x' => 'roof',
		     't' => 'n' },
 'tcnico'	=> { 'x' => 'technical',
		     't' => 'a' },
 'teja'		=> { 'x' => 'tile',
 		     't' => 'n' },
 'tejado'	=> { 'x' => 'roof',
		     '#' => 'tiled roof',
 		     't' => 'n' },
 'telfono'	=> { 'x' => 'telephone',
 		     't' => 'n' },
 'televisin'	=> { 'x' => 'television',
 		     't' => 'n',
		     'g' => 'f' },
 'tema'		=> { 'x' => 'theme',
		     't' => 'n' },
 'tenedor'	=> { 'x' => 'fork',
 		     't' => 'n' },
 'tender'	=> { 'x' => 'extend',
		     '#' => 'spread out, stretch',
		     't' => 'v' },
 'tentacin'	=> { 'x' => 'temptation',
 		     't' => 'n' },
 'tenue'	=> { 'x' => 'thin',
		     '#' => 'slender, light',
		     't' => 'a' },
 'tericamente'	=> { 'x' => 'theoretically',
		     't' => 'adv' },
 'tequila'	=> { 'x' => 'tequila',
 		     't' => 'n' },
 'terminal'	=> { 'x' => 'terminal',
		     't' => 'n' },
 'terminar'	=> { 'x' => 'finish',
		     't' => 'v' },
 'trmino'	=> { 'x' => 'end',
		     '#' => 'terminus, conclusion, boundary',
 		     't' => 'n' },
 'terraza'	=> { 'x' => 'balcony',
		     't' => 'n' },
 'terrn'	=> { 'x' => 'lump',
		     '#' => 'clod',
 		     't' => 'v',
		     'g' => 'm' },
 'testarudo'	=> { 'x' => 'stubborn',
 		     't' => 'a' },
 'texto'	=> { 'x' => 'text',
		     't' => 'n' },
 'tiempo'	=> { 'x' => 'time',
		     't' => 'n' },
 'tierra'	=> { 'x' => 'earth',
		     't' => 'n' },
 'ta'		=> { 'x' => 'aunt',
		     't' => 'n' },
 'tijuana'	=> { 'x' => 'tijuana',
		     '#' => 'place; proper',
 		     't' => 'n' },
 'to'		=> { 'x' => 'uncle',
		     't' => 'n' },
 'tipo'		=> { 'x' => 'type',
		     't' => 'n' },
 'tipografa'	=> { 'x' => 'typography',
		     't' => 'n' },
 'tirar'	=> { 'x' => 'throw',
		     't' => 'v' },
 'tiro'		=> { 'x' => 'shot',
		     '#' => 'throw',
 		     't' => 'n' },
 'tiroteo'	=> { 'x' => 'shooting',
		     '#' => 'exchange of shots',
 		     't' => 'n' },
 'titular'	=> { 'x' => 'headline',
		     '#' => 'also m/f holder',
 		     't' => 'n',
		     'g' => 'm' },
 'ttulo'	=> { 'x' => 'title',
		     't' => 'n' },
 'tocar'	=> { 'x' => 'touch',
		     '#' => 'play instrument',
		     't' => 'v' },
 'tocino'	=> { 'x' => 'bacon',
		     't' => 'n' },
 'tomar'	=> { 'x' => 'take',
		     't' => 'v' },
 'tonto'	=> { 'x' => 'silly',
 		     't' => 'a',
		     '#' => 'slang, mexican or latin?' },
 'torcer'	=> { 'x' => 'twist',
		     '#' => 'turn, sprain',
 		     't' => 'v' },
 'tormenta'	=> { 'x' => 'storm',
		     '#' => 'turmoil',
		     't' => 'n' },
 'tornar'	=> { 'x' => 'return',
		     '#' => 'give back, transform, go back; refl. become',
 		     't' => 'v' },
 'toro'		=> { 'x' => 'bull',
		     '#' => 'animal',
 		     't' => 'n' },
 'torpe'	=> { 'x' => 'clumsy',
		     '#' => 'awkward, dim, slow',
 		     't' => 'a' },
 'trabajar'	=> { 'x' => 'work',
		     't' => 'v' },
 'trabajo'	=> { 'x' => 'work',
		     't' => 'n' },
 'trabajlico'	=> { 'x' => 'workaholic',
		     '#' => 'adj & noun' },
 'trabajoso'	=> { 'x' => 'hard',
 		     't' => 'a' },
 'traducir'	=> { 'x' => 'translate',
 		     't' => 'v' },
 'traductor'	=> { 'x' => 'translator',
 		     't' => 'n' },
 'traer'	=> { 'x' => 'bring',
		     '#' => 'carry',
		     't' => 'v' },
 'trajeron'	=> { 'x' => '(they) carried',
		     't' => 'v',
		     'r' => 'traer' },
 'tranco'	=> { 'x' => 'stride',
		     't' => 'n' },
 'tranquilo'	=> { 'x' => 'quiet',
		     '#' => 'tranquil',
 		     't' => 'a' },
 'transcurrir'	=> { 'x' => 'pass',
		     '#' => 'take place',
 		     't' => 'v' },
 'transformar'	=> { 'x' => 'transform',
 		     't' => 'v' },
 'transicin'	=> { 'x' => 'transition',
 		     't' => 'n' },
 'transitar'	=> { 'x' => 'go',
		     '#' => 'from place to place, transit',
		     't' => 'v' },
 'transmisin'	=> { 'x' => 'transmission',
 		     't' => 'n',
		     'g' => 'f' },
 'transmitir'	=> { 'x' => 'transmit',
		     't' => 'v' },
 'trquea'	=> { 'x' => 'windpipe',
		     '#' => 'trachea',
 		     't' => 'n' },
 'traste'	=> { 'x' => 'fret',
		     '#' => '(guitar)',
 		     't' => 'n',
		     '#' => 'another meaning in "ciega sordomuda"' },
 'tratar'	=> { 'x' => 'try',
		     't' => 'v' },
 'tremendo'	=> { 'x' => 'terrible',
		     't' => 'a' },
 'tren'		=> { 'x' => 'train',
 		     't' => 'n' },
 'trepar'	=> { 'x' => 'climb',
		     't' => 'v' },
 'triste'	=> { 'x' => 'sad',
		     't' => 'a' },
 'tristeza'	=> { 'x' => 'sadness',
		     '#' => 'melancholy',
 		     't' => 'n' },
 'triunfar'	=> { 'x' => 'triumph',
 		     't' => 'v' },
 'triunfo'	=> { 'x' => 'triumph',
 		     't' => 'n' },
 'tropezar'	=> { 'x' => 'trip',
		     '#' => 'stumble',
		     't' => 'v' },
  'tropiezar'	=> { 'x' => 'trip',
		     '#' => 'virtual base',
  		     'r' => 'tropezar',
		     't' => 'v' },
 'truco'	=> { 'x' => 'trick',
		     '#' => 'knack',
 		     't' => 'n' },
 'turco'	=> { 'x' => 'turkish',
		     '#' => 'adj. and noun?' },
 'turista'	=> { 'x' => 'tourist',
		     't' => 'n' },
 'ltimo'	=> { 'x' => 'last',
		     't' => 'a',
		     '#' => 'can use as a noun' },
 'unido'	=> { 'x' => 'united',
		     't' => 'a' },
 'unin'	=> { 'x' => 'union',
		     't' => 'n',
		     'g' => 'f' },
 'universal'	=> { 'x' => 'universal',
 		     't' => 'a' },
 'universidad'	=> { 'x' => 'university',
		     't' => 'n',
		     'g' => 'f' },
 'ua'		=> { 'x' => 'nail',
		     '#' => 'claw',
 		     't' => 'n' },
 'uruguay'	=> { 'x' => 'uruguay',
		     '#' => 'proper; place',
 		     't' => 'n' },
 'urgente'	=> { 'x' => 'urgent',
		     't' => 'a' },
 'usar'		=> { 'x' => 'use',
		     't' => 'v' },
 'uso'		=> { 'x' => 'use',
		     't' => 'n' },
 'usual'	=> { 'x' => 'usual',
 		     't' => 'a' },
 'usuario'	=> { 'x' => 'user',
		     't' => 'n' },
 'til'		=> { 'x' => 'useful',
		     't' => 'a' },
 'utilidad'	=> { 'x' => 'utility',
		     't' => 'n' },
 'utilizar'	=> { 'x' => 'use',
		     '#' => 'utilize',
		     't' => 'v' },
 'vaca'		=> { 'x' => 'cow',
		     '#' => 'animal',
 		     't' => 'n' },
 'vaco'	=> { 'x' => 'empty',
		     't' => 'a' },
 'vaina'	=> { 'x' => 'sheath',
 		     't' => 'n' },
 'valer'	=> { 'x' => 'be worth',
		     '#' => '(2 words)',
		     't' => 'v' },
 'valor'	=> { 'x' => 'courage',
		     '#' => 'value, worth, price, valour',
 		     't' => 'n',
		     'g' => 'm' },
 'vlvula'	=> { 'x' => 'valve',
		     't' => 'n' },
 'valle'	=> { 'x' => 'valley',
		     't' => 'n' },
 'variante'	=> { 'x' => 'variant',
 		     't' => 'n',
		     'g' => 'f' },
 'varianza'	=> { 'x' => 'variance',
 		     't' => 'n' },
 'varias'	=> { 'x' => 'several',
		     '#' => 'various, (others)',
 		     'g' => 'f',
		     'n' => 'p' },
 'variedad'	=> { 'x' => 'variety',
		     't' => 'n' },
 'varios'	=> { 'x' => 'several',
		     '#' => 'various, (others)',
 		     'g' => 'm',
		     'n' => 'p' },
 'vaso'		=> { 'x' => 'glass',
		     '#' => 'container',
		     't' => 'n' },
 'vasto'	=> { 'x' => 'vast',
		     '#' => 'huge',
 		     't' => 'a' },
 'vecino'	=> { 'x' => 'neighbour',
		     '#' => 'person; resident',
 		     't' => 'n' },
 'vehculo'	=> { 'x' => 'vehicle',
 		     't' => 'n' },
 'velocidad'	=> { 'x' => 'velocity',
		     '#' => 'speed',
 		     't' => 'n' },
 'vendedor'	=> { 'x' => 'seller',
		     't' => 'n' },
 'vender'	=> { 'x' => 'sell',
		     't' => 'v' },
 'venir'	=> { 'x' => 'come',
		     't' => 'v' },
  'vienir'	=> { 'x' => 'come',
		     '#' => 'virtual base for pres.',
		     't' => 'v' },
  'vengir'	=> { 'x' => 'come',
		     '#' => 'virtual base for pres. subj. and "i come"',
		     't' => 'v' },
  'vendr'	=> { 'x' => 'come',
		     '#' => 'virtual base for fut. and condit.',
		     't' => 'v' },
 'ventaja'	=> { 'x' => 'advantage',
		     't' => 'n' },
 'ventana'	=> { 'x' => 'window',
		     't' => 'n' },
 'ver'		=> { 'x' => 'see',
		     '#' => 'look',
		     't' => 'v' },
  'veo'		=> { 'x' => '(i) see',
		     't' => 'v',
		     'r' => 'ver' },
  'vi'		=> { 'x' => '(i) saw',
		     't' => 'v',
		     'r' => 'ver' },
  'viste'	=> { 'x' => '(you) saw',
		     't' => 'v',
		     'r' => 'ver' },
  'vio'		=> { 'x' => '(it) saw',
		     't' => 'v',
		     'r' => 'ver' },
  'visto'	=> { 'x' => 'seen',
		     't' => 'v',
		     'r' => 'ver' },
 'verbo'	=> { 'x' => 'verb',
 		     't' => 'n' },
 'verdad'	=> { 'x' => 'truth',
		     '#' => 'true',
		     't' => 'n' },
 'vereda'	=> { 'x' => 'path',
		     '#' => 'pavement, sidewalk, footpath',
		     't' => 'n' },
 'verde'	=> { 'x' => 'green',
		     't' => 'a' },
 'vestido'	=> { 'x' => 'dress',
		     't' => 'n' },
 'versin'	=> { 'x' => 'version',
		     't' => 'n',
		     'g' => 'f' },
 'vestir'	=> { 'x' => 'dress',
		     '#' => 'wear',
		     't' => 'v' },
 'vestbulo'	=> { 'x' => 'hall',
		     '#' => 'foyer',
		     't' => 'n' },
 'vez'		=> { 'x' => 'time',
		     't' => 'n',
		     'g' => 'f' },
 'viajar'	=> { 'x' => 'travel',
		     't' => 'v' },
 'viajero'	=> { 'x' => 'traveller',
		     't' => 'n' },
 'vida'		=> { 'x' => 'life',
		     't' => 'n' },
 'vidrio'	=> { 'x' => 'glass',
		     't' => 'n' },
 'viejo'	=> { 'x' => 'old',
		     't' => 'a' },
 'viento'	=> { 'x' => 'wind',
 		     't' => 'n' },
 'vilo'		=> { 'x' => 'air',
		     '#' => '(en vilo = in the air)',
		     't' => 'n' },
 'vincular'	=> { 'x' => 'link',
		     '#' => 'bond',
 		     't' => 'v' },
 'vnculo'	=> { 'x' => 'link',
		     '#' => 'bond',
 		     't' => 'n' },
 'vino'		=> { 'x' => 'wine',
		     '#' => 'he/she/it came',
		     't' => 'n' },
 'virtud'	=> { 'x' => 'virtue',
 		     't' => 'n',
		     'g' => 'f' },
 'virtuoso'	=> { 'x' => 'virtuous',
		     '#' => 'also n virtuoso',
 		     't' => 'a' },
 'visa'		=> { 'x' => 'visa',
		     't' => 'n' },
 'visera'	=> { 'x' => 'visor',
		     't' => 'n' },
 'visin'	=> { 'x' => 'vision',
 		     't' => 'n' },
 'visita'	=> { 'x' => 'visit',
		     't' => 'n' },
 'visitar'	=> { 'x' => 'visit',
		     't' => 'v' },
 'vista'	=> { 'x' => 'sight',
		     '#' => 'view',
		     't' => 'n' },
 'vistazo'	=> { 'x' => 'glance',
 		     't' => 'n' },
 'vivir'	=> { 'x' => 'live',
		     't' => 'v' },
 'volante'	=> { 'x' => 'flying',
		     '#' => 'steering wheel, balance' },
 'volar'	=> { 'x' => 'fly',
 		     't' => 'v' },
 'voluntario'	=> { 'x' => 'volunteer',
		     '#' => 'adj. adn noun; voluntary' },
 'volver'	=> { 'x' => 'return',
		     '#' => 'irreg.',
		     't' => 'v' },
  'vuelto'	=> { 'x' => 'returned',
  		     'r' => 'volver',
		     't' => 'v',
		     'c' => 'pp' },
  'vuelver'	=> { 'x' => 'return',
		     '#' => 'virtual base',
  		     'r' => 'volver',
		     't' => 'v' },
 'votar'	=> { 'x' => 'vote',
		     't' => 'v' },
 'voz'		=> { 'x' => 'voice',
 		     't' => 'n' },
 'vuelta'	=> { 'x' => 'turn',
		     't' => 'n' },
 'zapato'	=> { 'x' => 'shoe',
		     't' => 'n' },
 'zona'		=> { 'x' => 'zone',
		     't' => 'n' },
 'zumbido'	=> { 'x' => 'buzzing' },
);
}

1;

